package org.jboss.errai.bus.client.framework;

import java.lang.annotation.Annotation;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.enterprise.event.Event;
import org.ext.uberfire.social.activities.model.PagedSocialQuery;
import org.ext.uberfire.social.activities.model.SocialActivitiesEvent;
import org.ext.uberfire.social.activities.model.SocialEventType;
import org.ext.uberfire.social.activities.model.SocialPaged;
import org.ext.uberfire.social.activities.model.SocialUser;
import org.ext.uberfire.social.activities.service.SocialActivitiesAPI;
import org.ext.uberfire.social.activities.service.SocialAdapter;
import org.ext.uberfire.social.activities.service.SocialEventTypeRepositoryAPI;
import org.ext.uberfire.social.activities.service.SocialPredicate;
import org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI;
import org.ext.uberfire.social.activities.service.SocialTimelineRulesQueryAPI;
import org.ext.uberfire.social.activities.service.SocialTypeTimelinePagedRepositoryAPI;
import org.ext.uberfire.social.activities.service.SocialUserRepositoryAPI;
import org.ext.uberfire.social.activities.service.SocialUserServiceAPI;
import org.ext.uberfire.social.activities.service.SocialUserTimelinePagedRepositoryAPI;
import org.ext.uberfire.social.activities.service.StressTestAPI;
import org.guvnor.common.services.project.builder.model.BuildResults;
import org.guvnor.common.services.project.builder.model.IncrementalBuildResults;
import org.guvnor.common.services.project.builder.service.BuildService;
import org.guvnor.common.services.project.model.GAV;
import org.guvnor.common.services.project.model.MavenRepositoryMetadata;
import org.guvnor.common.services.project.model.Module;
import org.guvnor.common.services.project.model.ModuleRepositories;
import org.guvnor.common.services.project.model.POM;
import org.guvnor.common.services.project.model.Package;
import org.guvnor.common.services.project.model.WorkspaceProject;
import org.guvnor.common.services.project.service.DeploymentMode;
import org.guvnor.common.services.project.service.ModuleRepositoriesService;
import org.guvnor.common.services.project.service.ModuleRepositoryResolver;
import org.guvnor.common.services.project.service.ModuleService;
import org.guvnor.common.services.project.service.POMService;
import org.guvnor.common.services.project.service.WorkspaceProjectSearchService;
import org.guvnor.common.services.project.service.WorkspaceProjectService;
import org.guvnor.common.services.shared.config.AppConfigService;
import org.guvnor.common.services.shared.config.ResourceConfigService;
import org.guvnor.common.services.shared.metadata.MetadataService;
import org.guvnor.common.services.shared.metadata.model.Metadata;
import org.guvnor.common.services.shared.test.TestRunnerService;
import org.guvnor.structure.navigator.FileNavigatorService;
import org.guvnor.structure.navigator.NavigatorContent;
import org.guvnor.structure.organizationalunit.OrganizationalUnit;
import org.guvnor.structure.organizationalunit.OrganizationalUnitSearchService;
import org.guvnor.structure.organizationalunit.OrganizationalUnitService;
import org.guvnor.structure.repositories.Branch;
import org.guvnor.structure.repositories.Repository;
import org.guvnor.structure.repositories.RepositoryEnvironmentConfigurations;
import org.guvnor.structure.repositories.RepositoryInfo;
import org.guvnor.structure.repositories.RepositorySearchService;
import org.guvnor.structure.repositories.RepositoryService;
import org.guvnor.structure.repositories.RepositoryServiceEditor;
import org.jboss.errai.bus.client.api.base.MessageBuilder;
import org.jboss.errai.bus.client.api.builder.RemoteCallSendable;
import org.jboss.errai.bus.client.api.messaging.MessageBus;
import org.jboss.errai.common.client.api.ErrorCallback;
import org.jboss.errai.common.client.api.RemoteCallback;
import org.jboss.errai.common.client.api.interceptor.InterceptedCall;
import org.jboss.errai.common.client.api.interceptor.RemoteCallContext;
import org.jboss.errai.common.client.framework.CallContextStatus;
import org.jboss.errai.common.client.framework.ProxyProvider;
import org.jboss.errai.common.client.framework.RemoteServiceProxyFactory;
import org.jboss.errai.common.client.util.AsyncBeanFactory;
import org.jboss.errai.common.client.util.CreationalCallback;
import org.jboss.errai.security.client.local.interceptors.AuthenticationServiceInterceptor;
import org.jboss.errai.security.shared.api.Group;
import org.jboss.errai.security.shared.api.Role;
import org.jboss.errai.security.shared.api.identity.User;
import org.jboss.errai.security.shared.service.AuthenticationService;
import org.jboss.errai.security.shared.service.NonCachingUserService;
import org.kie.soup.project.datamodel.imports.Imports;
import org.kie.workbench.common.forms.dynamic.model.config.SelectorData;
import org.kie.workbench.common.forms.dynamic.service.shared.BackendSelectorDataProviderService;
import org.kie.workbench.common.forms.dynamic.service.shared.FormRenderingContext;
import org.kie.workbench.common.services.datamodel.model.PackageDataModelOracleIncrementalPayload;
import org.kie.workbench.common.services.datamodel.service.IncrementalDataModelService;
import org.kie.workbench.common.stunner.bpmn.service.BPMNDiagramService;
import org.kie.workbench.common.stunner.bpmn.service.DataTypesService;
import org.kie.workbench.common.stunner.bpmn.service.ProjectType;
import org.kie.workbench.common.stunner.bpmn.workitem.service.WorkItemDefinitionLookupService;
import org.kie.workbench.common.stunner.core.diagram.Diagram;
import org.kie.workbench.common.stunner.core.graph.Element;
import org.kie.workbench.common.stunner.core.lookup.LookupManager.LookupResponse;
import org.kie.workbench.common.stunner.core.lookup.diagram.DiagramLookupRequest;
import org.kie.workbench.common.stunner.core.registry.factory.FactoryRegistry;
import org.kie.workbench.common.stunner.core.service.DiagramLookupService;
import org.kie.workbench.common.stunner.core.service.DiagramService;
import org.kie.workbench.common.stunner.core.service.FactoryService;
import org.kie.workbench.common.stunner.core.validation.ValidationService;
import org.kie.workbench.common.stunner.forms.service.FormGenerationService;
import org.uberfire.backend.authz.AuthorizationService;
import org.uberfire.backend.plugin.RuntimePluginService;
import org.uberfire.backend.vfs.DirectoryStream;
import org.uberfire.backend.vfs.DirectoryStream.Filter;
import org.uberfire.backend.vfs.Path;
import org.uberfire.backend.vfs.VFSLockService;
import org.uberfire.backend.vfs.VFSService;
import org.uberfire.backend.vfs.impl.LockInfo;
import org.uberfire.backend.vfs.impl.LockResult;
import org.uberfire.backend.vfs.impl.VFSCacheInterceptor;
import org.uberfire.experimental.service.backend.BackendExperimentalFeatureDefRegistry;
import org.uberfire.experimental.service.backend.BackendExperimentalFeaturesRegistryService;
import org.uberfire.experimental.service.backend.ExperimentalFeaturesSession;
import org.uberfire.experimental.service.definition.ExperimentalFeatureDefinition;
import org.uberfire.experimental.service.editor.EditableExperimentalFeature;
import org.uberfire.experimental.service.editor.FeaturesEditorService;
import org.uberfire.ext.apps.api.AppsPersistenceAPI;
import org.uberfire.ext.apps.api.Directory;
import org.uberfire.ext.editor.commons.file.DefaultMetadata;
import org.uberfire.ext.editor.commons.service.CopyService;
import org.uberfire.ext.editor.commons.service.DeleteService;
import org.uberfire.ext.editor.commons.service.RenameService;
import org.uberfire.ext.editor.commons.service.htmleditor.HtmlEditorService;
import org.uberfire.ext.editor.commons.version.VersionService;
import org.uberfire.ext.layout.editor.api.LayoutServices;
import org.uberfire.ext.layout.editor.api.PerspectiveServices;
import org.uberfire.ext.layout.editor.api.editor.LayoutTemplate;
import org.uberfire.ext.layout.editor.api.editor.LayoutTemplate.Style;
import org.uberfire.ext.plugin.model.DynamicMenu;
import org.uberfire.ext.plugin.model.LayoutEditorModel;
import org.uberfire.ext.plugin.model.Media;
import org.uberfire.ext.plugin.model.Plugin;
import org.uberfire.ext.plugin.model.PluginContent;
import org.uberfire.ext.plugin.model.PluginType;
import org.uberfire.ext.plugin.service.PluginServices;
import org.uberfire.ext.properties.editor.model.PropertyEditorCategory;
import org.uberfire.ext.properties.editor.service.BeanPropertyEditorBuilderService;
import org.uberfire.ext.security.management.api.AbstractEntityManager.SearchRequest;
import org.uberfire.ext.security.management.api.AbstractEntityManager.SearchResponse;
import org.uberfire.ext.security.management.api.GroupManagerSettings;
import org.uberfire.ext.security.management.api.RoleManagerSettings;
import org.uberfire.ext.security.management.api.UserManagerSettings;
import org.uberfire.ext.security.management.api.service.GroupManagerService;
import org.uberfire.ext.security.management.api.service.RoleManagerService;
import org.uberfire.ext.security.management.api.service.UserManagerService;
import org.uberfire.ext.services.shared.preferences.UserPreference;
import org.uberfire.ext.services.shared.preferences.UserPreferencesService;
import org.uberfire.ext.services.shared.preferences.UserPreferencesType;
import org.uberfire.java.nio.base.version.VersionRecord;
import org.uberfire.mvp.Command;
import org.uberfire.mvp.ParameterizedCommand;
import org.uberfire.preferences.shared.PreferenceScope;
import org.uberfire.preferences.shared.PreferenceScopeResolver;
import org.uberfire.preferences.shared.PreferenceStorage;
import org.uberfire.preferences.shared.PreferenceStore;
import org.uberfire.preferences.shared.bean.BasePreferencePortable;
import org.uberfire.preferences.shared.bean.PreferenceBeanServerStore;
import org.uberfire.preferences.shared.bean.PreferenceHierarchyElement;
import org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo;
import org.uberfire.preferences.shared.impl.PreferenceScopedValue;
import org.uberfire.security.authz.AuthorizationPolicy;
import org.uberfire.spaces.Space;
import org.uberfire.ssh.service.shared.editor.PortableSSHPublicKey;
import org.uberfire.ssh.service.shared.editor.SSHKeyEditorService;
import org.uberfire.workbench.model.PerspectiveDefinition;
import org.uberfire.workbench.model.SplashScreenFilter;
import org.uberfire.workbench.services.WorkbenchServices;

public class RpcProxyLoaderImpl implements RpcProxyLoader { public void loadProxies(final MessageBus bus) {
    class org_ext_uberfire_social_activities_service_SocialUserTimelinePagedRepositoryAPIImpl extends AbstractRpcProxy implements SocialUserTimelinePagedRepositoryAPI { public PagedSocialQuery getUserTimeline(final SocialUser a0, final SocialPaged a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialUserTimelinePagedRepositoryAPI").endpoint("getUserTimeline:org.ext.uberfire.social.activities.model.SocialUser:org.ext.uberfire.social.activities.model.SocialPaged:", qualifiers, new Object[] { a0, a1 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialUserTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PagedSocialQuery getUserTimeline(final SocialUser a0, final SocialPaged a1, final SocialPredicate a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialUserTimelinePagedRepositoryAPI").endpoint("getUserTimeline:org.ext.uberfire.social.activities.model.SocialUser:org.ext.uberfire.social.activities.model.SocialPaged:org.ext.uberfire.social.activities.service.SocialPredicate:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialUserTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PagedSocialQuery getUserTimeline(final SocialUser a0, final SocialPaged a1, final Map a2, final SocialPredicate a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialUserTimelinePagedRepositoryAPI").endpoint("getUserTimeline:org.ext.uberfire.social.activities.model.SocialUser:org.ext.uberfire.social.activities.model.SocialPaged:java.util.Map:org.ext.uberfire.social.activities.service.SocialPredicate:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialUserTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SocialUserTimelinePagedRepositoryAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_SocialUserTimelinePagedRepositoryAPIImpl();
      }
    });
    class org_guvnor_structure_repositories_RepositoryServiceImpl extends AbstractRpcProxy implements RepositoryService { public RepositoryInfo getRepositoryInfo(final Space a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getRepositoryInfo:org.uberfire.spaces.Space:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(RepositoryInfo.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getRepositoryHistory(final Space a0, final String a1, final int a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getRepositoryHistory:org.uberfire.spaces.Space:java.lang.String:int:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getRepositoryHistory(final Space a0, final String a1, final int a2, final int a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getRepositoryHistory:org.uberfire.spaces.Space:java.lang.String:int:int:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getRepositoryHistoryAll(final Space a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getRepositoryHistoryAll:org.uberfire.spaces.Space:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Repository getRepositoryFromSpace(final Space a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getRepositoryFromSpace:org.uberfire.spaces.Space:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Repository.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Repository getRepository(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getRepository:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Repository.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Repository getRepository(final Space a0, final Path a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getRepository:org.uberfire.spaces.Space:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0, a1 }).respondTo(Repository.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getAllRepositories(final Space a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getAllRepositories:org.uberfire.spaces.Space:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getAllRepositoriesFromAllUserSpaces() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getAllRepositoriesFromAllUserSpaces:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getRepositories(final Space a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("getRepositories:org.uberfire.spaces.Space:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Repository createRepository(final OrganizationalUnit a0, final String a1, final String a2, final RepositoryEnvironmentConfigurations a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("createRepository:org.guvnor.structure.organizationalunit.OrganizationalUnit:java.lang.String:java.lang.String:org.guvnor.structure.repositories.RepositoryEnvironmentConfigurations:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Repository.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public String normalizeRepositoryName(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("normalizeRepositoryName:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public boolean validateRepositoryName(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("validateRepositoryName:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public void addGroup(final Repository a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("addGroup:org.guvnor.structure.repositories.Repository:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
      }

      public void removeGroup(final Repository a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("removeGroup:org.guvnor.structure.repositories.Repository:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
      }

      public void removeRepository(final Space a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("removeRepository:org.uberfire.spaces.Space:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
      }

      public void removeRepositories(final Space a0, final Set a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryService").endpoint("removeRepositories:org.uberfire.spaces.Space:java.util.Set:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(RepositoryService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_structure_repositories_RepositoryServiceImpl();
      }
    });
    class org_uberfire_ext_layout_editor_api_LayoutServicesImpl extends AbstractRpcProxy implements LayoutServices { public String convertLayoutToString(final LayoutTemplate a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.LayoutServices").endpoint("convertLayoutToString:org.uberfire.ext.layout.editor.api.editor.LayoutTemplate:", qualifiers, new Object[] { a0 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_LayoutServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LayoutTemplate convertLayoutFromString(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.LayoutServices").endpoint("convertLayoutFromString:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(LayoutTemplate.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_LayoutServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(LayoutServices.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_layout_editor_api_LayoutServicesImpl();
      }
    });
    class org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl extends AbstractRpcProxy implements SocialTimeLineRepositoryAPI { public List getLastEventTimeline(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("getLastEventTimeline:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getLastEventTimeline(final String a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("getLastEventTimeline:java.lang.String:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getLastEventTimeline(final SocialAdapter a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("getLastEventTimeline:org.ext.uberfire.social.activities.service.SocialAdapter:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void saveTypeEvent(final SocialActivitiesEvent a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("saveTypeEvent:org.ext.uberfire.social.activities.model.SocialActivitiesEvent:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
      }

      public void saveUserEvent(final SocialActivitiesEvent a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("saveUserEvent:org.ext.uberfire.social.activities.model.SocialActivitiesEvent:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
      }

      public List getLastUserTimeline(final SocialUser a0, final SocialPredicate a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("getLastUserTimeline:org.ext.uberfire.social.activities.model.SocialUser:org.ext.uberfire.social.activities.service.SocialPredicate:", qualifiers, new Object[] { a0, a1 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getLastUserTimeline(final SocialUser a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("getLastUserTimeline:org.ext.uberfire.social.activities.model.SocialUser:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getLastUserTimeline(final SocialUser a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("getLastUserTimeline:org.ext.uberfire.social.activities.model.SocialUser:", qualifiers, new Object[] { a0 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Integer numberOfPages(final SocialEventType a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimeLineRepositoryAPI").endpoint("numberOfPages:org.ext.uberfire.social.activities.model.SocialEventType:", qualifiers, new Object[] { a0 }).respondTo(Integer.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return 0;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SocialTimeLineRepositoryAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_SocialTimeLineRepositoryAPIImpl();
      }
    });
    class org_guvnor_common_services_shared_metadata_MetadataServiceImpl extends AbstractRpcProxy implements MetadataService { public Metadata getMetadata(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.metadata.MetadataService").endpoint("getMetadata:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Metadata.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_metadata_MetadataServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getTags(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.metadata.MetadataService").endpoint("getTags:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_metadata_MetadataServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map configAttrs(final Map a0, final Metadata a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.metadata.MetadataService").endpoint("configAttrs:java.util.Map:org.guvnor.common.services.shared.metadata.model.Metadata:", qualifiers, new Object[] { a0, a1 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_metadata_MetadataServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map setUpAttributes(final Path a0, final Metadata a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.metadata.MetadataService").endpoint("setUpAttributes:org.uberfire.backend.vfs.Path:org.guvnor.common.services.shared.metadata.model.Metadata:", qualifiers, new Object[] { a0, a1 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_metadata_MetadataServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(MetadataService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_shared_metadata_MetadataServiceImpl();
      }
    });
    class org_ext_uberfire_social_activities_service_SocialEventTypeRepositoryAPIImpl extends AbstractRpcProxy implements SocialEventTypeRepositoryAPI { public List findAll() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialEventTypeRepositoryAPI").endpoint("findAll:", qualifiers, new Object[] { }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialEventTypeRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public SocialEventType findType(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialEventTypeRepositoryAPI").endpoint("findType:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(SocialEventType.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialEventTypeRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SocialEventTypeRepositoryAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_SocialEventTypeRepositoryAPIImpl();
      }
    });
    class org_guvnor_structure_organizationalunit_OrganizationalUnitSearchServiceImpl extends AbstractRpcProxy implements OrganizationalUnitSearchService { public Collection searchByName(final String a0, final int a1, final boolean a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitSearchService").endpoint("searchByName:java.lang.String:int:boolean:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitSearchServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection searchById(final Collection a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitSearchService").endpoint("searchById:java.util.Collection:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitSearchServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(OrganizationalUnitSearchService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_structure_organizationalunit_OrganizationalUnitSearchServiceImpl();
      }
    });
    class org_kie_workbench_common_stunner_core_service_FactoryServiceImpl extends AbstractRpcProxy implements FactoryService { public Object newDefinition(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.FactoryService").endpoint("newDefinition:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_FactoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Object newDefinition(final Class a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.FactoryService").endpoint("newDefinition:java.lang.Class:", qualifiers, new Object[] { a0 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_FactoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Element newElement(final String a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.FactoryService").endpoint("newElement:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Element.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_FactoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Element newElement(final String a0, final Class a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.FactoryService").endpoint("newElement:java.lang.String:java.lang.Class:", qualifiers, new Object[] { a0, a1 }).respondTo(Element.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_FactoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Diagram newDiagram(final String a0, final String a1, final org.kie.workbench.common.stunner.core.diagram.Metadata a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.FactoryService").endpoint("newDiagram:java.lang.String:java.lang.String:org.kie.workbench.common.stunner.core.diagram.Metadata:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Diagram.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_FactoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Diagram newDiagram(final String a0, final Class a1, final org.kie.workbench.common.stunner.core.diagram.Metadata a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.FactoryService").endpoint("newDiagram:java.lang.String:java.lang.Class:org.kie.workbench.common.stunner.core.diagram.Metadata:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Diagram.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_FactoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public FactoryRegistry registry() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.FactoryService").endpoint("registry:", qualifiers, new Object[] { }).respondTo(FactoryRegistry.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_FactoryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(FactoryService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_stunner_core_service_FactoryServiceImpl();
      }
    });
    class org_uberfire_workbench_services_WorkbenchServicesImpl extends AbstractRpcProxy implements WorkbenchServices { public void save(final String a0, final PerspectiveDefinition a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("save:java.lang.String:org.uberfire.workbench.model.PerspectiveDefinition:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
      }

      public void save(final SplashScreenFilter a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("save:org.uberfire.workbench.model.SplashScreenFilter:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
      }

      public Set loadPerspectives() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("loadPerspectives:", qualifiers, new Object[] { }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PerspectiveDefinition loadPerspective(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("loadPerspective:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(PerspectiveDefinition.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void removePerspectiveState(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("removePerspectiveState:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
      }

      public void removePerspectiveStates() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("removePerspectiveStates:", qualifiers, new Object[] { }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
      }

      public SplashScreenFilter loadSplashScreenFilter(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("loadSplashScreenFilter:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(SplashScreenFilter.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map loadDefaultEditorsMap() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("loadDefaultEditorsMap:", qualifiers, new Object[] { }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void saveDefaultEditors(final Map a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("saveDefaultEditors:java.util.Map:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
      }

      public boolean isWorkbenchOnCluster() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.workbench.services.WorkbenchServices").endpoint("isWorkbenchOnCluster:", qualifiers, new Object[] { }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_workbench_services_WorkbenchServicesImpl.this.sendRequest(bus, sendable);
        return false;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(WorkbenchServices.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_workbench_services_WorkbenchServicesImpl();
      }
    });
    class org_uberfire_backend_authz_AuthorizationServiceImpl extends AbstractRpcProxy implements AuthorizationService { public AuthorizationPolicy loadPolicy() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.authz.AuthorizationService").endpoint("loadPolicy:", qualifiers, new Object[] { }).respondTo(AuthorizationPolicy.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_authz_AuthorizationServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void savePolicy(final AuthorizationPolicy a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.authz.AuthorizationService").endpoint("savePolicy:org.uberfire.security.authz.AuthorizationPolicy:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_authz_AuthorizationServiceImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(AuthorizationService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_backend_authz_AuthorizationServiceImpl();
      }
    });
    class org_uberfire_ext_editor_commons_version_VersionServiceImpl extends AbstractRpcProxy implements VersionService { public List getVersions(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.version.VersionService").endpoint("getVersions:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_version_VersionServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path getPathToPreviousVersion(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.version.VersionService").endpoint("getPathToPreviousVersion:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_version_VersionServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path restore(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.version.VersionService").endpoint("restore:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_version_VersionServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(VersionService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_editor_commons_version_VersionServiceImpl();
      }
    });
    class org_uberfire_experimental_service_editor_FeaturesEditorServiceImpl extends AbstractRpcProxy implements FeaturesEditorService { public void save(final EditableExperimentalFeature a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.experimental.service.editor.FeaturesEditorService").endpoint("save:org.uberfire.experimental.service.editor.EditableExperimentalFeature:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_experimental_service_editor_FeaturesEditorServiceImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(FeaturesEditorService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_experimental_service_editor_FeaturesEditorServiceImpl();
      }
    });
    class org_uberfire_ssh_service_shared_editor_SSHKeyEditorServiceImpl extends AbstractRpcProxy implements SSHKeyEditorService { public Collection getUserKeys() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ssh.service.shared.editor.SSHKeyEditorService").endpoint("getUserKeys:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ssh_service_shared_editor_SSHKeyEditorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void deleteKey(final PortableSSHPublicKey a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ssh.service.shared.editor.SSHKeyEditorService").endpoint("deleteKey:org.uberfire.ssh.service.shared.editor.PortableSSHPublicKey:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ssh_service_shared_editor_SSHKeyEditorServiceImpl.this.sendRequest(bus, sendable);
      }

      public void addKey(final String a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ssh.service.shared.editor.SSHKeyEditorService").endpoint("addKey:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ssh_service_shared_editor_SSHKeyEditorServiceImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SSHKeyEditorService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ssh_service_shared_editor_SSHKeyEditorServiceImpl();
      }
    });
    class org_kie_workbench_common_forms_dynamic_service_shared_BackendSelectorDataProviderServiceImpl extends AbstractRpcProxy implements BackendSelectorDataProviderService { public Map availableProviders() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.forms.dynamic.service.shared.BackendSelectorDataProviderService").endpoint("availableProviders:", qualifiers, new Object[] { }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_forms_dynamic_service_shared_BackendSelectorDataProviderServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public SelectorData getDataFromProvider(final FormRenderingContext a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.forms.dynamic.service.shared.BackendSelectorDataProviderService").endpoint("getDataFromProvider:org.kie.workbench.common.forms.dynamic.service.shared.FormRenderingContext:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(SelectorData.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_forms_dynamic_service_shared_BackendSelectorDataProviderServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(BackendSelectorDataProviderService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_forms_dynamic_service_shared_BackendSelectorDataProviderServiceImpl();
      }
    });
    class org_uberfire_ext_properties_editor_service_BeanPropertyEditorBuilderServiceImpl extends AbstractRpcProxy implements BeanPropertyEditorBuilderService { public PropertyEditorCategory extract(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.properties.editor.service.BeanPropertyEditorBuilderService").endpoint("extract:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(PropertyEditorCategory.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_properties_editor_service_BeanPropertyEditorBuilderServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PropertyEditorCategory extract(final String a0, final Object a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.properties.editor.service.BeanPropertyEditorBuilderService").endpoint("extract:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1 }).respondTo(PropertyEditorCategory.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_properties_editor_service_BeanPropertyEditorBuilderServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(BeanPropertyEditorBuilderService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_properties_editor_service_BeanPropertyEditorBuilderServiceImpl();
      }
    });
    class org_uberfire_preferences_shared_PreferenceStorageImpl extends AbstractRpcProxy implements PreferenceStorage { public boolean exists(final PreferenceScope a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("exists:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public boolean exists(final PreferenceScopeResolutionStrategyInfo a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("exists:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public Object read(final PreferenceScope a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("read:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Object read(final PreferenceScopeResolutionStrategyInfo a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("read:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PreferenceScopedValue readWithScope(final PreferenceScopeResolutionStrategyInfo a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("readWithScope:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(PreferenceScopedValue.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void write(final PreferenceScope a0, final String a1, final Object a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("write:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
      }

      public void delete(final PreferenceScope a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("delete:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
      }

      public Collection allKeys(final PreferenceScope a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("allKeys:org.uberfire.preferences.shared.PreferenceScope:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection allKeys(final List a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStorage").endpoint("allKeys:java.util.List:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStorageImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(PreferenceStorage.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_preferences_shared_PreferenceStorageImpl();
      }
    });
    class org_guvnor_common_services_shared_config_AppConfigServiceImpl extends AbstractRpcProxy implements AppConfigService { public Map loadPreferences() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.config.AppConfigService").endpoint("loadPreferences:", qualifiers, new Object[] { }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_config_AppConfigServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public long getTimestamp() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.config.AppConfigService").endpoint("getTimestamp:", qualifiers, new Object[] { }).respondTo(Long.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_config_AppConfigServiceImpl.this.sendRequest(bus, sendable);
        return 0L;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(AppConfigService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_shared_config_AppConfigServiceImpl();
      }
    });
    class org_kie_workbench_common_stunner_forms_service_FormGenerationServiceImpl extends AbstractRpcProxy implements FormGenerationService { public void generateProcessForm(final Diagram a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.forms.service.FormGenerationService").endpoint("generateProcessForm:org.kie.workbench.common.stunner.core.diagram.Diagram:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_forms_service_FormGenerationServiceImpl.this.sendRequest(bus, sendable);
      }

      public void generateAllForms(final Diagram a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.forms.service.FormGenerationService").endpoint("generateAllForms:org.kie.workbench.common.stunner.core.diagram.Diagram:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_forms_service_FormGenerationServiceImpl.this.sendRequest(bus, sendable);
      }

      public void generateSelectedForms(final Diagram a0, final String[] a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.forms.service.FormGenerationService").endpoint("generateSelectedForms:org.kie.workbench.common.stunner.core.diagram.Diagram:java.lang.String[]:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_forms_service_FormGenerationServiceImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(FormGenerationService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_stunner_forms_service_FormGenerationServiceImpl();
      }
    });
    class org_uberfire_ext_plugin_service_PluginServicesImpl extends AbstractRpcProxy implements PluginServices { public String getMediaServletURI() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("getMediaServletURI:", qualifiers, new Object[] { }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection listRuntimePlugins() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("listRuntimePlugins:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection listPluginRuntimePlugins(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("listPluginRuntimePlugins:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection listPlugins() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("listPlugins:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Plugin createNewPlugin(final String a0, final PluginType a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("createNewPlugin:java.lang.String:org.uberfire.ext.plugin.model.PluginType:", qualifiers, new Object[] { a0, a1 }).respondTo(Plugin.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PluginContent getPluginContent(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("getPluginContent:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(PluginContent.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void deleteMedia(final Media a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("deleteMedia:org.uberfire.ext.plugin.model.Media:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
      }

      public DynamicMenu getDynamicMenuContent(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("getDynamicMenuContent:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(DynamicMenu.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path save(final Plugin a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("save:org.uberfire.ext.plugin.model.Plugin:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LayoutEditorModel getLayoutEditor(final Path a0, final PluginType a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("getLayoutEditor:org.uberfire.backend.vfs.Path:org.uberfire.ext.plugin.model.PluginType:", qualifiers, new Object[] { a0, a1 }).respondTo(LayoutEditorModel.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path saveMenu(final DynamicMenu a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("saveMenu:org.uberfire.ext.plugin.model.DynamicMenu:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path saveLayout(final LayoutEditorModel a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("saveLayout:org.uberfire.ext.plugin.model.LayoutEditorModel:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection listDynamicMenus() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("listDynamicMenus:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection listLayoutEditor(final PluginType a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("listLayoutEditor:org.uberfire.ext.plugin.model.PluginType:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void delete(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("delete:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
      }

      public Path copy(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path copy(final Path a0, final String a1, final Path a2, final String a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path saveAndRename(final Path a0, final String a1, final DefaultMetadata a2, final Plugin a3, final String a4) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("saveAndRename:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.ext.editor.commons.file.DefaultMetadata:org.uberfire.ext.plugin.model.Plugin:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3, a4 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path rename(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("rename:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path save(final Path a0, final Plugin a1, final DefaultMetadata a2, final String a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.plugin.service.PluginServices").endpoint("save:org.uberfire.backend.vfs.Path:org.uberfire.ext.plugin.model.Plugin:org.uberfire.ext.editor.commons.file.DefaultMetadata:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_plugin_service_PluginServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(PluginServices.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_plugin_service_PluginServicesImpl();
      }
    });
    class org_jboss_errai_security_shared_service_AuthenticationServiceImpl extends AbstractRpcProxy implements AuthenticationService { public User login(final String a0, final String a1) {
        try {
          final CallContextStatus status = new CallContextStatus(AuthenticationServiceInterceptor.class);
          final RemoteCallContext callContext = new RemoteCallContext() {
            public String getMethodName() {
              return "login";
            }
            public Class getReturnType() {
              return User.class;
            }
            public Annotation[] getAnnotations() {
              return new Annotation[] { };
            }
            public Annotation[] getTypeAnnotations() {
              return new Annotation[] { };
            }
            public Object proceed() {
              status.proceed();
              if (status.getNextInterceptor() == null) {
                RemoteCallSendable sendable = MessageBuilder.createCall().call("org.jboss.errai.security.shared.service.AuthenticationService").endpoint("login:java.lang.String:java.lang.String:", qualifiers, getParameters()).respondTo(User.class, remoteCallback).errorsHandledBy(errorCallback);
                org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.sendRequest(bus, sendable);
              } else if (status.getNextInterceptor() == AuthenticationServiceInterceptor.class) {
                final RemoteCallContext ctx = this;
                final CreationalCallback icc = new CreationalCallback() {
                  public void callback(final Object beanInstance) {
                    status.setProceeding(false);
                    ((AuthenticationServiceInterceptor) beanInstance).aroundInvoke(ctx);
                    if ((!status.isProceeding()) && (AuthenticationServiceInterceptor.class == status.getNextInterceptor())) {
                      remoteCallback.callback(ctx.getResult());
                    }
                  }
                };
                org.jboss.errai.ioc.client.container.IOC.getAsyncBeanManager().lookupBeans(AuthenticationServiceInterceptor.class).iterator().next().getInstance(icc);
              }
              return null;
            }
            public void proceed(final RemoteCallback interceptorCallback) {
              final RemoteCallback providedCallback = org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.remoteCallback;
              remoteCallback = new RemoteCallback() {
                public void callback(Object response) {
                  RemoteCallback intCallback = interceptorCallback;
                  setResult(response);
                  intCallback.callback(getResult());
                  providedCallback.callback(getResult());
                }
              };
              proceed();
            }
            public void proceed(RemoteCallback interceptorCallback, final ErrorCallback interceptorErrorCallback) {
              final ErrorCallback providedErrorCallback = org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.errorCallback;
              errorCallback = new ErrorCallback() {
                public boolean error(Object message, Throwable throwable) {
                  interceptorErrorCallback.error(message, throwable);
                  if (getResult() != null) {
                    remoteCallback.callback(getResult());
                    return false;
                  } else if (providedErrorCallback != null) {
                    return providedErrorCallback.error(message, throwable);
                  }
                  return true;
                }
              };
              proceed(interceptorCallback);
            }
          };
          callContext.setParameters(new Object[] { a0, a1 });
          callContext.proceed();
        } catch (Throwable throwable) {
          if (errorCallback != null) {
            if (errorCallback.error(null, throwable)) {
              invokeDefaultErrorHandlers(throwable);
            }
          } else {
            invokeDefaultErrorHandlers(throwable);
          }
        }
        return null;
      }

      public boolean isLoggedIn() {
        try {
          final CallContextStatus status = new CallContextStatus(AuthenticationServiceInterceptor.class);
          final RemoteCallContext callContext = new RemoteCallContext() {
            public String getMethodName() {
              return "isLoggedIn";
            }
            public Class getReturnType() {
              return boolean.class;
            }
            public Annotation[] getAnnotations() {
              return new Annotation[] { };
            }
            public Annotation[] getTypeAnnotations() {
              return new Annotation[] { };
            }
            public Object proceed() {
              status.proceed();
              if (status.getNextInterceptor() == null) {
                RemoteCallSendable sendable = MessageBuilder.createCall().call("org.jboss.errai.security.shared.service.AuthenticationService").endpoint("isLoggedIn:", qualifiers, getParameters()).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
                org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.sendRequest(bus, sendable);
              } else if (status.getNextInterceptor() == AuthenticationServiceInterceptor.class) {
                final RemoteCallContext ctx = this;
                final CreationalCallback icc = new CreationalCallback() {
                  public void callback(final Object beanInstance) {
                    status.setProceeding(false);
                    ((AuthenticationServiceInterceptor) beanInstance).aroundInvoke(ctx);
                    if ((!status.isProceeding()) && (AuthenticationServiceInterceptor.class == status.getNextInterceptor())) {
                      remoteCallback.callback(ctx.getResult());
                    }
                  }
                };
                org.jboss.errai.ioc.client.container.IOC.getAsyncBeanManager().lookupBeans(AuthenticationServiceInterceptor.class).iterator().next().getInstance(icc);
              }
              return null;
            }
            public void proceed(final RemoteCallback interceptorCallback) {
              final RemoteCallback providedCallback = org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.remoteCallback;
              remoteCallback = new RemoteCallback() {
                public void callback(Object response) {
                  RemoteCallback intCallback = interceptorCallback;
                  setResult(response);
                  intCallback.callback(getResult());
                  providedCallback.callback(getResult());
                }
              };
              proceed();
            }
            public void proceed(RemoteCallback interceptorCallback, final ErrorCallback interceptorErrorCallback) {
              final ErrorCallback providedErrorCallback = org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.errorCallback;
              errorCallback = new ErrorCallback() {
                public boolean error(Object message, Throwable throwable) {
                  interceptorErrorCallback.error(message, throwable);
                  if (getResult() != null) {
                    remoteCallback.callback(getResult());
                    return false;
                  } else if (providedErrorCallback != null) {
                    return providedErrorCallback.error(message, throwable);
                  }
                  return true;
                }
              };
              proceed(interceptorCallback);
            }
          };
          callContext.setParameters(new Object[] { });
          callContext.proceed();
        } catch (Throwable throwable) {
          if (errorCallback != null) {
            if (errorCallback.error(null, throwable)) {
              invokeDefaultErrorHandlers(throwable);
            }
          } else {
            invokeDefaultErrorHandlers(throwable);
          }
        }
        return false;
      }

      public void logout() {
        try {
          final CallContextStatus status = new CallContextStatus(AuthenticationServiceInterceptor.class);
          final RemoteCallContext callContext = new RemoteCallContext() {
            public String getMethodName() {
              return "logout";
            }
            public Class getReturnType() {
              return void.class;
            }
            public Annotation[] getAnnotations() {
              return new Annotation[] { };
            }
            public Annotation[] getTypeAnnotations() {
              return new Annotation[] { };
            }
            public Object proceed() {
              status.proceed();
              if (status.getNextInterceptor() == null) {
                RemoteCallSendable sendable = MessageBuilder.createCall().call("org.jboss.errai.security.shared.service.AuthenticationService").endpoint("logout:", qualifiers, getParameters()).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
                org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.sendRequest(bus, sendable);
              } else if (status.getNextInterceptor() == AuthenticationServiceInterceptor.class) {
                final RemoteCallContext ctx = this;
                final CreationalCallback icc = new CreationalCallback() {
                  public void callback(final Object beanInstance) {
                    status.setProceeding(false);
                    ((AuthenticationServiceInterceptor) beanInstance).aroundInvoke(ctx);
                    if ((!status.isProceeding()) && (AuthenticationServiceInterceptor.class == status.getNextInterceptor())) {
                      remoteCallback.callback(ctx.getResult());
                    }
                  }
                };
                org.jboss.errai.ioc.client.container.IOC.getAsyncBeanManager().lookupBeans(AuthenticationServiceInterceptor.class).iterator().next().getInstance(icc);
              }
              return null;
            }
            public void proceed(final RemoteCallback interceptorCallback) {
              final RemoteCallback providedCallback = org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.remoteCallback;
              remoteCallback = new RemoteCallback() {
                public void callback(Object response) {
                  RemoteCallback intCallback = interceptorCallback;
                  setResult(response);
                  intCallback.callback(getResult());
                  providedCallback.callback(getResult());
                }
              };
              proceed();
            }
            public void proceed(RemoteCallback interceptorCallback, final ErrorCallback interceptorErrorCallback) {
              final ErrorCallback providedErrorCallback = org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.errorCallback;
              errorCallback = new ErrorCallback() {
                public boolean error(Object message, Throwable throwable) {
                  interceptorErrorCallback.error(message, throwable);
                  if (getResult() != null) {
                    remoteCallback.callback(getResult());
                    return false;
                  } else if (providedErrorCallback != null) {
                    return providedErrorCallback.error(message, throwable);
                  }
                  return true;
                }
              };
              proceed(interceptorCallback);
            }
          };
          callContext.setParameters(new Object[] { });
          callContext.proceed();
        } catch (Throwable throwable) {
          if (errorCallback != null) {
            if (errorCallback.error(null, throwable)) {
              invokeDefaultErrorHandlers(throwable);
            }
          } else {
            invokeDefaultErrorHandlers(throwable);
          }
        }
      }

      public User getUser() {
        try {
          final CallContextStatus status = new CallContextStatus(AuthenticationServiceInterceptor.class);
          final RemoteCallContext callContext = new RemoteCallContext() {
            public String getMethodName() {
              return "getUser";
            }
            public Class getReturnType() {
              return User.class;
            }
            public Annotation[] getAnnotations() {
              return new Annotation[] { };
            }
            public Annotation[] getTypeAnnotations() {
              return new Annotation[] { };
            }
            public Object proceed() {
              status.proceed();
              if (status.getNextInterceptor() == null) {
                RemoteCallSendable sendable = MessageBuilder.createCall().call("org.jboss.errai.security.shared.service.AuthenticationService").endpoint("getUser:", qualifiers, getParameters()).respondTo(User.class, remoteCallback).errorsHandledBy(errorCallback);
                org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.sendRequest(bus, sendable);
              } else if (status.getNextInterceptor() == AuthenticationServiceInterceptor.class) {
                final RemoteCallContext ctx = this;
                final CreationalCallback icc = new CreationalCallback() {
                  public void callback(final Object beanInstance) {
                    status.setProceeding(false);
                    ((AuthenticationServiceInterceptor) beanInstance).aroundInvoke(ctx);
                    if ((!status.isProceeding()) && (AuthenticationServiceInterceptor.class == status.getNextInterceptor())) {
                      remoteCallback.callback(ctx.getResult());
                    }
                  }
                };
                org.jboss.errai.ioc.client.container.IOC.getAsyncBeanManager().lookupBeans(AuthenticationServiceInterceptor.class).iterator().next().getInstance(icc);
              }
              return null;
            }
            public void proceed(final RemoteCallback interceptorCallback) {
              final RemoteCallback providedCallback = org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.remoteCallback;
              remoteCallback = new RemoteCallback() {
                public void callback(Object response) {
                  RemoteCallback intCallback = interceptorCallback;
                  setResult(response);
                  intCallback.callback(getResult());
                  providedCallback.callback(getResult());
                }
              };
              proceed();
            }
            public void proceed(RemoteCallback interceptorCallback, final ErrorCallback interceptorErrorCallback) {
              final ErrorCallback providedErrorCallback = org_jboss_errai_security_shared_service_AuthenticationServiceImpl.this.errorCallback;
              errorCallback = new ErrorCallback() {
                public boolean error(Object message, Throwable throwable) {
                  interceptorErrorCallback.error(message, throwable);
                  if (getResult() != null) {
                    remoteCallback.callback(getResult());
                    return false;
                  } else if (providedErrorCallback != null) {
                    return providedErrorCallback.error(message, throwable);
                  }
                  return true;
                }
              };
              proceed(interceptorCallback);
            }
          };
          callContext.setParameters(new Object[] { });
          callContext.proceed();
        } catch (Throwable throwable) {
          if (errorCallback != null) {
            if (errorCallback.error(null, throwable)) {
              invokeDefaultErrorHandlers(throwable);
            }
          } else {
            invokeDefaultErrorHandlers(throwable);
          }
        }
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(AuthenticationService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_jboss_errai_security_shared_service_AuthenticationServiceImpl();
      }
    });
    class org_uberfire_experimental_service_backend_BackendExperimentalFeatureDefRegistryImpl extends AbstractRpcProxy implements BackendExperimentalFeatureDefRegistry { public Collection loadFeatureDefinitions(final Collection a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.experimental.service.backend.BackendExperimentalFeatureDefRegistry").endpoint("loadFeatureDefinitions:java.util.Collection:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_experimental_service_backend_BackendExperimentalFeatureDefRegistryImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public ExperimentalFeatureDefinition getFeatureById(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.experimental.service.backend.BackendExperimentalFeatureDefRegistry").endpoint("getFeatureById:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(ExperimentalFeatureDefinition.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_experimental_service_backend_BackendExperimentalFeatureDefRegistryImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getAllFeatures() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.experimental.service.backend.BackendExperimentalFeatureDefRegistry").endpoint("getAllFeatures:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_experimental_service_backend_BackendExperimentalFeatureDefRegistryImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getGlobalFeatures() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.experimental.service.backend.BackendExperimentalFeatureDefRegistry").endpoint("getGlobalFeatures:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_experimental_service_backend_BackendExperimentalFeatureDefRegistryImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getUserFeatures() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.experimental.service.backend.BackendExperimentalFeatureDefRegistry").endpoint("getUserFeatures:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_experimental_service_backend_BackendExperimentalFeatureDefRegistryImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(BackendExperimentalFeatureDefRegistry.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_experimental_service_backend_BackendExperimentalFeatureDefRegistryImpl();
      }
    });
    class org_kie_workbench_common_stunner_bpmn_service_BPMNDiagramServiceImpl extends AbstractRpcProxy implements BPMNDiagramService { public ProjectType getProjectType(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.bpmn.service.BPMNDiagramService").endpoint("getProjectType:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(ProjectType.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_bpmn_service_BPMNDiagramServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(BPMNDiagramService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_stunner_bpmn_service_BPMNDiagramServiceImpl();
      }
    });
    class org_guvnor_common_services_project_service_ModuleRepositoryResolverImpl extends AbstractRpcProxy implements ModuleRepositoryResolver { public Set getRemoteRepositoriesMetaData() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleRepositoryResolver").endpoint("getRemoteRepositoriesMetaData:", qualifiers, new Object[] { }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleRepositoryResolverImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Set getRemoteRepositoriesMetaData(final Module a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleRepositoryResolver").endpoint("getRemoteRepositoriesMetaData:org.guvnor.common.services.project.model.Module:", qualifiers, new Object[] { a0 }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleRepositoryResolverImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Set getRepositoriesResolvingArtifact(final GAV a0, final MavenRepositoryMetadata[] a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleRepositoryResolver").endpoint("getRepositoriesResolvingArtifact:org.guvnor.common.services.project.model.GAV:org.guvnor.common.services.project.model.MavenRepositoryMetadata[]:", qualifiers, new Object[] { a0, a1 }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleRepositoryResolverImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Set getRepositoriesResolvingArtifact(final GAV a0, final Module a1, final MavenRepositoryMetadata[] a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleRepositoryResolver").endpoint("getRepositoriesResolvingArtifact:org.guvnor.common.services.project.model.GAV:org.guvnor.common.services.project.model.Module:org.guvnor.common.services.project.model.MavenRepositoryMetadata[]:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleRepositoryResolverImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Set getRepositoriesResolvingArtifact(final String a0, final MavenRepositoryMetadata[] a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleRepositoryResolver").endpoint("getRepositoriesResolvingArtifact:java.lang.String:org.guvnor.common.services.project.model.MavenRepositoryMetadata[]:", qualifiers, new Object[] { a0, a1 }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleRepositoryResolverImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(ModuleRepositoryResolver.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_project_service_ModuleRepositoryResolverImpl();
      }
    });
    class org_kie_workbench_common_stunner_bpmn_service_DataTypesServiceImpl extends AbstractRpcProxy implements DataTypesService { public List getDataTypeNames(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.bpmn.service.DataTypesService").endpoint("getDataTypeNames:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_bpmn_service_DataTypesServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(DataTypesService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_stunner_bpmn_service_DataTypesServiceImpl();
      }
    });
    class org_ext_uberfire_social_activities_service_SocialActivitiesAPIImpl extends AbstractRpcProxy implements SocialActivitiesAPI { public void register(final SocialActivitiesEvent a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialActivitiesAPI").endpoint("register:org.ext.uberfire.social.activities.model.SocialActivitiesEvent:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialActivitiesAPIImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SocialActivitiesAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_SocialActivitiesAPIImpl();
      }
    });
    class org_uberfire_backend_vfs_VFSLockServiceImpl extends AbstractRpcProxy implements VFSLockService { public LockResult acquireLock(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSLockService").endpoint("acquireLock:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(LockResult.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSLockServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LockResult releaseLock(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSLockService").endpoint("releaseLock:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(LockResult.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSLockServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LockResult forceReleaseLock(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSLockService").endpoint("forceReleaseLock:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(LockResult.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSLockServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LockInfo retrieveLockInfo(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSLockService").endpoint("retrieveLockInfo:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(LockInfo.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSLockServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List retrieveLockInfos(final Path a0, final boolean a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSLockService").endpoint("retrieveLockInfos:org.uberfire.backend.vfs.Path:boolean:", qualifiers, new Object[] { a0, a1 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSLockServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(VFSLockService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_backend_vfs_VFSLockServiceImpl();
      }
    });
    class org_kie_workbench_common_stunner_core_service_DiagramLookupServiceImpl extends AbstractRpcProxy implements DiagramLookupService { public LookupResponse lookup(final DiagramLookupRequest a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.DiagramLookupService").endpoint("lookup:org.kie.workbench.common.stunner.core.lookup.diagram.DiagramLookupRequest:", qualifiers, new Object[] { a0 }).respondTo(LookupResponse.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_DiagramLookupServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(DiagramLookupService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_stunner_core_service_DiagramLookupServiceImpl();
      }
    });
    class org_kie_workbench_common_stunner_bpmn_workitem_service_WorkItemDefinitionLookupServiceImpl extends AbstractRpcProxy implements WorkItemDefinitionLookupService { public Collection execute(final org.kie.workbench.common.stunner.core.diagram.Metadata a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.bpmn.workitem.service.WorkItemDefinitionLookupService").endpoint("execute:org.kie.workbench.common.stunner.core.diagram.Metadata:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_bpmn_workitem_service_WorkItemDefinitionLookupServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(WorkItemDefinitionLookupService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_stunner_bpmn_workitem_service_WorkItemDefinitionLookupServiceImpl();
      }
    });
    class org_kie_workbench_common_stunner_core_validation_ValidationServiceImpl extends AbstractRpcProxy implements ValidationService { public Collection validate(final Diagram a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.validation.ValidationService").endpoint("validate:org.kie.workbench.common.stunner.core.diagram.Diagram:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_validation_ValidationServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(ValidationService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_stunner_core_validation_ValidationServiceImpl();
      }
    });
    class org_guvnor_structure_navigator_FileNavigatorServiceImpl extends AbstractRpcProxy implements FileNavigatorService { public NavigatorContent listContent(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.navigator.FileNavigatorService").endpoint("listContent:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(NavigatorContent.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_navigator_FileNavigatorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List listRepositories(final Space a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.navigator.FileNavigatorService").endpoint("listRepositories:org.uberfire.spaces.Space:", qualifiers, new Object[] { a0 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_navigator_FileNavigatorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(FileNavigatorService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_structure_navigator_FileNavigatorServiceImpl();
      }
    });
    class org_guvnor_common_services_project_service_WorkspaceProjectSearchServiceImpl extends AbstractRpcProxy implements WorkspaceProjectSearchService { public Collection searchByName(final String a0, final int a1, final boolean a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectSearchService").endpoint("searchByName:java.lang.String:int:boolean:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectSearchServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(WorkspaceProjectSearchService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_project_service_WorkspaceProjectSearchServiceImpl();
      }
    });
    class org_uberfire_backend_plugin_RuntimePluginServiceImpl extends AbstractRpcProxy implements RuntimePluginService { public Collection listFrameworksContent() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.plugin.RuntimePluginService").endpoint("listFrameworksContent:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_plugin_RuntimePluginServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection listPluginsContent() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.plugin.RuntimePluginService").endpoint("listPluginsContent:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_plugin_RuntimePluginServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public String getTemplateContent(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.plugin.RuntimePluginService").endpoint("getTemplateContent:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_plugin_RuntimePluginServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public String getRuntimePluginTemplateContent(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.plugin.RuntimePluginService").endpoint("getRuntimePluginTemplateContent:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_plugin_RuntimePluginServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getRuntimePlugins() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.plugin.RuntimePluginService").endpoint("getRuntimePlugins:", qualifiers, new Object[] { }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_plugin_RuntimePluginServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(RuntimePluginService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_backend_plugin_RuntimePluginServiceImpl();
      }
    });
    class org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl extends AbstractRpcProxy implements HtmlEditorService { public Path copy(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.htmleditor.HtmlEditorService").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path copy(final Path a0, final String a1, final Path a2, final String a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.htmleditor.HtmlEditorService").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void delete(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.htmleditor.HtmlEditorService").endpoint("delete:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl.this.sendRequest(bus, sendable);
      }

      public String load(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.htmleditor.HtmlEditorService").endpoint("load:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path saveAndRename(final Path a0, final String a1, final DefaultMetadata a2, final String a3, final String a4) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.htmleditor.HtmlEditorService").endpoint("saveAndRename:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.ext.editor.commons.file.DefaultMetadata:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3, a4 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path rename(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.htmleditor.HtmlEditorService").endpoint("rename:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path save(final Path a0, final String a1, final DefaultMetadata a2, final String a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.htmleditor.HtmlEditorService").endpoint("save:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.ext.editor.commons.file.DefaultMetadata:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(HtmlEditorService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_editor_commons_service_htmleditor_HtmlEditorServiceImpl();
      }
    });
    class org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl extends AbstractRpcProxy implements GroupManagerService { public void assignUsers(final String a0, final Collection a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.GroupManagerService").endpoint("assignUsers:java.lang.String:java.util.Collection:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl.this.sendRequest(bus, sendable);
      }

      public SearchResponse search(final SearchRequest a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.GroupManagerService").endpoint("search:org.uberfire.ext.security.management.api.AbstractEntityManager.SearchRequest:", qualifiers, new Object[] { a0 }).respondTo(SearchResponse.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Group get(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.GroupManagerService").endpoint("get:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Group.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Group create(final Group a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.GroupManagerService").endpoint("create:org.jboss.errai.security.shared.api.Group:", qualifiers, new Object[] { a0 }).respondTo(Group.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Group update(final Group a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.GroupManagerService").endpoint("update:org.jboss.errai.security.shared.api.Group:", qualifiers, new Object[] { a0 }).respondTo(Group.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void delete(final String[] a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.GroupManagerService").endpoint("delete:java.lang.String[]:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl.this.sendRequest(bus, sendable);
      }

      public GroupManagerSettings getSettings() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.GroupManagerService").endpoint("getSettings:", qualifiers, new Object[] { }).respondTo(GroupManagerSettings.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(GroupManagerService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_security_management_api_service_GroupManagerServiceImpl();
      }
    });
    class org_guvnor_common_services_shared_config_ResourceConfigServiceImpl extends AbstractRpcProxy implements ResourceConfigService { public Map configAttrs(final Map a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.config.ResourceConfigService").endpoint("configAttrs:java.util.Map:", qualifiers, new Object[] { a0 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_config_ResourceConfigServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(ResourceConfigService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_shared_config_ResourceConfigServiceImpl();
      }
    });
    class org_uberfire_preferences_shared_PreferenceStoreImpl extends AbstractRpcProxy implements PreferenceStore { public PreferenceScopeResolutionStrategyInfo getDefaultScopeResolutionStrategyInfo() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("getDefaultScopeResolutionStrategyInfo:", qualifiers, new Object[] { }).respondTo(PreferenceScopeResolutionStrategyInfo.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PreferenceScopeResolver getDefaultScopeResolver() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("getDefaultScopeResolver:", qualifiers, new Object[] { }).respondTo(PreferenceScopeResolver.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void put(final PreferenceScope a0, final String a1, final Object a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("put:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void put(final PreferenceScopeResolutionStrategyInfo a0, final String a1, final Object a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("put:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void put(final String a0, final Object a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("put:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void put(final PreferenceScope a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("put:org.uberfire.preferences.shared.PreferenceScope:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void put(final PreferenceScopeResolutionStrategyInfo a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("put:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void put(final Map a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("put:java.util.Map:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void putIfAbsent(final PreferenceScope a0, final String a1, final Object a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("putIfAbsent:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void putIfAbsent(final PreferenceScopeResolutionStrategyInfo a0, final String a1, final Object a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("putIfAbsent:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void putIfAbsent(final String a0, final Object a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("putIfAbsent:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void putIfAbsent(final PreferenceScope a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("putIfAbsent:org.uberfire.preferences.shared.PreferenceScope:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void putIfAbsent(final PreferenceScopeResolutionStrategyInfo a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("putIfAbsent:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void putIfAbsent(final Map a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("putIfAbsent:java.util.Map:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public Object get(final PreferenceScope a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("get:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Object get(final PreferenceScope a0, final String a1, final Object a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("get:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Object get(final PreferenceScopeResolutionStrategyInfo a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("get:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Object get(final PreferenceScopeResolutionStrategyInfo a0, final String a1, final Object a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("get:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Object get(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("get:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Object get(final String a0, final Object a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("get:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1 }).respondTo(Object.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PreferenceScopedValue getScoped(final PreferenceScopeResolutionStrategyInfo a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("getScoped:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(PreferenceScopedValue.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PreferenceScopedValue getScoped(final PreferenceScopeResolutionStrategyInfo a0, final String a1, final Object a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("getScoped:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(PreferenceScopedValue.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PreferenceScopedValue getScoped(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("getScoped:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(PreferenceScopedValue.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PreferenceScopedValue getScoped(final String a0, final Object a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("getScoped:java.lang.String:java.lang.Object:", qualifiers, new Object[] { a0, a1 }).respondTo(PreferenceScopedValue.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map search(final PreferenceScope a0, final Collection a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("search:org.uberfire.preferences.shared.PreferenceScope:java.util.Collection:", qualifiers, new Object[] { a0, a1 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map search(final PreferenceScopeResolutionStrategyInfo a0, final Collection a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("search:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.util.Collection:", qualifiers, new Object[] { a0, a1 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map search(final Collection a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("search:java.util.Collection:", qualifiers, new Object[] { a0 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map searchScoped(final PreferenceScopeResolutionStrategyInfo a0, final Collection a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("searchScoped:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:java.util.Collection:", qualifiers, new Object[] { a0, a1 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map searchScoped(final Collection a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("searchScoped:java.util.Collection:", qualifiers, new Object[] { a0 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map all(final PreferenceScope a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("all:org.uberfire.preferences.shared.PreferenceScope:", qualifiers, new Object[] { a0 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map all(final PreferenceScopeResolutionStrategyInfo a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("all:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:", qualifiers, new Object[] { a0 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map all() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("all:", qualifiers, new Object[] { }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map allScoped(final PreferenceScopeResolutionStrategyInfo a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("allScoped:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:", qualifiers, new Object[] { a0 }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map allScoped() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("allScoped:", qualifiers, new Object[] { }).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void remove(final PreferenceScope a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("remove:org.uberfire.preferences.shared.PreferenceScope:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }

      public void remove(final List a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.PreferenceStore").endpoint("remove:java.util.List:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_PreferenceStoreImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(PreferenceStore.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_preferences_shared_PreferenceStoreImpl();
      }
    });
    class org_uberfire_ext_apps_api_AppsPersistenceAPIImpl extends AbstractRpcProxy implements AppsPersistenceAPI { public Directory getRootDirectory() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.apps.api.AppsPersistenceAPI").endpoint("getRootDirectory:", qualifiers, new Object[] { }).respondTo(Directory.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_apps_api_AppsPersistenceAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Directory createDirectory(final Directory a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.apps.api.AppsPersistenceAPI").endpoint("createDirectory:org.uberfire.ext.apps.api.Directory:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Directory.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_apps_api_AppsPersistenceAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Boolean deleteDirectory(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.apps.api.AppsPersistenceAPI").endpoint("deleteDirectory:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_apps_api_AppsPersistenceAPIImpl.this.sendRequest(bus, sendable);
        return false;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(AppsPersistenceAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_apps_api_AppsPersistenceAPIImpl();
      }
    });
    class org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl extends AbstractRpcProxy implements OrganizationalUnitService { public OrganizationalUnit getOrganizationalUnit(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("getOrganizationalUnit:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(OrganizationalUnit.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getAllOrganizationalUnits() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("getAllOrganizationalUnits:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getAllUserSpaces() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("getAllUserSpaces:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getOrganizationalUnits() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("getOrganizationalUnits:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getOrganizationalUnits(final Repository a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("getOrganizationalUnits:org.guvnor.structure.repositories.Repository:", qualifiers, new Object[] { a0 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public OrganizationalUnit createOrganizationalUnit(final String a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("createOrganizationalUnit:java.lang.String:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(OrganizationalUnit.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public OrganizationalUnit createOrganizationalUnit(final String a0, final String a1, final String a2, final Collection a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("createOrganizationalUnit:java.lang.String:java.lang.String:java.lang.String:java.util.Collection:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(OrganizationalUnit.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public OrganizationalUnit createOrganizationalUnit(final String a0, final String a1, final String a2, final Collection a3, final Collection a4) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("createOrganizationalUnit:java.lang.String:java.lang.String:java.lang.String:java.util.Collection:java.util.Collection:", qualifiers, new Object[] { a0, a1, a2, a3, a4 }).respondTo(OrganizationalUnit.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public OrganizationalUnit updateOrganizationalUnit(final String a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("updateOrganizationalUnit:java.lang.String:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(OrganizationalUnit.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public OrganizationalUnit updateOrganizationalUnit(final String a0, final String a1, final String a2, final Collection a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("updateOrganizationalUnit:java.lang.String:java.lang.String:java.lang.String:java.util.Collection:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(OrganizationalUnit.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void addRepository(final OrganizationalUnit a0, final Repository a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("addRepository:org.guvnor.structure.organizationalunit.OrganizationalUnit:org.guvnor.structure.repositories.Repository:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
      }

      public void removeRepository(final OrganizationalUnit a0, final Repository a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("removeRepository:org.guvnor.structure.organizationalunit.OrganizationalUnit:org.guvnor.structure.repositories.Repository:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
      }

      public void addGroup(final OrganizationalUnit a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("addGroup:org.guvnor.structure.organizationalunit.OrganizationalUnit:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
      }

      public void removeGroup(final OrganizationalUnit a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("removeGroup:org.guvnor.structure.organizationalunit.OrganizationalUnit:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
      }

      public void removeOrganizationalUnit(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("removeOrganizationalUnit:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
      }

      public OrganizationalUnit getParentOrganizationalUnit(final Repository a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("getParentOrganizationalUnit:org.guvnor.structure.repositories.Repository:", qualifiers, new Object[] { a0 }).respondTo(OrganizationalUnit.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public String getSanitizedDefaultGroupId(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("getSanitizedDefaultGroupId:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Boolean isValidGroupId(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.organizationalunit.OrganizationalUnitService").endpoint("isValidGroupId:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(OrganizationalUnitService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_structure_organizationalunit_OrganizationalUnitServiceImpl();
      }
    });
    class org_ext_uberfire_social_activities_service_SocialTimelineRulesQueryAPIImpl extends AbstractRpcProxy implements SocialTimelineRulesQueryAPI { public List executeAllRules() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimelineRulesQueryAPI").endpoint("executeAllRules:", qualifiers, new Object[] { }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimelineRulesQueryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List executeSpecificRule(final Map a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimelineRulesQueryAPI").endpoint("executeSpecificRule:java.util.Map:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimelineRulesQueryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getAllCached() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimelineRulesQueryAPI").endpoint("getAllCached:", qualifiers, new Object[] { }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimelineRulesQueryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getTypeCached(final String[] a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimelineRulesQueryAPI").endpoint("getTypeCached:java.lang.String[]:", qualifiers, new Object[] { a0 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimelineRulesQueryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List getNEventsFromEachType(final int a0, final String[] a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTimelineRulesQueryAPI").endpoint("getNEventsFromEachType:int:java.lang.String[]:", qualifiers, new Object[] { a0, a1 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTimelineRulesQueryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SocialTimelineRulesQueryAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_SocialTimelineRulesQueryAPIImpl();
      }
    });
    class org_guvnor_common_services_project_service_ModuleRepositoriesServiceImpl extends AbstractRpcProxy implements ModuleRepositoriesService { public ModuleRepositories create(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleRepositoriesService").endpoint("create:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(ModuleRepositories.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleRepositoriesServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path save(final Path a0, final ModuleRepositories a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleRepositoriesService").endpoint("save:org.uberfire.backend.vfs.Path:org.guvnor.common.services.project.model.ModuleRepositories:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleRepositoriesServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public ModuleRepositories load(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleRepositoriesService").endpoint("load:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(ModuleRepositories.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleRepositoriesServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(ModuleRepositoriesService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_project_service_ModuleRepositoriesServiceImpl();
      }
    });
    class org_jboss_errai_security_shared_service_NonCachingUserServiceImpl extends AbstractRpcProxy implements NonCachingUserService { public User getUser() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.jboss.errai.security.shared.service.NonCachingUserService").endpoint("getUser:", qualifiers, new Object[] { }).respondTo(User.class, remoteCallback).errorsHandledBy(errorCallback);
        org_jboss_errai_security_shared_service_NonCachingUserServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(NonCachingUserService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_jboss_errai_security_shared_service_NonCachingUserServiceImpl();
      }
    });
    class org_uberfire_ext_editor_commons_service_CopyServiceImpl extends AbstractRpcProxy implements CopyService { public void copyIfExists(final Collection a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.CopyService").endpoint("copyIfExists:java.util.Collection:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_CopyServiceImpl.this.sendRequest(bus, sendable);
      }

      public boolean hasRestriction(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.CopyService").endpoint("hasRestriction:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_CopyServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public Path copy(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.CopyService").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_CopyServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path copy(final Path a0, final String a1, final Path a2, final String a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.CopyService").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_CopyServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(CopyService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_editor_commons_service_CopyServiceImpl();
      }
    });
    class org_uberfire_ext_services_shared_preferences_UserPreferencesServiceImpl extends AbstractRpcProxy implements UserPreferencesService { public void saveUserPreferences(final UserPreference a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.services.shared.preferences.UserPreferencesService").endpoint("saveUserPreferences:org.uberfire.ext.services.shared.preferences.UserPreference:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_services_shared_preferences_UserPreferencesServiceImpl.this.sendRequest(bus, sendable);
      }

      public UserPreference loadUserPreferences(final String a0, final UserPreferencesType a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.services.shared.preferences.UserPreferencesService").endpoint("loadUserPreferences:java.lang.String:org.uberfire.ext.services.shared.preferences.UserPreferencesType:", qualifiers, new Object[] { a0, a1 }).respondTo(UserPreference.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_services_shared_preferences_UserPreferencesServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public UserPreference loadUserPreferences(final UserPreference a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.services.shared.preferences.UserPreferencesService").endpoint("loadUserPreferences:org.uberfire.ext.services.shared.preferences.UserPreference:", qualifiers, new Object[] { a0 }).respondTo(UserPreference.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_services_shared_preferences_UserPreferencesServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(UserPreferencesService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_services_shared_preferences_UserPreferencesServiceImpl();
      }
    });
    class org_ext_uberfire_social_activities_service_SocialUserRepositoryAPIImpl extends AbstractRpcProxy implements SocialUserRepositoryAPI { public SocialUser findSocialUser(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialUserRepositoryAPI").endpoint("findSocialUser:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(SocialUser.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialUserRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public List findAllUsers() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialUserRepositoryAPI").endpoint("findAllUsers:", qualifiers, new Object[] { }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialUserRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SocialUserRepositoryAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_SocialUserRepositoryAPIImpl();
      }
    });
    class org_ext_uberfire_social_activities_service_SocialTypeTimelinePagedRepositoryAPIImpl extends AbstractRpcProxy implements SocialTypeTimelinePagedRepositoryAPI { public PagedSocialQuery getEventTimeline(final String a0, final SocialPaged a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTypeTimelinePagedRepositoryAPI").endpoint("getEventTimeline:java.lang.String:org.ext.uberfire.social.activities.model.SocialPaged:", qualifiers, new Object[] { a0, a1 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTypeTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PagedSocialQuery getEventTimeline(final SocialAdapter a0, final SocialPaged a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTypeTimelinePagedRepositoryAPI").endpoint("getEventTimeline:org.ext.uberfire.social.activities.service.SocialAdapter:org.ext.uberfire.social.activities.model.SocialPaged:", qualifiers, new Object[] { a0, a1 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTypeTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PagedSocialQuery getEventTimeline(final String a0, final SocialPaged a1, final SocialPredicate a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTypeTimelinePagedRepositoryAPI").endpoint("getEventTimeline:java.lang.String:org.ext.uberfire.social.activities.model.SocialPaged:org.ext.uberfire.social.activities.service.SocialPredicate:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTypeTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PagedSocialQuery getEventTimeline(final String a0, final SocialPaged a1, final Map a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTypeTimelinePagedRepositoryAPI").endpoint("getEventTimeline:java.lang.String:org.ext.uberfire.social.activities.model.SocialPaged:java.util.Map:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTypeTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PagedSocialQuery getEventTimeline(final String a0, final SocialPaged a1, final Map a2, final SocialPredicate a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTypeTimelinePagedRepositoryAPI").endpoint("getEventTimeline:java.lang.String:org.ext.uberfire.social.activities.model.SocialPaged:java.util.Map:org.ext.uberfire.social.activities.service.SocialPredicate:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTypeTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PagedSocialQuery getEventTimeline(final SocialAdapter a0, final SocialPaged a1, final Map a2, final SocialPredicate a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialTypeTimelinePagedRepositoryAPI").endpoint("getEventTimeline:org.ext.uberfire.social.activities.service.SocialAdapter:org.ext.uberfire.social.activities.model.SocialPaged:java.util.Map:org.ext.uberfire.social.activities.service.SocialPredicate:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(PagedSocialQuery.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialTypeTimelinePagedRepositoryAPIImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SocialTypeTimelinePagedRepositoryAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_SocialTypeTimelinePagedRepositoryAPIImpl();
      }
    });
    class org_kie_workbench_common_services_datamodel_service_IncrementalDataModelServiceImpl extends AbstractRpcProxy implements IncrementalDataModelService { public PackageDataModelOracleIncrementalPayload getUpdates(final Path a0, final Imports a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.services.datamodel.service.IncrementalDataModelService").endpoint("getUpdates:org.uberfire.backend.vfs.Path:org.kie.soup.project.datamodel.imports.Imports:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(PackageDataModelOracleIncrementalPayload.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_services_datamodel_service_IncrementalDataModelServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(IncrementalDataModelService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_services_datamodel_service_IncrementalDataModelServiceImpl();
      }
    });
    class org_ext_uberfire_social_activities_service_StressTestAPIImpl extends AbstractRpcProxy implements StressTestAPI { public int stress(final int a0, final int a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.StressTestAPI").endpoint("stress:int:int:", qualifiers, new Object[] { a0, a1 }).respondTo(Integer.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_StressTestAPIImpl.this.sendRequest(bus, sendable);
        return 0;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(StressTestAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_StressTestAPIImpl();
      }
    });
    class org_guvnor_common_services_shared_test_TestRunnerServiceImpl extends AbstractRpcProxy implements TestRunnerService { public void runAllTests(final String a0, final Path a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.test.TestRunnerService").endpoint("runAllTests:java.lang.String:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_test_TestRunnerServiceImpl.this.sendRequest(bus, sendable);
      }

      public void runAllTests(final String a0, final Path a1, final Event a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.shared.test.TestRunnerService").endpoint("runAllTests:java.lang.String:org.uberfire.backend.vfs.Path:javax.enterprise.event.Event:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_shared_test_TestRunnerServiceImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(TestRunnerService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_shared_test_TestRunnerServiceImpl();
      }
    });
    class org_guvnor_structure_repositories_RepositorySearchServiceImpl extends AbstractRpcProxy implements RepositorySearchService { public Collection searchByAlias(final String a0, final int a1, final boolean a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositorySearchService").endpoint("searchByAlias:java.lang.String:int:boolean:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositorySearchServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection searchById(final Collection a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositorySearchService").endpoint("searchById:java.util.Collection:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositorySearchServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(RepositorySearchService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_structure_repositories_RepositorySearchServiceImpl();
      }
    });
    class org_kie_workbench_common_stunner_core_service_DiagramServiceImpl extends AbstractRpcProxy implements DiagramService { public Diagram getDiagramByPath(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.DiagramService").endpoint("getDiagramByPath:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Diagram.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_DiagramServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public boolean accepts(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.DiagramService").endpoint("accepts:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_DiagramServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public Path create(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.DiagramService").endpoint("create:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_DiagramServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public org.kie.workbench.common.stunner.core.diagram.Metadata saveOrUpdate(final Diagram a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.DiagramService").endpoint("saveOrUpdate:org.kie.workbench.common.stunner.core.diagram.Diagram:", qualifiers, new Object[] { a0 }).respondTo(org.kie.workbench.common.stunner.core.diagram.Metadata.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_DiagramServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path saveOrUpdateSvg(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.DiagramService").endpoint("saveOrUpdateSvg:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_DiagramServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public boolean delete(final Diagram a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.DiagramService").endpoint("delete:org.kie.workbench.common.stunner.core.diagram.Diagram:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_DiagramServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public String getRawContent(final Diagram a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.kie.workbench.common.stunner.core.service.DiagramService").endpoint("getRawContent:org.kie.workbench.common.stunner.core.diagram.Diagram:", qualifiers, new Object[] { a0 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_kie_workbench_common_stunner_core_service_DiagramServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(DiagramService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_kie_workbench_common_stunner_core_service_DiagramServiceImpl();
      }
    });
    class org_ext_uberfire_social_activities_service_SocialUserServiceAPIImpl extends AbstractRpcProxy implements SocialUserServiceAPI { public void userFollowAnotherUser(final String a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialUserServiceAPI").endpoint("userFollowAnotherUser:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialUserServiceAPIImpl.this.sendRequest(bus, sendable);
      }

      public void userUnfollowAnotherUser(final String a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialUserServiceAPI").endpoint("userUnfollowAnotherUser:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialUserServiceAPIImpl.this.sendRequest(bus, sendable);
      }

      public void update(final SocialUser[] a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.ext.uberfire.social.activities.service.SocialUserServiceAPI").endpoint("update:org.ext.uberfire.social.activities.model.SocialUser[]:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_ext_uberfire_social_activities_service_SocialUserServiceAPIImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(SocialUserServiceAPI.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_ext_uberfire_social_activities_service_SocialUserServiceAPIImpl();
      }
    });
    class org_uberfire_ext_security_management_api_service_RoleManagerServiceImpl extends AbstractRpcProxy implements RoleManagerService { public SearchResponse search(final SearchRequest a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.RoleManagerService").endpoint("search:org.uberfire.ext.security.management.api.AbstractEntityManager.SearchRequest:", qualifiers, new Object[] { a0 }).respondTo(SearchResponse.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_RoleManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Role get(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.RoleManagerService").endpoint("get:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Role.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_RoleManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Role create(final Role a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.RoleManagerService").endpoint("create:org.jboss.errai.security.shared.api.Role:", qualifiers, new Object[] { a0 }).respondTo(Role.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_RoleManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Role update(final Role a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.RoleManagerService").endpoint("update:org.jboss.errai.security.shared.api.Role:", qualifiers, new Object[] { a0 }).respondTo(Role.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_RoleManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void delete(final String[] a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.RoleManagerService").endpoint("delete:java.lang.String[]:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_RoleManagerServiceImpl.this.sendRequest(bus, sendable);
      }

      public RoleManagerSettings getSettings() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.RoleManagerService").endpoint("getSettings:", qualifiers, new Object[] { }).respondTo(RoleManagerSettings.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_RoleManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(RoleManagerService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_security_management_api_service_RoleManagerServiceImpl();
      }
    });
    class org_uberfire_experimental_service_backend_BackendExperimentalFeaturesRegistryServiceImpl extends AbstractRpcProxy implements BackendExperimentalFeaturesRegistryService { public ExperimentalFeaturesSession getExperimentalFeaturesSession() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.experimental.service.backend.BackendExperimentalFeaturesRegistryService").endpoint("getExperimentalFeaturesSession:", qualifiers, new Object[] { }).respondTo(ExperimentalFeaturesSession.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_experimental_service_backend_BackendExperimentalFeaturesRegistryServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(BackendExperimentalFeaturesRegistryService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_experimental_service_backend_BackendExperimentalFeaturesRegistryServiceImpl();
      }
    });
    class org_guvnor_structure_repositories_RepositoryServiceEditorImpl extends AbstractRpcProxy implements RepositoryServiceEditor { public List revertHistory(final String a0, final Path a1, final String a2, final VersionRecord a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.structure.repositories.RepositoryServiceEditor").endpoint("revertHistory:java.lang.String:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.java.nio.base.version.VersionRecord:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(List.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_structure_repositories_RepositoryServiceEditorImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(RepositoryServiceEditor.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_structure_repositories_RepositoryServiceEditorImpl();
      }
    });
    class org_guvnor_common_services_project_service_POMServiceImpl extends AbstractRpcProxy implements POMService { public Path create(final Path a0, final POM a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.POMService").endpoint("create:org.uberfire.backend.vfs.Path:org.guvnor.common.services.project.model.POM:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_POMServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path save(final Path a0, final POM a1, final Metadata a2, final String a3, final boolean a4) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.POMService").endpoint("save:org.uberfire.backend.vfs.Path:org.guvnor.common.services.project.model.POM:org.guvnor.common.services.shared.metadata.model.Metadata:java.lang.String:boolean:", qualifiers, new Object[] { a0, a1, a2, a3, a4 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_POMServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public POM load(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.POMService").endpoint("load:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(POM.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_POMServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path save(final Path a0, final POM a1, final Metadata a2, final String a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.POMService").endpoint("save:org.uberfire.backend.vfs.Path:org.guvnor.common.services.project.model.POM:org.guvnor.common.services.shared.metadata.model.Metadata:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_POMServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(POMService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_project_service_POMServiceImpl();
      }
    });
    class org_uberfire_ext_editor_commons_service_DeleteServiceImpl extends AbstractRpcProxy implements DeleteService { public void deleteIfExists(final Collection a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.DeleteService").endpoint("deleteIfExists:java.util.Collection:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_DeleteServiceImpl.this.sendRequest(bus, sendable);
      }

      public boolean hasRestriction(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.DeleteService").endpoint("hasRestriction:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_DeleteServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public void delete(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.DeleteService").endpoint("delete:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_DeleteServiceImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(DeleteService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_editor_commons_service_DeleteServiceImpl();
      }
    });
    class org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl extends AbstractRpcProxy implements WorkspaceProjectService { public Collection getAllWorkspaceProjects() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("getAllWorkspaceProjects:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getAllWorkspaceProjects(final OrganizationalUnit a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("getAllWorkspaceProjects:org.guvnor.structure.organizationalunit.OrganizationalUnit:", qualifiers, new Object[] { a0 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection getAllWorkspaceProjectsByName(final OrganizationalUnit a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("getAllWorkspaceProjectsByName:org.guvnor.structure.organizationalunit.OrganizationalUnit:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public boolean spaceHasNoProjectsWithName(final OrganizationalUnit a0, final String a1, final WorkspaceProject a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("spaceHasNoProjectsWithName:org.guvnor.structure.organizationalunit.OrganizationalUnit:java.lang.String:org.guvnor.common.services.project.model.WorkspaceProject:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public WorkspaceProject newProject(final OrganizationalUnit a0, final POM a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("newProject:org.guvnor.structure.organizationalunit.OrganizationalUnit:org.guvnor.common.services.project.model.POM:", qualifiers, new Object[] { a0, a1 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public WorkspaceProject newProject(final OrganizationalUnit a0, final POM a1, final DeploymentMode a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("newProject:org.guvnor.structure.organizationalunit.OrganizationalUnit:org.guvnor.common.services.project.model.POM:org.guvnor.common.services.project.service.DeploymentMode:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public String createFreshProjectName(final OrganizationalUnit a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("createFreshProjectName:org.guvnor.structure.organizationalunit.OrganizationalUnit:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public WorkspaceProject resolveProject(final Repository a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("resolveProject:org.guvnor.structure.repositories.Repository:", qualifiers, new Object[] { a0 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public WorkspaceProject resolveProject(final Space a0, final Branch a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("resolveProject:org.uberfire.spaces.Space:org.guvnor.structure.repositories.Branch:", qualifiers, new Object[] { a0, a1 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public WorkspaceProject resolveProject(final Space a0, final Module a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("resolveProject:org.uberfire.spaces.Space:org.guvnor.common.services.project.model.Module:", qualifiers, new Object[] { a0, a1 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public WorkspaceProject resolveProject(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("resolveProject:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public WorkspaceProject resolveProject(final Space a0, final Path a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("resolveProject:org.uberfire.spaces.Space:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0, a1 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public WorkspaceProject resolveProject(final Space a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("resolveProject:org.uberfire.spaces.Space:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public WorkspaceProject resolveProjectByRepositoryAlias(final Space a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.WorkspaceProjectService").endpoint("resolveProjectByRepositoryAlias:org.uberfire.spaces.Space:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(WorkspaceProject.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(WorkspaceProjectService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_project_service_WorkspaceProjectServiceImpl();
      }
    });
    class org_guvnor_common_services_project_service_ModuleServiceImpl extends AbstractRpcProxy implements ModuleService { public Module resolveModule(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolveModule:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Module.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Module resolveModule(final Path a0, final boolean a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolveModule:org.uberfire.backend.vfs.Path:boolean:", qualifiers, new Object[] { a0, a1 }).respondTo(Module.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Module resolveParentModule(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolveParentModule:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Module.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Module resolveToParentModule(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolveToParentModule:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Module.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Set resolvePackages(final Module a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolvePackages:org.guvnor.common.services.project.model.Module:", qualifiers, new Object[] { a0 }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Set resolvePackages(final Package a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolvePackages:org.guvnor.common.services.project.model.Package:", qualifiers, new Object[] { a0 }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Package resolveDefaultPackage(final Module a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolveDefaultPackage:org.guvnor.common.services.project.model.Module:", qualifiers, new Object[] { a0 }).respondTo(Package.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Package resolveDefaultWorkspacePackage(final Module a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolveDefaultWorkspacePackage:org.guvnor.common.services.project.model.Module:", qualifiers, new Object[] { a0 }).respondTo(Package.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Package resolveParentPackage(final Package a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolveParentPackage:org.guvnor.common.services.project.model.Package:", qualifiers, new Object[] { a0 }).respondTo(Package.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path resolveDefaultPath(final Package a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolveDefaultPath:org.guvnor.common.services.project.model.Package:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public boolean isPom(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("isPom:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public Package resolvePackage(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("resolvePackage:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Package.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Set getAllModules(final Branch a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("getAllModules:org.guvnor.structure.repositories.Branch:", qualifiers, new Object[] { a0 }).respondTo(Set.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Module newModule(final Path a0, final POM a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("newModule:org.uberfire.backend.vfs.Path:org.guvnor.common.services.project.model.POM:", qualifiers, new Object[] { a0, a1 }).respondTo(Module.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Module newModule(final Path a0, final POM a1, final DeploymentMode a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("newModule:org.uberfire.backend.vfs.Path:org.guvnor.common.services.project.model.POM:org.guvnor.common.services.project.service.DeploymentMode:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Module.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Package newPackage(final Package a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("newPackage:org.guvnor.common.services.project.model.Package:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Package.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path rename(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("rename:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void delete(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("delete:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
      }

      public void copy(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
      }

      public void reImport(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.service.ModuleService").endpoint("reImport:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_service_ModuleServiceImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(ModuleService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_project_service_ModuleServiceImpl();
      }
    });
    class org_uberfire_backend_vfs_VFSServiceImpl extends AbstractRpcProxy implements VFSService { public Path get(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("get:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public DirectoryStream newDirectoryStream(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("newDirectoryStream:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(DirectoryStream.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public DirectoryStream newDirectoryStream(final Path a0, final Filter a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("newDirectoryStream:org.uberfire.backend.vfs.Path:org.uberfire.backend.vfs.DirectoryStream.Filter:", qualifiers, new Object[] { a0, a1 }).respondTo(DirectoryStream.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path createDirectory(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("createDirectory:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path createDirectory(final Path a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("createDirectory:org.uberfire.backend.vfs.Path:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path createDirectories(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("createDirectories:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path createDirectories(final Path a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("createDirectories:org.uberfire.backend.vfs.Path:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Map readAttributes(final Path a0) {
        try {
          final CallContextStatus status = new CallContextStatus(VFSCacheInterceptor.class);
          final RemoteCallContext callContext = new RemoteCallContext() {
            public String getMethodName() {
              return "readAttributes";
            }
            public Class getReturnType() {
              return Map.class;
            }
            public Annotation[] getAnnotations() {
              return new Annotation[] { new InterceptedCall() {
                  public Class annotationType() {
                    return InterceptedCall.class;
                  }
                  public String toString() {
                    return "@org.jboss.errai.common.client.api.interceptor.InterceptedCall(value=[class org.uberfire.backend.vfs.impl.VFSCacheInterceptor])";
                  }
                  public Class[] value() {
                    return new Class[] { VFSCacheInterceptor.class };
                  }
              } };
            }
            public Annotation[] getTypeAnnotations() {
              return new Annotation[] { };
            }
            public Object proceed() {
              status.proceed();
              if (status.getNextInterceptor() == null) {
                RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("readAttributes:org.uberfire.backend.vfs.Path:", qualifiers, getParameters()).respondTo(Map.class, remoteCallback).errorsHandledBy(errorCallback);
                org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
              } else if (status.getNextInterceptor() == VFSCacheInterceptor.class) {
                final RemoteCallContext ctx = this;
                final CreationalCallback icc = new CreationalCallback() {
                  public void callback(final Object beanInstance) {
                    status.setProceeding(false);
                    ((VFSCacheInterceptor) beanInstance).aroundInvoke(ctx);
                    if ((!status.isProceeding()) && (VFSCacheInterceptor.class == status.getNextInterceptor())) {
                      remoteCallback.callback(ctx.getResult());
                    }
                  }
                };
                AsyncBeanFactory.createBean(new VFSCacheInterceptor(), icc);
              }
              return null;
            }
            public void proceed(final RemoteCallback interceptorCallback) {
              final RemoteCallback providedCallback = org_uberfire_backend_vfs_VFSServiceImpl.this.remoteCallback;
              remoteCallback = new RemoteCallback() {
                public void callback(Object response) {
                  RemoteCallback intCallback = interceptorCallback;
                  setResult(response);
                  intCallback.callback(getResult());
                  providedCallback.callback(getResult());
                }
              };
              proceed();
            }
            public void proceed(RemoteCallback interceptorCallback, final ErrorCallback interceptorErrorCallback) {
              final ErrorCallback providedErrorCallback = org_uberfire_backend_vfs_VFSServiceImpl.this.errorCallback;
              errorCallback = new ErrorCallback() {
                public boolean error(Object message, Throwable throwable) {
                  interceptorErrorCallback.error(message, throwable);
                  if (getResult() != null) {
                    remoteCallback.callback(getResult());
                    return false;
                  } else if (providedErrorCallback != null) {
                    return providedErrorCallback.error(message, throwable);
                  }
                  return true;
                }
              };
              proceed(interceptorCallback);
            }
          };
          callContext.setParameters(new Object[] { a0 });
          callContext.proceed();
        } catch (Throwable throwable) {
          if (errorCallback != null) {
            if (errorCallback.error(null, throwable)) {
              invokeDefaultErrorHandlers(throwable);
            }
          } else {
            invokeDefaultErrorHandlers(throwable);
          }
        }
        return null;
      }

      public void setAttributes(final Path a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("setAttributes:org.uberfire.backend.vfs.Path:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
      }

      public void delete(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("delete:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
      }

      public boolean deleteIfExists(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("deleteIfExists:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public Path copy(final Path a0, final Path a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("copy:org.uberfire.backend.vfs.Path:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path move(final Path a0, final Path a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("move:org.uberfire.backend.vfs.Path:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public String readAllString(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("readAllString:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(String.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path write(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("write:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path write(final Path a0, final String a1, final Map a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("write:org.uberfire.backend.vfs.Path:java.lang.String:java.util.Map:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public boolean isRegularFile(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("isRegularFile:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public boolean isRegularFile(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("isRegularFile:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public boolean isDirectory(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("isDirectory:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public boolean isDirectory(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.backend.vfs.VFSService").endpoint("isDirectory:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_backend_vfs_VFSServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(VFSService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_backend_vfs_VFSServiceImpl();
      }
    });
    class org_uberfire_ext_security_management_api_service_UserManagerServiceImpl extends AbstractRpcProxy implements UserManagerService { public void assignGroups(final String a0, final Collection a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("assignGroups:java.lang.String:java.util.Collection:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
      }

      public void assignRoles(final String a0, final Collection a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("assignRoles:java.lang.String:java.util.Collection:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
      }

      public void changePassword(final String a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("changePassword:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
      }

      public SearchResponse search(final SearchRequest a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("search:org.uberfire.ext.security.management.api.AbstractEntityManager.SearchRequest:", qualifiers, new Object[] { a0 }).respondTo(SearchResponse.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public User get(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("get:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(User.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public User create(final User a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("create:org.jboss.errai.security.shared.api.identity.User:", qualifiers, new Object[] { a0 }).respondTo(User.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public User update(final User a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("update:org.jboss.errai.security.shared.api.identity.User:", qualifiers, new Object[] { a0 }).respondTo(User.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void delete(final String[] a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("delete:java.lang.String[]:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
      }

      public UserManagerSettings getSettings() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("getSettings:", qualifiers, new Object[] { }).respondTo(UserManagerSettings.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public User getUser(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.security.management.api.service.UserManagerService").endpoint("getUser:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(User.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_security_management_api_service_UserManagerServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(UserManagerService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_security_management_api_service_UserManagerServiceImpl();
      }
    });
    class org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl extends AbstractRpcProxy implements PerspectiveServices { public Plugin createNewPerspective(final String a0, final Style a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("createNewPerspective:java.lang.String:org.uberfire.ext.layout.editor.api.editor.LayoutTemplate.Style:", qualifiers, new Object[] { a0, a1 }).respondTo(Plugin.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Collection listLayoutTemplates() {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("listLayoutTemplates:", qualifiers, new Object[] { }).respondTo(Collection.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LayoutTemplate getLayoutTemplate(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("getLayoutTemplate:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(LayoutTemplate.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LayoutTemplate getLayoutTemplate(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("getLayoutTemplate:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(LayoutTemplate.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LayoutTemplate getLayoutTemplate(final Plugin a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("getLayoutTemplate:org.uberfire.ext.plugin.model.Plugin:", qualifiers, new Object[] { a0 }).respondTo(LayoutTemplate.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public LayoutTemplate convertToLayoutTemplate(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("convertToLayoutTemplate:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(LayoutTemplate.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path saveLayoutTemplate(final Path a0, final LayoutTemplate a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("saveLayoutTemplate:org.uberfire.backend.vfs.Path:org.uberfire.ext.layout.editor.api.editor.LayoutTemplate:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path copy(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path copy(final Path a0, final String a1, final Path a2, final String a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("copy:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void delete(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("delete:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
      }

      public Path saveAndRename(final Path a0, final String a1, final DefaultMetadata a2, final LayoutTemplate a3, final String a4) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("saveAndRename:org.uberfire.backend.vfs.Path:java.lang.String:org.uberfire.ext.editor.commons.file.DefaultMetadata:org.uberfire.ext.layout.editor.api.editor.LayoutTemplate:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3, a4 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path rename(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("rename:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public Path save(final Path a0, final LayoutTemplate a1, final DefaultMetadata a2, final String a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.layout.editor.api.PerspectiveServices").endpoint("save:org.uberfire.backend.vfs.Path:org.uberfire.ext.layout.editor.api.editor.LayoutTemplate:org.uberfire.ext.editor.commons.file.DefaultMetadata:java.lang.String:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(PerspectiveServices.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_layout_editor_api_PerspectiveServicesImpl();
      }
    });
    class org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl extends AbstractRpcProxy implements PreferenceBeanServerStore { public BasePreferencePortable load(final BasePreferencePortable a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("load:org.uberfire.preferences.shared.bean.BasePreferencePortable:", qualifiers, new Object[] { a0 }).respondTo(BasePreferencePortable.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public BasePreferencePortable load(final BasePreferencePortable a0, final PreferenceScopeResolutionStrategyInfo a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("load:org.uberfire.preferences.shared.bean.BasePreferencePortable:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:", qualifiers, new Object[] { a0, a1 }).respondTo(BasePreferencePortable.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void save(final BasePreferencePortable a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:org.uberfire.preferences.shared.bean.BasePreferencePortable:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final BasePreferencePortable a0, final PreferenceScopeResolutionStrategyInfo a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:org.uberfire.preferences.shared.bean.BasePreferencePortable:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final BasePreferencePortable a0, final PreferenceScope a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:org.uberfire.preferences.shared.bean.BasePreferencePortable:org.uberfire.preferences.shared.PreferenceScope:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final Collection a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:java.util.Collection:", qualifiers, new Object[] { a0 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final Collection a0, final PreferenceScopeResolutionStrategyInfo a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:java.util.Collection:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final Collection a0, final PreferenceScope a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:java.util.Collection:org.uberfire.preferences.shared.PreferenceScope:", qualifiers, new Object[] { a0, a1 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public PreferenceHierarchyElement buildHierarchyStructureForPreference(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("buildHierarchyStructureForPreference:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(PreferenceHierarchyElement.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public PreferenceHierarchyElement buildHierarchyStructureForPreference(final String a0, final PreferenceScopeResolutionStrategyInfo a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("buildHierarchyStructureForPreference:java.lang.String:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:", qualifiers, new Object[] { a0, a1 }).respondTo(PreferenceHierarchyElement.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public void load(final BasePreferencePortable a0, final ParameterizedCommand a1, final ParameterizedCommand a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("load:org.uberfire.preferences.shared.bean.BasePreferencePortable:org.uberfire.mvp.ParameterizedCommand:org.uberfire.mvp.ParameterizedCommand:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void load(final BasePreferencePortable a0, final PreferenceScopeResolutionStrategyInfo a1, final ParameterizedCommand a2, final ParameterizedCommand a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("load:org.uberfire.preferences.shared.bean.BasePreferencePortable:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:org.uberfire.mvp.ParameterizedCommand:org.uberfire.mvp.ParameterizedCommand:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final BasePreferencePortable a0, final Command a1, final ParameterizedCommand a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:org.uberfire.preferences.shared.bean.BasePreferencePortable:org.uberfire.mvp.Command:org.uberfire.mvp.ParameterizedCommand:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final BasePreferencePortable a0, final PreferenceScopeResolutionStrategyInfo a1, final Command a2, final ParameterizedCommand a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:org.uberfire.preferences.shared.bean.BasePreferencePortable:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:org.uberfire.mvp.Command:org.uberfire.mvp.ParameterizedCommand:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final BasePreferencePortable a0, final PreferenceScope a1, final Command a2, final ParameterizedCommand a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:org.uberfire.preferences.shared.bean.BasePreferencePortable:org.uberfire.preferences.shared.PreferenceScope:org.uberfire.mvp.Command:org.uberfire.mvp.ParameterizedCommand:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final Collection a0, final Command a1, final ParameterizedCommand a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:java.util.Collection:org.uberfire.mvp.Command:org.uberfire.mvp.ParameterizedCommand:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final Collection a0, final PreferenceScopeResolutionStrategyInfo a1, final Command a2, final ParameterizedCommand a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:java.util.Collection:org.uberfire.preferences.shared.impl.PreferenceScopeResolutionStrategyInfo:org.uberfire.mvp.Command:org.uberfire.mvp.ParameterizedCommand:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }

      public void save(final Collection a0, final PreferenceScope a1, final Command a2, final ParameterizedCommand a3) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.preferences.shared.bean.PreferenceBeanServerStore").endpoint("save:java.util.Collection:org.uberfire.preferences.shared.PreferenceScope:org.uberfire.mvp.Command:org.uberfire.mvp.ParameterizedCommand:", qualifiers, new Object[] { a0, a1, a2, a3 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl.this.sendRequest(bus, sendable);
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(PreferenceBeanServerStore.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_preferences_shared_bean_PreferenceBeanServerStoreImpl();
      }
    });
    class org_uberfire_ext_editor_commons_service_RenameServiceImpl extends AbstractRpcProxy implements RenameService { public void renameIfExists(final Collection a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.RenameService").endpoint("renameIfExists:java.util.Collection:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(void.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_RenameServiceImpl.this.sendRequest(bus, sendable);
      }

      public boolean hasRestriction(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.RenameService").endpoint("hasRestriction:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_RenameServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public Path rename(final Path a0, final String a1, final String a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.RenameService").endpoint("rename:org.uberfire.backend.vfs.Path:java.lang.String:java.lang.String:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(Path.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_RenameServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(RenameService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_editor_commons_service_RenameServiceImpl();
      }
    });
    class org_guvnor_common_services_project_builder_service_BuildServiceImpl extends AbstractRpcProxy implements BuildService { public BuildResults build(final Module a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("build:org.guvnor.common.services.project.model.Module:", qualifiers, new Object[] { a0 }).respondTo(BuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public BuildResults buildAndDeploy(final Module a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("buildAndDeploy:org.guvnor.common.services.project.model.Module:", qualifiers, new Object[] { a0 }).respondTo(BuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public BuildResults buildAndDeploy(final Module a0, final DeploymentMode a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("buildAndDeploy:org.guvnor.common.services.project.model.Module:org.guvnor.common.services.project.service.DeploymentMode:", qualifiers, new Object[] { a0, a1 }).respondTo(BuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public BuildResults buildAndDeploy(final Module a0, final boolean a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("buildAndDeploy:org.guvnor.common.services.project.model.Module:boolean:", qualifiers, new Object[] { a0, a1 }).respondTo(BuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public BuildResults buildAndDeploy(final Module a0, final boolean a1, final DeploymentMode a2) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("buildAndDeploy:org.guvnor.common.services.project.model.Module:boolean:org.guvnor.common.services.project.service.DeploymentMode:", qualifiers, new Object[] { a0, a1, a2 }).respondTo(BuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public boolean isBuilt(final Module a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("isBuilt:org.guvnor.common.services.project.model.Module:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public IncrementalBuildResults addPackageResource(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("addPackageResource:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(IncrementalBuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public IncrementalBuildResults deletePackageResource(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("deletePackageResource:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(IncrementalBuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public IncrementalBuildResults updatePackageResource(final Path a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("updatePackageResource:org.uberfire.backend.vfs.Path:", qualifiers, new Object[] { a0 }).respondTo(IncrementalBuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }

      public IncrementalBuildResults applyBatchResourceChanges(final Module a0, final Map a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.guvnor.common.services.project.builder.service.BuildService").endpoint("applyBatchResourceChanges:org.guvnor.common.services.project.model.Module:java.util.Map:", qualifiers, new Object[] { a0, a1 }).respondTo(IncrementalBuildResults.class, remoteCallback).errorsHandledBy(errorCallback);
        org_guvnor_common_services_project_builder_service_BuildServiceImpl.this.sendRequest(bus, sendable);
        return null;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(BuildService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_guvnor_common_services_project_builder_service_BuildServiceImpl();
      }
    });
    class org_uberfire_ext_editor_commons_service_ValidationServiceImpl extends AbstractRpcProxy implements org.uberfire.ext.editor.commons.service.ValidationService { public boolean isFileNameValid(final Path a0, final String a1) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.ValidationService").endpoint("isFileNameValid:org.uberfire.backend.vfs.Path:java.lang.String:", qualifiers, new Object[] { a0, a1 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_ValidationServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }

      public boolean isFileNameValid(final String a0) {
        RemoteCallSendable sendable = MessageBuilder.createCall().call("org.uberfire.ext.editor.commons.service.ValidationService").endpoint("isFileNameValid:java.lang.String:", qualifiers, new Object[] { a0 }).respondTo(Boolean.class, remoteCallback).errorsHandledBy(errorCallback);
        org_uberfire_ext_editor_commons_service_ValidationServiceImpl.this.sendRequest(bus, sendable);
        return false;
      }
    }
    RemoteServiceProxyFactory.addRemoteProxy(org.uberfire.ext.editor.commons.service.ValidationService.class, new ProxyProvider() {
      public Object getProxy() {
        return new org_uberfire_ext_editor_commons_service_ValidationServiceImpl();
      }
    });
  }
}