package org.jboss.errai.ioc.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.HeaderPanel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.web.bindery.event.shared.EventBus;
import elemental2.dom.CustomElementRegistry.DefineOptionsType;
import elemental2.dom.Document.RegisterElementOptionsType;
import elemental2.dom.Document.RegisterOptionsType;
import elemental2.dom.Element.ScrollIntoViewTopType;
import elemental2.dom.File.FilePropertiesType;
import elemental2.dom.HTMLAnchorElement;
import elemental2.dom.HTMLButtonElement;
import elemental2.dom.HTMLDivElement;
import elemental2.dom.HTMLDocument;
import elemental2.dom.HTMLElement;
import elemental2.dom.HTMLElement.AttachShadowOptionsType;
import elemental2.dom.HTMLInputElement;
import elemental2.dom.HTMLLIElement;
import elemental2.dom.HTMLLabelElement;
import elemental2.dom.HTMLOListElement;
import elemental2.dom.HTMLSlotElement.AssignedNodesOptionsType;
import elemental2.dom.HTMLTableRowElement;
import elemental2.dom.HTMLTextAreaElement;
import elemental2.dom.HTMLUListElement;
import elemental2.dom.HashChangeEvent.HashChangeEventEventInitDictType;
import elemental2.dom.InputDeviceCapabilities.InputDeviceCapabilitiesOptionsType;
import elemental2.dom.PageTransitionEvent.PageTransitionEventEventInitDictType;
import elemental2.dom.PopStateEvent.PopStateEventEventInitDictType;
import elemental2.dom.RTCConfigurationInterface_.IceServersFieldType;
import elemental2.dom.RTCConfigurationInterface_.UrlsFieldType;
import elemental2.dom.RelatedEvent.RelatedEventEventInitDictType;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.enterprise.event.Event;
import javax.enterprise.inject.Instance;
import javax.validation.Validator;
import org.guvnor.common.services.project.categories.Decision;
import org.guvnor.common.services.project.categories.Form;
import org.guvnor.common.services.project.categories.Model;
import org.guvnor.common.services.project.categories.Optimization;
import org.guvnor.common.services.project.categories.Process;
import org.guvnor.common.services.project.editor.type.POMResourceTypeDefinition;
import org.guvnor.common.services.project.preferences.GAVPreferencesBeanGeneratedImpl;
import org.guvnor.common.services.project.preferences.GeneralPreferencesBeanGeneratedImpl;
import org.guvnor.common.services.workingset.client.WorkingSetManager;
import org.gwtbootstrap3.client.ui.Anchor;
import org.gwtbootstrap3.client.ui.CheckBox;
import org.gwtbootstrap3.client.ui.Container;
import org.gwtbootstrap3.client.ui.TextBox;
import org.gwtbootstrap3.client.ui.gwt.FlowPanel;
import org.gwtbootstrap3.extras.toggleswitch.client.ui.ToggleSwitch;
import org.jboss.errai.bus.client.api.ClientMessageBus;
import org.jboss.errai.bus.client.api.messaging.RequestDispatcher;
import org.jboss.errai.common.client.api.BatchCaller;
import org.jboss.errai.common.client.api.Caller;
import org.jboss.errai.common.client.dom.Div;
import org.jboss.errai.common.client.dom.Document;
import org.jboss.errai.common.client.dom.Heading;
import org.jboss.errai.common.client.dom.Image;
import org.jboss.errai.common.client.dom.Input;
import org.jboss.errai.common.client.dom.Label;
import org.jboss.errai.common.client.dom.ListItem;
import org.jboss.errai.common.client.dom.Paragraph;
import org.jboss.errai.common.client.dom.RadioInput;
import org.jboss.errai.common.client.dom.Span;
import org.jboss.errai.common.client.dom.TableCell;
import org.jboss.errai.common.client.dom.TableSection;
import org.jboss.errai.common.client.dom.TextArea;
import org.jboss.errai.common.client.dom.TextInput;
import org.jboss.errai.common.client.dom.UnorderedList;
import org.jboss.errai.common.client.dom.elemental2.Elemental2DomUtil;
import org.jboss.errai.databinding.client.DataBinderProvider;
import org.jboss.errai.databinding.client.DataBindingModuleBootstrapper;
import org.jboss.errai.databinding.client.ListComponentProvider;
import org.jboss.errai.databinding.client.api.DataBinder;
import org.jboss.errai.databinding.client.components.ListComponent;
import org.jboss.errai.enterprise.client.cdi.CDIEventTypeLookup;
import org.jboss.errai.enterprise.client.cdi.EventProvider;
import org.jboss.errai.enterprise.client.cdi.InstanceProvider;
import org.jboss.errai.enterprise.client.cdi.JsTypeEventObserver;
import org.jboss.errai.enterprise.client.cdi.WindowEventObservers;
import org.jboss.errai.enterprise.client.cdi.api.CDI;
import org.jboss.errai.ioc.client.api.Disposer;
import org.jboss.errai.ioc.client.api.InitBallot;
import org.jboss.errai.ioc.client.api.ManagedInstance;
import org.jboss.errai.ioc.client.api.builtin.CallerProvider;
import org.jboss.errai.ioc.client.api.builtin.DisposerProvider;
import org.jboss.errai.ioc.client.api.builtin.IOCBeanManagerProvider;
import org.jboss.errai.ioc.client.api.builtin.InitBallotProvider;
import org.jboss.errai.ioc.client.api.builtin.ManagedInstanceProvider;
import org.jboss.errai.ioc.client.api.builtin.RootPanelProvider;
import org.jboss.errai.ioc.client.container.ApplicationScopedContext;
import org.jboss.errai.ioc.client.container.Context;
import org.jboss.errai.ioc.client.container.ContextManager;
import org.jboss.errai.ioc.client.container.ContextManagerImpl;
import org.jboss.errai.ioc.client.container.DependentScopeContext;
import org.jboss.errai.ioc.client.container.Factory;
import org.jboss.errai.ioc.client.container.JsTypeProvider;
import org.jboss.errai.ioc.client.container.SyncBeanManager;
import org.jboss.errai.ioc.client.lifecycle.impl.AccessImpl;
import org.jboss.errai.ioc.client.lifecycle.impl.CreationImpl;
import org.jboss.errai.ioc.client.lifecycle.impl.DestructionImpl;
import org.jboss.errai.ioc.client.lifecycle.impl.LifecycleListenerRegistrarImpl;
import org.jboss.errai.ioc.client.lifecycle.impl.StateChangeImpl;
import org.jboss.errai.ioc.support.bus.client.BatchCallerProvider;
import org.jboss.errai.ioc.support.bus.client.MessageBusProvider;
import org.jboss.errai.ioc.support.bus.client.RequestDispatcherProvider;
import org.jboss.errai.ioc.support.bus.client.Sender;
import org.jboss.errai.ioc.support.bus.client.SenderProvider;
import org.jboss.errai.security.client.local.callback.DefaultBusSecurityErrorCallback;
import org.jboss.errai.security.client.local.callback.DefaultRestSecurityErrorCallback;
import org.jboss.errai.security.client.local.context.BasicUserCacheImpl;
import org.jboss.errai.security.client.local.context.SecurityContextImpl;
import org.jboss.errai.security.client.local.context.SecurityContextImpl.SecurityRolesConstraintPage;
import org.jboss.errai.security.client.local.handler.DefaultSecurityExceptionHandler;
import org.jboss.errai.security.client.local.interceptors.AuthenticationServiceInterceptor;
import org.jboss.errai.security.client.local.interceptors.ClientSecurityRoleInterceptor;
import org.jboss.errai.security.client.local.nav.SecurityContextHoldingSingleton;
import org.jboss.errai.security.client.local.roles.ClientRequiredRolesExtractorImpl;
import org.jboss.errai.security.client.local.storage.StorageHandlerProvider;
import org.jboss.errai.security.client.local.storage.UserStorageHandler;
import org.jboss.errai.security.client.local.style.RoleStyleBindingProvider;
import org.jboss.errai.security.shared.api.identity.User;
import org.jboss.errai.ui.client.local.producer.Elemental2HTMLElementProvider;
import org.jboss.errai.ui.client.local.producer.HTMLElementProvider;
import org.jboss.errai.ui.client.local.spi.TranslationService;
import org.jboss.errai.ui.client.local.spi.TranslationServiceProvider;
import org.jboss.errai.ui.client.widget.ListWidget;
import org.jboss.errai.ui.client.widget.ListWidgetProvider;
import org.jboss.errai.ui.client.widget.LocaleListBox;
import org.jboss.errai.ui.client.widget.LocaleSelector;
import org.jboss.errai.ui.nav.client.local.HistoryTokenFactory;
import org.jboss.errai.ui.nav.client.local.Navigation;
import org.jboss.errai.ui.nav.client.local.NavigationPanel;
import org.jboss.errai.ui.nav.client.local.NavigationPanelProvider;
import org.jboss.errai.ui.nav.client.local.PageTransitionProvider;
import org.jboss.errai.ui.nav.client.local.TransitionAnchor;
import org.jboss.errai.ui.nav.client.local.TransitionAnchorFactory;
import org.jboss.errai.ui.nav.client.local.TransitionAnchorFactoryProvider;
import org.jboss.errai.ui.nav.client.local.TransitionAnchorProvider;
import org.jboss.errai.ui.nav.client.local.TransitionTo;
import org.jboss.errai.ui.nav.client.local.TransitionToRole;
import org.jboss.errai.ui.nav.client.local.TransitionToRoleProvider;
import org.jboss.errai.ui.nav.client.local.URLPatternMatcher;
import org.jboss.errai.ui.nav.client.local.URLPatternMatcherProvider;
import org.jboss.errai.ui.nav.client.local.spi.NavigationGraph;
import org.jboss.errai.ui.shared.ServerTemplateProvider;
import org.jboss.errai.validation.client.ValidatorProvider;
import org.jboss.errai.validation.client.dynamic.DynamicValidator;
import org.kie.workbench.common.forms.adf.engine.client.formGeneration.ClientFieldElementProcessor;
import org.kie.workbench.common.forms.adf.engine.client.formGeneration.ClientFormGenerator;
import org.kie.workbench.common.forms.adf.engine.client.formGeneration.ClientMetaDataEntryManager;
import org.kie.workbench.common.forms.adf.engine.client.formGeneration.util.ClientPropertuValueExtractor;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.layout.LayoutGenerator;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.HasMaxLengthFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.HasPlaceHolderFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.HasRowsFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.TextAreaFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.nestedForms.MultipleSubFormFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.nestedForms.SubFormFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.selectors.ListBoxFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.selectors.SelectorFieldInitilizer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.slider.DoubleSliderFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.formGeneration.processing.fields.fieldInitializers.slider.IntegerSliderFieldInitializer;
import org.kie.workbench.common.forms.adf.engine.shared.impl.FormBuildingServiceImpl;
import org.kie.workbench.common.forms.common.rendering.client.FormWidgetsEntryPoint;
import org.kie.workbench.common.forms.common.rendering.client.widgets.decimalBox.DecimalBox;
import org.kie.workbench.common.forms.common.rendering.client.widgets.decimalBox.DecimalBoxViewImpl;
import org.kie.workbench.common.forms.common.rendering.client.widgets.integerBox.IntegerBox;
import org.kie.workbench.common.forms.common.rendering.client.widgets.integerBox.IntegerBoxViewImpl;
import org.kie.workbench.common.forms.common.rendering.client.widgets.picture.PictureInput;
import org.kie.workbench.common.forms.common.rendering.client.widgets.picture.widget.PictureWidget;
import org.kie.workbench.common.forms.common.rendering.client.widgets.picture.widget.PictureWidgetViewImpl;
import org.kie.workbench.common.forms.common.rendering.client.widgets.typeahead.BindableTypeAhead;
import org.kie.workbench.common.forms.common.rendering.client.widgets.typeahead.BindableTypeAheadViewImpl;
import org.kie.workbench.common.forms.crud.client.CrudComponentEntryPoint;
import org.kie.workbench.common.forms.crud.client.component.CrudComponent;
import org.kie.workbench.common.forms.crud.client.component.CrudComponentViewImpl;
import org.kie.workbench.common.forms.crud.client.component.formDisplay.embedded.EmbeddedFormDisplayer;
import org.kie.workbench.common.forms.crud.client.component.formDisplay.embedded.EmbeddedFormDisplayerViewImpl;
import org.kie.workbench.common.forms.crud.client.component.formDisplay.modal.ModalFormDisplayer;
import org.kie.workbench.common.forms.crud.client.component.formDisplay.modal.ModalFormDisplayerViewImpl;
import org.kie.workbench.common.forms.dynamic.client.DynamicFormRenderer;
import org.kie.workbench.common.forms.dynamic.client.DynamicFormRendererViewImpl;
import org.kie.workbench.common.forms.dynamic.client.DynamicRendererEntryPoint;
import org.kie.workbench.common.forms.dynamic.client.config.ClientSelectorDataProviderManager;
import org.kie.workbench.common.forms.dynamic.client.helper.MapModelBindingHelper;
import org.kie.workbench.common.forms.dynamic.client.init.FormHandlerGeneratorManager;
import org.kie.workbench.common.forms.dynamic.client.init.impl.DynamicFormHandlerGenerator;
import org.kie.workbench.common.forms.dynamic.client.init.impl.StaticFormHandlerGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.FieldLayoutComponent;
import org.kie.workbench.common.forms.dynamic.client.rendering.FieldRendererManagerImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.FormGeneratorDriver;
import org.kie.workbench.common.forms.dynamic.client.rendering.FormLayoutGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.checkbox.CheckBoxFormGroup;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.checkbox.CheckBoxFormGroupViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.configError.ConfigErrorDisplayer;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.configError.ConfigErrorDisplayerViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.def.DefaultFormGroup;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.def.DefaultFormGroupViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.nestedForm.collapse.CollapsibleFormGroup;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.nestedForm.collapse.CollapsibleFormGroupViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.nestedForm.fieldSet.FieldSetFormGroup;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.nestedForm.fieldSet.FieldSetFormGroupViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.slider.SliderFormGroup;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.impl.slider.SliderFormGroupViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.labels.help.FieldHelp;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.labels.help.FieldHelpViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.labels.label.FieldLabel;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.labels.label.FieldLabelViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.labels.required.FieldRequired;
import org.kie.workbench.common.forms.dynamic.client.rendering.formGroups.labels.required.FieldRequiredViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.CheckBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.DecimalBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.IntegerBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.SliderFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.TextAreaFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.TextBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.date.DatePickerFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.MultipleInputFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.MultipleInput;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.MultipleInputViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.MultipleInputComponent;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.MultipleInputComponentViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.BooleanEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.CharacterEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.DateEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.DateTimePickerCell;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.DateTimePickerPresenter;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.DateTimePickerPresenterViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.EditableColumnGeneratorManagerImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.StringEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.numeric.decimals.BigDecimalEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.numeric.decimals.DoubleEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.numeric.decimals.FloatEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.numeric.integers.BigIntegerEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.numeric.integers.ByteEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.numeric.integers.IntegerEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.numeric.integers.LongEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.creator.input.widget.impl.numeric.integers.ShortEditableColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.selector.MultipleSelectorFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.selector.input.MultipleSelectorInput;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.lov.selector.input.MultipleSelectorInputViewImpl;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.picture.PictureFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.multipleSubform.ColumnGeneratorManager;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.multipleSubform.MultipleSubFormFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.multipleSubform.MultipleSubFormWidget;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.multipleSubform.binding.DynamicBindingHelper;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.multipleSubform.binding.StaticBindingHelper;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.multipleSubform.columns.impl.BooleanColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.multipleSubform.columns.impl.StringColumnGenerator;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.subform.SubFormFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.relations.subform.widget.SubFormWidget;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.listBox.CharacterListBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.listBox.DecimalListBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.listBox.EnumListBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.listBox.IntegerListBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.listBox.StringListBoxFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.radioGroup.CharacterRadioGroupFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.radioGroup.DecimalRadioGroupFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.radioGroup.IntegerRadioGroupFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.rendering.renderers.selectors.radioGroup.StringRadioGroupFieldRenderer;
import org.kie.workbench.common.forms.dynamic.client.service.ClientFieldManagerImpl;
import org.kie.workbench.common.forms.dynamic.service.shared.adf.DynamicFormModelGenerator;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.checkBox.provider.CheckBoxFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.datePicker.provider.DatePickerFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.decimalBox.provider.DecimalBoxFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.image.provider.PictureFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.integerBox.provider.IntegerBoxFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.lists.input.MultipleInputProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.lists.selector.MultipleSelectorProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.selectors.listBox.provider.ListBoxFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.selectors.radioGroup.provider.RadioGroupFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.slider.provider.SliderFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.textArea.provider.TextAreaFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.textBox.provider.TextBoxFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.relations.multipleSubform.provider.MultipleSubFormFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.relations.subForm.provider.SubFormFieldProvider;
import org.kie.workbench.common.forms.fields.shared.model.meta.entries.FieldPlaceHolderEntryProcessor;
import org.kie.workbench.common.forms.processing.engine.handling.ProcessingEngineEntryPoint;
import org.kie.workbench.common.forms.processing.engine.handling.impl.DefaultModelValidator;
import org.kie.workbench.common.forms.processing.engine.handling.impl.FieldChangeHandlerManagerImpl;
import org.kie.workbench.common.forms.processing.engine.handling.impl.FieldStateValidatorImpl;
import org.kie.workbench.common.forms.processing.engine.handling.impl.FormHandlerImpl;
import org.kie.workbench.common.forms.processing.engine.handling.impl.FormValidatorImpl;
import org.kie.workbench.common.forms.service.shared.meta.processing.impl.processors.FieldLabelEntryProcessor;
import org.kie.workbench.common.forms.service.shared.meta.processing.impl.processors.FieldReadOnlyEntryProcessor;
import org.kie.workbench.common.forms.service.shared.meta.processing.impl.processors.FieldRequiredEntryProcessor;
import org.kie.workbench.common.forms.service.shared.meta.processing.impl.processors.FieldTypeEntryProcessor;
import org.kie.workbench.common.stunner.bpmn.BPMNDefinitionSet;
import org.kie.workbench.common.stunner.bpmn.StunnerBPMNEntryPoint;
import org.kie.workbench.common.stunner.bpmn.client.BPMNShapeSet;
import org.kie.workbench.common.stunner.bpmn.client.BPMNThumbnailProvider;
import org.kie.workbench.common.stunner.bpmn.client.canvas.controls.BPMNCanvasShortcutsControlImpl;
import org.kie.workbench.common.stunner.bpmn.client.canvas.controls.BPMNCommonActionsToolboxFactory;
import org.kie.workbench.common.stunner.bpmn.client.canvas.controls.BPMNToolboxControl;
import org.kie.workbench.common.stunner.bpmn.client.canvas.controls.keyboard.shortcut.AppendEmbeddedSubprocessShortcut;
import org.kie.workbench.common.stunner.bpmn.client.canvas.controls.keyboard.shortcut.AppendNoneEndEventShortcut;
import org.kie.workbench.common.stunner.bpmn.client.canvas.controls.keyboard.shortcut.AppendNoneTaskShortcut;
import org.kie.workbench.common.stunner.bpmn.client.canvas.controls.keyboard.shortcut.AppendParallelGatewayShortcut;
import org.kie.workbench.common.stunner.bpmn.client.components.palette.BPMNPaletteDefinitionBuilder;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.AdHocOrderingProvider;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.DefaultRouteFormProvider;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.ProcessCompensationRefProvider;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.ProcessErrorRefProvider;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.ProcessEscalationRefProvider;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.ProcessMessageRefProvider;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.ProcessSignalRefProvider;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.SignalScopeProvider;
import org.kie.workbench.common.stunner.bpmn.client.dataproviders.VariablesProvider;
import org.kie.workbench.common.stunner.bpmn.client.diagram.DiagramTypeServiceImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.AssigneeEditorFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.formGroup.AssigneeFormGroup;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.formGroup.AssigneeFormGroupViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.widget.AssigneeEditorWidget;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.widget.AssigneeEditorWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.widget.AssigneeListItem;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.widget.AssigneeLiveSearchEntryCreationEditor;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.widget.AssigneeLiveSearchEntryCreationEditorViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assigneeEditor.widget.AssigneeLiveSearchService;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assignmentsEditor.ActivityDataIOEditor;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assignmentsEditor.ActivityDataIOEditorViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assignmentsEditor.ActivityDataIOEditorWidget;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assignmentsEditor.ActivityDataIOEditorWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assignmentsEditor.AssignmentListItemWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assignmentsEditor.AssignmentsEditorFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.assignmentsEditor.AssignmentsEditorWidget;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.cm.roles.RolesEditorFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.cm.roles.RolesEditorWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.cm.roles.RolesListItemWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.comboBoxEditor.ComboBoxFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.comboBoxEditor.ComboBoxFixedValuesWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.comboBoxEditor.ComboBoxWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.comboBoxEditor.ConditionalComboBoxFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.scriptEditor.ScriptTypeFieldEditorPresenter;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.scriptEditor.ScriptTypeFieldEditorView;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.scriptEditor.ScriptTypeFieldEditorWidget;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.scriptEditor.ScriptTypeFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.scriptEditor.ScriptTypeListFieldEditorPresenter;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.scriptEditor.ScriptTypeListFieldEditorWidget;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.scriptEditor.ScriptTypeListFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.timerEditor.TimerSettingsFieldEditorPresenter;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.timerEditor.TimerSettingsFieldEditorView;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.timerEditor.TimerSettingsFieldEditorWidget;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.timerEditor.TimerSettingsFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.variablesEditor.VariableDeleteHandler;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.variablesEditor.VariableListItemWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.variablesEditor.VariablesEditorFieldRenderer;
import org.kie.workbench.common.stunner.bpmn.client.forms.fields.variablesEditor.VariablesEditorWidgetViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.filters.BPMNDiagramFilterProvider;
import org.kie.workbench.common.stunner.bpmn.client.forms.widgets.ComboBox;
import org.kie.workbench.common.stunner.bpmn.client.forms.widgets.ComboBoxViewImpl;
import org.kie.workbench.common.stunner.bpmn.client.forms.widgets.VariableNameTextBox;
import org.kie.workbench.common.stunner.bpmn.client.preferences.BPMNTextPreferences;
import org.kie.workbench.common.stunner.bpmn.client.resources.BPMNIconsStrip;
import org.kie.workbench.common.stunner.bpmn.client.resources.BPMNSVGViewFactoryImpl;
import org.kie.workbench.common.stunner.bpmn.client.session.BPMNSessionInitializer;
import org.kie.workbench.common.stunner.bpmn.client.shape.factory.BPMNShapeFactory;
import org.kie.workbench.common.stunner.bpmn.client.workitem.WorkItemDefinitionClientRegistry;
import org.kie.workbench.common.stunner.bpmn.definition.adapter.binding.BPMNDefinitionSetDefinitionAdapterImpl;
import org.kie.workbench.common.stunner.bpmn.definition.adapter.binding.BPMNDefinitionSetDefinitionSetAdapterImpl;
import org.kie.workbench.common.stunner.bpmn.definition.adapter.binding.BPMNDefinitionSetDefinitionSetProxyImpl;
import org.kie.workbench.common.stunner.bpmn.definition.adapter.binding.BPMNDefinitionSetPropertyAdapterImpl;
import org.kie.workbench.common.stunner.bpmn.definition.adapter.binding.BPMNDefinitionSetPropertySetAdapterImpl;
import org.kie.workbench.common.stunner.bpmn.definition.adapter.binding.BPMNDefinitionSetRuleAdapterImpl;
import org.kie.workbench.common.stunner.bpmn.definition.factory.BPMNDefinitionSetModelFactoryImpl;
import org.kie.workbench.common.stunner.bpmn.definition.morph.BPMNDefinitionSetMorphDefinitionProvider;
import org.kie.workbench.common.stunner.bpmn.factory.BPMNDiagramFactoryImpl;
import org.kie.workbench.common.stunner.bpmn.factory.BPMNGraphFactoryImpl;
import org.kie.workbench.common.stunner.bpmn.forms.serializer.KeyValueSerializer;
import org.kie.workbench.common.stunner.bpmn.forms.serializer.cm.CaseRoleSerializer;
import org.kie.workbench.common.stunner.bpmn.forms.service.adf.processing.processors.fields.AssigneeFieldInitializer;
import org.kie.workbench.common.stunner.bpmn.forms.service.adf.processing.processors.fields.AssignmentsEditorFieldInitializer;
import org.kie.workbench.common.stunner.bpmn.forms.service.adf.processing.processors.fields.ScriptTypeFieldInitializer;
import org.kie.workbench.common.stunner.bpmn.forms.service.adf.processing.processors.fields.ScriptTypeListFieldInitializer;
import org.kie.workbench.common.stunner.bpmn.forms.service.adf.processing.processors.fields.VariablesEditorFieldInitializer;
import org.kie.workbench.common.stunner.bpmn.forms.service.adf.processing.processors.fields.cm.RolesEditorFieldInitializer;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.AssigneeEditorFieldProvider;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.AssignmentsEditorFieldProvider;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.ComboBoxFieldProvider;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.ConditionalComboBoxFieldProvider;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.ScriptTypeFieldProvider;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.ScriptTypeListFieldProvider;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.TimerSettingsFieldProvider;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.VariablesEditorFieldProvider;
import org.kie.workbench.common.stunner.bpmn.forms.service.fieldProviders.cm.RolesEditorFieldProvider;
import org.kie.workbench.common.stunner.bpmn.resource.BPMNDefinitionSetResourceType;
import org.kie.workbench.common.stunner.bpmn.workitem.ServiceTaskFactory;
import org.kie.workbench.common.stunner.bpmn.workitem.WorkItemDefinitionCacheRegistry;
import org.kie.workbench.common.stunner.bpmn.workitem.WorkItemDefinitionRegistry;
import org.kie.workbench.common.stunner.client.lienzo.StunnerLienzoCore;
import org.kie.workbench.common.stunner.client.lienzo.canvas.LienzoLayer;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.ConnectionAcceptorControlImpl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.ContainmentAcceptorControlImpl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.ControlPointControlImpl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.DockingAcceptorControlImpl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.LienzoMultipleSelectionControl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.LienzoSelectionControl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.LocationControlImpl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.PanControlImpl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.ResizeControlImpl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.controls.ZoomWheelControlImpl;
import org.kie.workbench.common.stunner.client.lienzo.canvas.export.LienzoCanvasExport;
import org.kie.workbench.common.stunner.client.lienzo.canvas.index.bounds.CanvasBoundsIndexerImpl;
import org.kie.workbench.common.stunner.client.lienzo.components.drag.ConnectorDragProxyImpl;
import org.kie.workbench.common.stunner.client.lienzo.components.drag.PrimitiveDragProxyImpl;
import org.kie.workbench.common.stunner.client.lienzo.components.drag.ShapeViewDragProxyImpl;
import org.kie.workbench.common.stunner.client.lienzo.components.glyph.LienzoGlyphRenderers;
import org.kie.workbench.common.stunner.client.lienzo.components.glyph.LienzoImageStripGlyphRenderer;
import org.kie.workbench.common.stunner.client.lienzo.components.glyph.LienzoImageStripLoader;
import org.kie.workbench.common.stunner.client.lienzo.components.glyph.LienzoPictureGlyphRenderer;
import org.kie.workbench.common.stunner.client.lienzo.components.glyph.LienzoShapeGlyphRenderer;
import org.kie.workbench.common.stunner.client.lienzo.components.glyph.LienzoSvgDataUriGlyphRenderer;
import org.kie.workbench.common.stunner.client.lienzo.components.glyph.ShapeGlyphDragHandlerImpl;
import org.kie.workbench.common.stunner.client.lienzo.components.toolbox.actions.CommonActionsToolboxView;
import org.kie.workbench.common.stunner.client.lienzo.components.toolbox.actions.FlowActionsToolboxView;
import org.kie.workbench.common.stunner.client.lienzo.components.toolbox.actions.MorphActionsToolboxView;
import org.kie.workbench.common.stunner.client.lienzo.components.views.LienzoTextTooltip;
import org.kie.workbench.common.stunner.client.lienzo.util.LienzoPanelUtils;
import org.kie.workbench.common.stunner.client.lienzo.wires.StunnerWiresControlFactory;
import org.kie.workbench.common.stunner.client.lienzo.wires.StunnerWiresHandlerFactory;
import org.kie.workbench.common.stunner.client.lienzo.wires.WiresManagerFactoryImpl;
import org.kie.workbench.common.stunner.client.widgets.StunnerWidgetsEntryPoint;
import org.kie.workbench.common.stunner.client.widgets.canvas.actions.IntegerTextBox;
import org.kie.workbench.common.stunner.client.widgets.canvas.actions.TextEditorMultiLineBox;
import org.kie.workbench.common.stunner.client.widgets.canvas.actions.TextEditorMultiLineBoxView;
import org.kie.workbench.common.stunner.client.widgets.canvas.actions.TextEditorSingleLineBox;
import org.kie.workbench.common.stunner.client.widgets.canvas.actions.TextEditorSingleLineBoxView;
import org.kie.workbench.common.stunner.client.widgets.canvas.view.CanvasView;
import org.kie.workbench.common.stunner.client.widgets.canvas.view.LienzoPanel;
import org.kie.workbench.common.stunner.client.widgets.canvas.wires.WiresCanvasPresenter;
import org.kie.workbench.common.stunner.client.widgets.canvas.wires.WiresCanvasView;
import org.kie.workbench.common.stunner.client.widgets.components.glyph.BS3IconTypeGlyphRenderer;
import org.kie.workbench.common.stunner.client.widgets.components.glyph.DOMGlyphRenderers;
import org.kie.workbench.common.stunner.client.widgets.components.glyph.ElementShapeGlyphRenderer;
import org.kie.workbench.common.stunner.client.widgets.components.glyph.ImageElementGlyphRenderer;
import org.kie.workbench.common.stunner.client.widgets.components.glyph.ImageStripDOMGlyphRenderer;
import org.kie.workbench.common.stunner.client.widgets.components.glyph.SvgElementGlyphRenderer;
import org.kie.workbench.common.stunner.client.widgets.explorer.navigator.diagrams.DiagramNavigatorItemImpl;
import org.kie.workbench.common.stunner.client.widgets.explorer.navigator.diagrams.DiagramsNavigatorImpl;
import org.kie.workbench.common.stunner.client.widgets.explorer.navigator.item.NavigatorThumbnailItemView;
import org.kie.workbench.common.stunner.client.widgets.explorer.navigator.view.BootstrapNavigatorView;
import org.kie.workbench.common.stunner.client.widgets.explorer.tree.TreeExplorer;
import org.kie.workbench.common.stunner.client.widgets.explorer.tree.TreeExplorerView;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.MenuDevCommandsBuilder;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.ShapeSetsMenuItemsBuilder;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.ClearCommandHistoryDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.LogBoundsDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.LogCommandHistoryDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.LogDefinitionDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.LogGraphDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.LogNodeEdgesDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.LogSelectedEdgeDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.LogSelectedItemsDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.LogSessionDevCommand;
import org.kie.workbench.common.stunner.client.widgets.menu.dev.impl.SwitchLogLevelDevCommand;
import org.kie.workbench.common.stunner.client.widgets.notification.NotificationsObserver;
import org.kie.workbench.common.stunner.client.widgets.palette.BS3PaletteWidgetImpl;
import org.kie.workbench.common.stunner.client.widgets.palette.BS3PaletteWidgetViewImpl;
import org.kie.workbench.common.stunner.client.widgets.palette.DefaultPaletteFactory;
import org.kie.workbench.common.stunner.client.widgets.palette.categories.DefinitionPaletteCategoryWidget;
import org.kie.workbench.common.stunner.client.widgets.palette.categories.DefinitionPaletteCategoryWidgetViewImpl;
import org.kie.workbench.common.stunner.client.widgets.palette.categories.group.DefinitionPaletteGroupWidget;
import org.kie.workbench.common.stunner.client.widgets.palette.categories.group.DefinitionPaletteGroupWidgetViewImpl;
import org.kie.workbench.common.stunner.client.widgets.palette.categories.items.DefinitionPaletteItemWidget;
import org.kie.workbench.common.stunner.client.widgets.palette.categories.items.DefinitionPaletteItemWidgetViewImpl;
import org.kie.workbench.common.stunner.client.widgets.palette.collapsed.CollapsedDefinitionPaletteItemWidget;
import org.kie.workbench.common.stunner.client.widgets.palette.collapsed.CollapsedDefinitionPaletteItemWidgetViewImpl;
import org.kie.workbench.common.stunner.client.widgets.popups.PopupUtil;
import org.kie.workbench.common.stunner.client.widgets.presenters.diagram.impl.DefaultDiagramEditor;
import org.kie.workbench.common.stunner.client.widgets.presenters.diagram.impl.DefaultDiagramViewer;
import org.kie.workbench.common.stunner.client.widgets.presenters.session.impl.SessionContainer;
import org.kie.workbench.common.stunner.client.widgets.presenters.session.impl.SessionEditorImpl;
import org.kie.workbench.common.stunner.client.widgets.presenters.session.impl.SessionEditorPresenter;
import org.kie.workbench.common.stunner.client.widgets.presenters.session.impl.SessionPresenterView;
import org.kie.workbench.common.stunner.client.widgets.presenters.session.impl.SessionPreviewImpl;
import org.kie.workbench.common.stunner.client.widgets.presenters.session.impl.SessionViewerImpl;
import org.kie.workbench.common.stunner.client.widgets.presenters.session.impl.SessionViewerPresenter;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.ClearToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.CopyToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.CutToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.DeleteSelectionToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.ExportToBpmnToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.ExportToJpgToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.ExportToPdfToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.ExportToPngToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.ExportToSvgToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.PasteToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.RedoToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.SaveToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.SwitchGridToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.UndoToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.ValidateToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.command.VisitGraphToolbarCommand;
import org.kie.workbench.common.stunner.client.widgets.toolbar.impl.DefaultToolbarView;
import org.kie.workbench.common.stunner.client.widgets.toolbar.impl.ManagedEditorToolbar;
import org.kie.workbench.common.stunner.client.widgets.toolbar.impl.ManagedToolbar;
import org.kie.workbench.common.stunner.client.widgets.toolbar.impl.ManagedViewerToolbar;
import org.kie.workbench.common.stunner.client.widgets.toolbar.item.ToolbarItem;
import org.kie.workbench.common.stunner.client.widgets.toolbar.item.ToolbarItemView;
import org.kie.workbench.common.stunner.client.widgets.views.AnimatedFloatingWidgetView;
import org.kie.workbench.common.stunner.client.widgets.views.ImageElementRendererViewImpl;
import org.kie.workbench.common.stunner.client.widgets.views.LoadingBox;
import org.kie.workbench.common.stunner.client.widgets.views.WidgetElementRendererViewImpl;
import org.kie.workbench.common.stunner.client.widgets.views.WidgetWrapperViewImpl;
import org.kie.workbench.common.stunner.client.widgets.views.session.NoCanvasView;
import org.kie.workbench.common.stunner.client.widgets.views.session.ScreenErrorViewImpl;
import org.kie.workbench.common.stunner.client.widgets.views.session.ScreenPanelViewImpl;
import org.kie.workbench.common.stunner.cm.CaseManagementDefinitionSet;
import org.kie.workbench.common.stunner.cm.CaseManagementEntryPoint;
import org.kie.workbench.common.stunner.cm.client.CaseManagementShapeSet;
import org.kie.workbench.common.stunner.cm.client.canvas.CaseManagementCanvasHandler;
import org.kie.workbench.common.stunner.cm.client.canvas.CaseManagementCanvasPresenter;
import org.kie.workbench.common.stunner.cm.client.canvas.CaseManagementCanvasView;
import org.kie.workbench.common.stunner.cm.client.canvas.controls.CaseManagementLocationControlImpl;
import org.kie.workbench.common.stunner.cm.client.canvas.controls.builder.CaseManagementElementBuilderControl;
import org.kie.workbench.common.stunner.cm.client.canvas.controls.builder.CaseManagementNodeBuilderControl;
import org.kie.workbench.common.stunner.cm.client.canvas.controls.builder.CaseManagementObserverBuilderControl;
import org.kie.workbench.common.stunner.cm.client.canvas.controls.containment.CaseManagementContainmentAcceptorControlImpl;
import org.kie.workbench.common.stunner.cm.client.command.CaseManagementCanvasCommandFactory;
import org.kie.workbench.common.stunner.cm.client.palette.CaseManagementPaletteDefinitionBuilder;
import org.kie.workbench.common.stunner.cm.client.preferences.CaseManagementPreferencesRegistry;
import org.kie.workbench.common.stunner.cm.client.resources.CaseManagementSVGViewFactoryImpl;
import org.kie.workbench.common.stunner.cm.client.session.CaseManagementEditorSession;
import org.kie.workbench.common.stunner.cm.client.session.CaseManagementViewerSession;
import org.kie.workbench.common.stunner.cm.client.session.command.impl.CaseManagementDeleteSelectionSessionCommand;
import org.kie.workbench.common.stunner.cm.client.session.command.impl.CaseManagementVisitGraphSessionCommand;
import org.kie.workbench.common.stunner.cm.client.session.command.impl.PasteSelectionSessionCommand;
import org.kie.workbench.common.stunner.cm.client.shape.factory.CaseManagementShapeDefFactory;
import org.kie.workbench.common.stunner.cm.client.shape.factory.CaseManagementShapeDefFunctionalFactory;
import org.kie.workbench.common.stunner.cm.client.shape.factory.CaseManagementShapeFactory;
import org.kie.workbench.common.stunner.cm.client.toolbar.command.CaseManagementDeleteSelectionToolbarCommand;
import org.kie.workbench.common.stunner.cm.client.toolbar.command.CaseManagementVisitGraphToolbarCommand;
import org.kie.workbench.common.stunner.cm.client.toolbar.impl.CaseManagementEditorToolbar;
import org.kie.workbench.common.stunner.cm.client.wires.CaseManagementContainmentStateHolder;
import org.kie.workbench.common.stunner.cm.client.wires.CaseManagementControlFactory;
import org.kie.workbench.common.stunner.cm.client.wires.CaseManagementWiresManagerFactory;
import org.kie.workbench.common.stunner.cm.definition.adapter.binding.CaseManagementDefinitionSetDefinitionAdapterImpl;
import org.kie.workbench.common.stunner.cm.definition.adapter.binding.CaseManagementDefinitionSetDefinitionSetAdapterImpl;
import org.kie.workbench.common.stunner.cm.definition.adapter.binding.CaseManagementDefinitionSetDefinitionSetProxyImpl;
import org.kie.workbench.common.stunner.cm.definition.adapter.binding.CaseManagementDefinitionSetPropertyAdapterImpl;
import org.kie.workbench.common.stunner.cm.definition.adapter.binding.CaseManagementDefinitionSetPropertySetAdapterImpl;
import org.kie.workbench.common.stunner.cm.definition.adapter.binding.CaseManagementDefinitionSetRuleAdapterImpl;
import org.kie.workbench.common.stunner.cm.definition.factory.CaseManagementDefinitionSetModelFactoryImpl;
import org.kie.workbench.common.stunner.cm.definition.formBuilder.provider.ModuleFormGenerationResourcesProvider;
import org.kie.workbench.common.stunner.cm.factory.CaseManagementDiagramFactoryImpl;
import org.kie.workbench.common.stunner.cm.factory.CaseManagementGraphFactoryImpl;
import org.kie.workbench.common.stunner.cm.resource.CaseManagementDefinitionSetResourceType;
import org.kie.workbench.common.stunner.core.StunnerCoreEntryPoint;
import org.kie.workbench.common.stunner.core.client.api.ClientDefinitionManager;
import org.kie.workbench.common.stunner.core.client.api.ClientFactoryManager;
import org.kie.workbench.common.stunner.core.client.api.SessionManagerImpl;
import org.kie.workbench.common.stunner.core.client.api.ShapeManagerImpl;
import org.kie.workbench.common.stunner.core.client.canvas.CanvasHandlerImpl;
import org.kie.workbench.common.stunner.core.client.canvas.command.DefaultCanvasCommandFactory;
import org.kie.workbench.common.stunner.core.client.canvas.controls.actions.CanvasInPlaceTextEditorControlMultiLine;
import org.kie.workbench.common.stunner.core.client.canvas.controls.actions.CanvasInPlaceTextEditorControlSingleLine;
import org.kie.workbench.common.stunner.core.client.canvas.controls.actions.DefaultTextPropertyProviderImpl;
import org.kie.workbench.common.stunner.core.client.canvas.controls.actions.TextPropertyProviderFactoryImpl;
import org.kie.workbench.common.stunner.core.client.canvas.controls.builder.impl.EdgeBuilderControlImpl;
import org.kie.workbench.common.stunner.core.client.canvas.controls.builder.impl.ElementBuilderControlImpl;
import org.kie.workbench.common.stunner.core.client.canvas.controls.builder.impl.NodeBuilderControlImpl;
import org.kie.workbench.common.stunner.core.client.canvas.controls.builder.impl.ObserverBuilderControl;
import org.kie.workbench.common.stunner.core.client.canvas.controls.clipboard.ClipboardControlCleaner;
import org.kie.workbench.common.stunner.core.client.canvas.controls.clipboard.LocalClipboardControl;
import org.kie.workbench.common.stunner.core.client.canvas.controls.keyboard.KeyEventHandler;
import org.kie.workbench.common.stunner.core.client.canvas.controls.keyboard.KeyboardControlImpl;
import org.kie.workbench.common.stunner.core.client.canvas.controls.toolbox.DefaultToolboxControl;
import org.kie.workbench.common.stunner.core.client.canvas.util.CanvasFileExport;
import org.kie.workbench.common.stunner.core.client.canvas.util.CanvasLayoutUtils;
import org.kie.workbench.common.stunner.core.client.command.CanvasCommandManagerImpl;
import org.kie.workbench.common.stunner.core.client.command.RequestCommandManager;
import org.kie.workbench.common.stunner.core.client.command.SessionCommandManagerImpl;
import org.kie.workbench.common.stunner.core.client.components.drag.NodeDragProxyImpl;
import org.kie.workbench.common.stunner.core.client.components.drag.ShapeDragProxyImpl;
import org.kie.workbench.common.stunner.core.client.components.layout.LayoutHelper;
import org.kie.workbench.common.stunner.core.client.components.layout.OpenDiagramLayoutExecutor;
import org.kie.workbench.common.stunner.core.client.components.palette.CollapsedPaletteDefinitionBuilder;
import org.kie.workbench.common.stunner.core.client.components.palette.ExpandedPaletteDefinitionBuilder;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.BPMNCreateNodeAction;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.CommonActionsToolboxFactory;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.CreateConnectorAction;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.CreateNodeAction;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.DeleteNodeAction;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.FlowActionsToolboxFactory;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.MorphActionsToolboxFactory;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.MorphNodeAction;
import org.kie.workbench.common.stunner.core.client.components.toolbox.actions.ToolboxDomainLookups;
import org.kie.workbench.common.stunner.core.client.components.views.CanvasDefinitionTooltip;
import org.kie.workbench.common.stunner.core.client.definition.adapter.binding.ClientBindableAdapterFactory;
import org.kie.workbench.common.stunner.core.client.definition.adapter.binding.ClientBindableMorphAdapter;
import org.kie.workbench.common.stunner.core.client.error.DiagramClientErrorHandler;
import org.kie.workbench.common.stunner.core.client.event.screen.ScreenEventPublisher;
import org.kie.workbench.common.stunner.core.client.i18n.ClientTranslationMessages;
import org.kie.workbench.common.stunner.core.client.i18n.ClientTranslationService;
import org.kie.workbench.common.stunner.core.client.preferences.DefaultPreferencesRegistry;
import org.kie.workbench.common.stunner.core.client.preferences.StunnerPreferencesEntryPoint;
import org.kie.workbench.common.stunner.core.client.preferences.StunnerPreferencesRegistries;
import org.kie.workbench.common.stunner.core.client.preferences.StunnerPreferencesRegistryLoader;
import org.kie.workbench.common.stunner.core.client.registry.impl.ClientDefinitionsCacheRegistry;
import org.kie.workbench.common.stunner.core.client.registry.impl.ClientRegistryFactoryImpl;
import org.kie.workbench.common.stunner.core.client.resources.StunnerCommonIconsStrip;
import org.kie.workbench.common.stunner.core.client.rule.ClientRuleManager;
import org.kie.workbench.common.stunner.core.client.service.ClientDiagramServiceImpl;
import org.kie.workbench.common.stunner.core.client.service.ClientFactoryService;
import org.kie.workbench.common.stunner.core.client.session.command.ManagedClientSessionCommands;
import org.kie.workbench.common.stunner.core.client.session.command.impl.ClearSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.CopySelectionSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.CutSelectionSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.DeleteSelectionSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.ExportToBpmnSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.ExportToJpgSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.ExportToPdfSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.ExportToPngSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.ExportToSvgSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.RedoSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.SaveDiagramSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.SwitchGridSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.UndoSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.ValidateSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.command.impl.VisitGraphSessionCommand;
import org.kie.workbench.common.stunner.core.client.session.impl.DefaultEditorSession;
import org.kie.workbench.common.stunner.core.client.session.impl.DefaultViewerSession;
import org.kie.workbench.common.stunner.core.client.session.impl.ManagedSession;
import org.kie.workbench.common.stunner.core.client.session.impl.SessionLoader;
import org.kie.workbench.common.stunner.core.client.shape.ImageStripRegistry;
import org.kie.workbench.common.stunner.core.client.shape.factory.DelegateShapeFactory;
import org.kie.workbench.common.stunner.core.client.shape.factory.ShapeDefFunctionalFactory;
import org.kie.workbench.common.stunner.core.client.util.SvgDataUriGenerator;
import org.kie.workbench.common.stunner.core.client.validation.ClientDiagramValidator;
import org.kie.workbench.common.stunner.core.client.validation.ClientModelValidator;
import org.kie.workbench.common.stunner.core.client.validation.canvas.CanvasDiagramValidator;
import org.kie.workbench.common.stunner.core.command.util.RedoCommandHandler;
import org.kie.workbench.common.stunner.core.definition.adapter.AdapterManagerImpl;
import org.kie.workbench.common.stunner.core.definition.adapter.bootstrap.BootstrapAdapterFactory;
import org.kie.workbench.common.stunner.core.definition.clone.CloneManagerImpl;
import org.kie.workbench.common.stunner.core.definition.clone.DeepCloneProcess;
import org.kie.workbench.common.stunner.core.definition.clone.DefaultCloneProcess;
import org.kie.workbench.common.stunner.core.definition.clone.NoneCloneProcess;
import org.kie.workbench.common.stunner.core.factory.impl.DiagramFactoryImpl;
import org.kie.workbench.common.stunner.core.factory.impl.EdgeFactoryImpl;
import org.kie.workbench.common.stunner.core.factory.impl.GraphFactoryImpl;
import org.kie.workbench.common.stunner.core.factory.impl.NodeFactoryImpl;
import org.kie.workbench.common.stunner.core.graph.command.GraphCommandManagerImpl;
import org.kie.workbench.common.stunner.core.graph.command.impl.GraphCommandFactory;
import org.kie.workbench.common.stunner.core.graph.processing.index.bounds.GraphBoundsIndexerImpl;
import org.kie.workbench.common.stunner.core.graph.processing.index.map.MapIndexBuilder;
import org.kie.workbench.common.stunner.core.graph.processing.layout.sugiyama.SugiyamaLayoutService;
import org.kie.workbench.common.stunner.core.graph.processing.layout.sugiyama.step01.ReverseEdgesCycleBreaker;
import org.kie.workbench.common.stunner.core.graph.processing.layout.sugiyama.step02.LongestPathVertexLayerer;
import org.kie.workbench.common.stunner.core.graph.processing.layout.sugiyama.step03.DefaultVertexOrdering;
import org.kie.workbench.common.stunner.core.graph.processing.layout.sugiyama.step03.LayerCrossingCount;
import org.kie.workbench.common.stunner.core.graph.processing.layout.sugiyama.step03.MedianVertexLayerPositioning;
import org.kie.workbench.common.stunner.core.graph.processing.layout.sugiyama.step03.VerticesTransposer;
import org.kie.workbench.common.stunner.core.graph.processing.layout.sugiyama.step04.DefaultVertexPositioning;
import org.kie.workbench.common.stunner.core.graph.processing.traverse.content.ChildrenTraverseProcessorImpl;
import org.kie.workbench.common.stunner.core.graph.processing.traverse.content.FullContentTraverseProcessorImpl;
import org.kie.workbench.common.stunner.core.graph.processing.traverse.content.ViewTraverseProcessorImpl;
import org.kie.workbench.common.stunner.core.graph.processing.traverse.tree.TreeWalkTraverseProcessorImpl;
import org.kie.workbench.common.stunner.core.graph.util.GraphUtils;
import org.kie.workbench.common.stunner.core.lookup.definition.DefinitionLookupManagerImpl;
import org.kie.workbench.common.stunner.core.lookup.domain.CommonDomainLookups;
import org.kie.workbench.common.stunner.core.lookup.rule.RuleLookupManagerImpl;
import org.kie.workbench.common.stunner.core.preferences.StunnerDiagramEditorPreferencesBeanGeneratedImpl;
import org.kie.workbench.common.stunner.core.preferences.StunnerPreferencesBeanGeneratedImpl;
import org.kie.workbench.common.stunner.core.registry.impl.DefaultDefinitionsCacheRegistry;
import org.kie.workbench.common.stunner.core.registry.impl.DefinitionsCacheRegistry;
import org.kie.workbench.common.stunner.core.rule.CachedRuleManager;
import org.kie.workbench.common.stunner.core.rule.RuleManagerImpl;
import org.kie.workbench.common.stunner.core.rule.ext.RuleExtensionMultiHandler;
import org.kie.workbench.common.stunner.core.rule.ext.impl.ConnectorParentsMatchConnectionHandler;
import org.kie.workbench.common.stunner.core.rule.ext.impl.ConnectorParentsMatchContainmentHandler;
import org.kie.workbench.common.stunner.core.rule.ext.impl.ConnectorParentsMatchHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.CardinalityEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.ConnectionEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.ConnectorCardinalityEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.ContainmentEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.DockingEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.EdgeCardinalityEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.ElementCardinalityEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.GraphConnectionEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.NodeContainmentEvaluationHandler;
import org.kie.workbench.common.stunner.core.rule.handler.impl.NodeDockingEvaluationHandler;
import org.kie.workbench.common.stunner.core.util.ClassUtils;
import org.kie.workbench.common.stunner.core.util.DefinitionUtils;
import org.kie.workbench.common.stunner.core.validation.impl.GraphValidatorImpl;
import org.kie.workbench.common.stunner.forms.client.FormsClientEntryPoint;
import org.kie.workbench.common.stunner.forms.client.components.toolbox.FormGenerationToolboxAction;
import org.kie.workbench.common.stunner.forms.client.fields.colorPicker.ColorPickerFieldRenderer;
import org.kie.workbench.common.stunner.forms.client.fields.colorPicker.ColorPickerWidget;
import org.kie.workbench.common.stunner.forms.client.gen.ClientFormGenerationManager;
import org.kie.workbench.common.stunner.forms.client.notifications.FormGenerationNotifier;
import org.kie.workbench.common.stunner.forms.client.session.command.GenerateDiagramFormsSessionCommand;
import org.kie.workbench.common.stunner.forms.client.session.command.GenerateProcessFormsSessionCommand;
import org.kie.workbench.common.stunner.forms.client.session.command.GenerateSelectedFormsSessionCommand;
import org.kie.workbench.common.stunner.forms.client.widgets.FormPropertiesWidget;
import org.kie.workbench.common.stunner.forms.client.widgets.FormPropertiesWidgetViewImpl;
import org.kie.workbench.common.stunner.forms.client.widgets.FormsCanvasSessionHandler;
import org.kie.workbench.common.stunner.forms.client.widgets.container.FormsContainer;
import org.kie.workbench.common.stunner.forms.client.widgets.container.FormsContainerViewImpl;
import org.kie.workbench.common.stunner.forms.client.widgets.container.displayer.FormDisplayer;
import org.kie.workbench.common.stunner.forms.client.widgets.container.displayer.FormDisplayerViewImpl;
import org.kie.workbench.common.stunner.forms.service.fieldProviders.ColorPickerFieldProvider;
import org.kie.workbench.common.stunner.shapes.client.ConnectorGlyphLienzoRenderer;
import org.kie.workbench.common.stunner.shapes.client.factory.BasicShapesFactory;
import org.kie.workbench.common.stunner.shapes.client.factory.PictureProvidersManager;
import org.kie.workbench.common.stunner.shapes.client.view.ShapeViewFactory;
import org.kie.workbench.common.stunner.standalone.client.ShowcaseEntryPoint;
import org.kie.workbench.common.stunner.standalone.client.navbar.AppNavBar;
import org.kie.workbench.common.stunner.standalone.client.perspectives.AuthoringPerspective;
import org.kie.workbench.common.stunner.standalone.client.perspectives.AuthoringPerspectiveActivity;
import org.kie.workbench.common.stunner.standalone.client.perspectives.CanvasPerspective;
import org.kie.workbench.common.stunner.standalone.client.perspectives.CanvasPerspectiveActivity;
import org.kie.workbench.common.stunner.standalone.client.perspectives.HomePerspective;
import org.kie.workbench.common.stunner.standalone.client.perspectives.HomePerspectiveActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.DiagramPresenterScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.DiagramPresenterScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.DiagramsNavigatorScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.DiagramsNavigatorScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.NotificationsScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.NotificationsScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionDiagramEditorScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionDiagramEditorScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionDiagramPreviewScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionDiagramPreviewScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionDiagramViewerScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionDiagramViewerScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionPropertiesScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionPropertiesScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionScreenViewImpl;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionTreeExplorerScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.SessionTreeExplorerScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.screens.ShowcaseDiagramService;
import org.kie.workbench.common.stunner.standalone.client.screens.WelcomeScreen;
import org.kie.workbench.common.stunner.standalone.client.screens.WelcomeScreenActivity;
import org.kie.workbench.common.stunner.standalone.client.widgets.Notifications;
import org.kie.workbench.common.stunner.standalone.client.widgets.NotificationsView;
import org.kie.workbench.common.stunner.svg.client.shape.factory.SVGShapeFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.uberfire.backend.vfs.Path;
import org.uberfire.backend.vfs.impl.ObservablePathImpl;
import org.uberfire.client.DocksEntryPoint;
import org.uberfire.client.JSEntryPoint;
import org.uberfire.client.RuntimePluginsServiceProxyBackendImpl;
import org.uberfire.client.VFSLockServiceProxyBackendImpl;
import org.uberfire.client.VFSServiceProxyBackendImpl;
import org.uberfire.client.WorkbenchBackendEntryPoint;
import org.uberfire.client.WorkbenchServicesProxyBackendImpl;
import org.uberfire.client.authz.DefaultWorkbenchController;
import org.uberfire.client.authz.EditorTreeProvider;
import org.uberfire.client.authz.PerspectiveTreeProvider;
import org.uberfire.client.docks.UberfireDocksImpl;
import org.uberfire.client.docks.view.DocksBars;
import org.uberfire.client.docks.view.menu.MenuBuilder;
import org.uberfire.client.editor.JSNativeEditor;
import org.uberfire.client.editor.type.JSNativeClientResourceType;
import org.uberfire.client.exporter.EditorJSExporter;
import org.uberfire.client.exporter.PerspectiveJSExporter;
import org.uberfire.client.exporter.PlaceManagerJSExporter;
import org.uberfire.client.exporter.PluginJSExporter;
import org.uberfire.client.exporter.ResourceTypeDefinitionJSExporter;
import org.uberfire.client.exporter.SplashScreenJSExporter;
import org.uberfire.client.exporter.UberfireJSAPIExporter;
import org.uberfire.client.exporter.VFSJSExporter;
import org.uberfire.client.menu.CustomSplashHelp;
import org.uberfire.client.menu.SplashScreenMenuPresenter;
import org.uberfire.client.menu.WorkbenchViewModeSwitcherMenuBuilder;
import org.uberfire.client.menu.WorkbenchViewModeSwitcherPresenter;
import org.uberfire.client.mvp.Activity;
import org.uberfire.client.mvp.ActivityBeansCache;
import org.uberfire.client.mvp.ActivityBeansInfo;
import org.uberfire.client.mvp.ActivityLifecycleErrorHandler;
import org.uberfire.client.mvp.ActivityManagerImpl;
import org.uberfire.client.mvp.CategoriesManagerCache;
import org.uberfire.client.mvp.ForceUnlockEventObserver;
import org.uberfire.client.mvp.LockDemandDetector;
import org.uberfire.client.mvp.LockManager;
import org.uberfire.client.mvp.LockManagerImpl;
import org.uberfire.client.mvp.LockTarget;
import org.uberfire.client.mvp.LockTarget.TitleProvider;
import org.uberfire.client.mvp.PerspectiveActivity;
import org.uberfire.client.mvp.PerspectiveManagerImpl;
import org.uberfire.client.mvp.PlaceHistoryHandler;
import org.uberfire.client.mvp.PlaceManager;
import org.uberfire.client.mvp.PlaceManagerImpl;
import org.uberfire.client.mvp.PlaceRequestHistoryMapperImpl;
import org.uberfire.client.mvp.PlaceStatus;
import org.uberfire.client.mvp.ResourceTypeManagerCache;
import org.uberfire.client.mvp.WorkbenchActivity;
import org.uberfire.client.perspective.JSNativePerspective;
import org.uberfire.client.promise.Promises;
import org.uberfire.client.resources.i18n.PermissionTreeI18NImpl;
import org.uberfire.client.screen.JSNativeScreen;
import org.uberfire.client.splash.JSNativeSplashScreen;
import org.uberfire.client.util.Clipboard;
import org.uberfire.client.views.pfly.HTML5DndSeleniumSupport;
import org.uberfire.client.views.pfly.PatternFlyEntryPoint;
import org.uberfire.client.views.pfly.dnd.CompassWidgetImpl;
import org.uberfire.client.views.pfly.listbar.ListBarWidgetImpl;
import org.uberfire.client.views.pfly.menu.PartContextMenusView;
import org.uberfire.client.views.pfly.menu.SplashScreenMenuView;
import org.uberfire.client.views.pfly.menu.UserMenu;
import org.uberfire.client.views.pfly.menu.UserMenuViewImpl;
import org.uberfire.client.views.pfly.menu.UtilityMenuBarView;
import org.uberfire.client.views.pfly.menu.WorkbenchMenuBarView;
import org.uberfire.client.views.pfly.menu.WorkbenchMenuCompactNavBarView;
import org.uberfire.client.views.pfly.menu.WorkbenchMenuStandardNavBarView;
import org.uberfire.client.views.pfly.menu.WorkbenchViewModeSwitcherView;
import org.uberfire.client.views.pfly.menu.megamenu.WorkbenchMegaMenuView;
import org.uberfire.client.views.pfly.menu.megamenu.contextmenuitem.ChildContextMenuItemView;
import org.uberfire.client.views.pfly.menu.megamenu.contextmenuitem.GroupContextMenuItemView;
import org.uberfire.client.views.pfly.menu.megamenu.menuitem.ChildMenuItemView;
import org.uberfire.client.views.pfly.menu.megamenu.menuitem.GroupMenuItemView;
import org.uberfire.client.views.pfly.menu.megamenu.menuitem.IconMenuItemView;
import org.uberfire.client.views.pfly.modal.Bs3Modal;
import org.uberfire.client.views.pfly.multipage.MultiPageEditorImpl;
import org.uberfire.client.views.pfly.multipage.MultiPageEditorViewImpl;
import org.uberfire.client.views.pfly.multiscreen.MultiScreenMenuBuilder;
import org.uberfire.client.views.pfly.multiscreen.MultiScreenPartWidget;
import org.uberfire.client.views.pfly.multiscreen.MultiScreenView;
import org.uberfire.client.views.pfly.notfound.ActivityNotFoundView;
import org.uberfire.client.views.pfly.notifications.NotificationPopupsManagerView;
import org.uberfire.client.views.pfly.popup.PopupViewImpl;
import org.uberfire.client.views.pfly.splash.SplashModalFooter;
import org.uberfire.client.views.pfly.splash.SplashViewImpl;
import org.uberfire.client.views.pfly.tab.MultiTabWorkbenchPanelView;
import org.uberfire.client.views.pfly.tab.ResizeTabPanel;
import org.uberfire.client.views.pfly.tab.TabPanelWithDropdowns;
import org.uberfire.client.views.pfly.tab.UberTabPanel;
import org.uberfire.client.views.pfly.toolbar.WorkbenchToolBarView;
import org.uberfire.client.views.pfly.widgets.Button;
import org.uberfire.client.views.pfly.widgets.ConfirmPopup;
import org.uberfire.client.views.pfly.widgets.DateRangePicker;
import org.uberfire.client.views.pfly.widgets.DropdownMenu;
import org.uberfire.client.views.pfly.widgets.DropdownMenuItem;
import org.uberfire.client.views.pfly.widgets.Elemental2Producer;
import org.uberfire.client.views.pfly.widgets.ErrorPopup;
import org.uberfire.client.views.pfly.widgets.ErrorPopupView;
import org.uberfire.client.views.pfly.widgets.FormGroup;
import org.uberfire.client.views.pfly.widgets.FormLabel;
import org.uberfire.client.views.pfly.widgets.InlineNotification;
import org.uberfire.client.views.pfly.widgets.JQueryProducer.JQuery;
import org.uberfire.client.views.pfly.widgets.KebabMenu;
import org.uberfire.client.views.pfly.widgets.KebabMenuItem;
import org.uberfire.client.views.pfly.widgets.Modal;
import org.uberfire.client.views.pfly.widgets.Select;
import org.uberfire.client.workbench.DefaultBeanFactory;
import org.uberfire.client.workbench.LayoutSelection;
import org.uberfire.client.workbench.PanelManagerImpl;
import org.uberfire.client.workbench.StandaloneEditorPerspective;
import org.uberfire.client.workbench.VFSLockServiceProxyClientImpl;
import org.uberfire.client.workbench.VFSServiceProxyClientImpl;
import org.uberfire.client.workbench.Workbench;
import org.uberfire.client.workbench.WorkbenchLayoutImpl;
import org.uberfire.client.workbench.WorkbenchLayoutInfoImpl;
import org.uberfire.client.workbench.docks.UberfireDocksContainer;
import org.uberfire.client.workbench.panels.impl.AdaptiveWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.AdaptiveWorkbenchPanelView;
import org.uberfire.client.workbench.panels.impl.ClosableSimpleWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.ClosableSimpleWorkbenchPanelView;
import org.uberfire.client.workbench.panels.impl.LayoutPanelPresenter;
import org.uberfire.client.workbench.panels.impl.LayoutPanelView;
import org.uberfire.client.workbench.panels.impl.MultiListWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.MultiListWorkbenchPanelView;
import org.uberfire.client.workbench.panels.impl.MultiScreenWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.MultiScreenWorkbenchPanelView;
import org.uberfire.client.workbench.panels.impl.MultiTabWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.SimpleDnDWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.SimpleDnDWorkbenchPanelView;
import org.uberfire.client.workbench.panels.impl.SimpleWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.SimpleWorkbenchPanelView;
import org.uberfire.client.workbench.panels.impl.SplitLayoutPanelPresenter;
import org.uberfire.client.workbench.panels.impl.SplitLayoutPanelView;
import org.uberfire.client.workbench.panels.impl.StaticWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.StaticWorkbenchPanelView;
import org.uberfire.client.workbench.panels.impl.TemplatedWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.TemplatedWorkbenchPanelView;
import org.uberfire.client.workbench.panels.impl.UnanchoredStaticWorkbenchPanelPresenter;
import org.uberfire.client.workbench.panels.impl.UnanchoredStaticWorkbenchPanelView;
import org.uberfire.client.workbench.panels.support.PartManager;
import org.uberfire.client.workbench.pmgr.nswe.part.WorkbenchPartPresenterDefault;
import org.uberfire.client.workbench.pmgr.nswe.part.WorkbenchPartView;
import org.uberfire.client.workbench.pmgr.unanchored.part.UnanchoredWorkbenchPartPresenter;
import org.uberfire.client.workbench.pmgr.unanchored.part.UnanchoredWorkbenchPartView;
import org.uberfire.client.workbench.type.AnyResourceType;
import org.uberfire.client.workbench.type.ClientResourceType;
import org.uberfire.client.workbench.type.DotResourceType;
import org.uberfire.client.workbench.widgets.common.ErrorPopupPresenter;
import org.uberfire.client.workbench.widgets.dnd.CompassDropController;
import org.uberfire.client.workbench.widgets.dnd.WorkbenchDragAndDropManager;
import org.uberfire.client.workbench.widgets.dnd.WorkbenchPickupDragController;
import org.uberfire.client.workbench.widgets.listbar.ResizeFlowPanel;
import org.uberfire.client.workbench.widgets.menu.PartContextMenusPresenter;
import org.uberfire.client.workbench.widgets.menu.UtilityMenuBarPresenter;
import org.uberfire.client.workbench.widgets.menu.WorkbenchMenuBarPresenter;
import org.uberfire.client.workbench.widgets.menu.WorkbenchMenuBarProducer;
import org.uberfire.client.workbench.widgets.menu.megamenu.WorkbenchMegaMenuPresenter;
import org.uberfire.client.workbench.widgets.menu.megamenu.WorkbenchMegaMenuProducer;
import org.uberfire.client.workbench.widgets.menu.megamenu.contextmenuitem.ChildContextMenuItemPresenter;
import org.uberfire.client.workbench.widgets.menu.megamenu.contextmenuitem.GroupContextMenuItemPresenter;
import org.uberfire.client.workbench.widgets.menu.megamenu.menuitem.ChildMenuItemPresenter;
import org.uberfire.client.workbench.widgets.menu.megamenu.menuitem.GroupMenuItemPresenter;
import org.uberfire.client.workbench.widgets.menu.megamenu.menuitem.IconMenuItemPresenter;
import org.uberfire.client.workbench.widgets.notifications.NotificationManager;
import org.uberfire.client.workbench.widgets.panel.StaticFocusedResizePanel;
import org.uberfire.client.workbench.widgets.toolbar.WorkbenchToolBarPresenter;
import org.uberfire.experimental.client.UberfireExperimentalEntryPoint;
import org.uberfire.experimental.client.disabled.component.DisabledFeatureComponent;
import org.uberfire.experimental.client.disabled.component.DisabledFeatureComponentViewImpl;
import org.uberfire.experimental.client.disabled.screen.DisabledFeatureActivity;
import org.uberfire.experimental.client.editor.ExperimentalFeaturesEditorScreen;
import org.uberfire.experimental.client.editor.ExperimentalFeaturesEditorScreenActivity;
import org.uberfire.experimental.client.editor.ExperimentalFeaturesEditorScreenViewImpl;
import org.uberfire.experimental.client.editor.group.ExperimentalFeaturesGroup;
import org.uberfire.experimental.client.editor.group.ExperimentalFeaturesGroupViewImpl;
import org.uberfire.experimental.client.editor.group.feature.ExperimentalFeatureEditor;
import org.uberfire.experimental.client.editor.group.feature.ExperimentalFeatureEditorViewImpl;
import org.uberfire.experimental.client.perspective.ExperimentalFeaturesPerspective;
import org.uberfire.experimental.client.perspective.ExperimentalFeaturesPerspectiveActivity;
import org.uberfire.experimental.client.service.auth.ExperimentalActivitiesAuthorizationManagerImpl;
import org.uberfire.experimental.client.service.impl.CDIClientFeatureDefRegistry;
import org.uberfire.experimental.client.service.impl.ClientExperimentalFeaturesRegistryServiceImpl;
import org.uberfire.experimental.client.workbench.type.ExperimentalAwareClientTypeRegistryImpl;
import org.uberfire.ext.apps.client.AppsEntryPoint;
import org.uberfire.ext.apps.client.AppsPerspective;
import org.uberfire.ext.apps.client.AppsPerspectiveActivity;
import org.uberfire.ext.apps.client.home.AppsHomePresenter;
import org.uberfire.ext.apps.client.home.AppsHomePresenterActivity;
import org.uberfire.ext.apps.client.home.AppsHomeView;
import org.uberfire.ext.editor.commons.client.BaseEditorEntryPoint;
import org.uberfire.ext.editor.commons.client.file.CopyPopUpViewProducer;
import org.uberfire.ext.editor.commons.client.file.RestoreUtil;
import org.uberfire.ext.editor.commons.client.file.exports.FileExportProducer;
import org.uberfire.ext.editor.commons.client.file.exports.ImageFileExport;
import org.uberfire.ext.editor.commons.client.file.exports.PdfFileExport;
import org.uberfire.ext.editor.commons.client.file.exports.TextFileExport;
import org.uberfire.ext.editor.commons.client.file.exports.jso.FileExportScriptInjector;
import org.uberfire.ext.editor.commons.client.file.exports.svg.SvgFileExport;
import org.uberfire.ext.editor.commons.client.file.popups.CopyPopUpPresenter;
import org.uberfire.ext.editor.commons.client.file.popups.CopyPopUpPresenter.View;
import org.uberfire.ext.editor.commons.client.file.popups.CopyPopUpView;
import org.uberfire.ext.editor.commons.client.file.popups.DeletePopUpPresenter;
import org.uberfire.ext.editor.commons.client.file.popups.DeletePopUpView;
import org.uberfire.ext.editor.commons.client.file.popups.RenamePopUpPresenter;
import org.uberfire.ext.editor.commons.client.file.popups.RenamePopUpView;
import org.uberfire.ext.editor.commons.client.file.popups.RestorePopUpPresenter;
import org.uberfire.ext.editor.commons.client.file.popups.RestorePopUpView;
import org.uberfire.ext.editor.commons.client.file.popups.SavePopUpPresenter;
import org.uberfire.ext.editor.commons.client.file.popups.SavePopUpView;
import org.uberfire.ext.editor.commons.client.file.popups.commons.ToggleCommentPresenter;
import org.uberfire.ext.editor.commons.client.file.popups.commons.ToggleCommentView;
import org.uberfire.ext.editor.commons.client.history.SaveButton;
import org.uberfire.ext.editor.commons.client.history.VersionHistoryPresenter;
import org.uberfire.ext.editor.commons.client.history.VersionHistoryPresenterViewImpl;
import org.uberfire.ext.editor.commons.client.history.VersionMenuDropDownButton;
import org.uberfire.ext.editor.commons.client.history.VersionMenuDropDownButtonViewImpl;
import org.uberfire.ext.editor.commons.client.history.VersionRecordManager;
import org.uberfire.ext.editor.commons.client.htmleditor.HtmlEditor;
import org.uberfire.ext.editor.commons.client.htmleditor.HtmlEditorActivity;
import org.uberfire.ext.editor.commons.client.htmleditor.HtmlEditorLibraryLoader;
import org.uberfire.ext.editor.commons.client.htmleditor.HtmlEditorPresenter;
import org.uberfire.ext.editor.commons.client.htmleditor.HtmlEditorView;
import org.uberfire.ext.editor.commons.client.htmleditor.HtmlResourceType;
import org.uberfire.ext.editor.commons.client.menu.BasicFileMenuBuilderImpl;
import org.uberfire.ext.editor.commons.client.menu.DownloadMenuItem;
import org.uberfire.ext.editor.commons.client.menu.RestoreVersionCommandProvider;
import org.uberfire.ext.editor.commons.client.menu.common.SaveAndRenameCommandBuilder;
import org.uberfire.ext.editor.commons.client.validation.DefaultFileNameValidator;
import org.uberfire.ext.editor.commons.file.exports.FileExportsPreferencesBeanGeneratedImpl;
import org.uberfire.ext.layout.editor.client.LayoutComponentPaletteScreen;
import org.uberfire.ext.layout.editor.client.LayoutEditorEntryPoint;
import org.uberfire.ext.layout.editor.client.LayoutEditorPluginImpl;
import org.uberfire.ext.layout.editor.client.LayoutEditorPresenter;
import org.uberfire.ext.layout.editor.client.LayoutEditorPropertiesScreen;
import org.uberfire.ext.layout.editor.client.LayoutEditorView;
import org.uberfire.ext.layout.editor.client.components.columns.ColumnWithComponents;
import org.uberfire.ext.layout.editor.client.components.columns.ColumnWithComponentsView;
import org.uberfire.ext.layout.editor.client.components.columns.ComponentColumn;
import org.uberfire.ext.layout.editor.client.components.columns.ComponentColumnView;
import org.uberfire.ext.layout.editor.client.components.container.ContainerView;
import org.uberfire.ext.layout.editor.client.components.rows.EmptyDropRow;
import org.uberfire.ext.layout.editor.client.components.rows.EmptyDropRowView;
import org.uberfire.ext.layout.editor.client.components.rows.Row;
import org.uberfire.ext.layout.editor.client.components.rows.RowView;
import org.uberfire.ext.layout.editor.client.generator.BootstrapLayoutGenerator;
import org.uberfire.ext.layout.editor.client.generator.BootstrapLayoutGeneratorDriver;
import org.uberfire.ext.layout.editor.client.infra.DnDManager;
import org.uberfire.ext.layout.editor.client.infra.DragHelperComponentColumn;
import org.uberfire.ext.layout.editor.client.infra.LayoutDragComponentHelper;
import org.uberfire.ext.layout.editor.client.infra.LayoutEditorCssHelper;
import org.uberfire.ext.layout.editor.client.infra.experimental.DisabledExperimentalLayoutComponent;
import org.uberfire.ext.layout.editor.client.widgets.KebabWidget;
import org.uberfire.ext.layout.editor.client.widgets.LayoutComponentPalettePresenter;
import org.uberfire.ext.layout.editor.client.widgets.LayoutComponentPaletteView;
import org.uberfire.ext.layout.editor.client.widgets.LayoutDragComponentGroupPresenter;
import org.uberfire.ext.layout.editor.client.widgets.LayoutDragComponentGroupView;
import org.uberfire.ext.layout.editor.client.widgets.LayoutDragComponentWidget;
import org.uberfire.ext.layout.editor.client.widgets.LayoutEditorPropertiesPresenter;
import org.uberfire.ext.layout.editor.client.widgets.LayoutEditorPropertiesView;
import org.uberfire.ext.layout.editor.client.widgets.LayoutElementPropertiesPresenter;
import org.uberfire.ext.layout.editor.client.widgets.LayoutElementPropertiesView;
import org.uberfire.ext.plugin.client.RuntimePluginsEntryPoint;
import org.uberfire.ext.plugin.client.config.PluginConfigService;
import org.uberfire.ext.plugin.client.editor.DynamicMenuEditorPresenter;
import org.uberfire.ext.plugin.client.editor.DynamicMenuEditorPresenterActivity;
import org.uberfire.ext.plugin.client.editor.DynamicMenuEditorView;
import org.uberfire.ext.plugin.client.editor.EditorPlugInEditorPresenter;
import org.uberfire.ext.plugin.client.editor.EditorPlugInEditorPresenterActivity;
import org.uberfire.ext.plugin.client.editor.EditorPlugInEditorView;
import org.uberfire.ext.plugin.client.editor.ScreenEditorPresenter;
import org.uberfire.ext.plugin.client.editor.ScreenEditorPresenterActivity;
import org.uberfire.ext.plugin.client.editor.ScreenEditorView;
import org.uberfire.ext.plugin.client.editor.SplashEditorPresenter;
import org.uberfire.ext.plugin.client.editor.SplashEditorPresenterActivity;
import org.uberfire.ext.plugin.client.editor.SplashEditorView;
import org.uberfire.ext.plugin.client.explorer.PluginExplorerScreen;
import org.uberfire.ext.plugin.client.explorer.PluginExplorerScreenActivity;
import org.uberfire.ext.plugin.client.info.PluginsInfo;
import org.uberfire.ext.plugin.client.perspective.PlugInAuthoringPerspective;
import org.uberfire.ext.plugin.client.perspective.PlugInAuthoringPerspectiveActivity;
import org.uberfire.ext.plugin.client.perspective.editor.PerspectiveEditorPresenter;
import org.uberfire.ext.plugin.client.perspective.editor.PerspectiveEditorPresenterActivity;
import org.uberfire.ext.plugin.client.perspective.editor.PerspectiveEditorView;
import org.uberfire.ext.plugin.client.perspective.editor.api.PerspectiveEditorCoreGroupProvider;
import org.uberfire.ext.plugin.client.perspective.editor.api.PerspectiveEditorNavGroupProvider;
import org.uberfire.ext.plugin.client.perspective.editor.generator.PerspectiveEditorGenerator;
import org.uberfire.ext.plugin.client.perspective.editor.layout.editor.HTMLLayoutDragComponent;
import org.uberfire.ext.plugin.client.perspective.editor.layout.editor.PerspectiveEditorSettings;
import org.uberfire.ext.plugin.client.perspective.editor.layout.editor.ScreenLayoutDragComponent;
import org.uberfire.ext.plugin.client.perspective.editor.layout.editor.TargetDivDragComponent;
import org.uberfire.ext.plugin.client.perspective.editor.layout.editor.popups.EditHTMLPresenter;
import org.uberfire.ext.plugin.client.perspective.editor.layout.editor.popups.EditHTMLView;
import org.uberfire.ext.plugin.client.plugins.RuntimePluginStartup;
import org.uberfire.ext.plugin.client.resources.i18n.PerspectiveEditorPaletteI18nImpl;
import org.uberfire.ext.plugin.client.security.PluginController;
import org.uberfire.ext.plugin.client.type.DynamicMenuResourceType;
import org.uberfire.ext.plugin.client.type.EditorPluginResourceType;
import org.uberfire.ext.plugin.client.type.PerspectiveLayoutPluginResourceType;
import org.uberfire.ext.plugin.client.type.ScreenPluginResourceType;
import org.uberfire.ext.plugin.client.type.SplashPluginResourceType;
import org.uberfire.ext.plugin.client.validation.PluginNameValidator;
import org.uberfire.ext.plugin.client.widget.media.MediaLibraryWidget;
import org.uberfire.ext.plugin.client.widget.navigator.PluginNavList;
import org.uberfire.ext.plugin.client.widget.plugin.GeneralPluginEditor;
import org.uberfire.ext.plugin.client.widget.popup.NewPluginPopUp;
import org.uberfire.ext.plugin.client.widget.popup.NewPluginPopUpViewImpl;
import org.uberfire.ext.plugin.type.DynamicMenuResourceTypeDefinition;
import org.uberfire.ext.properties.editor.client.PropertyEditorWidget;
import org.uberfire.ext.properties.editor.client.fields.BooleanField;
import org.uberfire.ext.properties.editor.client.fields.ColorField;
import org.uberfire.ext.properties.editor.client.fields.ComboField;
import org.uberfire.ext.properties.editor.client.fields.SecretTextField;
import org.uberfire.ext.properties.editor.client.fields.TextField;
import org.uberfire.ext.security.management.client.ClientSecurityExceptionMessageResolver;
import org.uberfire.ext.security.management.client.ClientUserSystemManager;
import org.uberfire.ext.security.management.client.ResourceBundleEntryPoint;
import org.uberfire.ext.security.management.client.editor.group.GroupEditorDriver;
import org.uberfire.ext.security.management.client.editor.role.RoleEditorDriver;
import org.uberfire.ext.security.management.client.editor.user.UserEditorDriver;
import org.uberfire.ext.security.management.client.perspectives.SecurityManagementPerspective;
import org.uberfire.ext.security.management.client.perspectives.SecurityManagementPerspectiveActivity;
import org.uberfire.ext.security.management.client.screens.BaseScreenView;
import org.uberfire.ext.security.management.client.screens.editor.GroupEditorScreen;
import org.uberfire.ext.security.management.client.screens.editor.GroupEditorScreenActivity;
import org.uberfire.ext.security.management.client.screens.editor.RoleEditorScreen;
import org.uberfire.ext.security.management.client.screens.editor.RoleEditorScreenActivity;
import org.uberfire.ext.security.management.client.screens.editor.UserEditorScreen;
import org.uberfire.ext.security.management.client.screens.editor.UserEditorScreenActivity;
import org.uberfire.ext.security.management.client.screens.explorer.SecurityExplorerScreen;
import org.uberfire.ext.security.management.client.screens.explorer.SecurityExplorerScreenActivity;
import org.uberfire.ext.security.management.client.screens.explorer.SecurityExplorerView;
import org.uberfire.ext.security.management.client.screens.home.SecurityManagementHomeScreen;
import org.uberfire.ext.security.management.client.screens.home.SecurityManagementHomeScreenActivity;
import org.uberfire.ext.security.management.client.screens.home.SecurityManagementHomeView;
import org.uberfire.ext.security.management.client.widgets.management.ChangePassword;
import org.uberfire.ext.security.management.client.widgets.management.ChangePasswordView;
import org.uberfire.ext.security.management.client.widgets.management.CreateEntity;
import org.uberfire.ext.security.management.client.widgets.management.CreateEntityView;
import org.uberfire.ext.security.management.client.widgets.management.editor.AssignedEntitiesExplorerView;
import org.uberfire.ext.security.management.client.widgets.management.editor.AssignedEntitiesInlineEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.AssignedEntitiesModalEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.ACLEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.ACLEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.ACLSettings;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.ACLSettingsView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.ACLViewer;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.ACLViewerView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.PriorityDropDown;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.LeafPermissionNodeEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.LeafPermissionNodeEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.LeafPermissionNodeViewer;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.LeafPermissionNodeViewerView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.MultiplePermissionNodeEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.MultiplePermissionNodeEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.MultiplePermissionNodeViewer;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.MultiplePermissionNodeViewerView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.PermissionExceptionSwitch;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.PermissionExceptionSwitchView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.PermissionSwitch;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.PermissionSwitchView;
import org.uberfire.ext.security.management.client.widgets.management.editor.acl.node.PermissionWidgetFactory;
import org.uberfire.ext.security.management.client.widgets.management.editor.group.GroupEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.group.GroupEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.group.GroupUsersAssignment;
import org.uberfire.ext.security.management.client.widgets.management.editor.group.workflow.GroupCreationWorkflow;
import org.uberfire.ext.security.management.client.widgets.management.editor.group.workflow.GroupEditorWorkflow;
import org.uberfire.ext.security.management.client.widgets.management.editor.role.RoleEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.role.RoleEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.role.workflow.RoleEditorWorkflow;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.NewUserAttributeEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.NewUserAttributeEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.UserAssignedGroupsEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.UserAssignedGroupsExplorer;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.UserAssignedRolesEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.UserAssignedRolesExplorer;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.UserAttributesEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.UserAttributesEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.UserEditor;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.UserEditorView;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.workflow.UserCreationWorkflow;
import org.uberfire.ext.security.management.client.widgets.management.editor.user.workflow.UserEditorWorkflow;
import org.uberfire.ext.security.management.client.widgets.management.editor.workflow.EntityWorkflowViewImpl;
import org.uberfire.ext.security.management.client.widgets.management.explorer.EntitiesExplorerViewImpl;
import org.uberfire.ext.security.management.client.widgets.management.explorer.GroupsExplorer;
import org.uberfire.ext.security.management.client.widgets.management.explorer.RolesExplorer;
import org.uberfire.ext.security.management.client.widgets.management.explorer.UserGroupsExplorer;
import org.uberfire.ext.security.management.client.widgets.management.explorer.UserGroupsExplorerView;
import org.uberfire.ext.security.management.client.widgets.management.explorer.UsersExplorer;
import org.uberfire.ext.security.management.client.widgets.management.list.EntitiesListView;
import org.uberfire.ext.security.management.client.widgets.management.list.GroupsList;
import org.uberfire.ext.security.management.client.widgets.management.list.RolesList;
import org.uberfire.ext.security.management.client.widgets.management.list.UsersList;
import org.uberfire.ext.security.management.client.widgets.popup.ConfirmBox;
import org.uberfire.ext.security.management.client.widgets.popup.ConfirmBoxView;
import org.uberfire.ext.security.management.client.widgets.popup.ErrorBox;
import org.uberfire.ext.security.management.client.widgets.popup.ErrorBoxView;
import org.uberfire.ext.security.management.client.widgets.popup.LoadingBoxView;
import org.uberfire.ext.widgets.common.client.CommonsEntryPoint;
import org.uberfire.ext.widgets.common.client.ace.AceEditor;
import org.uberfire.ext.widgets.common.client.breadcrumbs.UberfireBreadcrumbs;
import org.uberfire.ext.widgets.common.client.breadcrumbs.UberfireBreadcrumbsView;
import org.uberfire.ext.widgets.common.client.breadcrumbs.header.UberfireBreadcrumbsContainerImpl;
import org.uberfire.ext.widgets.common.client.breadcrumbs.widget.DefaultBreadcrumbsPresenter;
import org.uberfire.ext.widgets.common.client.breadcrumbs.widget.DefaultBreadcrumbsView;
import org.uberfire.ext.widgets.common.client.common.BusyIndicatorView;
import org.uberfire.ext.widgets.common.client.dropdown.LiveSearchDropDown;
import org.uberfire.ext.widgets.common.client.dropdown.LiveSearchDropDownView;
import org.uberfire.ext.widgets.common.client.dropdown.PerspectiveDropDown;
import org.uberfire.ext.widgets.common.client.dropdown.footer.LiveSearchFooter;
import org.uberfire.ext.widgets.common.client.dropdown.footer.LiveSearchFooterViewImpl;
import org.uberfire.ext.widgets.common.client.dropdown.items.LiveSearchSelectorDropDownItem;
import org.uberfire.ext.widgets.common.client.dropdown.items.LiveSearchSelectorDropDownItemViewImpl;
import org.uberfire.ext.widgets.common.client.dropdown.noItems.NoItemsComponent;
import org.uberfire.ext.widgets.common.client.dropdown.noItems.NoItemsComponentViewImpl;
import org.uberfire.ext.widgets.common.client.menu.MenuItemDividerView;
import org.uberfire.ext.widgets.common.client.menu.MenuItemFactory;
import org.uberfire.ext.widgets.common.client.menu.MenuItemHeaderView;
import org.uberfire.ext.widgets.common.client.menu.MenuItemWithIconView;
import org.uberfire.ext.widgets.common.client.select.SelectComponent;
import org.uberfire.ext.widgets.common.client.select.SelectOptionComponent;
import org.uberfire.ext.widgets.common.client.select.SelectOptionView;
import org.uberfire.ext.widgets.common.client.select.SelectView;
import org.uberfire.ext.widgets.common.client.tables.FilterPagedTable;
import org.uberfire.ext.widgets.common.client.tables.PagedTable;
import org.uberfire.ext.widgets.common.client.tables.SimpleTable;
import org.uberfire.ext.widgets.common.client.tables.popup.NewFilterPopup;
import org.uberfire.ext.widgets.common.client.tables.popup.NewTabFilterPopup;
import org.uberfire.ext.widgets.core.client.CoreEntryPoint;
import org.uberfire.ext.widgets.core.client.editors.defaulteditor.DefaultFileEditorPresenter;
import org.uberfire.ext.widgets.core.client.editors.defaulteditor.DefaultFileEditorPresenterActivity;
import org.uberfire.ext.widgets.core.client.editors.defaulteditor.DefaultFileEditorView;
import org.uberfire.ext.widgets.core.client.editors.metafile.MetaFileEditorPresenter;
import org.uberfire.ext.widgets.core.client.editors.metafile.MetaFileEditorPresenterActivity;
import org.uberfire.ext.widgets.core.client.editors.texteditor.TextEditorView;
import org.uberfire.ext.widgets.core.client.editors.texteditor.TextEditorWorkbenchEditor;
import org.uberfire.ext.widgets.core.client.editors.texteditor.TextEditorWorkbenchEditorActivity;
import org.uberfire.ext.widgets.core.client.editors.texteditor.TextResourceType;
import org.uberfire.ext.widgets.core.client.screens.iframe.IFrameScreenPresenter;
import org.uberfire.ext.widgets.core.client.screens.iframe.IFrameScreenPresenterActivity;
import org.uberfire.ext.widgets.core.client.screens.iframe.IFrameScreenView;
import org.uberfire.ext.widgets.core.client.tree.Tree;
import org.uberfire.ext.widgets.core.client.wizards.WizardPageTitle;
import org.uberfire.ext.widgets.core.client.wizards.WizardViewImpl;
import org.uberfire.ext.widgets.core.client.workbench.widgets.popups.activities.notfound.ActivityNotFoundPresenter;
import org.uberfire.ext.widgets.core.client.workbench.widgets.popups.activities.notfound.ActivityNotFoundPresenterActivity;
import org.uberfire.ext.widgets.table.client.TableEntryPoint;
import org.uberfire.mvp.Command;
import org.uberfire.mvp.PlaceRequest;
import org.uberfire.mvp.impl.ExternalPathPlaceRequest;
import org.uberfire.preferences.client.scope.ClientUsernameProvider;
import org.uberfire.preferences.client.scope.PreferenceScopeResolutionStrategyIOCProvider;
import org.uberfire.preferences.client.store.PreferenceBeanStoreImpl;
import org.uberfire.preferences.client.store.PreferenceStore;
import org.uberfire.preferences.client.store.PreferenceStoreIOCProvider;
import org.uberfire.preferences.shared.PreferenceScopeResolutionStrategy;
import org.uberfire.preferences.shared.PreferenceScopeTypes;
import org.uberfire.preferences.shared.bean.mock.PortablePreferenceMockBeanGeneratedImpl;
import org.uberfire.preferences.shared.impl.DefaultPreferenceScopeTypes;
import org.uberfire.preferences.shared.impl.PreferenceScopeFactoryImpl;
import org.uberfire.preferences.shared.impl.PreferenceScopeTypesProducer;
import org.uberfire.preferences.shared.impl.PreferenceScopeValidatorImpl;
import org.uberfire.rpc.SessionInfo;
import org.uberfire.security.Resource;
import org.uberfire.security.ResourceAction;
import org.uberfire.security.ResourceType;
import org.uberfire.security.authz.ResourceActionRef;
import org.uberfire.security.authz.RuntimeResource;
import org.uberfire.security.client.SecurityEntryPoint;
import org.uberfire.security.client.authz.AuthorizationManagerHelper;
import org.uberfire.security.client.authz.tree.impl.DefaultPermissionTreeFactory;
import org.uberfire.security.impl.authz.DefaultAuthorizationManager;
import org.uberfire.security.impl.authz.DefaultPermissionManager;
import org.uberfire.security.impl.authz.DefaultPermissionTypeRegistry;
import org.uberfire.ssh.client.SSHClientEntryPoint;
import org.uberfire.ssh.client.editor.SSHKeysEditorPerspective;
import org.uberfire.ssh.client.editor.SSHKeysEditorPerspectiveActivity;
import org.uberfire.ssh.client.editor.SSHKeysEditorScreen;
import org.uberfire.ssh.client.editor.SSHKeysEditorScreenActivity;
import org.uberfire.ssh.client.editor.component.SSHKeysEditor;
import org.uberfire.ssh.client.editor.component.SSHKeysEditorViewImpl;
import org.uberfire.ssh.client.editor.component.creation.NewSSHKeyModal;
import org.uberfire.ssh.client.editor.component.creation.NewSSHKeyModalViewImpl;
import org.uberfire.ssh.client.editor.component.empty.SSHKeysEditorEmptyStateDisplayer;
import org.uberfire.ssh.client.editor.component.empty.SSHKeysEditorEmptyStateDisplayerViewImpl;
import org.uberfire.ssh.client.editor.component.keys.SSHKeysDisplayer;
import org.uberfire.ssh.client.editor.component.keys.SSHKeysDisplayerViewImpl;
import org.uberfire.ssh.client.editor.component.keys.key.SSHKeyEditor;
import org.uberfire.ssh.client.editor.component.keys.key.SSHKeyEditorViewImpl;
import org.uberfire.workbench.category.Others;
import org.uberfire.workbench.category.Undefined;
import org.uberfire.workbench.model.ActivityResourceType;
import org.uberfire.workbench.model.CompassPosition;
import org.uberfire.workbench.model.ContextDefinition;
import org.uberfire.workbench.model.ContextDisplayMode;
import org.uberfire.workbench.model.CustomPanelDefinition;
import org.uberfire.workbench.model.PanelDefinition;
import org.uberfire.workbench.model.PartDefinition;
import org.uberfire.workbench.model.PerspectiveDefinition;
import org.uberfire.workbench.model.Position;
import org.uberfire.workbench.model.impl.PanelDefinitionImpl;
import org.uberfire.workbench.model.menu.HasEnabledStateChangeListeners;
import org.uberfire.workbench.model.menu.MenuCustom;
import org.uberfire.workbench.model.menu.MenuGroup;
import org.uberfire.workbench.model.menu.MenuItem;
import org.uberfire.workbench.model.menu.MenuItemCommand;
import org.uberfire.workbench.model.menu.MenuItemPerspective;
import org.uberfire.workbench.model.menu.MenuItemPlain;
import org.uberfire.workbench.model.menu.MenuPosition;
import org.uberfire.workbench.model.menu.MenuVisitor;
import org.uberfire.workbench.model.menu.Menus;
import org.uberfire.workbench.model.menu.impl.DefaultMenuGroup;
import org.uberfire.workbench.model.menu.impl.DefaultMenus;
import org.uberfire.workbench.model.toolbar.ToolBar;
import org.uberfire.workbench.model.toolbar.ToolBarIcon;
import org.uberfire.workbench.model.toolbar.ToolBarItem;
import org.uberfire.workbench.type.ResourceTypeDefinition;

public class BootstrapperImpl implements Bootstrapper {
  {
    addLookups_0();
    new CDI().initLookupTable(CDIEventTypeLookup.get());
    new DataBindingModuleBootstrapper().run();
  }

  private ContextManager contextManager = new ContextManagerImpl();
  private static final Logger logger = LoggerFactory.getLogger(Bootstrapper.class);
  public abstract class Type_factory__o_u_c_VFSLockServiceProxyBackendImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<VFSLockServiceProxyBackendImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_w_ServiceTaskFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<ServiceTaskFactory> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_BigIntegerEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<BigIntegerEditableColumnGenerator> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_GroupsExplorer__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupsExplorer> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_BPMNThumbnailProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNThumbnailProvider> { }
  public abstract class Provider_factory__o_j_e_u_n_c_l_NavigationPanel__quals__j_e_i_Any_j_e_i_Default extends Factory<NavigationPanel> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_PanControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PanControlImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_f_p_ModuleFormGenerationResourcesProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ModuleFormGenerationResourcesProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_ChangePasswordView__quals__j_e_i_Any_j_e_i_Default extends Factory<ChangePasswordView> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_c_ComponentColumnView__quals__j_e_i_Any_j_e_i_Default extends Factory<ComponentColumnView> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_GroupEditorScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupEditorScreen> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_r_IntegerRadioGroupFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerRadioGroupFieldRenderer> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_UserAttributesEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<UserAttributesEditorView> { }
  public abstract class ContextualProvider_factory__o_j_e_c_c_a_Caller__quals__Universal extends Factory<Caller> { }
  public abstract class Producer_factory__o_j_e_b_c_a_ClientMessageBus__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientMessageBus> { }
  public abstract class Type_factory__o_u_e_e_c_c_m_DownloadMenuItem__quals__j_e_i_Any_j_e_i_Default extends Factory<DownloadMenuItem> { }
  public abstract class ExtensionProvided_factory__o_g_e_t_c_u_ToggleSwitch__quals__j_e_i_Any_j_e_i_Default extends Factory<ToggleSwitch> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_l_s_s_LayerCrossingCount__quals__j_e_i_Any_j_e_i_Default extends Factory<LayerCrossingCount> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToPngSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToPngSessionCommand> { }
  public abstract class ContextualProvider_factory__o_j_e_u_n_c_l_TransitionAnchorFactory__quals__Universal extends Factory<TransitionAnchorFactory> { }
  public abstract class Type_factory__o_k_w_c_f_c_c_c_f_e_EmbeddedFormDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<EmbeddedFormDisplayerViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_i_FormHandlerGeneratorManager__quals__j_e_i_Any_j_e_i_Default extends Factory<FormHandlerGeneratorManager> { }
  public abstract class JsType_factory__o_u_s_ResourceAction__quals__Universal extends Factory<ResourceAction> { }
  public abstract class Type_factory__o_u_c_w_VFSServiceProxyClientImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<VFSServiceProxyClientImpl> { }
  public abstract class Type_factory__o_u_e_l_e_c_LayoutEditorEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutEditorEntryPoint> { }
  public abstract class Type_factory__o_u_c_w_Workbench__quals__j_e_i_Any_j_e_i_Default extends Factory<Workbench> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_r_RoleEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<RoleEditor> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_ActivityDataIOEditorWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityDataIOEditorWidget> { }
  public abstract class JsType_factory__o_u_s_a_ResourceActionRef__quals__Universal extends Factory<ResourceActionRef> { }
  public abstract class Type_factory__o_u_c_w_p_i_MultiListWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiListWorkbenchPanelPresenter> { }
  public abstract class Type_factory__o_g_c_s_p_c_Optimization__quals__j_e_i_Any_j_e_i_Default extends Factory<Optimization> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_DeleteNodeAction__quals__j_e_i_Any_j_e_i_Default extends Factory<DeleteNodeAction> { }
  public abstract class JsType_factory__o_u_s_ResourceType__quals__Universal extends Factory<ResourceType> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_l_s_SugiyamaLayoutService__quals__j_e_i_Any_j_e_i_Default extends Factory<SugiyamaLayoutService> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_g_LienzoPictureGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoPictureGlyphRenderer> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_l_EntitiesListView__quals__j_e_i_Any_j_e_i_Default extends Factory<EntitiesListView> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_s_DoubleSliderFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<DoubleSliderFieldInitializer> { }
  public abstract class Type_factory__o_k_w_c_s_f_s_f_ColorPickerFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ColorPickerFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_LienzoMultipleSelectionControl__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_s_MultipleSelection extends Factory<LienzoMultipleSelectionControl> { }
  public abstract class JsType_factory__e_d_HTMLSlotElement_AssignedNodesOptionsType__quals__Universal extends Factory<AssignedNodesOptionsType> { }
  public abstract class Type_factory__o_u_c_m_ActivityBeansInfo__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityBeansInfo> { }
  public abstract class Type_factory__o_u_e_w_c_c_m_MenuItemWithIconView__quals__j_e_i_Any_o_u_e_w_c_c_m_MenuItemWithIcon extends Factory<MenuItemWithIconView> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_DynamicFormRendererViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicFormRendererViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_ACLEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<ACLEditorView> { }
  public abstract class JsType_factory__e_d_RTCConfigurationInterface_UrlsFieldType__quals__Universal extends Factory<UrlsFieldType> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_g_GroupEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupEditorView> { }
  public abstract class Type_factory__o_u_e_c_UberfireExperimentalEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<UberfireExperimentalEntryPoint> { }
  public abstract class Type_factory__o_u_c_w_p_u_p_UnanchoredWorkbenchPartView__quals__j_e_i_Any_j_e_i_Default extends Factory<UnanchoredWorkbenchPartView> { }
  public abstract class Type_factory__o_j_e_s_c_l_h_DefaultSecurityExceptionHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultSecurityExceptionHandler> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_h_MapModelBindingHelper__quals__j_e_i_Any_j_e_i_Default extends Factory<MapModelBindingHelper> { }
  public abstract class Type_factory__o_u_e_c_d_c_DisabledFeatureComponentViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DisabledFeatureComponentViewImpl> { }
  public abstract class Producer_factory__o_u_p_s_PreferenceScopeTypes__quals__j_e_i_Any_o_u_a_Customizable extends Factory<PreferenceScopeTypes> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutDragComponentGroupView__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutDragComponentGroupView> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_UserEditorScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<UserEditorScreen> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_d_i_DefaultDiagramViewer__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultDiagramViewer> { }
  public abstract class ExtensionProvided_factory__o_g_c_u_TextBox__quals__j_e_i_Any_j_e_i_Default extends Factory<TextBox> { }
  public abstract class Type_factory__o_u_c_v_p_w_Select__quals__j_e_i_Any_j_e_i_Default extends Factory<Select> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_g_ImageStripDOMGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ImageStripDOMGlyphRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_f_SVGShapeFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<SVGShapeFactory> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_VisitGraphToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<VisitGraphToolbarCommand> { }
  public abstract class Type_factory__o_u_w_m_i_PanelDefinitionImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PanelDefinitionImpl> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_f_c_ColorPickerFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ColorPickerFieldRenderer> { }
  public abstract class Type_factory__o_u_e_e_c_f_e_FileExportsPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FileExportsPreferencesBeanGeneratedImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_ClientFactoryService__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientFactoryService> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_t_p_TextAreaFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<TextAreaFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_TextAreaFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<TextAreaFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_SessionCommandManagerImpl__quals__j_e_i_Any_o_k_w_c_s_c_c_s_Session extends Factory<SessionCommandManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_r_ClientRuleManager__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientRuleManager> { }
  public abstract class JsType_factory__e_d_PopStateEvent_PopStateEventEventInitDictType__quals__Universal extends Factory<PopStateEventEventInitDictType> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_g_BS3IconTypeGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<BS3IconTypeGlyphRenderer> { }
  public abstract class Type_factory__o_u_c_v_p_w_ErrorPopupView__quals__j_e_i_Any_j_e_i_Default extends Factory<ErrorPopupView> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_WelcomeScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<WelcomeScreenActivity> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_MultiplePermissionNodeViewerView__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiplePermissionNodeViewerView> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_TimerSettingsFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<TimerSettingsFieldProvider> { }
  public abstract class Type_factory__o_u_s_c_e_c_SSHKeysEditorViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeysEditorViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_l_r_FieldRequiredViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldRequiredViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_s_c_GenerateSelectedFormsSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<GenerateSelectedFormsSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_ToolboxDomainLookups__quals__j_e_i_Any_j_e_i_Default extends Factory<ToolboxDomainLookups> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_c_ComboBoxFixedValuesWidgetViewImpl__quals__j_e_i_Any_o_k_w_c_s_b_c_f_f_c_a_FixedValues extends Factory<ComboBoxFixedValuesWidgetViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_i_ManagedEditorToolbar__quals__j_e_i_Any_j_e_i_Default extends Factory<ManagedEditorToolbar> { }
  public abstract class Type_factory__o_u_c_v_p_m_MultiScreenView__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiScreenView> { }
  public abstract class Type_factory__o_u_e_p_c_e_EditorPlugInEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<EditorPlugInEditorPresenter> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_UserGroupsExplorer__quals__j_e_i_Any_j_e_i_Default extends Factory<UserGroupsExplorer> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_f_p_ModuleFormGenerationResourcesProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<org.kie.workbench.common.stunner.bpmn.definition.formBuilder.provider.ModuleFormGenerationResourcesProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_g_w_GroupEditorWorkflow__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupEditorWorkflow> { }
  public abstract class Type_factory__o_u_e_s_m_c_p_SecurityManagementPerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityManagementPerspective> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_f_c_ColorPickerWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<ColorPickerWidget> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_NodeDockingEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<NodeDockingEvaluationHandler> { }
  public abstract class JsType_factory__o_u_w_m_m_MenuGroup__quals__Universal extends Factory<MenuGroup> { }
  public abstract class Type_factory__o_u_e_c_e_g_f_ExperimentalFeatureEditorViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalFeatureEditorViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_e_RuleExtensionMultiHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<RuleExtensionMultiHandler> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_ProcessCompensationRefProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ProcessCompensationRefProvider> { }
  public abstract class Type_factory__o_u_c_w_p_i_AdaptiveWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<AdaptiveWorkbenchPanelView> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_r_RuleLookupManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<RuleLookupManagerImpl> { }
  public abstract class ExtensionProvided_factory__o_g_c_u_Container__quals__j_e_i_Any_j_e_i_Default extends Factory<Container> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_MultipleInputComponentViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleInputComponentViewImpl> { }
  public abstract class Type_factory__o_u_c_w_w_d_WorkbenchDragAndDropManager__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchDragAndDropManager> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_n_NotificationsObserver__quals__j_e_i_Any_j_e_i_Default extends Factory<NotificationsObserver> { }
  public abstract class Type_factory__o_u_e_l_e_c_LayoutEditorPropertiesScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutEditorPropertiesScreen> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_LogCommandHistoryDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<LogCommandHistoryDevCommand> { }
  public abstract class Type_factory__o_g_c_s_p_e_t_POMResourceTypeDefinition__quals__j_e_i_Any_j_e_i_Default extends Factory<POMResourceTypeDefinition> { }
  public abstract class Type_factory__o_u_c_m_LockManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<LockManagerImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_PriorityDropDown__quals__j_e_i_Any_j_e_i_Default extends Factory<PriorityDropDown> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_BPMNCreateNodeAction__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNCreateNodeAction> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_d_ShapeDragProxyImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ShapeDragProxyImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_s_SelectComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<SelectComponent> { }
  public abstract class Type_factory__o_u_c_m_PerspectiveManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveManagerImpl> { }
  public abstract class Type_factory__o_u_c_w_w_c_ErrorPopupPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ErrorPopupPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_c_NoneCloneProcess__quals__j_e_i_Any_j_e_i_Default extends Factory<NoneCloneProcess> { }
  public abstract class ContextualProvider_factory__j_e_e_Event__quals__Universal extends Factory<Event> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_r_CharacterRadioGroupFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<CharacterRadioGroupFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_i_p_IntegerBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerBoxFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_FormGeneratorDriver__quals__j_e_i_Any_j_e_i_Default extends Factory<FormGeneratorDriver> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_RedoSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<RedoSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_t_p_TextBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<TextBoxFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetDefinitionSetProxyImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDefinitionSetDefinitionSetProxyImpl> { }
  public abstract class Type_factory__o_g_c_s_w_c_WorkingSetManager__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkingSetManager> { }
  public abstract class Type_factory__o_j_e_s_c_l_i_ClientSecurityRoleInterceptor__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientSecurityRoleInterceptor> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_ProcessEscalationRefProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ProcessEscalationRefProvider> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_c_CaseRoleSerializer__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseRoleSerializer> { }
  public abstract class Type_factory__o_u_c_w_StandaloneEditorPerspective__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<StandaloneEditorPerspective> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_i_ManagedSession__quals__j_e_i_Any_j_e_i_Default extends Factory<ManagedSession> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionWidgetFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<PermissionWidgetFactory> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_CopyPopUpView__quals__j_e_i_Any_o_u_a_FallbackImplementation extends Factory<CopyPopUpView> { }
  public abstract class ContextualProvider_factory__o_j_e_i_c_a_InitBallot__quals__Universal extends Factory<InitBallot> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_c_CollapsedDefinitionPaletteItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CollapsedDefinitionPaletteItemWidgetViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_t_c_ViewTraverseProcessorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ViewTraverseProcessorImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_e_DiagramClientErrorHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<DiagramClientErrorHandler> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_ShortEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<ShortEditableColumnGenerator> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_w_UserCreationWorkflow__quals__j_e_i_Any_j_e_i_Default extends Factory<UserCreationWorkflow> { }
  public abstract class Type_factory__o_u_w_c_Others__quals__j_e_i_Any_j_e_i_Default extends Factory<Others> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Input__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<Input> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_d_ConnectorDragProxyImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ConnectorDragProxyImpl> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionDiagramPreviewScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SessionDiagramPreviewScreenActivity> { }
  public abstract class JsType_factory__e_d_File_FilePropertiesType__quals__Universal extends Factory<FilePropertiesType> { }
  public abstract class Type_factory__o_j_e_c_c_d_e_Elemental2DomUtil__quals__j_e_i_Any_j_e_i_Default extends Factory<Elemental2DomUtil> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_VersionRecordManager__quals__j_e_i_Any_j_e_i_Default extends Factory<VersionRecordManager> { }
  public abstract class Type_factory__o_u_c_w_p_i_StaticWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<StaticWorkbenchPanelPresenter> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_DateTimePickerPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<DateTimePickerPresenter> { }
  public abstract class Type_factory__o_u_c_u_Clipboard__quals__j_e_i_Any_j_e_i_Default extends Factory<Clipboard> { }
  public abstract class Type_factory__o_u_e_s_m_c_ClientUserSystemManager__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientUserSystemManager> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_AssigneeEditorFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<AssigneeEditorFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_c_r_RolesEditorFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<RolesEditorFieldRenderer> { }
  public abstract class Type_factory__o_u_c_v_p_w_Button__quals__j_e_i_Any_j_e_i_Default extends Factory<Button> { }
  public abstract class Type_factory__o_u_e_w_t_c_TableEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<TableEntryPoint> { }
  public abstract class Type_factory__o_u_e_a_c_h_AppsHomePresenterActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<AppsHomePresenterActivity> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_d_DefaultFileEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultFileEditorPresenter> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_s_i_MultipleSelectorInputViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleSelectorInputViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionPropertiesScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SessionPropertiesScreenActivity> { }
  public abstract class Type_factory__o_u_c_v_p_t_TabPanelWithDropdowns__quals__j_e_i_Any_j_e_i_Default extends Factory<TabPanelWithDropdowns> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_r_RowView__quals__j_e_i_Any_j_e_i_Default extends Factory<RowView> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_g_ShapeGlyphDragHandlerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ShapeGlyphDragHandlerImpl> { }
  public abstract class JsType_factory__o_u_w_m_t_ToolBarIcon__quals__Universal extends Factory<ToolBarIcon> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_r_m_p_MultipleSubFormFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleSubFormFieldProvider> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_g_PerspectiveEditorGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveEditorGenerator> { }
  public abstract class JsType_factory__o_u_c_m_LockManager__quals__Universal extends Factory<LockManager> { }
  public abstract class Type_factory__o_u_c_e_UberfireJSAPIExporter__quals__j_e_i_Any_j_e_i_Default extends Factory<UberfireJSAPIExporter> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_w_WiresCanvasPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<WiresCanvasPresenter> { }
  public abstract class Provider_factory__o_j_e_c_c_a_BatchCaller__quals__j_e_i_Any_j_e_i_Default extends Factory<BatchCaller> { }
  public abstract class Type_factory__o_u_c_w_p_i_SimpleWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<SimpleWorkbenchPanelPresenter> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_p_w_PictureWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<PictureWidget> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionDiagramEditorScreen__quals__j_e_i_Any_o_k_w_c_s_c_c_a_DiagramEditor extends Factory<SessionDiagramEditorScreen> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_UserAssignedGroupsExplorer__quals__j_e_i_Any_j_e_i_Default extends Factory<UserAssignedGroupsExplorer> { }
  public abstract class JsType_factory__e_d_Document_RegisterOptionsType__quals__Universal extends Factory<RegisterOptionsType> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_VariablesProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<VariablesProvider> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_c_c_BPMNCommonActionsToolboxFactory__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<BPMNCommonActionsToolboxFactory> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_c_ClientSelectorDataProviderManager__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientSelectorDataProviderManager> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_CanvasHandlerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasHandlerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_CutSelectionSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<CutSelectionSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_v_VariablesEditorFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<VariablesEditorFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_DeleteSelectionSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<DeleteSelectionSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_w_VariableNameTextBox__quals__j_e_i_Any_o_k_w_c_s_b_c_StunnerSpecific extends Factory<VariableNameTextBox> { }
  public abstract class Type_factory__o_u_c_v_p_w_FormLabel__quals__j_e_i_Any_j_e_i_Default extends Factory<FormLabel> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_t_TextEditorWorkbenchEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<TextEditorWorkbenchEditor> { }
  public abstract class Type_factory__o_u_e_l_e_c_LayoutComponentPaletteScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutComponentPaletteScreen> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_PopupUtil__quals__j_e_i_Any_j_e_i_Default extends Factory<PopupUtil> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_DefaultPaletteFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultPaletteFactory> { }
  public abstract class ContextualProvider_factory__o_j_e_u_n_c_l_TransitionAnchor__quals__Universal extends Factory<TransitionAnchor> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_CaseManagementShapeSet__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementShapeSet> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_FieldRendererManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldRendererManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_r_BPMNIconsStrip__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<BPMNIconsStrip> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_AdHocOrderingProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<AdHocOrderingProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_SecurityExplorerScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityExplorerScreen> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_c_RolesEditorFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<RolesEditorFieldInitializer> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_SwitchGridSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<SwitchGridSessionCommand> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_l_e_HTMLLayoutDragComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<HTMLLayoutDragComponent> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_g_DOMGlyphRenderers__quals__j_e_i_Any_j_e_i_Default extends Factory<DOMGlyphRenderers> { }
  public abstract class Type_factory__o_u_c_WorkbenchBackendEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchBackendEntryPoint> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_c_Container__quals__j_e_i_Any_j_e_i_Default extends Factory<org.uberfire.ext.layout.editor.client.components.container.Container> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeEditorWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeEditorWidgetViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_s_l_d_f_p_ModuleFormGenerationResourcesProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.selectors.listBox.definition.formBuilder.provider.ModuleFormGenerationResourcesProvider> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_c_p_BPMNPaletteDefinitionBuilder__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<BPMNPaletteDefinitionBuilder> { }
  public abstract class Type_factory__o_u_s_i_a_DefaultPermissionManager__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultPermissionManager> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_c_RolesEditorFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<RolesEditorFieldProvider> { }
  public abstract class Type_factory__o_u_c_VFSServiceProxyBackendImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<VFSServiceProxyBackendImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_e_n_d_DiagramNavigatorItemImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DiagramNavigatorItemImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_d_a_b_ClientBindableAdapterFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientBindableAdapterFactory> { }
  public abstract class Type_factory__o_u_e_p_e_c_f_BooleanField__quals__j_e_i_Any_j_e_i_Default extends Factory<BooleanField> { }
  public abstract class Type_factory__o_j_e_u_n_c_l_Navigation__quals__j_e_i_Any_j_e_i_Default extends Factory<Navigation> { }
  public abstract class Type_factory__o_u_c_w_w_m_m_m_IconMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<IconMenuItemPresenter> { }
  public abstract class JsType_factory__o_u_w_m_m_MenuPosition__quals__Universal extends Factory<MenuPosition> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_LogDefinitionDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<LogDefinitionDevCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_b_CaseManagementElementBuilderControl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor_o_k_w_c_s_c_c_c_c_b_i_Element extends Factory<CaseManagementElementBuilderControl> { }
  public abstract class Type_factory__o_u_e_w_c_c_w_w_p_a_n_ActivityNotFoundPresenterActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ActivityNotFoundPresenterActivity> { }
  public abstract class Type_factory__o_u_c_v_p_m_m_WorkbenchMegaMenuView__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchMegaMenuView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_b_CaseManagementObserverBuilderControl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor_o_k_w_c_s_c_c_c_c_b_i_Observer extends Factory<CaseManagementObserverBuilderControl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_e_n_d_DiagramsNavigatorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DiagramsNavigatorImpl> { }
  public abstract class Type_factory__o_u_c_v_p_m_MultiScreenMenuBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiScreenMenuBuilder> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_CaseManagementDeleteSelectionSessionCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementDeleteSelectionSessionCommand> { }
  public abstract class Type_factory__o_u_e_w_c_c_s_SelectOptionView__quals__j_e_i_Any_j_e_i_Default extends Factory<SelectOptionView> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_r_EmptyDropRowView__quals__j_e_i_Any_j_e_i_Default extends Factory<EmptyDropRowView> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_l_CharacterListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<CharacterListBoxFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeLiveSearchEntryCreationEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeLiveSearchEntryCreationEditor> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_s_p_SliderFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<SliderFieldProvider> { }
  public abstract class Type_factory__o_u_c_v_p_PatternFlyEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<PatternFlyEntryPoint> { }
  public abstract class Type_factory__o_u_c_v_p_m_WorkbenchMenuStandardNavBarView__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchMenuStandardNavBarView> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionTreeExplorerScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionTreeExplorerScreen> { }
  public abstract class Type_factory__o_u_c_DocksEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<DocksEntryPoint> { }
  public abstract class Type_factory__o_j_e_s_c_l_c_SecurityContextImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityContextImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_c_CollapsedDefinitionPaletteItemWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<CollapsedDefinitionPaletteItemWidget> { }
  public abstract class Type_factory__o_u_e_p_c_e_DynamicMenuEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicMenuEditorView> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_f_AssigneeFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeFormGroupViewImpl> { }
  public abstract class Type_factory__o_u_e_c_e_ExperimentalFeaturesEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ExperimentalFeaturesEditorScreenActivity> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_l_r_FieldRequired__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldRequired> { }
  public abstract class Type_factory__o_j_e_i_s_b_c_BatchCallerProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<BatchCallerProvider> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_l_LayoutGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutGenerator> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_NewUserAttributeEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<NewUserAttributeEditorView> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_AssignmentsEditorFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<AssignmentsEditorFieldInitializer> { }
  public abstract class Producer_factory__o_u_e_e_c_c_f_e_ImageFileExport__quals__j_e_i_Any_j_e_i_Default extends Factory<ImageFileExport> { }
  public abstract class Type_factory__o_j_e_u_n_c_l_TransitionToRoleProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<TransitionToRoleProvider> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_s_ClientFieldManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientFieldManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_ShapeSetsMenuItemsBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<ShapeSetsMenuItemsBuilder> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_c_c_k_s_AppendNoneEndEventShortcut__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<AppendNoneEndEventShortcut> { }
  public abstract class JsType_factory__j_u_Set__quals__Universal extends Factory<Set> { }
  public abstract class Type_factory__o_u_e_p_c_i_PluginsInfo__quals__j_e_i_Any_j_e_i_Default extends Factory<PluginsInfo> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionDiagramEditorScreenActivity__quals__j_e_i_Any_j_i_Named_o_k_w_c_s_c_c_a_DiagramEditor extends Factory<SessionDiagramEditorScreenActivity> { }
  public abstract class Producer_factory__e_d_HTMLDocument__quals__j_e_i_Any_j_e_i_Default extends Factory<HTMLDocument> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_e_s_ScreenEventPublisher__quals__j_e_i_Any_j_e_i_Default extends Factory<ScreenEventPublisher> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_p_CaseManagementPaletteDefinitionBuilder__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementPaletteDefinitionBuilder> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_c_ConfigErrorDisplayer__quals__j_e_i_Any_j_e_i_Default extends Factory<ConfigErrorDisplayer> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_w_CaseManagementControlFactory__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementControlFactory> { }
  public abstract class Type_factory__o_u_e_w_c_c_m_MenuItemHeaderView__quals__j_e_i_Any_o_u_e_w_c_c_m_MenuItemHeader extends Factory<MenuItemHeaderView> { }
  public abstract class Type_factory__o_u_e_w_c_c_w_WizardPageTitle__quals__j_e_i_Any_j_e_i_Default extends Factory<WizardPageTitle> { }
  public abstract class Type_factory__o_u_c_w_DefaultBeanFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultBeanFactory> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_SaveButton__quals__j_e_i_Any_j_e_i_Default extends Factory<SaveButton> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_g_w_GroupCreationWorkflow__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupCreationWorkflow> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_BPMNShapeSet__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNShapeSet> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionExceptionSwitchView__quals__j_e_i_Any_j_e_i_Default extends Factory<PermissionExceptionSwitchView> { }
  public abstract class Type_factory__o_u_e_c_d_s_DisabledFeatureActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<DisabledFeatureActivity> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_ClientDiagramServiceImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientDiagramServiceImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_LocationControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<LocationControlImpl> { }
  public abstract class Type_factory__o_u_c_WorkbenchServicesProxyBackendImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchServicesProxyBackendImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_n_f_FieldSetFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldSetFormGroupViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_PasteToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<PasteToolbarCommand> { }
  public abstract class Type_factory__o_u_s_c_e_c_e_SSHKeysEditorEmptyStateDisplayer__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeysEditorEmptyStateDisplayer> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_n_c_CollapsibleFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CollapsibleFormGroupViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_AssignedEntitiesExplorerView__quals__j_e_i_Any_j_e_i_Default extends Factory<AssignedEntitiesExplorerView> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_t_a_CommonActionsToolboxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_CommonActionsToolbox extends Factory<CommonActionsToolboxView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_v_LienzoPanel__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoPanel> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_m_c_i_StringColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<StringColumnGenerator> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_w_StunnerWiresHandlerFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerWiresHandlerFactory> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_DiagramsNavigatorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<DiagramsNavigatorScreenActivity> { }
  public abstract class Type_factory__o_u_c_w_p_s_PartManager__quals__j_e_i_Any_j_e_i_Default extends Factory<PartManager> { }
  public abstract class Type_factory__o_u_e_w_c_c_w_w_p_a_n_ActivityNotFoundPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityNotFoundPresenter> { }
  public abstract class Type_factory__o_j_e_i_c_l_i_AccessImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<AccessImpl> { }
  public abstract class Type_factory__o_u_w_c_Undefined__quals__j_e_i_Any_j_e_i_Default extends Factory<Undefined> { }
  public abstract class Type_factory__o_u_e_p_c_e_PluginExplorerScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<PluginExplorerScreenActivity> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_s_i_SessionPresenterView__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionPresenterView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_i_ToolbarItem__quals__j_e_i_Any_j_e_i_Default extends Factory<ToolbarItem> { }
  public abstract class Type_factory__o_u_c_v_p_t_WorkbenchToolBarView__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchToolBarView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_v_ClientDiagramValidator__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientDiagramValidator> { }
  public abstract class ContextualProvider_factory__o_j_e_u_n_c_l_TransitionTo__quals__Universal extends Factory<TransitionTo> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_TextBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<TextBoxFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_CreateNodeAction__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_FlowActionsToolbox extends Factory<CreateNodeAction> { }
  public abstract class Type_factory__o_u_e_w_c_c_t_PagedTable__quals__j_e_i_Any_j_e_i_Default extends Factory<PagedTable> { }
  public abstract class ExtensionProvided_factory__o_k_w_c_s_b_c_f_w_VariableNameTextBox__quals__j_e_i_Any_j_e_i_Default extends Factory<VariableNameTextBox> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToPdfSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToPdfSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_c_g_DefinitionPaletteGroupWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionPaletteGroupWidget> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_DefaultCanvasCommandFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultCanvasCommandFactory> { }
  public abstract class Type_factory__o_k_w_c_s_b_r_BPMNDefinitionSetResourceType__quals__j_e_i_Any_j_e_i_Default_o_u_w_a_VisibleAsset extends Factory<BPMNDefinitionSetResourceType> { }
  public abstract class Type_factory__o_k_w_c_f_s_s_m_p_i_p_FieldRequiredEntryProcessor__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldRequiredEntryProcessor> { }
  public abstract class Type_factory__o_k_w_c_f_c_c_c_CrudComponentViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CrudComponentViewImpl> { }
  public abstract class Type_factory__o_u_e_c_s_i_CDIClientFeatureDefRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<CDIClientFeatureDefRegistry> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_d_i_DefaultDiagramEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultDiagramEditor> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_m_BPMNDefinitionSetMorphDefinitionProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSetMorphDefinitionProvider> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_m_m_e_FieldPlaceHolderEntryProcessor__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldPlaceHolderEntryProcessor> { }
  public abstract class Type_factory__o_u_c_v_p_n_ActivityNotFoundView__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityNotFoundView> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_t_BindableTypeAhead__quals__j_e_i_Any_j_e_i_Default extends Factory<BindableTypeAhead> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_HtmlEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<HtmlEditor> { }
  public abstract class Type_factory__o_u_c_w_VFSLockServiceProxyClientImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<VFSLockServiceProxyClientImpl> { }
  public abstract class JsType_factory__e_d_Document_RegisterElementOptionsType__quals__Universal extends Factory<RegisterElementOptionsType> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_AssignmentListItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<AssignmentListItemWidgetViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_s_SliderFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SliderFormGroupViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_n_c_CollapsibleFormGroup__quals__j_e_i_Any_j_e_i_Default extends Factory<CollapsibleFormGroup> { }
  public abstract class Type_factory__o_u_c_v_p_w_KebabMenuItem__quals__j_e_i_Any_j_e_i_Default extends Factory<KebabMenuItem> { }
  public abstract class Type_factory__o_u_c_e_SplashScreenJSExporter__quals__j_e_i_Any_j_e_i_Default extends Factory<SplashScreenJSExporter> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_ScriptTypeListFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ScriptTypeListFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_c_t_FormGenerationToolboxAction__quals__j_e_i_Any_j_e_i_Default extends Factory<FormGenerationToolboxAction> { }
  public abstract class JsType_factory__j_u_Map__quals__Universal extends Factory<Map> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_CheckBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<CheckBoxFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_LoadingBox__quals__j_e_i_Any_j_e_i_Default extends Factory<LoadingBox> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_c_ToggleCommentView__quals__j_e_i_Any_j_e_i_Default extends Factory<ToggleCommentView> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_l_h_FieldHelpViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldHelpViewImpl> { }
  public abstract class Type_factory__o_u_c_v_p_s_SplashModalFooter__quals__j_e_i_Any_j_e_i_Default extends Factory<SplashModalFooter> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_d_DefaultFileEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<DefaultFileEditorPresenterActivity> { }
  public abstract class Type_factory__o_u_e_p_c_e_PluginExplorerScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<PluginExplorerScreen> { }
  public abstract class Type_factory__o_u_c_v_p_m_PartContextMenusView__quals__j_e_i_Any_j_e_i_Default extends Factory<PartContextMenusView> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_FieldLayoutComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldLayoutComponent> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_c_f_ClientMetaDataEntryManager__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientMetaDataEntryManager> { }
  public abstract class Type_factory__o_u_c_v_p_t_ResizeTabPanel__quals__j_e_i_Any_o_u_c_v_p_t_Resize extends Factory<ResizeTabPanel> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_UndoToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<UndoToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_ClearSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ClearSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_g_LienzoImageStripGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoImageStripGlyphRenderer> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_s_ListBoxFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<ListBoxFieldInitializer> { }
  public abstract class Type_factory__o_k_w_c_f_p_e_h_i_FieldChangeHandlerManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldChangeHandlerManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionScreenViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionScreenViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_s_c_GenerateDiagramFormsSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<GenerateDiagramFormsSessionCommand> { }
  public abstract class JsType_factory__o_u_w_m_PartDefinition__quals__Universal extends Factory<PartDefinition> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Paragraph__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<Paragraph> { }
  public abstract class Type_factory__o_u_c_w_p_i_UnanchoredStaticWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<UnanchoredStaticWorkbenchPanelPresenter> { }
  public abstract class JsType_factory__o_u_w_m_m_MenuItemCommand__quals__Universal extends Factory<MenuItemCommand> { }
  public abstract class Type_factory__o_u_e_p_c_t_DynamicMenuResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicMenuResourceType> { }
  public abstract class ExtensionProvided_factory__o_g_c_u_Anchor__quals__j_e_i_Any_j_e_i_Default extends Factory<Anchor> { }
  public abstract class JsType_factory__e_d_HashChangeEvent_HashChangeEventEventInitDictType__quals__Universal extends Factory<HashChangeEventEventInitDictType> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeFieldEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeFieldEditorPresenter> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_UnorderedList__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<UnorderedList> { }
  public abstract class Type_factory__o_u_e_s_m_c_e_u_UserEditorDriver__quals__j_e_i_Any_j_e_i_Default extends Factory<UserEditorDriver> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_l_GroupsList__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupsList> { }
  public abstract class Type_factory__o_u_c_a_EditorTreeProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<EditorTreeProvider> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_DiagramsNavigatorScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<DiagramsNavigatorScreen> { }
  public abstract class Type_factory__o_k_w_c_f_s_s_m_p_i_p_FieldReadOnlyEntryProcessor__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldReadOnlyEntryProcessor> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_p_BPMNTextPreferences__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNTextPreferences> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_l_IntegerListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerListBoxFieldRenderer> { }
  public abstract class Provider_factory__c_g_g_u_c_u_RootPanel__quals__j_e_i_Any_j_e_i_Default extends Factory<RootPanel> { }
  public abstract class Producer_factory__o_k_w_c_s_c_r_i_DefinitionsCacheRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionsCacheRegistry> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_CaseManagementLocationControlImpl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementLocationControlImpl> { }
  public abstract class Type_factory__o_u_p_s_i_PreferenceScopeTypesProducer__quals__j_e_i_Any_j_e_i_Default extends Factory<PreferenceScopeTypesProducer> { }
  public abstract class Type_factory__o_u_e_p_c_t_PerspectiveLayoutPluginResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveLayoutPluginResourceType> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_DateTimePickerCell__quals__j_e_i_Any_j_e_i_Default extends Factory<DateTimePickerCell> { }
  public abstract class Type_factory__o_u_c_w_p_i_StaticWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<StaticWorkbenchPanelView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_d_NodeDragProxyImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<NodeDragProxyImpl> { }
  public abstract class Type_factory__o_u_c_v_p_HTML5DndSeleniumSupport__quals__j_e_i_Any_j_e_i_Default extends Factory<HTML5DndSeleniumSupport> { }
  public abstract class Type_factory__o_k_w_c_s_c_p_StunnerPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerPreferencesBeanGeneratedImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_i_LiveSearchSelectorDropDownItemViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<LiveSearchSelectorDropDownItemViewImpl> { }
  public abstract class Type_factory__o_u_c_w_p_i_MultiScreenWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiScreenWorkbenchPanelPresenter> { }
  public abstract class Producer_factory__o_u_c_w_w_m_WorkbenchMenuBarPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchMenuBarPresenter> { }
  public abstract class JsType_factory__o_u_w_m_t_ToolBar__quals__Universal extends Factory<ToolBar> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_HasRowsFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<HasRowsFieldInitializer> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_KeyValueSerializer__quals__j_e_i_Any_j_e_i_Default extends Factory<KeyValueSerializer> { }
  public abstract class Type_factory__o_k_w_c_s_c_p_StunnerDiagramEditorPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerDiagramEditorPreferencesBeanGeneratedImpl> { }
  public abstract class Type_factory__o_g_c_s_p_p_GAVPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<GAVPreferencesBeanGeneratedImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_l_DecimalListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<DecimalListBoxFieldRenderer> { }
  public abstract class Type_factory__o_u_e_s_m_c_e_g_GroupEditorDriver__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupEditorDriver> { }
  public abstract class Type_factory__o_u_s_c_e_c_k_k_SSHKeyEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeyEditor> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_i_m_MapIndexBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<MapIndexBuilder> { }
  public abstract class Type_factory__o_u_e_p_c_e_SplashEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<SplashEditorView> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_NodeContainmentEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<NodeContainmentEvaluationHandler> { }
  public abstract class Type_factory__o_u_c_v_p_m_WorkbenchViewModeSwitcherView__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchViewModeSwitcherView> { }
  public abstract class Type_factory__o_u_c_a_PerspectiveTreeProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveTreeProvider> { }
  public abstract class Type_factory__o_u_c_w_w_m_PartContextMenusPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<PartContextMenusPresenter> { }
  public abstract class Type_factory__o_u_c_v_p_m_SplashScreenMenuView__quals__j_e_i_Any_j_e_i_Default extends Factory<SplashScreenMenuView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_c_i_DefinitionPaletteItemWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionPaletteItemWidget> { }
  public abstract class Type_factory__o_j_e_i_c_l_i_DestructionImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DestructionImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_c_g_DefinitionPaletteGroupWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionPaletteGroupWidgetViewImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_b_UberfireBreadcrumbsView__quals__j_e_i_Any_j_e_i_Default extends Factory<UberfireBreadcrumbsView> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_HtmlEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<HtmlEditorView> { }
  public abstract class Type_factory__o_u_c_w_w_d_CompassDropController__quals__j_e_i_Any_j_e_i_Default extends Factory<CompassDropController> { }
  public abstract class Type_factory__o_u_s_c_e_c_SSHKeysEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeysEditor> { }
  public abstract class Type_factory__o_k_w_c_f_p_e_h_i_DefaultModelValidator__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultModelValidator> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_i_DefaultToolbarView__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultToolbarView> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_ProcessMessageRefProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ProcessMessageRefProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_ACLEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<ACLEditor> { }
  public abstract class Type_factory__o_u_e_p_c_t_ScreenPluginResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<ScreenPluginResourceType> { }
  public abstract class Type_factory__o_u_e_l_e_c_g_BootstrapLayoutGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<BootstrapLayoutGenerator> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_ProcessErrorRefProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ProcessErrorRefProvider> { }
  public abstract class Type_factory__o_u_e_p_c_v_PluginNameValidator__quals__j_e_i_Any_j_e_i_Default extends Factory<PluginNameValidator> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_ImageElementRendererViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ImageElementRendererViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_UserGroupsExplorerView__quals__j_e_i_Any_j_e_i_Default extends Factory<UserGroupsExplorerView> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_l_UsersList__quals__j_e_i_Any_j_e_i_Default extends Factory<UsersList> { }
  public abstract class Type_factory__o_u_c_v_p_w_FormGroup__quals__j_e_i_Any_j_e_i_Default extends Factory<FormGroup> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_i_DefaultDefinitionsCacheRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultDefinitionsCacheRegistry> { }
  public abstract class Type_factory__o_j_e_s_c_l_c_DefaultRestSecurityErrorCallback__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultRestSecurityErrorCallback> { }
  public abstract class Type_factory__o_u_c_v_p_t_MultiTabWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<MultiTabWorkbenchPanelView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_w_CaseManagementContainmentStateHolder__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementContainmentStateHolder> { }
  public abstract class Type_factory__o_u_c_v_p_m_m_c_ChildContextMenuItemView__quals__j_e_i_Any_j_e_i_Default extends Factory<ChildContextMenuItemView> { }
  public abstract class Type_factory__o_j_e_u_n_c_l_HistoryTokenFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<HistoryTokenFactory> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_r_s_p_SubFormFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<SubFormFieldProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_p_SecurityManagementPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SecurityManagementPerspectiveActivity> { }
  public abstract class Type_factory__o_j_e_s_c_l_r_ClientRequiredRolesExtractorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientRequiredRolesExtractorImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_ManagedClientSessionCommands__quals__j_e_i_Any_j_e_i_Default extends Factory<ManagedClientSessionCommands> { }
  public abstract class Type_factory__o_u_e_c_e_g_f_ExperimentalFeatureEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalFeatureEditor> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToBpmnSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToBpmnSessionCommand> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_SecurityExplorerView__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityExplorerView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_CommonActionsToolboxFactory__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_CommonActionsToolbox extends Factory<CommonActionsToolboxFactory> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_RuleManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<RuleManagerImpl> { }
  public abstract class JsType_factory__e_d_RTCConfigurationInterface_IceServersFieldType__quals__Universal extends Factory<IceServersFieldType> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_g_GroupUsersAssignment__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupUsersAssignment> { }
  public abstract class Type_factory__o_j_e_i_c_a_b_InitBallotProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<InitBallotProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_c_ClipboardControlCleaner__quals__j_e_i_Any_j_e_i_Default extends Factory<ClipboardControlCleaner> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_g_ClientFormGenerationManager__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientFormGenerationManager> { }
  public abstract class Type_factory__o_u_e_w_c_c_m_MenuItemFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<MenuItemFactory> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_DateEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<DateEditableColumnGenerator> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_m_c_i_BooleanColumnGenerator__quals__j_e_i_Any extends Factory<BooleanColumnGenerator> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_s_r_p_RadioGroupFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<RadioGroupFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_t_TimerSettingsFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<TimerSettingsFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_c_LocalClipboardControl__quals__j_e_i_Any_j_e_i_Default extends Factory<LocalClipboardControl> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_RestorePopUpView__quals__j_e_i_Any_j_e_i_Default extends Factory<RestorePopUpView> { }
  public abstract class JsType_factory__o_u_m_Command__quals__Universal extends Factory<Command> { }
  public abstract class Type_factory__o_u_c_RuntimePluginsServiceProxyBackendImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<RuntimePluginsServiceProxyBackendImpl> { }
  public abstract class Type_factory__o_j_e_u_c_w_LocaleListBox__quals__j_e_i_Any_j_e_i_Default extends Factory<LocaleListBox> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_e_LienzoCanvasExport__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoCanvasExport> { }
  public abstract class Type_factory__o_u_e_p_c_e_SplashEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<SplashEditorPresenterActivity> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_s_ScreenErrorViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ScreenErrorViewImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_m_MetaFileEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<MetaFileEditorPresenterActivity> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_w_Notifications__quals__j_e_i_Any_j_e_i_Default extends Factory<Notifications> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_FormsClientEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<FormsClientEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_FlowActionsToolboxFactory__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_FlowActionsToolbox extends Factory<FlowActionsToolboxFactory> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_LiveSearchDropDownView__quals__j_e_i_Any_j_e_i_Default extends Factory<LiveSearchDropDownView> { }
  public abstract class Type_factory__o_u_c_w_w_m_UtilityMenuBarPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<UtilityMenuBarPresenter> { }
  public abstract class Type_factory__o_u_e_p_e_c_f_TextField__quals__j_e_i_Any_j_e_i_Default extends Factory<TextField> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_t_c_ChildrenTraverseProcessorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ChildrenTraverseProcessorImpl> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_l_e_TargetDivDragComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<TargetDivDragComponent> { }
  public abstract class JsType_factory__o_u_m_PlaceRequest__quals__Universal extends Factory<PlaceRequest> { }
  public abstract class Type_factory__o_k_w_c_s_c_f_CaseManagementDiagramFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDiagramFactoryImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_WidgetWrapperViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<WidgetWrapperViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_DockingAcceptorControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DockingAcceptorControlImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_l_s_s_LongestPathVertexLayerer__quals__j_e_i_Any_j_e_i_Default extends Factory<LongestPathVertexLayerer> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_ContainmentAcceptorControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ContainmentAcceptorControlImpl> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_ShowcaseEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<ShowcaseEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_IntegerEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerEditableColumnGenerator> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_WidgetElementRendererViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<WidgetElementRendererViewImpl> { }
  public abstract class Type_factory__o_u_s_c_e_SSHKeysEditorScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeysEditorScreen> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_ExportToPngToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToPngToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_w_WiresCanvasView__quals__j_e_i_Any_j_e_i_Default extends Factory<WiresCanvasView> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_w_StunnerWiresControlFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerWiresControlFactory> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_LogSelectedEdgeDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<LogSelectedEdgeDevCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_r_BPMNSVGViewFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNSVGViewFactoryImpl> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_s_SelectorFieldInitilizer__quals__j_e_i_Any_j_e_i_Default extends Factory<SelectorFieldInitilizer> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_c_f_ClientFormGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientFormGenerator> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_DeletePopUpPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<DeletePopUpPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_w_ComboBox__quals__j_e_i_Any_j_e_i_Default extends Factory<ComboBox> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionPropertiesScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionPropertiesScreen> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeLiveSearchService__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeLiveSearchService> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_c_f_u_ClientPropertuValueExtractor__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientPropertuValueExtractor> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_NewUserAttributeEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<NewUserAttributeEditor> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_a_TextEditorSingleLineBox__quals__j_e_i_Any_o_k_w_c_s_c_c_c_c_a_SingleLineTextEditorBox extends Factory<TextEditorSingleLineBox> { }
  public abstract class Type_factory__o_u_c_w_p_i_MultiListWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<MultiListWorkbenchPanelView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_ClearCommandHistoryDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ClearCommandHistoryDevCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_t_c_CaseManagementDeleteSelectionToolbarCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementDeleteSelectionToolbarCommand> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_c_ColumnWithComponentsView__quals__j_e_i_Any_j_e_i_Default extends Factory<ColumnWithComponentsView> { }
  public abstract class Type_factory__o_u_c_w_w_n_ActivityNotFoundPresenter__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<org.uberfire.client.workbench.widgets.notfound.ActivityNotFoundPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_EdgeCardinalityEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<EdgeCardinalityEvaluationHandler> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_w_ComboBoxViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ComboBoxViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_a_TextEditorMultiLineBoxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_c_a_MultiLineTextEditorBox extends Factory<TextEditorMultiLineBoxView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_i_ToolbarItemView__quals__j_e_i_Any_j_e_i_Default extends Factory<ToolbarItemView> { }
  public abstract class Type_factory__o_u_c_w_p_i_ClosableSimpleWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ClosableSimpleWorkbenchPanelPresenter> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_w_UserEditorWorkflow__quals__j_e_i_Any_j_e_i_Default extends Factory<UserEditorWorkflow> { }
  public abstract class Type_factory__o_u_c_m_LockDemandDetector__quals__j_e_i_Any_j_e_i_Default extends Factory<LockDemandDetector> { }
  public abstract class Type_factory__o_u_e_p_e_c_f_ColorField__quals__j_e_i_Any_j_e_i_Default extends Factory<ColorField> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLDivElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLDivElement> { }
  public abstract class Type_factory__o_u_e_w_c_c_CommonsEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<CommonsEntryPoint> { }
  public abstract class Type_factory__o_j_e_s_c_l_s_RoleStyleBindingProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<RoleStyleBindingProvider> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_d_p_DecimalBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<DecimalBoxFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_u_CanvasLayoutUtils__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasLayoutUtils> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_ACLSettingsView__quals__j_e_i_Any_j_e_i_Default extends Factory<ACLSettingsView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_ExportToJpgToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToJpgToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_i_DefaultEditorSession__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultEditorSession> { }
  public abstract class Type_factory__o_u_e_w_c_c_b_h_UberfireBreadcrumbsContainerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<UberfireBreadcrumbsContainerImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_h_SecurityManagementHomeScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityManagementHomeScreen> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_g_LienzoImageStripLoader__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoImageStripLoader> { }
  public abstract class Type_factory__o_u_c_w_p_i_SimpleWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SimpleWorkbenchPanelView> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_v_ShapeViewFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<ShapeViewFactory> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_RoleEditorScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<RoleEditorScreen> { }
  public abstract class JsType_factory__o_j_e_i_c_c_JsTypeProvider__quals__Universal extends Factory<JsTypeProvider> { }
  public abstract class ContextualProvider_factory__e_d_HTMLElement__quals__Universal extends Factory<HTMLElement> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_w_WiresManagerFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<WiresManagerFactoryImpl> { }
  public abstract class Type_factory__o_u_c_w_p_i_SimpleDnDWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default extends Factory<SimpleDnDWorkbenchPanelView> { }
  public abstract class Type_factory__o_u_p_s_i_PreferenceScopeFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PreferenceScopeFactoryImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeListItem__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeListItem> { }
  public abstract class Type_factory__o_u_s_c_e_c_k_k_SSHKeyEditorViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeyEditorViewImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_f_LiveSearchFooter__quals__j_e_i_Any_j_e_i_Default extends Factory<LiveSearchFooter> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_h_SecurityManagementHomeView__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityManagementHomeView> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLTextAreaElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLTextAreaElement> { }
  public abstract class Type_factory__o_u_c_v_p_m_WorkbenchMenuBarView__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchMenuBarView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_i_SessionLoader__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionLoader> { }
  public abstract class Type_factory__o_j_e_e_c_c_EventProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<EventProvider> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_d_DoubleEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<DoubleEditableColumnGenerator> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_g_SvgElementGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<SvgElementGlyphRenderer> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_w_EntityWorkflowViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<EntityWorkflowViewImpl> { }
  public abstract class JsType_factory__o_u_w_m_m_MenuItem__quals__Universal extends Factory<MenuItem> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_r_DecimalRadioGroupFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<DecimalRadioGroupFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_LogSelectedItemsDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<LogSelectedItemsDevCommand> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutComponentPalettePresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutComponentPalettePresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_l_LayoutHelper__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutHelper> { }
  public abstract class ContextualProvider_factory__j_e_i_Instance__quals__Universal extends Factory<Instance> { }
  public abstract class DynamicValidatorFactory extends Factory<DynamicValidator> { }
  public abstract class ExtensionProvided_factory__c_g_g_u_c_u_HeaderPanel__quals__j_e_i_Any_j_e_i_Default extends Factory<HeaderPanel> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_AssigneeFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeFieldInitializer> { }
  public abstract class Producer_factory__c_g_w_b_e_s_EventBus__quals__j_e_i_Any_j_e_i_Default extends Factory<EventBus> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_f_CaseManagementShapeDefFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementShapeDefFactory> { }
  public abstract class Type_factory__o_u_c_e_PlaceManagerJSExporter__quals__j_e_i_Any_j_e_i_Default extends Factory<PlaceManagerJSExporter> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_p_AuthoringPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<AuthoringPerspectiveActivity> { }
  public abstract class Type_factory__o_u_s_c_e_c_k_SSHKeysDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeysDisplayerViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_r_RoleEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<RoleEditorView> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_a_PerspectiveEditorCoreGroupProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveEditorCoreGroupProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_LogNodeEdgesDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<LogNodeEdgesDevCommand> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Heading__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<Heading> { }
  public abstract class Type_factory__o_u_e_p_c_e_ScreenEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<ScreenEditorView> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_l_h_FieldHelp__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldHelp> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_w_c_FormsContainer__quals__j_e_i_Any_j_e_i_Default extends Factory<FormsContainer> { }
  public abstract class Type_factory__o_u_c_v_p_m_MultiPageEditorViewImpl__quals__j_e_i_Any_o_u_c_w_w_m_Multiple extends Factory<MultiPageEditorViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_UserEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<UserEditor> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_SwitchLogLevelDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<SwitchLogLevelDevCommand> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_d_DefaultFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultFormGroupViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_d_DefaultFormGroup__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultFormGroup> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_MultiplePermissionNodeEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiplePermissionNodeEditorView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_DeleteSelectionToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<DeleteSelectionToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_r_i_ClientRegistryFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientRegistryFactoryImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeListFieldEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeListFieldEditorPresenter> { }
  public abstract class Type_factory__o_u_c_v_p_m_m_m_GroupMenuItemView__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupMenuItemView> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_s_MultipleSelectorFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleSelectorFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_BPMNGraphFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNGraphFactoryImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_l_s_s_DefaultVertexPositioning__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultVertexPositioning> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionDiagramViewerScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SessionDiagramViewerScreenActivity> { }
  public abstract class Type_factory__o_k_w_c_f_p_e_h_ProcessingEngineEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<ProcessingEngineEntryPoint> { }
  public abstract class Type_factory__o_u_e_p_c_e_SplashEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<SplashEditorPresenter> { }
  public abstract class Producer_factory__o_u_c_w_w_m_m_WorkbenchMegaMenuPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchMegaMenuPresenter> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_t_BindableTypeAheadViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BindableTypeAheadViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_f_CaseManagementGraphFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementGraphFactoryImpl> { }
  public abstract class Type_factory__o_u_c_w_p_i_ClosableSimpleWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ClosableSimpleWorkbenchPanelView> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_c_c_BPMNCanvasShortcutsControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNCanvasShortcutsControlImpl> { }
  public abstract class Type_factory__o_j_e_i_c_a_b_CallerProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<CallerProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_a_CanvasInPlaceTextEditorControlMultiLine__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_a_MultiLineTextEditorBox extends Factory<CanvasInPlaceTextEditorControlMultiLine> { }
  public abstract class Type_factory__o_u_c_r_i_PermissionTreeI18NImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PermissionTreeI18NImpl> { }
  public abstract class JsType_factory__e_d_HTMLElement_AttachShadowOptionsType__quals__Universal extends Factory<AttachShadowOptionsType> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_v_CanvasView__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_CaseManagementEditorSession__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementEditorSession> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_n_SubFormFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<SubFormFieldInitializer> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_e_i_ConnectorParentsMatchConnectionHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<ConnectorParentsMatchConnectionHandler> { }
  public abstract class Type_factory__o_u_c_v_p_w_DropdownMenu__quals__j_e_i_Any_j_e_i_Default extends Factory<DropdownMenu> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_p_LoadingBox__quals__j_e_i_Any_j_e_i_Default extends Factory<org.uberfire.ext.security.management.client.widgets.popup.LoadingBox> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_n_MultipleSubFormFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleSubFormFieldInitializer> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Span__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<Span> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLInputElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLInputElement> { }
  public abstract class Type_factory__o_u_b_v_i_ObservablePathImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ObservablePathImpl> { }
  public abstract class ContextualProvider_factory__o_u_p_c_s_PreferenceStore__quals__Universal extends Factory<PreferenceStore> { }
  public abstract class Type_factory__o_u_e_p_c_e_DynamicMenuEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<DynamicMenuEditorPresenterActivity> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_DynamicRendererEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicRendererEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_ConnectionEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<ConnectionEvaluationHandler> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLOListElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLOListElement> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_a_AdapterManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<AdapterManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_GraphConnectionEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<GraphConnectionEvaluationHandler> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_t_DefaultToolboxControl__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultToolboxControl> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_a_b_BootstrapAdapterFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<BootstrapAdapterFactory> { }
  public abstract class Type_factory__o_u_e_p_c_p_PlugInAuthoringPerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<PlugInAuthoringPerspective> { }
  public abstract class Type_factory__o_u_c_v_p_t_UberTabPanel__quals__j_e_i_Any_j_e_i_Default extends Factory<UberTabPanel> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_c_CheckBoxFormGroup__quals__j_e_i_Any_j_e_i_Default extends Factory<CheckBoxFormGroup> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_b_i_ElementBuilderControlImpl__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_b_i_Element extends Factory<ElementBuilderControlImpl> { }
  public abstract class JsType_factory__o_u_c_m_Activity__quals__Universal extends Factory<Activity> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLButtonElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLButtonElement> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_i_ManagedToolbar__quals__j_e_i_Any_j_e_i_Default extends Factory<ManagedToolbar> { }
  public abstract class Type_factory__o_u_e_p_c_e_EditorPlugInEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<EditorPlugInEditorPresenterActivity> { }
  public abstract class Type_factory__o_u_s_c_SecurityEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityEntryPoint> { }
  public abstract class Type_factory__o_u_c_s_JSNativeSplashScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<JSNativeSplashScreen> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_NotificationsScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<NotificationsScreen> { }
  public abstract class Type_factory__o_j_e_u_c_w_LocaleSelector__quals__j_e_i_Any_j_e_i_Default extends Factory<LocaleSelector> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutComponentPaletteView__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutComponentPaletteView> { }
  public abstract class Type_factory__o_j_e_s_c_l_c_SecurityContextImpl_SecurityRolesConstraintPage__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityRolesConstraintPage> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_m_MultipleSubFormFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleSubFormFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_f_i_GraphFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<GraphFactoryImpl> { }
  public abstract class Type_factory__o_j_e_i_c_l_i_CreationImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CreationImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_UsersExplorer__quals__j_e_i_Any_j_e_i_Default extends Factory<UsersExplorer> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_DecimalBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<DecimalBoxFieldRenderer> { }
  public abstract class Type_factory__o_j_e_d_c_ListComponentProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ListComponentProvider> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_w_FormPropertiesWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<FormPropertiesWidget> { }
  public abstract class JsType_factory__o_u_w_m_m_MenuItemPerspective__quals__Universal extends Factory<MenuItemPerspective> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_s_i_SessionEditorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionEditorImpl> { }
  public abstract class Type_factory__o_u_p_s_i_DefaultPreferenceScopeTypes__quals__j_e_i_Any_o_u_a_FallbackImplementation extends Factory<DefaultPreferenceScopeTypes> { }
  public abstract class JsType_factory__o_u_w_m_ContextDefinition__quals__Universal extends Factory<ContextDefinition> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_PerspectiveEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveEditorView> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_ResizeControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ResizeControlImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_b_UberfireBreadcrumbs__quals__j_e_i_Any_j_e_i_Default extends Factory<UberfireBreadcrumbs> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_f_CaseManagementShapeFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementShapeFactory> { }
  public abstract class Type_factory__o_u_e_l_e_c_LayoutEditorPluginImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutEditorPluginImpl> { }
  public abstract class Type_factory__o_u_s_i_a_DefaultPermissionTypeRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultPermissionTypeRegistry> { }
  public abstract class Type_factory__o_u_c_v_p_s_SplashViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SplashViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_g_LienzoGlyphRenderers__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoGlyphRenderers> { }
  public abstract class JsType_factory__o_u_c_m_PerspectiveActivity__quals__Universal extends Factory<PerspectiveActivity> { }
  public abstract class Type_factory__o_u_e_c_e_g_ExperimentalFeaturesGroup__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalFeaturesGroup> { }
  public abstract class JsType_factory__o_j_e_e_c_c_JsTypeEventObserver__quals__Universal extends Factory<JsTypeEventObserver> { }
  public abstract class Type_factory__o_u_c_w_WorkbenchLayoutImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchLayoutImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeEditorWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeEditorWidget> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_FormLayoutGenerator__quals__j_e_i_Any extends Factory<FormLayoutGenerator> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_HtmlEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<HtmlEditorPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_p_CanvasPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<CanvasPerspectiveActivity> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_d_BigDecimalEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<BigDecimalEditableColumnGenerator> { }
  public abstract class Type_factory__o_u_c_e_PluginJSExporter__quals__j_e_i_Any_j_e_i_Default extends Factory<PluginJSExporter> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_c_p_CheckBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<CheckBoxFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_MultipleInputFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleInputFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_f_i_DiagramFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DiagramFactoryImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_MultipleInputComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleInputComponent> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_s_i_SessionEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionEditorPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_UndoSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<UndoSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_f_c_c_CrudComponentEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<CrudComponentEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_CaseManagementCanvasView__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementCanvasView> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_l_i_MultipleInputProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleInputProvider> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_SavePopUpPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<SavePopUpPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_CanvasCommandManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasCommandManagerImpl> { }
  public abstract class Type_factory__o_u_c_v_p_m_m_m_ChildMenuItemView__quals__j_e_i_Any_j_e_i_Default extends Factory<ChildMenuItemView> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_l_s_s_DefaultVertexOrdering__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultVertexOrdering> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_d_CommonDomainLookups__quals__j_e_i_Any_j_e_i_Default extends Factory<CommonDomainLookups> { }
  public abstract class JsType_factory__o_u_w_m_m_i_DefaultMenuGroup__quals__Universal extends Factory<DefaultMenuGroup> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_StunnerLienzoCore__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerLienzoCore> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_g_LienzoShapeGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoShapeGlyphRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_c_c_BPMNToolboxControl__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<BPMNToolboxControl> { }
  public abstract class Producer_factory__o_j_e_u_n_c_l_URLPatternMatcher__quals__j_e_i_Any_j_e_i_Default extends Factory<URLPatternMatcher> { }
  public abstract class Type_factory__o_u_c_d_UberfireDocksImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<UberfireDocksImpl> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_HtmlEditorActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<HtmlEditorActivity> { }
  public abstract class Type_factory__o_u_c_w_p_i_SimpleDnDWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<SimpleDnDWorkbenchPanelPresenter> { }
  public abstract class Type_factory__o_u_p_c_s_PreferenceStoreIOCProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<PreferenceStoreIOCProvider> { }
  public abstract class Type_factory__o_u_c_v_p_m_MultiScreenPartWidget__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<MultiScreenPartWidget> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_v_VariableDeleteHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<VariableDeleteHandler> { }
  public abstract class Type_factory__o_j_e_i_c_l_i_LifecycleListenerRegistrarImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<LifecycleListenerRegistrarImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_MultipleInputViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleInputViewImpl> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_RenamePopUpView__quals__j_e_i_Any_j_e_i_Default extends Factory<RenamePopUpView> { }
  public abstract class Type_factory__o_u_e_c_e_ExperimentalFeaturesEditorScreenViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalFeaturesEditorScreenViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_a_ClientDefinitionManager__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientDefinitionManager> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_CardinalityEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<CardinalityEvaluationHandler> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionDiagramPreviewScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionDiagramPreviewScreen> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_LiveSearchDropDown__quals__j_e_i_Any_j_e_i_Default extends Factory<LiveSearchDropDown> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_d_DatePickerFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<DatePickerFieldRenderer> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_EntitiesExplorerViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<EntitiesExplorerViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_h_SecurityManagementHomeScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SecurityManagementHomeScreenActivity> { }
  public abstract class ContextualProvider_factory__o_j_e_c_c_d_HTMLElement__quals__Universal extends Factory<org.jboss.errai.common.client.dom.HTMLElement> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_i_b_GraphBoundsIndexerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<GraphBoundsIndexerImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_b_w_DefaultBreadcrumbsView__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultBreadcrumbsView> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_c_f_ClientFieldElementProcessor__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientFieldElementProcessor> { }
  public abstract class Type_factory__o_u_e_a_c_AppsEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<AppsEntryPoint> { }
  public abstract class Producer_factory__o_u_e_e_c_c_f_e_TextFileExport__quals__j_e_i_Any_j_e_i_Default extends Factory<TextFileExport> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_v_VariableListItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<VariableListItemWidgetViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_BS3PaletteWidgetImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BS3PaletteWidgetImpl> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutDragComponentGroupPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutDragComponentGroupPresenter> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_LongEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<LongEditableColumnGenerator> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToSvgSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToSvgSessionCommand> { }
  public abstract class Type_factory__o_j_e_v_c_ValidatorProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ValidatorProvider> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_VersionHistoryPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<VersionHistoryPresenter> { }
  public abstract class Type_factory__o_u_p_s_b_m_PortablePreferenceMockBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PortablePreferenceMockBeanGeneratedImpl> { }
  public abstract class Type_factory__o_u_c_w_p_i_MultiTabWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiTabWorkbenchPanelPresenter> { }
  public abstract class JsType_factory__o_u_w_m_CustomPanelDefinition__quals__Universal extends Factory<CustomPanelDefinition> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_l_s_MultipleSelectorProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleSelectorProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_v_LienzoTextTooltip__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoTextTooltip> { }
  public abstract class JsType_factory__o_u_b_v_Path__quals__Universal extends Factory<Path> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_g_ImageElementGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ImageElementGlyphRenderer> { }
  public abstract class Type_factory__o_u_c_v_p_w_Elemental2Producer__quals__j_e_i_Any_j_e_i_Default extends Factory<Elemental2Producer> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_e_t_TreeExplorer__quals__j_e_i_Any_j_e_i_Default extends Factory<TreeExplorer> { }
  public abstract class Type_factory__o_k_w_c_s_c_f_i_EdgeFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<EdgeFactoryImpl> { }
  public abstract class ContextualProvider_factory__o_j_e_i_c_a_Disposer__quals__Universal extends Factory<Disposer> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_TextAreaFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<TextAreaFieldInitializer> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_ComboBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ComboBoxFieldProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_p_ErrorBox__quals__j_e_i_Any_j_e_i_Default extends Factory<ErrorBox> { }
  public abstract class ContextualProvider_factory__o_j_e_d_c_c_ListComponent__quals__Universal extends Factory<ListComponent> { }
  public abstract class JsType_factory__e_d_InputDeviceCapabilities_InputDeviceCapabilitiesOptionsType__quals__Universal extends Factory<InputDeviceCapabilitiesOptionsType> { }
  public abstract class Type_factory__o_u_c_w_p_i_SplitLayoutPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SplitLayoutPanelView> { }
  public abstract class Type_factory__o_u_e_e_c_c_m_BasicFileMenuBuilderImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BasicFileMenuBuilderImpl> { }
  public abstract class Type_factory__o_u_c_v_p_m_ErrorPopupView__quals__j_e_i_Any_j_e_i_Default extends Factory<org.uberfire.client.views.pfly.modal.ErrorPopupView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_i_ClientTranslationMessages__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientTranslationMessages> { }
  public abstract class Type_factory__o_u_e_w_c_c_w_w_p_a_n_ActivityNotFoundView__quals__j_e_i_Any_j_e_i_Default extends Factory<org.uberfire.ext.widgets.core.client.workbench.widgets.popups.activities.notfound.ActivityNotFoundView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_p_ExpandedPaletteDefinitionBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<ExpandedPaletteDefinitionBuilder> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_RolesExplorer__quals__j_e_i_Any_j_e_i_Default extends Factory<RolesExplorer> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_w_FormPropertiesWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FormPropertiesWidgetViewImpl> { }
  public abstract class JsType_factory__o_u_w_m_m_HasEnabledStateChangeListeners__quals__Universal extends Factory<HasEnabledStateChangeListeners> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetDefinitionSetAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSetDefinitionSetAdapterImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_t_c_FullContentTraverseProcessorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FullContentTraverseProcessorImpl> { }
  public abstract class Type_factory__o_j_e_s_c_l_s_StorageHandlerProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<StorageHandlerProvider> { }
  public abstract class Type_factory__o_u_c_w_t_AnyResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<AnyResourceType> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_FormWidgetsEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<FormWidgetsEntryPoint> { }
  public abstract class Type_factory__o_u_c_m_PlaceHistoryHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<PlaceHistoryHandler> { }
  public abstract class Type_factory__o_g_c_s_p_c_Form__quals__j_e_i_Any_j_e_i_Default extends Factory<Form> { }
  public abstract class Type_factory__o_u_c_w_w_n_NotificationManager__quals__j_e_i_Any_j_e_i_Default extends Factory<NotificationManager> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_f_ShapeDefFunctionalFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<ShapeDefFunctionalFactory> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Select__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<org.jboss.errai.common.client.dom.Select> { }
  public abstract class Type_factory__o_u_e_w_c_c_b_w_DefaultBreadcrumbsPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultBreadcrumbsPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_BPMNDiagramFilterProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDiagramFilterProvider> { }
  public abstract class Type_factory__o_u_e_e_c_c_m_RestoreVersionCommandProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<RestoreVersionCommandProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_MultiplePermissionNodeViewer__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiplePermissionNodeViewer> { }
  public abstract class Type_factory__o_u_e_l_e_c_LayoutEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutEditorPresenter> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_d_p_DatePickerFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<DatePickerFieldProvider> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_c_ToggleCommentPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ToggleCommentPresenter> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_s_SubFormFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<SubFormFieldRenderer> { }
  public abstract class Type_factory__o_u_c_e_t_JSNativeClientResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<JSNativeClientResourceType> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_m_MultipleSubFormWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleSubFormWidget> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_c_ComponentColumn__quals__j_e_i_Any_j_e_i_Default extends Factory<ComponentColumn> { }
  public abstract class ContextualProvider_factory__o_j_e_u_n_c_l_TransitionToRole__quals__Universal extends Factory<TransitionToRole> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeFieldEditorWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeFieldEditorWidget> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_a_PerspectiveEditorNavGroupProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveEditorNavGroupProvider> { }
  public abstract class Type_factory__o_u_c_w_t_DotResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<DotResourceType> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_ConnectorGlyphLienzoRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ConnectorGlyphLienzoRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_p_CollapsedPaletteDefinitionBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<CollapsedPaletteDefinitionBuilder> { }
  public abstract class Producer_factory__o_j_e_c_c_d_Document__quals__j_e_i_Any_j_e_i_Default extends Factory<Document> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLAnchorElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLAnchorElement> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_v_c_CanvasDiagramValidator__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasDiagramValidator> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_ValidateSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ValidateSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetPropertySetAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDefinitionSetPropertySetAdapterImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_LienzoLayer__quals__j_e_i_Any_o_k_w_c_s_c_l_Lienzo extends Factory<LienzoLayer> { }
  public abstract class Provider_factory__o_j_e_s_c_l_s_UserStorageHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<UserStorageHandler> { }
  public abstract class ContextualProvider_factory__o_j_e_i_c_a_ManagedInstance__quals__Universal extends Factory<ManagedInstance> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_AssignedEntitiesInlineEditorView__quals__j_e_i_Any_o_u_e_s_m_c_w_m_e_AssignedEntitiesInlineEditor extends Factory<AssignedEntitiesInlineEditorView> { }
  public abstract class Type_factory__o_u_e_c_s_a_ExperimentalActivitiesAuthorizationManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalActivitiesAuthorizationManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_PasteSelectionSessionCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<PasteSelectionSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_ScriptTypeFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ScriptTypeFieldProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_p_LoadingBoxView__quals__j_e_i_Any_j_e_i_Default extends Factory<LoadingBoxView> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeListFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeListFieldRenderer> { }
  public abstract class JsType_factory__o_u_w_m_m_Menus__quals__Universal extends Factory<Menus> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_StringEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<StringEditableColumnGenerator> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_CaseManagementVisitGraphSessionCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementVisitGraphSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_StunnerBPMNEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerBPMNEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToJpgSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToJpgSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_NotificationsScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<NotificationsScreenActivity> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_f_PictureProvidersManager__quals__j_e_i_Any_j_e_i_Default extends Factory<PictureProvidersManager> { }
  public abstract class Type_factory__o_u_e_l_e_c_i_DragHelperComponentColumn__quals__j_e_i_Any_j_e_i_Default extends Factory<DragHelperComponentColumn> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_s_NoCanvasView__quals__j_e_i_Any_j_e_i_Default extends Factory<NoCanvasView> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_g_LienzoSvgDataUriGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoSvgDataUriGlyphRenderer> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Anchor__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<org.jboss.errai.common.client.dom.Anchor> { }
  public abstract class Type_factory__o_u_c_p_JSNativePerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<JSNativePerspective> { }
  public abstract class Type_factory__o_u_p_s_i_PreferenceScopeValidatorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PreferenceScopeValidatorImpl> { }
  public abstract class Type_factory__o_u_e_p_e_c_PropertyEditorWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<PropertyEditorWidget> { }
  public abstract class Type_factory__o_u_c_v_p_m_UserMenuViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<UserMenuViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionSwitch__quals__j_e_i_Any_j_e_i_Default extends Factory<PermissionSwitch> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_UserAssignedGroupsEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<UserAssignedGroupsEditor> { }
  public abstract class JsType_factory__o_u_c_m_PlaceStatus__quals__Universal extends Factory<PlaceStatus> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_w_FormsCanvasSessionHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<FormsCanvasSessionHandler> { }
  public abstract class Type_factory__o_k_w_c_f_p_e_h_i_FormValidatorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FormValidatorImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_AssignedEntitiesModalEditorView__quals__j_e_i_Any_o_u_e_s_m_c_w_m_e_AssignedEntitiesModalEditor extends Factory<AssignedEntitiesModalEditorView> { }
  public abstract class ContextualProvider_factory__o_u_p_s_PreferenceScopeResolutionStrategy__quals__Universal extends Factory<PreferenceScopeResolutionStrategy> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_DeletePopUpView__quals__j_e_i_Any_j_e_i_Default extends Factory<DeletePopUpView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_i_ClientTranslationService__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientTranslationService> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_t_TimerSettingsFieldEditorWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<TimerSettingsFieldEditorWidget> { }
  public abstract class Type_factory__o_u_e_c_w_t_ExperimentalAwareClientTypeRegistryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalAwareClientTypeRegistryImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_AssignmentsEditorFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<AssignmentsEditorFieldProvider> { }
  public abstract class JsType_factory__o_u_s_Resource__quals__Universal extends Factory<Resource> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_c_ConfigErrorDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ConfigErrorDisplayerViewImpl> { }
  public abstract class Type_factory__o_u_c_w_p_i_LayoutPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutPanelPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_DockingEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<DockingEvaluationHandler> { }
  public abstract class Type_factory__o_u_c_s_JSNativeScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<JSNativeScreen> { }
  public abstract class Type_factory__o_u_c_v_p_w_Modal__quals__j_e_i_Any_j_e_i_Default extends Factory<Modal> { }
  public abstract class Type_factory__o_u_c_a_DefaultWorkbenchController__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultWorkbenchController> { }
  public abstract class ContextualProvider_factory__o_j_e_d_c_a_DataBinder__quals__Universal extends Factory<DataBinder> { }
  public abstract class ExtensionProvided_factory__o_s_Logger__quals__Universal_2 extends Factory<Logger> { }
  public abstract class ExtensionProvided_factory__o_s_Logger__quals__Universal_1 extends Factory<Logger> { }
  public abstract class ExtensionProvided_factory__o_s_Logger__quals__Universal_4 extends Factory<Logger> { }
  public abstract class ExtensionProvided_factory__o_s_Logger__quals__Universal_3 extends Factory<Logger> { }
  public abstract class ExtensionProvided_factory__o_s_Logger__quals__Universal_6 extends Factory<Logger> { }
  public abstract class ExtensionProvided_factory__o_s_Logger__quals__Universal_5 extends Factory<Logger> { }
  public abstract class Type_factory__o_u_e_w_c_c_t_p_NewTabFilterPopup__quals__j_e_i_Any_j_e_i_Default extends Factory<NewTabFilterPopup> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_ConnectorCardinalityEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<ConnectorCardinalityEvaluationHandler> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeFieldEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeFieldEditorView> { }
  public abstract class Type_factory__o_k_w_c_f_s_s_m_p_i_p_FieldTypeEntryProcessor__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldTypeEntryProcessor> { }
  public abstract class JsType_factory__o_u_w_m_m_MenuItemPlain__quals__Universal extends Factory<MenuItemPlain> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_w_WorkItemDefinitionClientRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkItemDefinitionClientRegistry> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_HasPlaceHolderFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<HasPlaceHolderFieldInitializer> { }
  public abstract class Type_factory__o_k_w_c_f_s_s_m_p_i_p_FieldLabelEntryProcessor__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldLabelEntryProcessor> { }
  public abstract class Type_factory__o_k_w_c_f_p_e_h_i_FieldStateValidatorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldStateValidatorImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_c_c_k_s_AppendParallelGatewayShortcut__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<AppendParallelGatewayShortcut> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_w_CaseManagementWiresManagerFactory__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementWiresManagerFactory> { }
  public abstract class Type_factory__o_u_c_w_w_m_WorkbenchMenuBarProducer__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchMenuBarProducer> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_s_i_SessionViewerPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionViewerPresenter> { }
  public abstract class Type_factory__o_u_e_p_c_e_DynamicMenuEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicMenuEditorPresenter> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_MultiplePermissionNodeEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiplePermissionNodeEditor> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_s_i_MultipleSelectorInput__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleSelectorInput> { }
  public abstract class Type_factory__o_k_w_c_f_c_c_c_CrudComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<CrudComponent> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_RestorePopUpPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<RestorePopUpPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_ScriptTypeFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeFieldInitializer> { }
  public abstract class Type_factory__o_j_e_i_c_a_b_RootPanelProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<RootPanelProvider> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_TableCell__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<TableCell> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_c_DefaultCloneProcess__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultCloneProcess> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_ShowcaseDiagramService__quals__j_e_i_Any_j_e_i_Default extends Factory<ShowcaseDiagramService> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_l_s_s_VerticesTransposer__quals__j_e_i_Any_j_e_i_Default extends Factory<VerticesTransposer> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_c_ConditionalComboBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ConditionalComboBoxFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_BS3PaletteWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BS3PaletteWidgetViewImpl> { }
  public abstract class Producer_factory__o_j_e_u_n_c_l_s_NavigationGraph__quals__j_e_i_Any_j_e_i_Default extends Factory<NavigationGraph> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_ImageStripRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<ImageStripRegistry> { }
  public abstract class Producer_factory__o_u_e_e_c_c_f_e_s_SvgFileExport__quals__j_e_i_Any_j_e_i_Default extends Factory<SvgFileExport> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_i_DefaultViewerSession__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultViewerSession> { }
  public abstract class ExtensionProvided_factory__o_s_Logger__quals__Universal extends Factory<Logger> { }
  public abstract class Type_factory__o_u_e_c_s_i_ClientExperimentalFeaturesRegistryServiceImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientExperimentalFeaturesRegistryServiceImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_c_DefinitionPaletteCategoryWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionPaletteCategoryWidgetViewImpl> { }
  public abstract class Type_factory__o_u_c_m_PlaceRequestHistoryMapperImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PlaceRequestHistoryMapperImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_t_TextEditorWorkbenchEditorActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<TextEditorWorkbenchEditorActivity> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_f_AssigneeFormGroup__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeFormGroup> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_CreateConnectorAction__quals__j_e_i_Any_j_e_i_Default extends Factory<CreateConnectorAction> { }
  public abstract class JsType_factory__o_u_w_m_i_PanelDefinitionImpl__quals__Universal extends Factory<PanelDefinitionImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_s_s_a_DynamicFormModelGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicFormModelGenerator> { }
  public abstract class Type_factory__o_u_e_p_c_e_ScreenEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<ScreenEditorPresenterActivity> { }
  public abstract class Producer_factory__o_u_c_v_p_w_JQueryProducer_JQuery__quals__j_e_i_Any_j_e_i_Default extends Factory<JQuery> { }
  public abstract class Type_factory__o_j_e_u_c_w_ListWidgetProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ListWidgetProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_i_b_CanvasBoundsIndexerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasBoundsIndexerImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_s_i_IFrameScreenView__quals__j_e_i_Any_j_e_i_Default extends Factory<IFrameScreenView> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_m_MetaFileEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<MetaFileEditorPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionDiagramViewerScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionDiagramViewerScreen> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_e_i_ConnectorParentsMatchHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<ConnectorParentsMatchHandler> { }
  public abstract class Type_factory__o_u_e_p_c_r_i_PerspectiveEditorPaletteI18nImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveEditorPaletteI18nImpl> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutDragComponentWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutDragComponentWidget> { }
  public abstract class Type_factory__o_u_e_w_c_c_s_i_IFrameScreenPresenterActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<IFrameScreenPresenterActivity> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_e_n_v_BootstrapNavigatorView__quals__j_e_i_Any_j_e_i_Default extends Factory<BootstrapNavigatorView> { }
  public abstract class Type_factory__o_k_w_c_s_c_f_i_NodeFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<NodeFactoryImpl> { }
  public abstract class Type_factory__o_j_e_u_n_c_l_URLPatternMatcherProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<URLPatternMatcherProvider> { }
  public abstract class Type_factory__o_u_c_v_p_m_Bs3Modal__quals__j_e_i_Any_j_e_i_Default extends Factory<Bs3Modal> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_RestoreUtil__quals__j_e_i_Any_j_e_i_Default extends Factory<RestoreUtil> { }
  public abstract class Type_factory__o_u_e_p_c_p_RuntimePluginStartup__quals__j_e_i_Any_j_e_i_Default extends Factory<RuntimePluginStartup> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_CaseManagementDefinitionSetResourceType__quals__j_e_i_Any_j_e_i_Default_o_u_w_a_VisibleAsset extends Factory<CaseManagementDefinitionSetResourceType> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_ConnectionAcceptorControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ConnectionAcceptorControlImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_LogSessionDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<LogSessionDevCommand> { }
  public abstract class Type_factory__o_u_e_p_c_s_PluginController__quals__j_e_i_Any_j_e_i_Default extends Factory<PluginController> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_r_w_RoleEditorWorkflow__quals__j_e_i_Any_j_e_i_Default extends Factory<RoleEditorWorkflow> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_u_GraphUtils__quals__j_e_i_Any_j_e_i_Default extends Factory<GraphUtils> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_c_ContainerView__quals__j_e_i_Any_j_e_i_Default extends Factory<ContainerView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_MorphActionsToolboxFactory__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_MorphActionsToolbox extends Factory<MorphActionsToolboxFactory> { }
  public abstract class JsType_factory__o_u_w_m_ContextDisplayMode__quals__Universal extends Factory<ContextDisplayMode> { }
  public abstract class Type_factory__o_u_c_w_WorkbenchLayoutInfoImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchLayoutInfoImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_t_a_MorphActionsToolboxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_MorphActionsToolbox extends Factory<MorphActionsToolboxView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_SaveToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<SaveToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_t_a_FlowActionsToolboxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_FlowActionsToolbox extends Factory<FlowActionsToolboxView> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_TableSection__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<TableSection> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetDefinitionAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSetDefinitionAdapterImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_ScriptTypeListFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeListFieldInitializer> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Label__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<Label> { }
  public abstract class Provider_factory__j_v_Validator__quals__j_e_i_Any_j_e_i_Default extends Factory<Validator> { }
  public abstract class JsType_factory__o_u_w_m_PerspectiveDefinition__quals__Universal extends Factory<PerspectiveDefinition> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_f_CaseManagementShapeDefFunctionalFactory__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementShapeDefFunctionalFactory> { }
  public abstract class Type_factory__o_u_c_w_LayoutSelection__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutSelection> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_VisitGraphSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<VisitGraphSessionCommand> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_ACLViewerView__quals__j_e_i_Any_j_e_i_Default extends Factory<ACLViewerView> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_d_DefinitionLookupManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionLookupManagerImpl> { }
  public abstract class Type_factory__o_j_e_u_s_ServerTemplateProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ServerTemplateProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_g_ElementShapeGlyphRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ElementShapeGlyphRenderer> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLUListElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLUListElement> { }
  public abstract class ExtensionProvided_factory__o_g_c_u_Button__quals__j_e_i_Any_j_e_i_Default extends Factory<org.gwtbootstrap3.client.ui.Button> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_b_i_NodeBuilderControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<NodeBuilderControlImpl> { }
  public abstract class ExtensionProvided_factory__o_g_c_u_g_FlowPanel__quals__j_e_i_Any_j_e_i_Default extends Factory<FlowPanel> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_ValidateToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ValidateToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_c_GraphCommandManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<GraphCommandManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_CutToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<CutToolbarCommand> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_UserEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<UserEditorScreenActivity> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_a_ShapeManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ShapeManagerImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_CoreEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<CoreEntryPoint> { }
  public abstract class Type_factory__o_u_e_s_m_c_ClientSecurityExceptionMessageResolver__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientSecurityExceptionMessageResolver> { }
  public abstract class Type_factory__o_u_c_m_PlaceManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PlaceManagerImpl> { }
  public abstract class Type_factory__o_u_c_m_CustomSplashHelp__quals__j_e_i_Any_j_e_i_Default extends Factory<CustomSplashHelp> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_LeafPermissionNodeEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<LeafPermissionNodeEditorView> { }
  public abstract class Type_factory__o_u_e_p_e_c_f_SecretTextField__quals__j_e_i_Any_j_e_i_Default extends Factory<SecretTextField> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_VersionMenuDropDownButtonViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<VersionMenuDropDownButtonViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_t_TimerSettingsFieldEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<TimerSettingsFieldEditorPresenter> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_VersionHistoryPresenterViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<VersionHistoryPresenterViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_DiagramTypeServiceImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DiagramTypeServiceImpl> { }
  public abstract class Type_factory__o_u_s_i_a_DefaultAuthorizationManager__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultAuthorizationManager> { }
  public abstract class Type_factory__o_j_e_i_c_a_b_IOCBeanManagerProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<IOCBeanManagerProvider> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_p_w_PictureWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PictureWidgetViewImpl> { }
  public abstract class Type_factory__o_u_s_c_e_c_k_SSHKeysDisplayer__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeysDisplayer> { }
  public abstract class ExtensionProvided_factory__o_u_c_w_w_l_ResizeFlowPanel__quals__j_e_i_Any_j_e_i_Default extends Factory<ResizeFlowPanel> { }
  public abstract class Type_factory__o_u_e_p_c_p_PlugInAuthoringPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<PlugInAuthoringPerspectiveActivity> { }
  public abstract class Type_factory__o_u_c_v_p_w_KebabMenu__quals__j_e_i_Any_j_e_i_Default extends Factory<KebabMenu> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_ProcessSignalRefProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ProcessSignalRefProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_p_ErrorBoxView__quals__j_e_i_Any_j_e_i_Default extends Factory<ErrorBoxView> { }
  public abstract class Type_factory__o_k_w_c_s_c_StunnerCoreEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerCoreEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_n_AppNavBar__quals__j_e_i_Any_j_e_i_Default extends Factory<AppNavBar> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_d_DecimalBox__quals__j_e_i_Any_j_e_i_Default extends Factory<DecimalBox> { }
  public abstract class Type_factory__o_u_c_e_JSNativeEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<JSNativeEditor> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_DiagramPresenterScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<DiagramPresenterScreen> { }
  public abstract class Type_factory__o_u_e_w_c_c_s_SelectView__quals__j_e_i_Any_j_e_i_Default extends Factory<SelectView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_p_CaseManagementPreferencesRegistry__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementPreferencesRegistry> { }
  public abstract class Type_factory__o_u_e_p_c_e_ScreenEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ScreenEditorPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_SignalScopeProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<SignalScopeProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_LogBoundsDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<LogBoundsDevCommand> { }
  public abstract class JsType_factory__o_u_c_m_WorkbenchActivity__quals__Universal extends Factory<WorkbenchActivity> { }
  public abstract class Type_factory__o_j_e_u_n_c_l_TransitionAnchorFactoryProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<TransitionAnchorFactoryProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_CreateEntity__quals__j_e_i_Any_j_e_i_Default extends Factory<CreateEntity> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_ChangePassword__quals__j_e_i_Any_j_e_i_Default extends Factory<ChangePassword> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_i_IntegerBox__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerBox> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_l_OpenDiagramLayoutExecutor__quals__j_e_i_Any_j_e_i_Default extends Factory<OpenDiagramLayoutExecutor> { }
  public abstract class Producer_factory__o_k_w_c_s_b_w_WorkItemDefinitionRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkItemDefinitionRegistry> { }
  public abstract class Type_factory__o_u_c_m_WorkbenchViewModeSwitcherMenuBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchViewModeSwitcherMenuBuilder> { }
  public abstract class Type_factory__o_u_c_m_ActivityLifecycleErrorHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityLifecycleErrorHandler> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_p_CanvasPerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasPerspective> { }
  public abstract class Type_factory__o_u_c_w_p_i_UnanchoredStaticWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<UnanchoredStaticWorkbenchPanelView> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_m_ColumnGeneratorManager__quals__j_e_i_Any_j_e_i_Default extends Factory<ColumnGeneratorManager> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_TextInput__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<TextInput> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_w_NotificationsView__quals__j_e_i_Any_j_e_i_Default extends Factory<NotificationsView> { }
  public abstract class Type_factory__o_k_w_c_s_c_v_i_GraphValidatorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<GraphValidatorImpl> { }
  public abstract class Type_factory__o_u_e_l_e_c_i_DnDManager__quals__j_e_i_Any_j_e_i_Default extends Factory<DnDManager> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_p_PictureFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<PictureFieldRenderer> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_HtmlResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<HtmlResourceType> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_v_CanvasDefinitionTooltip__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasDefinitionTooltip> { }
  public abstract class Type_factory__o_j_e_i_s_b_c_RequestDispatcherProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<RequestDispatcherProvider> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_UserAttributesEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<UserAttributesEditor> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLLIElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLLIElement> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_f_BPMNDefinitionSetModelFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSetModelFactoryImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_e_i_ConnectorParentsMatchContainmentHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<ConnectorParentsMatchContainmentHandler> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_e_n_i_NavigatorThumbnailItemView__quals__j_e_i_Any_j_e_i_Default extends Factory<NavigatorThumbnailItemView> { }
  public abstract class Type_factory__o_u_c_v_p_w_InlineNotification__quals__j_e_i_Any_j_e_i_Default extends Factory<InlineNotification> { }
  public abstract class Type_factory__o_g_c_s_p_c_Process__quals__j_e_i_Any_j_e_i_Default extends Factory<Process> { }
  public abstract class JsType_factory__e_d_Element_ScrollIntoViewTopType__quals__Universal extends Factory<ScrollIntoViewTopType> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_CopyToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<CopyToolbarCommand> { }
  public abstract class Type_factory__o_u_s_c_e_c_c_NewSSHKeyModal__quals__j_e_i_Any_j_e_i_Default extends Factory<NewSSHKeyModal> { }
  public abstract class Type_factory__o_u_e_p_e_c_f_ComboField__quals__j_e_i_Any_j_e_i_Default extends Factory<ComboField> { }
  public abstract class Type_factory__o_u_s_c_a_t_i_DefaultPermissionTreeFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultPermissionTreeFactory> { }
  public abstract class JsType_factory__o_u_s_a_RuntimeResource__quals__Universal extends Factory<RuntimeResource> { }
  public abstract class Type_factory__o_u_c_m_ActivityBeansCache__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityBeansCache> { }
  public abstract class ExtensionProvided_factory__o_k_w_c_s_c_w_c_a_IntegerTextBox__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerTextBox> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_t_a_MorphNodeAction__quals__j_e_i_Any_j_e_i_Default extends Factory<MorphNodeAction> { }
  public abstract class Type_factory__o_u_c_w_w_m_m_m_ChildMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ChildMenuItemPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_c_ComboBoxWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ComboBoxWidgetViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_n_f_FieldSetFormGroup__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldSetFormGroup> { }
  public abstract class Type_factory__o_u_c_d_v_DocksBars__quals__j_e_i_Any_j_e_i_Default extends Factory<DocksBars> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_r_StunnerCommonIconsStrip__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerCommonIconsStrip> { }
  public abstract class Type_factory__o_u_e_p_c_w_p_NewPluginPopUpViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<NewPluginPopUpViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetRuleAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSetRuleAdapterImpl> { }
  public abstract class Type_factory__o_j_e_i_c_a_b_DisposerProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<DisposerProvider> { }
  public abstract class Type_factory__o_u_e_p_t_DynamicMenuResourceTypeDefinition__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicMenuResourceTypeDefinition> { }
  public abstract class Type_factory__o_j_e_i_s_b_c_MessageBusProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<MessageBusProvider> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_l_l_FieldLabel__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldLabel> { }
  public abstract class Producer_factory__o_u_e_e_c_c_f_e_PdfFileExport__quals__j_e_i_Any_j_e_i_Default extends Factory<PdfFileExport> { }
  public abstract class Type_factory__o_u_c_v_p_m_MultiPageEditorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<MultiPageEditorImpl> { }
  public abstract class Type_factory__o_j_e_i_s_b_c_SenderProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<SenderProvider> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_AssignmentsEditorWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<AssignmentsEditorWidget> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_RenamePopUpPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<RenamePopUpPresenter> { }
  public abstract class Type_factory__o_u_e_a_c_h_AppsHomeView__quals__j_e_i_Any_j_e_i_Default extends Factory<AppsHomeView> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_r_EmptyDropRow__quals__j_e_i_Any_j_e_i_Default extends Factory<EmptyDropRow> { }
  public abstract class Type_factory__o_j_e_i_c_a_b_ManagedInstanceProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ManagedInstanceProvider> { }
  public abstract class Type_factory__o_u_e_l_e_c_i_LayoutDragComponentHelper__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutDragComponentHelper> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_c_ComboBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ComboBoxFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_k_KeyEventHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<KeyEventHandler> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_c_i_GraphCommandFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<GraphCommandFactory> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_l_e_p_EditHTMLView__quals__j_e_i_Any_j_e_i_Default extends Factory<EditHTMLView> { }
  public abstract class JsType_factory__j_u_List__quals__Universal extends Factory<List> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_a_CanvasInPlaceTextEditorControlSingleLine__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_a_SingleLineTextEditorBox extends Factory<CanvasInPlaceTextEditorControlSingleLine> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Image__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<Image> { }
  public abstract class Type_factory__o_u_c_w_w_m_m_m_GroupMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupMenuItemPresenter> { }
  public abstract class Type_factory__o_u_e_c_e_ExperimentalFeaturesEditorScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalFeaturesEditorScreen> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_PasteSelectionSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<org.kie.workbench.common.stunner.core.client.session.command.impl.PasteSelectionSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_i_p_PictureFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<PictureFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_ByteEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<ByteEditableColumnGenerator> { }
  public abstract class Type_factory__o_j_e_s_c_l_i_AuthenticationServiceInterceptor__quals__j_e_i_Any_j_e_i_Default extends Factory<AuthenticationServiceInterceptor> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_CaseManagementCanvasHandler__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementCanvasHandler> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_l_s_s_ReverseEdgesCycleBreaker__quals__j_e_i_Any_j_e_i_Default extends Factory<ReverseEdgesCycleBreaker> { }
  public abstract class Type_factory__o_u_e_l_e_c_LayoutEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutEditorView> { }
  public abstract class Type_factory__o_u_c_p_Promises__quals__j_e_i_Any_j_e_i_Default extends Factory<Promises> { }
  public abstract class Type_factory__o_u_p_c_s_PreferenceBeanStoreImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PreferenceBeanStoreImpl> { }
  public abstract class Type_factory__o_u_c_v_p_l_ListBarWidgetImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ListBarWidgetImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_i_LogGraphDevCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<LogGraphDevCommand> { }
  public abstract class Type_factory__o_u_c_m_WorkbenchViewModeSwitcherPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchViewModeSwitcherPresenter> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_l_e_ScreenLayoutDragComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<ScreenLayoutDragComponent> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_ActivityDataIOEditorWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityDataIOEditorWidgetViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_c_c_k_s_AppendEmbeddedSubprocessShortcut__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<AppendEmbeddedSubprocessShortcut> { }
  public abstract class Type_factory__o_j_e_u_c_l_p_HTMLElementProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<HTMLElementProvider> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_AssigneeEditorFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeEditorFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_c_i_DefinitionPaletteItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionPaletteItemWidgetViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_SwitchGridToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<SwitchGridToolbarCommand> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_l_RolesList__quals__j_e_i_Any_j_e_i_Default extends Factory<RolesList> { }
  public abstract class JsType_factory__o_u_w_m_m_i_DefaultMenus__quals__Universal extends Factory<DefaultMenus> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_HtmlEditorLibraryLoader__quals__j_e_i_Any_j_e_i_Default extends Factory<HtmlEditorLibraryLoader> { }
  public abstract class Type_factory__o_u_e_e_c_c_v_DefaultFileNameValidator__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultFileNameValidator> { }
  public abstract class Type_factory__o_j_e_u_c_l_p_Elemental2HTMLElementProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<Elemental2HTMLElementProvider> { }
  public abstract class ExtensionProvided_factory__o_g_c_u_h_Span__quals__j_e_i_Any_j_e_i_Default extends Factory<org.gwtbootstrap3.client.ui.html.Span> { }
  public abstract class Type_factory__o_u_c_d_v_m_MenuBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<MenuBuilder> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_LienzoSelectionControl__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_s_SingleSelection extends Factory<LienzoSelectionControl> { }
  public abstract class Producer_factory__o_u_e_e_c_c_f_p_CopyPopUpPresenter_View__quals__j_e_i_Any_o_u_e_e_c_c_f_Customizable extends Factory<View> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_d_DefaultRouteFormProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultRouteFormProvider> { }
  public abstract class Type_factory__o_u_e_a_c_h_AppsHomePresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<AppsHomePresenter> { }
  public abstract class Type_factory__o_u_e_w_c_c_t_p_NewFilterPopup__quals__j_e_i_Any_j_e_i_Default extends Factory<NewFilterPopup> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_a_SessionManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_VariablesEditorFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<VariablesEditorFieldProvider> { }
  public abstract class Type_factory__o_u_e_w_c_c_t_FilterPagedTable__quals__j_e_i_Any_j_e_i_Default extends Factory<FilterPagedTable> { }
  public abstract class Type_factory__o_u_c_v_p_m_UtilityMenuBarView__quals__j_e_i_Any_j_e_i_Default extends Factory<UtilityMenuBarView> { }
  public abstract class Type_factory__o_k_w_c_s_c_CaseManagementDefinitionSet__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDefinitionSet> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_CopySelectionSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<CopySelectionSessionCommand> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_p_ConfirmBoxView__quals__j_e_i_Any_j_e_i_Default extends Factory<ConfirmBoxView> { }
  public abstract class Type_factory__o_u_e_l_e_c_g_BootstrapLayoutGeneratorDriver__quals__j_e_i_Any_j_e_i_Default extends Factory<BootstrapLayoutGeneratorDriver> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_ListItem__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ListItem> { }
  public abstract class Type_factory__o_u_s_c_e_SSHKeysEditorPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SSHKeysEditorPerspectiveActivity> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_BaseScreenView__quals__j_e_i_Any_j_e_i_Default extends Factory<BaseScreenView> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_w_c_FormsContainerViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FormsContainerViewImpl> { }
  public abstract class Producer_factory__o_u_r_SessionInfo__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionInfo> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_r_i_ClientDefinitionsCacheRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientDefinitionsCacheRegistry> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_s_w_SubFormWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<SubFormWidget> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_RoleEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<RoleEditorScreenActivity> { }
  public abstract class ContextualProvider_factory__o_j_e_u_c_w_ListWidget__quals__Universal extends Factory<ListWidget> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_n_NoItemsComponentViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<NoItemsComponentViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_a_TextPropertyProviderFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<TextPropertyProviderFactoryImpl> { }
  public abstract class Type_factory__o_u_e_l_e_c_i_e_DisabledExperimentalLayoutComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<DisabledExperimentalLayoutComponent> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_v_ClientModelValidator__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientModelValidator> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_f_LiveSearchFooterViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<LiveSearchFooterViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_UserEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<UserEditorView> { }
  public abstract class Type_factory__o_u_e_e_c_c_m_c_SaveAndRenameCommandBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<SaveAndRenameCommandBuilder> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_i_i_DynamicFormHandlerGenerator__quals__j_e_i_Any_o_k_w_c_f_d_s_s_DynamicContext extends Factory<DynamicFormHandlerGenerator> { }
  public abstract class Type_factory__o_k_w_c_f_p_e_h_i_FormHandlerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FormHandlerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_d_ShapeViewDragProxyImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ShapeViewDragProxyImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_c_r_RolesEditorWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<RolesEditorWidgetViewImpl> { }
  public abstract class Type_factory__o_u_e_c_e_g_ExperimentalFeaturesGroupViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalFeaturesGroupViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_CharacterEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<CharacterEditableColumnGenerator> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutElementPropertiesPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutElementPropertiesPresenter> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_i_LiveSearchSelectorDropDownItem__quals__j_e_i_Any_j_e_i_Default extends Factory<LiveSearchSelectorDropDownItem> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_SecurityExplorerScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SecurityExplorerScreenActivity> { }
  public abstract class Type_factory__o_k_w_c_f_f_s_f_b_s_l_p_ListBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ListBoxFieldProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_ContainmentEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<ContainmentEvaluationHandler> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_SessionTreeExplorerScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SessionTreeExplorerScreenActivity> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_HasMaxLengthFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<HasMaxLengthFieldInitializer> { }
  public abstract class JsType_factory__o_u_m_i_ExternalPathPlaceRequest__quals__Universal extends Factory<ExternalPathPlaceRequest> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_f_ConditionalComboBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ConditionalComboBoxFieldProvider> { }
  public abstract class Type_factory__o_j_e_e_c_c_InstanceProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<InstanceProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_b_CaseManagementNodeBuilderControl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementNodeBuilderControl> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_w_c_d_FormDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FormDisplayerViewImpl> { }
  public abstract class Type_factory__o_u_c_w_p_i_MultiScreenWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<MultiScreenWorkbenchPanelView> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_LeafPermissionNodeViewerView__quals__j_e_i_Any_j_e_i_Default extends Factory<LeafPermissionNodeViewerView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_b_i_ObserverBuilderControl__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_b_i_Observer extends Factory<ObserverBuilderControl> { }
  public abstract class Type_factory__o_u_c_v_p_n_NotificationPopupsManagerView__quals__j_e_i_Any_j_e_i_Default extends Factory<NotificationPopupsManagerView> { }
  public abstract class Type_factory__o_u_c_w_p_i_SplitLayoutPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<SplitLayoutPanelPresenter> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_CopyPopUpPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<CopyPopUpPresenter> { }
  public abstract class Type_factory__o_u_c_v_p_w_DateRangePicker__quals__j_e_i_Any_j_e_i_Default extends Factory<DateRangePicker> { }
  public abstract class Type_factory__o_u_c_w_w_m_m_c_GroupContextMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupContextMenuItemPresenter> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_d_DefaultFileEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultFileEditorView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_CaseManagementViewerSession__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementViewerSession> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_ControlPointControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ControlPointControlImpl> { }
  public abstract class JsType_factory__o_u_w_m_m_MenuCustom__quals__Universal extends Factory<MenuCustom> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_c_DeepCloneProcess__quals__j_e_i_Any_j_e_i_Default extends Factory<DeepCloneProcess> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_DateTimePickerPresenterViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DateTimePickerPresenterViewImpl> { }
  public abstract class JsType_factory__o_j_e_e_c_c_WindowEventObservers__quals__Universal extends Factory<WindowEventObservers> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_s_IntegerSliderFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerSliderFieldInitializer> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_LeafPermissionNodeViewer__quals__j_e_i_Any_j_e_i_Default extends Factory<LeafPermissionNodeViewer> { }
  public abstract class JsType_factory__e_d_RelatedEvent_RelatedEventEventInitDictType__quals__Universal extends Factory<RelatedEventEventInitDictType> { }
  public abstract class Type_factory__o_u_c_m_ForceUnlockEventObserver__quals__j_e_i_Any_j_e_i_Default extends Factory<ForceUnlockEventObserver> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_n_FormGenerationNotifier__quals__j_e_i_Any_j_e_i_Default extends Factory<FormGenerationNotifier> { }
  public abstract class JsType_factory__e_d_PageTransitionEvent_PageTransitionEventEventInitDictType__quals__Universal extends Factory<PageTransitionEventEventInitDictType> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_d_PrimitiveDragProxyImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PrimitiveDragProxyImpl> { }
  public abstract class Type_factory__o_u_c_w_w_d_WorkbenchPickupDragController__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchPickupDragController> { }
  public abstract class Type_factory__o_u_c_w_PanelManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PanelManagerImpl> { }
  public abstract class Type_factory__o_u_e_p_c_t_SplashPluginResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<SplashPluginResourceType> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_CaseManagementCanvasCommandFactory__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementCanvasCommandFactory> { }
  public abstract class Type_factory__o_u_e_w_c_c_s_SelectOptionComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<SelectOptionComponent> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetDefinitionSetProxyImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSetDefinitionSetProxyImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_m_b_DynamicBindingHelper__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicBindingHelper> { }
  public abstract class ExtensionProvided_factory__o_g_c_u_CheckBox__quals__j_e_i_Any_j_e_i_Default extends Factory<CheckBox> { }
  public abstract class ContextualProvider_factory__o_j_e_i_s_b_c_Sender__quals__Universal extends Factory<Sender> { }
  public abstract class Type_factory__o_u_e_p_c_w_n_PluginNavList__quals__j_e_i_Any_j_e_i_Default extends Factory<PluginNavList> { }
  public abstract class Type_factory__o_u_e_a_c_AppsPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<AppsPerspectiveActivity> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_LeafPermissionNodeEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<LeafPermissionNodeEditor> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_t_TextResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<TextResourceType> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_s_i_SessionPreviewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionPreviewImpl> { }
  public abstract class ExtensionProvided_factory__o_u_e_w_c_c_t_Tree__quals__j_e_i_Any_j_e_i_Default extends Factory<Tree> { }
  public abstract class Type_factory__o_u_c_m_SplashScreenMenuPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<SplashScreenMenuPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_c_DefinitionPaletteCategoryWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionPaletteCategoryWidget> { }
  public abstract class ExtensionProvided_factory__c_g_g_u_c_u_FlowPanel__quals__j_e_i_Any_j_e_i_Default extends Factory<com.google.gwt.user.client.ui.FlowPanel> { }
  public abstract class Type_factory__o_u_c_v_p_w_DropdownMenuItem__quals__j_e_i_Any_j_e_i_Default extends Factory<DropdownMenuItem> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_p_StunnerPreferencesRegistries__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerPreferencesRegistries> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_ExportToBpmnToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToBpmnToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_w_WorkItemDefinitionCacheRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkItemDefinitionCacheRegistry> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_h_i_ElementCardinalityEvaluationHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<ElementCardinalityEvaluationHandler> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_ACLSettings__quals__j_e_i_Any_j_e_i_Default extends Factory<ACLSettings> { }
  public abstract class Type_factory__o_g_c_s_p_p_GeneralPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<GeneralPreferencesBeanGeneratedImpl> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_e_j_FileExportScriptInjector__quals__j_e_i_Any_j_e_i_Default extends Factory<FileExportScriptInjector> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_a_TextEditorMultiLineBox__quals__j_e_i_Any_o_k_w_c_s_c_c_c_c_a_MultiLineTextEditorBox extends Factory<TextEditorMultiLineBox> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_p_StunnerPreferencesRegistryLoader__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerPreferencesRegistryLoader> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_f_BasicShapesFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<BasicShapesFactory> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_g_GroupEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupEditor> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_ExportToPdfToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToPdfToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetDefinitionAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDefinitionSetDefinitionAdapterImpl> { }
  public abstract class Type_factory__o_u_c_w_w_m_m_WorkbenchMegaMenuProducer__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchMegaMenuProducer> { }
  public abstract class Type_factory__o_u_e_p_c_w_p_NewPluginPopUp__quals__j_e_i_Any_j_e_i_Default extends Factory<NewPluginPopUp> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_RedoToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<RedoToolbarCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_ActivityDataIOEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityDataIOEditor> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_PerspectiveEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority extends Factory<PerspectiveEditorPresenterActivity> { }
  public abstract class Type_factory__o_u_c_m_CategoriesManagerCache__quals__j_e_i_Any_j_e_i_Default extends Factory<CategoriesManagerCache> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Div__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<Div> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_u_LienzoPanelUtils__quals__j_e_i_Any_j_e_i_Default extends Factory<LienzoPanelUtils> { }
  public abstract class Type_factory__o_k_w_c_s_b_BPMNDefinitionSet__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSet> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_r_m_b_StaticBindingHelper__quals__j_e_i_Any_j_e_i_Default extends Factory<StaticBindingHelper> { }
  public abstract class Type_factory__o_u_c_m_ResourceTypeManagerCache__quals__j_e_i_Any_j_e_i_Default extends Factory<ResourceTypeManagerCache> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_p_PictureInput__quals__j_e_i_Any_j_e_i_Default extends Factory<PictureInput> { }
  public abstract class Type_factory__o_u_e_w_c_c_e_t_TextEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<TextEditorView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_c_i_SaveDiagramSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<SaveDiagramSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetPropertySetAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSetPropertySetAdapterImpl> { }
  public abstract class JsType_factory__j_u_Collection__quals__Universal extends Factory<Collection> { }
  public abstract class JsType_factory__o_u_w_t_ResourceTypeDefinition__quals__Universal extends Factory<ResourceTypeDefinition> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_TextArea__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<TextArea> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_l_StringListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<StringListBoxFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_l_s_s_MedianVertexLayerPositioning__quals__j_e_i_Any_j_e_i_Default extends Factory<MedianVertexLayerPositioning> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_s_f_DelegateShapeFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<DelegateShapeFactory> { }
  public abstract class Type_factory__o_g_c_s_p_c_Model__quals__j_e_i_Any_j_e_i_Default extends Factory<Model> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_r_StringRadioGroupFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<StringRadioGroupFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeLiveSearchEntryCreationEditorViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<AssigneeLiveSearchEntryCreationEditorViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_l_l_FieldLabelViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FieldLabelViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_i_i_StaticFormHandlerGenerator__quals__j_e_i_Any_o_k_w_c_f_d_s_s_StaticContext extends Factory<StaticFormHandlerGenerator> { }
  public abstract class Type_factory__o_u_c_JSEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<JSEntryPoint> { }
  public abstract class Type_factory__o_u_c_w_p_i_TemplatedWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<TemplatedWorkbenchPanelView> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_VariablesEditorFieldInitializer__quals__j_e_i_Any_j_e_i_Default extends Factory<VariablesEditorFieldInitializer> { }
  public abstract class Type_factory__o_u_s_c_a_AuthorizationManagerHelper__quals__j_e_i_Any_j_e_i_Default extends Factory<AuthorizationManagerHelper> { }
  public abstract class Producer_factory__o_j_e_s_s_a_i_User__quals__j_e_i_Any_j_e_i_Default extends Factory<User> { }
  public abstract class Type_factory__o_j_e_d_c_DataBinderProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<DataBinderProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_AnimatedFloatingWidgetView__quals__j_e_i_Any_j_e_i_Default extends Factory<AnimatedFloatingWidgetView> { }
  public abstract class Type_factory__o_j_e_u_n_c_l_PageTransitionProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<PageTransitionProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetDefinitionSetAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDefinitionSetDefinitionSetAdapterImpl> { }
  public abstract class Type_factory__o_u_c_w_p_n_p_WorkbenchPartPresenterDefault__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchPartPresenterDefault> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_s_c_GenerateProcessFormsSessionCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<GenerateProcessFormsSessionCommand> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_t_TimerSettingsFieldEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<TimerSettingsFieldEditorView> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_p_SavePopUpView__quals__j_e_i_Any_j_e_i_Default extends Factory<SavePopUpView> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_i_ManagedViewerToolbar__quals__j_e_i_Any_j_e_i_Default extends Factory<ManagedViewerToolbar> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_d_DecimalBoxViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DecimalBoxViewImpl> { }
  public abstract class Type_factory__o_u_e_a_c_AppsPerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<AppsPerspective> { }
  public abstract class JsType_factory__o_u_c_m_PlaceManager__quals__Universal extends Factory<PlaceManager> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_f_CaseManagementDefinitionSetModelFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDefinitionSetModelFactoryImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_u_ClassUtils__quals__j_e_i_Any_j_e_i_Default extends Factory<ClassUtils> { }
  public abstract class Type_factory__o_k_w_c_s_c_r_CachedRuleManager__quals__j_e_i_Any_j_e_i_Default extends Factory<CachedRuleManager> { }
  public abstract class Type_factory__o_k_w_c_f_a_e_s_i_FormBuildingServiceImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<FormBuildingServiceImpl> { }
  public abstract class Type_factory__o_j_e_u_n_c_l_TransitionAnchorProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<TransitionAnchorProvider> { }
  public abstract class Type_factory__o_u_c_v_p_m_UserMenu__quals__j_e_i_Any_j_e_i_Default extends Factory<UserMenu> { }
  public abstract class Type_factory__o_u_c_v_p_d_CompassWidgetImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CompassWidgetImpl> { }
  public abstract class JsType_factory__o_u_w_m_t_ToolBarItem__quals__Universal extends Factory<ToolBarItem> { }
  public abstract class Type_factory__o_u_c_v_p_m_WorkbenchMenuCompactNavBarView__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchMenuCompactNavBarView> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_a_ClientFactoryManager__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientFactoryManager> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_UserAssignedRolesEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<UserAssignedRolesEditor> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetPropertyAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDefinitionSetPropertyAdapterImpl> { }
  public abstract class Type_factory__o_u_s_c_e_SSHKeysEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<SSHKeysEditorScreenActivity> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_t_i_CaseManagementEditorToolbar__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementEditorToolbar> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLLabelElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLLabelElement> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_s_BPMNSessionInitializer__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<BPMNSessionInitializer> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_PerspectiveEditorPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveEditorPresenter> { }
  public abstract class Type_factory__o_u_c_v_p_p_PopupViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<PopupViewImpl> { }
  public abstract class Type_factory__o_u_e_p_c_w_p_GeneralPluginEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<GeneralPluginEditor> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_CreateEntityView__quals__j_e_i_Any_j_e_i_Default extends Factory<CreateEntityView> { }
  public abstract class Type_factory__o_u_c_v_p_w_ConfirmPopup__quals__j_e_i_Any_j_e_i_Default extends Factory<ConfirmPopup> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_p_HomePerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<HomePerspective> { }
  public abstract class Type_factory__o_j_e_e_c_c_JsTypeEventObserver__quals__j_e_i_Any_j_e_i_Default extends Factory<JsTypeEventObserver> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_s_f_BPMNShapeFactory__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNShapeFactory> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_c_ColumnWithComponents__quals__j_e_i_Any_j_e_i_Default extends Factory<ColumnWithComponents> { }
  public abstract class Type_factory__o_u_c_w_w_t_WorkbenchToolBarPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchToolBarPresenter> { }
  public abstract class Type_factory__o_u_e_c_d_c_DisabledFeatureComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<DisabledFeatureComponent> { }
  public abstract class JsType_factory__o_u_c_m_LockTarget__quals__Universal extends Factory<LockTarget> { }
  public abstract class Type_factory__o_k_w_c_f_c_r_c_w_i_IntegerBoxViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerBoxViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_ClearToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ClearToolbarCommand> { }
  public abstract class Type_factory__o_u_e_e_c_c_BaseEditorEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<BaseEditorEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_s_f_c_w_c_d_FormDisplayer__quals__j_e_i_Any_j_e_i_Default extends Factory<FormDisplayer> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_MultipleInput__quals__j_e_i_Any_j_e_i_Default extends Factory<MultipleInput> { }
  public abstract class JsType_factory__e_d_CustomElementRegistry_DefineOptionsType__quals__Universal extends Factory<DefineOptionsType> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_IntegerBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<IntegerBoxFieldRenderer> { }
  public abstract class Type_factory__o_u_e_p_c_t_EditorPluginResourceType__quals__j_e_i_Any_j_e_i_Default extends Factory<EditorPluginResourceType> { }
  public abstract class Type_factory__o_u_e_l_e_c_i_LayoutEditorCssHelper__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutEditorCssHelper> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_n_NoItemsComponent__quals__j_e_i_Any_j_e_i_Default extends Factory<NoItemsComponent> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_AssignmentsEditorFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<AssignmentsEditorFieldRenderer> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_e_FileExportProducer__quals__j_e_i_Any_j_e_i_Default extends Factory<FileExportProducer> { }
  public abstract class JsType_factory__o_u_w_m_Position__quals__Universal extends Factory<Position> { }
  public abstract class ExtensionProvided_factory__e_d_HTMLTableRowElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HTMLTableRowElement> { }
  public abstract class Type_factory__o_u_e_p_c_e_EditorPlugInEditorView__quals__j_e_i_Any_j_e_i_Default extends Factory<EditorPlugInEditorView> { }
  public abstract class Type_factory__o_u_s_c_e_c_c_NewSSHKeyModalViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<NewSSHKeyModalViewImpl> { }
  public abstract class Type_factory__o_u_s_c_e_c_e_SSHKeysEditorEmptyStateDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeysEditorEmptyStateDisplayerViewImpl> { }
  public abstract class JsType_factory__o_u_w_m_PanelDefinition__quals__Universal extends Factory<PanelDefinition> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_c_c_k_s_AppendNoneTaskShortcut__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN extends Factory<AppendNoneTaskShortcut> { }
  public abstract class Type_factory__o_j_e_s_c_l_c_BasicUserCacheImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BasicUserCacheImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_k_KeyboardControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<KeyboardControlImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_g_p_t_t_TreeWalkTraverseProcessorImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<TreeWalkTraverseProcessorImpl> { }
  public abstract class Type_factory__o_u_e_e_c_c_f_CopyPopUpViewProducer__quals__j_e_i_Any_j_e_i_Default extends Factory<CopyPopUpViewProducer> { }
  public abstract class Type_factory__o_u_e_p_c_w_m_MediaLibraryWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<MediaLibraryWidget> { }
  public abstract class Type_factory__o_u_c_e_EditorJSExporter__quals__j_e_i_Any_j_e_i_Default extends Factory<EditorJSExporter> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_d_a_b_ClientBindableMorphAdapter__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientBindableMorphAdapter> { }
  public abstract class ExtensionProvided_factory__o_k_w_c_s_c_w_p_s_i_SessionContainer__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionContainer> { }
  public abstract class Type_factory__o_u_e_p_c_RuntimePluginsEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<RuntimePluginsEntryPoint> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Heading__quals__j_e_i_Any_j_e_i_Default_j_i_Named_1 extends Factory<Heading> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_e_t_TreeExplorerView__quals__j_e_i_Any_j_e_i_Default extends Factory<TreeExplorerView> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_p_AuthoringPerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<AuthoringPerspective> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_t_c_ExportToSvgToolbarCommand__quals__j_e_i_Any_j_e_i_Default extends Factory<ExportToSvgToolbarCommand> { }
  public abstract class JsType_factory__o_u_c_m_LockTarget_TitleProvider__quals__Universal extends Factory<TitleProvider> { }
  public abstract class Type_factory__o_u_e_w_c_c_t_SimpleTable__quals__j_e_i_Any_j_e_i_Default extends Factory<SimpleTable> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_s_l_EnumListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<EnumListBoxFieldRenderer> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_s_SliderFormGroup__quals__j_e_i_Any_j_e_i_Default extends Factory<SliderFormGroup> { }
  public abstract class ExtensionProvided_factory__c_g_g_u_c_u_Label__quals__j_e_i_Any_j_e_i_Default extends Factory<com.google.gwt.user.client.ui.Label> { }
  public abstract class Type_factory__o_j_e_e_c_c_WindowEventObservers__quals__j_e_i_Any_j_e_i_Default extends Factory<WindowEventObservers> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_a_ActivityDataIOEditorViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityDataIOEditorViewImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_d_FloatEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<FloatEditableColumnGenerator> { }
  public abstract class Type_factory__o_j_e_s_c_l_n_SecurityContextHoldingSingleton__quals__j_e_i_Any_j_e_i_Default extends Factory<SecurityContextHoldingSingleton> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_a_DefaultTextPropertyProviderImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultTextPropertyProviderImpl> { }
  public abstract class Type_factory__o_u_e_w_c_c_c_BusyIndicatorView__quals__j_e_i_Any_j_e_i_Default extends Factory<BusyIndicatorView> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutEditorPropertiesView__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutEditorPropertiesView> { }
  public abstract class Type_factory__o_u_e_w_c_c_m_MenuItemDividerView__quals__j_e_i_Any_o_u_e_w_c_c_m_MenuItemDivider extends Factory<MenuItemDividerView> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_u_UserAssignedRolesExplorer__quals__j_e_i_Any_j_e_i_Default extends Factory<UserAssignedRolesExplorer> { }
  public abstract class Type_factory__o_k_w_c_f_c_c_c_f_m_ModalFormDisplayer__quals__j_e_i_Any_j_e_i_Default extends Factory<ModalFormDisplayer> { }
  public abstract class Type_factory__o_k_w_c_f_c_c_c_f_e_EmbeddedFormDisplayer__quals__j_e_i_Any_j_e_i_Default extends Factory<EmbeddedFormDisplayer> { }
  public abstract class Type_factory__o_j_e_u_n_c_l_NavigationPanelProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<NavigationPanelProvider> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_p_StunnerPreferencesEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerPreferencesEntryPoint> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_p_ConfirmBox__quals__j_e_i_Any_j_e_i_Default extends Factory<ConfirmBox> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_DiagramPresenterScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<DiagramPresenterScreenActivity> { }
  public abstract class JsType_factory__o_j_e_i_c_JsArray__quals__Universal extends Factory<JsArray> { }
  public abstract class Type_factory__o_u_e_c_p_ExperimentalFeaturesPerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<ExperimentalFeaturesPerspective> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_r_CaseManagementSVGViewFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementSVGViewFactoryImpl> { }
  public abstract class JsType_factory__o_u_w_m_CompassPosition__quals__Universal extends Factory<CompassPosition> { }
  public abstract class Type_factory__o_u_e_c_p_ExperimentalFeaturesPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<ExperimentalFeaturesPerspectiveActivity> { }
  public abstract class Type_factory__o_u_c_m_ActivityManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ActivityManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_EditableColumnGeneratorManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<EditableColumnGeneratorManagerImpl> { }
  public abstract class Type_factory__o_u_e_e_c_c_h_VersionMenuDropDownButton__quals__j_e_i_Any_j_e_i_Default extends Factory<VersionMenuDropDownButton> { }
  public abstract class Type_factory__o_u_e_s_m_c_e_r_RoleEditorDriver__quals__j_e_i_Any_j_e_i_Default extends Factory<RoleEditorDriver> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionSwitchView__quals__j_e_i_Any_j_e_i_Default extends Factory<PermissionSwitchView> { }
  public abstract class JsType_factory__o_u_w_m_m_MenuVisitor__quals__Universal extends Factory<MenuVisitor> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_c_r_RolesListItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<RolesListItemWidgetViewImpl> { }
  public abstract class Type_factory__o_u_c_w_p_i_TemplatedWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<TemplatedWorkbenchPanelPresenter> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_DynamicFormRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<DynamicFormRenderer> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_s_ScreenPanelViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ScreenPanelViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_u_SvgDataUriGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<SvgDataUriGenerator> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_SliderFieldRenderer__quals__j_e_i_Any_j_e_i_Default extends Factory<SliderFieldRenderer> { }
  public abstract class Type_factory__o_u_p_c_s_PreferenceScopeResolutionStrategyIOCProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<PreferenceScopeResolutionStrategyIOCProvider> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_BooleanEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default extends Factory<BooleanEditableColumnGenerator> { }
  public abstract class Type_factory__o_u_c_e_PerspectiveJSExporter__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveJSExporter> { }
  public abstract class Type_factory__o_j_e_i_c_l_i_StateChangeImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<StateChangeImpl> { }
  public abstract class Provider_factory__o_j_e_b_c_a_m_RequestDispatcher__quals__j_e_i_Any_j_e_i_Default extends Factory<RequestDispatcher> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_c_CaseManagementContainmentAcceptorControlImpl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementContainmentAcceptorControlImpl> { }
  public abstract class Type_factory__o_u_s_c_SSHClientEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHClientEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_StunnerWidgetsEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<StunnerWidgetsEntryPoint> { }
  public abstract class Type_factory__o_k_w_c_s_b_f_BPMNDiagramFactoryImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDiagramFactoryImpl> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_KebabWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<KebabWidget> { }
  public abstract class Type_factory__o_k_w_c_s_c_l_c_c_ZoomWheelControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ZoomWheelControlImpl> { }
  public abstract class Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetPropertyAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<BPMNDefinitionSetPropertyAdapterImpl> { }
  public abstract class Type_factory__o_u_c_v_p_w_ErrorPopup__quals__j_e_i_Any_j_e_i_Default extends Factory<ErrorPopup> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeListFieldEditorWidget__quals__j_e_i_Any_j_e_i_Default extends Factory<ScriptTypeListFieldEditorWidget> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_l_e_p_EditHTMLPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<EditHTMLPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_c_a_TextEditorSingleLineBoxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_c_a_SingleLineTextEditorBox extends Factory<TextEditorSingleLineBoxView> { }
  public abstract class Type_factory__o_j_e_u_c_l_s_TranslationServiceProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<TranslationServiceProvider> { }
  public abstract class ExtensionProvided_factory__o_u_e_w_c_c_a_AceEditor__quals__j_e_i_Any_j_e_i_Default extends Factory<AceEditor> { }
  public abstract class Type_factory__o_u_p_c_s_ClientUsernameProvider__quals__j_e_i_Any_j_e_i_Default extends Factory<ClientUsernameProvider> { }
  public abstract class Type_factory__o_u_c_e_VFSJSExporter__quals__j_e_i_Any_j_e_i_Default extends Factory<VFSJSExporter> { }
  public abstract class Type_factory__o_u_e_w_c_c_d_PerspectiveDropDown__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveDropDown> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_c_b_i_EdgeBuilderControlImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<EdgeBuilderControlImpl> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_s_WelcomeScreen__quals__j_e_i_Any_j_e_i_Default extends Factory<WelcomeScreen> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_u_RedoCommandHandler__quals__j_e_i_Any_j_e_i_Default extends Factory<RedoCommandHandler> { }
  public abstract class Type_factory__o_u_c_w_p_i_AdaptiveWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<AdaptiveWorkbenchPanelPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_CaseManagementCanvasPresenter__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementCanvasPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_m_d_MenuDevCommandsBuilder__quals__j_e_i_Any_j_e_i_Default extends Factory<MenuDevCommandsBuilder> { }
  public abstract class Type_factory__o_k_w_c_s_b_c_f_f_v_VariablesEditorWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<VariablesEditorWidgetViewImpl> { }
  public abstract class Type_factory__o_u_e_s_m_c_s_e_GroupEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<GroupEditorScreenActivity> { }
  public abstract class Type_factory__o_j_e_s_c_l_c_DefaultBusSecurityErrorCallback__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultBusSecurityErrorCallback> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionExceptionSwitch__quals__j_e_i_Any_j_e_i_Default extends Factory<PermissionExceptionSwitch> { }
  public abstract class Type_factory__o_u_e_p_c_c_PluginConfigService__quals__j_e_i_Any_j_e_i_Default extends Factory<PluginConfigService> { }
  public abstract class Type_factory__o_k_w_c_f_d_c_r_f_i_c_CheckBoxFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CheckBoxFormGroupViewImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_p_s_i_SessionViewerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<SessionViewerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetRuleAdapterImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementDefinitionSetRuleAdapterImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_d_c_CloneManagerImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<CloneManagerImpl> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_RequestCommandManager__quals__j_e_i_Any_o_k_w_c_s_c_c_c_Request extends Factory<RequestCommandManager> { }
  public abstract class JsType_factory__o_u_c_w_t_ClientResourceType__quals__Universal extends Factory<ClientResourceType> { }
  public abstract class Type_factory__o_k_w_c_s_c_w_v_LoadingBoxView__quals__j_e_i_Any_j_e_i_Default extends Factory<org.kie.workbench.common.stunner.client.widgets.views.LoadingBoxView> { }
  public abstract class Type_factory__o_k_w_c_s_c_u_DefinitionUtils__quals__j_e_i_Any_j_e_i_Default extends Factory<DefinitionUtils> { }
  public abstract class Type_factory__o_k_w_c_s_s_c_p_HomePerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<HomePerspectiveActivity> { }
  public abstract class JsType_factory__o_u_w_m_ActivityResourceType__quals__Universal extends Factory<ActivityResourceType> { }
  public abstract class Type_factory__o_u_c_w_p_u_p_UnanchoredWorkbenchPartPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<UnanchoredWorkbenchPartPresenter> { }
  public abstract class Type_factory__o_g_c_s_p_c_Decision__quals__j_e_i_Any_j_e_i_Default extends Factory<Decision> { }
  public abstract class Type_factory__o_u_c_v_p_m_m_c_GroupContextMenuItemView__quals__j_e_i_Any_j_e_i_Default extends Factory<GroupContextMenuItemView> { }
  public abstract class Producer_factory__o_j_e_i_c_c_SyncBeanManager__quals__j_e_i_Any_j_e_i_Default extends Factory<SyncBeanManager> { }
  public abstract class Type_factory__o_u_e_w_c_c_s_i_IFrameScreenPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<IFrameScreenPresenter> { }
  public abstract class Type_factory__o_u_e_l_e_c_c_r_Row__quals__j_e_i_Any_j_e_i_Default extends Factory<Row> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_p_DefaultPreferencesRegistry__quals__j_e_i_Any_j_e_i_Default extends Factory<DefaultPreferencesRegistry> { }
  public abstract class Type_factory__o_k_w_c_f_c_c_c_f_m_ModalFormDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<ModalFormDisplayerViewImpl> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutEditorPropertiesPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutEditorPropertiesPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_t_c_CaseManagementVisitGraphToolbarCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor extends Factory<CaseManagementVisitGraphToolbarCommand> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_Button__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<org.jboss.errai.common.client.dom.Button> { }
  public abstract class Type_factory__o_u_s_c_e_SSHKeysEditorPerspective__quals__j_e_i_Any_j_e_i_Default extends Factory<SSHKeysEditorPerspective> { }
  public abstract class Type_factory__o_u_e_l_e_c_w_LayoutElementPropertiesView__quals__j_e_i_Any_j_e_i_Default extends Factory<LayoutElementPropertiesView> { }
  public abstract class Type_factory__o_u_e_w_c_c_w_WizardViewImpl__quals__j_e_i_Any_j_e_i_Default extends Factory<WizardViewImpl> { }
  public abstract class Type_factory__o_u_c_e_ResourceTypeDefinitionJSExporter__quals__j_e_i_Any_j_e_i_Default extends Factory<ResourceTypeDefinitionJSExporter> { }
  public abstract class ExtensionProvided_factory__o_u_c_w_w_p_StaticFocusedResizePanel__quals__j_e_i_Any_j_e_i_Default extends Factory<StaticFocusedResizePanel> { }
  public abstract class Type_factory__o_u_e_s_m_c_ResourceBundleEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<ResourceBundleEntryPoint> { }
  public abstract class Type_factory__o_u_c_w_p_n_p_WorkbenchPartView__quals__j_e_i_Any_j_e_i_Default extends Factory<WorkbenchPartView> { }
  public abstract class Type_factory__o_k_w_c_s_c_CaseManagementEntryPoint__quals__j_e_i_Any_j_e_i_Default extends Factory<CaseManagementEntryPoint> { }
  public abstract class Type_factory__o_u_c_w_w_m_m_c_ChildContextMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default extends Factory<ChildContextMenuItemPresenter> { }
  public abstract class Type_factory__o_k_w_c_s_c_c_c_u_CanvasFileExport__quals__j_e_i_Any_j_e_i_Default extends Factory<CanvasFileExport> { }
  public abstract class Type_factory__o_u_e_p_c_p_e_l_e_PerspectiveEditorSettings__quals__j_e_i_Any_j_e_i_Default extends Factory<PerspectiveEditorSettings> { }
  public abstract class Provider_factory__o_j_e_u_c_l_s_TranslationService__quals__j_e_i_Any_j_e_i_Default extends Factory<TranslationService> { }
  public abstract class Type_factory__o_u_c_v_p_m_m_m_IconMenuItemView__quals__j_e_i_Any_j_e_i_Default extends Factory<IconMenuItemView> { }
  public abstract class ExtensionProvided_factory__o_j_e_c_c_d_RadioInput__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<RadioInput> { }
  public abstract class Type_factory__o_u_c_w_d_UberfireDocksContainer__quals__j_e_i_Any_j_e_i_Default extends Factory<UberfireDocksContainer> { }
  public abstract class Type_factory__o_u_e_s_m_c_w_m_e_a_ACLViewer__quals__j_e_i_Any_j_e_i_Default extends Factory<ACLViewer> { }
  public abstract class Type_factory__o_u_c_w_p_i_LayoutPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named extends Factory<LayoutPanelView> { }
  private static void addLookups_0() {
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.core.client.session.event.SessionDestroyedEvent", "java.lang.Object");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.core.client.session.event.SessionDestroyedEvent", "org.uberfire.workbench.events.UberFireEvent");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.core.client.session.event.OnSessionErrorEvent", "org.kie.workbench.common.stunner.core.client.session.event.AbstractSessionEvent");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.client.widgets.event.SessionFocusedEvent", "java.lang.Object");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.client.widgets.event.SessionFocusedEvent", "org.uberfire.workbench.events.UberFireEvent");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.forms.client.event.FormPropertiesOpened", "java.lang.Object");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.forms.client.event.FormPropertiesOpened", "org.uberfire.workbench.events.UberFireEvent");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.client.widgets.event.LoadDiagramEvent", "java.lang.Object");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.client.widgets.event.LoadDiagramEvent", "org.uberfire.workbench.events.UberFireEvent");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.client.widgets.event.SessionDiagramOpenedEvent", "java.lang.Object");
    CDIEventTypeLookup.get().addLookup("org.kie.workbench.common.stunner.client.widgets.event.SessionDiagramOpenedEvent", "org.uberfire.workbench.events.UberFireEvent");
  }

  private void registerFactories0(final Context org_jboss_errai_ioc_client_container_ApplicationScopedContext_context, final Context org_jboss_errai_ioc_client_container_DependentScopeContext_context) {
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VFSLockServiceProxyBackendImpl>) GWT.create(Type_factory__o_u_c_VFSLockServiceProxyBackendImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ServiceTaskFactory>) GWT.create(Type_factory__o_k_w_c_s_b_w_ServiceTaskFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BigIntegerEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_BigIntegerEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupsExplorer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_GroupsExplorer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNThumbnailProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_BPMNThumbnailProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NavigationPanel>) GWT.create(Provider_factory__o_j_e_u_n_c_l_NavigationPanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PanControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_PanControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ModuleFormGenerationResourcesProvider>) GWT.create(Type_factory__o_k_w_c_s_c_d_f_p_ModuleFormGenerationResourcesProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ChangePasswordView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_ChangePasswordView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComponentColumnView>) GWT.create(Type_factory__o_u_e_l_e_c_c_c_ComponentColumnView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupEditorScreen>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_GroupEditorScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerRadioGroupFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_r_IntegerRadioGroupFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserAttributesEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_UserAttributesEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Caller>) GWT.create(ContextualProvider_factory__o_j_e_c_c_a_Caller__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClientMessageBus>) GWT.create(Producer_factory__o_j_e_b_c_a_ClientMessageBus__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DownloadMenuItem>) GWT.create(Type_factory__o_u_e_e_c_c_m_DownloadMenuItem__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ToggleSwitch>) GWT.create(ExtensionProvided_factory__o_g_e_t_c_u_ToggleSwitch__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayerCrossingCount>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_l_s_s_LayerCrossingCount__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToPngSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToPngSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TransitionAnchorFactory>) GWT.create(ContextualProvider_factory__o_j_e_u_n_c_l_TransitionAnchorFactory__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EmbeddedFormDisplayerViewImpl>) GWT.create(Type_factory__o_k_w_c_f_c_c_c_f_e_EmbeddedFormDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormHandlerGeneratorManager>) GWT.create(Type_factory__o_k_w_c_f_d_c_i_FormHandlerGeneratorManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ResourceAction>) GWT.create(JsType_factory__o_u_s_ResourceAction__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VFSServiceProxyClientImpl>) GWT.create(Type_factory__o_u_c_w_VFSServiceProxyClientImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LayoutEditorEntryPoint>) GWT.create(Type_factory__o_u_e_l_e_c_LayoutEditorEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Workbench>) GWT.create(Type_factory__o_u_c_w_Workbench__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RoleEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_r_RoleEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ActivityDataIOEditorWidget>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_ActivityDataIOEditorWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ResourceActionRef>) GWT.create(JsType_factory__o_u_s_a_ResourceActionRef__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiListWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_MultiListWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Optimization>) GWT.create(Type_factory__o_g_c_s_p_c_Optimization__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DeleteNodeAction>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_DeleteNodeAction__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ResourceType>) GWT.create(JsType_factory__o_u_s_ResourceType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SugiyamaLayoutService>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_l_s_SugiyamaLayoutService__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoPictureGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_g_LienzoPictureGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EntitiesListView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_l_EntitiesListView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DoubleSliderFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_s_DoubleSliderFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ColorPickerFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_f_s_f_ColorPickerFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoMultipleSelectionControl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_LienzoMultipleSelectionControl__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_s_MultipleSelection.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignedNodesOptionsType>) GWT.create(JsType_factory__e_d_HTMLSlotElement_AssignedNodesOptionsType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ActivityBeansInfo>) GWT.create(Type_factory__o_u_c_m_ActivityBeansInfo__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuItemWithIconView>) GWT.create(Type_factory__o_u_e_w_c_c_m_MenuItemWithIconView__quals__j_e_i_Any_o_u_e_w_c_c_m_MenuItemWithIcon.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DynamicFormRendererViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_DynamicFormRendererViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ACLEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_ACLEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UrlsFieldType>) GWT.create(JsType_factory__e_d_RTCConfigurationInterface_UrlsFieldType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_g_GroupEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<UberfireExperimentalEntryPoint>) GWT.create(Type_factory__o_u_e_c_UberfireExperimentalEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UnanchoredWorkbenchPartView>) GWT.create(Type_factory__o_u_c_w_p_u_p_UnanchoredWorkbenchPartView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultSecurityExceptionHandler>) GWT.create(Type_factory__o_j_e_s_c_l_h_DefaultSecurityExceptionHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<MapModelBindingHelper>) GWT.create(Type_factory__o_k_w_c_f_d_c_h_MapModelBindingHelper__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DisabledFeatureComponentViewImpl>) GWT.create(Type_factory__o_u_e_c_d_c_DisabledFeatureComponentViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PreferenceScopeTypes>) GWT.create(Producer_factory__o_u_p_s_PreferenceScopeTypes__quals__j_e_i_Any_o_u_a_Customizable.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutDragComponentGroupView>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutDragComponentGroupView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserEditorScreen>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_UserEditorScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultDiagramViewer>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_d_i_DefaultDiagramViewer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextBox>) GWT.create(ExtensionProvided_factory__o_g_c_u_TextBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Select>) GWT.create(Type_factory__o_u_c_v_p_w_Select__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ImageStripDOMGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_g_ImageStripDOMGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SVGShapeFactory>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_f_SVGShapeFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VisitGraphToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_VisitGraphToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PanelDefinitionImpl>) GWT.create(Type_factory__o_u_w_m_i_PanelDefinitionImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ColorPickerFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_f_c_f_c_ColorPickerFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FileExportsPreferencesBeanGeneratedImpl>) GWT.create(Type_factory__o_u_e_e_c_f_e_FileExportsPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientFactoryService>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_ClientFactoryService__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextAreaFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_t_p_TextAreaFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextAreaFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_TextAreaFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SessionCommandManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_SessionCommandManagerImpl__quals__j_e_i_Any_o_k_w_c_s_c_c_s_Session.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientRuleManager>) GWT.create(Type_factory__o_k_w_c_s_c_c_r_ClientRuleManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PopStateEventEventInitDictType>) GWT.create(JsType_factory__e_d_PopStateEvent_PopStateEventEventInitDictType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BS3IconTypeGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_g_BS3IconTypeGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ErrorPopupView>) GWT.create(Type_factory__o_u_c_v_p_w_ErrorPopupView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WelcomeScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_WelcomeScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiplePermissionNodeViewerView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_MultiplePermissionNodeViewerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TimerSettingsFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_TimerSettingsFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeysEditorViewImpl>) GWT.create(Type_factory__o_u_s_c_e_c_SSHKeysEditorViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldRequiredViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_l_r_FieldRequiredViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GenerateSelectedFormsSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_f_c_s_c_GenerateSelectedFormsSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ToolboxDomainLookups>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_ToolboxDomainLookups__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComboBoxFixedValuesWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_c_ComboBoxFixedValuesWidgetViewImpl__quals__j_e_i_Any_o_k_w_c_s_b_c_f_f_c_a_FixedValues.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ManagedEditorToolbar>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_i_ManagedEditorToolbar__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiScreenView>) GWT.create(Type_factory__o_u_c_v_p_m_MultiScreenView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EditorPlugInEditorPresenter>) GWT.create(Type_factory__o_u_e_p_c_e_EditorPlugInEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserGroupsExplorer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_UserGroupsExplorer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<org.kie.workbench.common.stunner.bpmn.definition.formBuilder.provider.ModuleFormGenerationResourcesProvider>) GWT.create(Type_factory__o_k_w_c_s_b_d_f_p_ModuleFormGenerationResourcesProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupEditorWorkflow>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_g_w_GroupEditorWorkflow__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SecurityManagementPerspective>) GWT.create(Type_factory__o_u_e_s_m_c_p_SecurityManagementPerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ColorPickerWidget>) GWT.create(Type_factory__o_k_w_c_s_f_c_f_c_ColorPickerWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<NodeDockingEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_NodeDockingEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuGroup>) GWT.create(JsType_factory__o_u_w_m_m_MenuGroup__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExperimentalFeatureEditorViewImpl>) GWT.create(Type_factory__o_u_e_c_e_g_f_ExperimentalFeatureEditorViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RuleExtensionMultiHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_e_RuleExtensionMultiHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ProcessCompensationRefProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_ProcessCompensationRefProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AdaptiveWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_AdaptiveWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<RuleLookupManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_r_RuleLookupManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Container>) GWT.create(ExtensionProvided_factory__o_g_c_u_Container__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleInputComponentViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_MultipleInputComponentViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchDragAndDropManager>) GWT.create(Type_factory__o_u_c_w_w_d_WorkbenchDragAndDropManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NotificationsObserver>) GWT.create(Type_factory__o_k_w_c_s_c_w_n_NotificationsObserver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LayoutEditorPropertiesScreen>) GWT.create(Type_factory__o_u_e_l_e_c_LayoutEditorPropertiesScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LogCommandHistoryDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_LogCommandHistoryDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<POMResourceTypeDefinition>) GWT.create(Type_factory__o_g_c_s_p_e_t_POMResourceTypeDefinition__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LockManagerImpl>) GWT.create(Type_factory__o_u_c_m_LockManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PriorityDropDown>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_PriorityDropDown__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNCreateNodeAction>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_BPMNCreateNodeAction__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ShapeDragProxyImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_d_ShapeDragProxyImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SelectComponent>) GWT.create(Type_factory__o_u_e_w_c_c_s_SelectComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveManagerImpl>) GWT.create(Type_factory__o_u_c_m_PerspectiveManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ErrorPopupPresenter>) GWT.create(Type_factory__o_u_c_w_w_c_ErrorPopupPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NoneCloneProcess>) GWT.create(Type_factory__o_k_w_c_s_c_d_c_NoneCloneProcess__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Event>) GWT.create(ContextualProvider_factory__j_e_e_Event__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CharacterRadioGroupFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_r_CharacterRadioGroupFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerBoxFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_i_p_IntegerBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormGeneratorDriver>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_FormGeneratorDriver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RedoSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_RedoSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextBoxFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_t_p_TextBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSetDefinitionSetProxyImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetDefinitionSetProxyImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkingSetManager>) GWT.create(Type_factory__o_g_c_s_w_c_WorkingSetManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClientSecurityRoleInterceptor>) GWT.create(Type_factory__o_j_e_s_c_l_i_ClientSecurityRoleInterceptor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ProcessEscalationRefProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_ProcessEscalationRefProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseRoleSerializer>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_c_CaseRoleSerializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StandaloneEditorPerspective>) GWT.create(Type_factory__o_u_c_w_StandaloneEditorPerspective__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ManagedSession>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_i_ManagedSession__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PermissionWidgetFactory>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionWidgetFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CopyPopUpView>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_CopyPopUpView__quals__j_e_i_Any_o_u_a_FallbackImplementation.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<InitBallot>) GWT.create(ContextualProvider_factory__o_j_e_i_c_a_InitBallot__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CollapsedDefinitionPaletteItemWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_c_CollapsedDefinitionPaletteItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ViewTraverseProcessorImpl>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_t_c_ViewTraverseProcessorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DiagramClientErrorHandler>) GWT.create(Type_factory__o_k_w_c_s_c_c_e_DiagramClientErrorHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ShortEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_ShortEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserCreationWorkflow>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_w_UserCreationWorkflow__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Others>) GWT.create(Type_factory__o_u_w_c_Others__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Input>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Input__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConnectorDragProxyImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_d_ConnectorDragProxyImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionDiagramPreviewScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionDiagramPreviewScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FilePropertiesType>) GWT.create(JsType_factory__e_d_File_FilePropertiesType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Elemental2DomUtil>) GWT.create(Type_factory__o_j_e_c_c_d_e_Elemental2DomUtil__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VersionRecordManager>) GWT.create(Type_factory__o_u_e_e_c_c_h_VersionRecordManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StaticWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_StaticWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DateTimePickerPresenter>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_DateTimePickerPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Clipboard>) GWT.create(Type_factory__o_u_c_u_Clipboard__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientUserSystemManager>) GWT.create(Type_factory__o_u_e_s_m_c_ClientUserSystemManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeEditorFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_AssigneeEditorFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RolesEditorFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_c_r_RolesEditorFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Button>) GWT.create(Type_factory__o_u_c_v_p_w_Button__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<TableEntryPoint>) GWT.create(Type_factory__o_u_e_w_t_c_TableEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AppsHomePresenterActivity>) GWT.create(Type_factory__o_u_e_a_c_h_AppsHomePresenterActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultFileEditorPresenter>) GWT.create(Type_factory__o_u_e_w_c_c_e_d_DefaultFileEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleSelectorInputViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_s_i_MultipleSelectorInputViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionPropertiesScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionPropertiesScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TabPanelWithDropdowns>) GWT.create(Type_factory__o_u_c_v_p_t_TabPanelWithDropdowns__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RowView>) GWT.create(Type_factory__o_u_e_l_e_c_c_r_RowView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ShapeGlyphDragHandlerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_g_ShapeGlyphDragHandlerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ToolBarIcon>) GWT.create(JsType_factory__o_u_w_m_t_ToolBarIcon__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleSubFormFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_r_m_p_MultipleSubFormFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveEditorGenerator>) GWT.create(Type_factory__o_u_e_p_c_p_e_g_PerspectiveEditorGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LockManager>) GWT.create(JsType_factory__o_u_c_m_LockManager__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<UberfireJSAPIExporter>) GWT.create(Type_factory__o_u_c_e_UberfireJSAPIExporter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WiresCanvasPresenter>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_w_WiresCanvasPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BatchCaller>) GWT.create(Provider_factory__o_j_e_c_c_a_BatchCaller__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SimpleWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_SimpleWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PictureWidget>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_p_w_PictureWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionDiagramEditorScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionDiagramEditorScreen__quals__j_e_i_Any_o_k_w_c_s_c_c_a_DiagramEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserAssignedGroupsExplorer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_UserAssignedGroupsExplorer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RegisterOptionsType>) GWT.create(JsType_factory__e_d_Document_RegisterOptionsType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariablesProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_VariablesProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNCommonActionsToolboxFactory>) GWT.create(Type_factory__o_k_w_c_s_b_c_c_c_BPMNCommonActionsToolboxFactory__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientSelectorDataProviderManager>) GWT.create(Type_factory__o_k_w_c_f_d_c_c_ClientSelectorDataProviderManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasHandlerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_CanvasHandlerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CutSelectionSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_CutSelectionSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariablesEditorFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_v_VariablesEditorFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DeleteSelectionSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_DeleteSelectionSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariableNameTextBox>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_w_VariableNameTextBox__quals__j_e_i_Any_o_k_w_c_s_b_c_StunnerSpecific.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormLabel>) GWT.create(Type_factory__o_u_c_v_p_w_FormLabel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextEditorWorkbenchEditor>) GWT.create(Type_factory__o_u_e_w_c_c_e_t_TextEditorWorkbenchEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LayoutComponentPaletteScreen>) GWT.create(Type_factory__o_u_e_l_e_c_LayoutComponentPaletteScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PopupUtil>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_PopupUtil__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultPaletteFactory>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_DefaultPaletteFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TransitionAnchor>) GWT.create(ContextualProvider_factory__o_j_e_u_n_c_l_TransitionAnchor__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementShapeSet>) GWT.create(Type_factory__o_k_w_c_s_c_c_CaseManagementShapeSet__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldRendererManagerImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_FieldRendererManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNIconsStrip>) GWT.create(Type_factory__o_k_w_c_s_b_c_r_BPMNIconsStrip__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AdHocOrderingProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_AdHocOrderingProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecurityExplorerScreen>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_SecurityExplorerScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RolesEditorFieldInitializer>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_c_RolesEditorFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SwitchGridSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_SwitchGridSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLLayoutDragComponent>) GWT.create(Type_factory__o_u_e_p_c_p_e_l_e_HTMLLayoutDragComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DOMGlyphRenderers>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_g_DOMGlyphRenderers__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchBackendEntryPoint>) GWT.create(Type_factory__o_u_c_WorkbenchBackendEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.uberfire.ext.layout.editor.client.components.container.Container>) GWT.create(Type_factory__o_u_e_l_e_c_c_c_Container__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeEditorWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeEditorWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.selectors.listBox.definition.formBuilder.provider.ModuleFormGenerationResourcesProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_s_l_d_f_p_ModuleFormGenerationResourcesProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNPaletteDefinitionBuilder>) GWT.create(Type_factory__o_k_w_c_s_b_c_c_p_BPMNPaletteDefinitionBuilder__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultPermissionManager>) GWT.create(Type_factory__o_u_s_i_a_DefaultPermissionManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RolesEditorFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_c_RolesEditorFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VFSServiceProxyBackendImpl>) GWT.create(Type_factory__o_u_c_VFSServiceProxyBackendImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DiagramNavigatorItemImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_e_n_d_DiagramNavigatorItemImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientBindableAdapterFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_d_a_b_ClientBindableAdapterFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BooleanField>) GWT.create(Type_factory__o_u_e_p_e_c_f_BooleanField__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Navigation>) GWT.create(Type_factory__o_j_e_u_n_c_l_Navigation__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IconMenuItemPresenter>) GWT.create(Type_factory__o_u_c_w_w_m_m_m_IconMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuPosition>) GWT.create(JsType_factory__o_u_w_m_m_MenuPosition__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LogDefinitionDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_LogDefinitionDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementElementBuilderControl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_b_CaseManagementElementBuilderControl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor_o_k_w_c_s_c_c_c_c_b_i_Element.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ActivityNotFoundPresenterActivity>) GWT.create(Type_factory__o_u_e_w_c_c_w_w_p_a_n_ActivityNotFoundPresenterActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchMegaMenuView>) GWT.create(Type_factory__o_u_c_v_p_m_m_WorkbenchMegaMenuView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementObserverBuilderControl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_b_CaseManagementObserverBuilderControl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor_o_k_w_c_s_c_c_c_c_b_i_Observer.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DiagramsNavigatorImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_e_n_d_DiagramsNavigatorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiScreenMenuBuilder>) GWT.create(Type_factory__o_u_c_v_p_m_MultiScreenMenuBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementDeleteSelectionSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_CaseManagementDeleteSelectionSessionCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SelectOptionView>) GWT.create(Type_factory__o_u_e_w_c_c_s_SelectOptionView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EmptyDropRowView>) GWT.create(Type_factory__o_u_e_l_e_c_c_r_EmptyDropRowView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CharacterListBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_l_CharacterListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeLiveSearchEntryCreationEditor>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeLiveSearchEntryCreationEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SliderFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_s_p_SliderFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PatternFlyEntryPoint>) GWT.create(Type_factory__o_u_c_v_p_PatternFlyEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchMenuStandardNavBarView>) GWT.create(Type_factory__o_u_c_v_p_m_WorkbenchMenuStandardNavBarView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionTreeExplorerScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionTreeExplorerScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DocksEntryPoint>) GWT.create(Type_factory__o_u_c_DocksEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SecurityContextImpl>) GWT.create(Type_factory__o_j_e_s_c_l_c_SecurityContextImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CollapsedDefinitionPaletteItemWidget>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_c_CollapsedDefinitionPaletteItemWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DynamicMenuEditorView>) GWT.create(Type_factory__o_u_e_p_c_e_DynamicMenuEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeFormGroupViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_f_AssigneeFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExperimentalFeaturesEditorScreenActivity>) GWT.create(Type_factory__o_u_e_c_e_ExperimentalFeaturesEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldRequired>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_l_r_FieldRequired__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BatchCallerProvider>) GWT.create(Type_factory__o_j_e_i_s_b_c_BatchCallerProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutGenerator>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_l_LayoutGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NewUserAttributeEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_NewUserAttributeEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignmentsEditorFieldInitializer>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_AssignmentsEditorFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ImageFileExport>) GWT.create(Producer_factory__o_u_e_e_c_c_f_e_ImageFileExport__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<TransitionToRoleProvider>) GWT.create(Type_factory__o_j_e_u_n_c_l_TransitionToRoleProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientFieldManagerImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_s_ClientFieldManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ShapeSetsMenuItemsBuilder>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_ShapeSetsMenuItemsBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AppendNoneEndEventShortcut>) GWT.create(Type_factory__o_k_w_c_s_b_c_c_c_k_s_AppendNoneEndEventShortcut__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Set>) GWT.create(JsType_factory__j_u_Set__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PluginsInfo>) GWT.create(Type_factory__o_u_e_p_c_i_PluginsInfo__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionDiagramEditorScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionDiagramEditorScreenActivity__quals__j_e_i_Any_j_i_Named_o_k_w_c_s_c_c_a_DiagramEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLDocument>) GWT.create(Producer_factory__e_d_HTMLDocument__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ScreenEventPublisher>) GWT.create(Type_factory__o_k_w_c_s_c_c_e_s_ScreenEventPublisher__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementPaletteDefinitionBuilder>) GWT.create(Type_factory__o_k_w_c_s_c_c_p_CaseManagementPaletteDefinitionBuilder__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConfigErrorDisplayer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_c_ConfigErrorDisplayer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementControlFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_w_CaseManagementControlFactory__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuItemHeaderView>) GWT.create(Type_factory__o_u_e_w_c_c_m_MenuItemHeaderView__quals__j_e_i_Any_o_u_e_w_c_c_m_MenuItemHeader.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WizardPageTitle>) GWT.create(Type_factory__o_u_e_w_c_c_w_WizardPageTitle__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultBeanFactory>) GWT.create(Type_factory__o_u_c_w_DefaultBeanFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SaveButton>) GWT.create(Type_factory__o_u_e_e_c_c_h_SaveButton__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupCreationWorkflow>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_g_w_GroupCreationWorkflow__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNShapeSet>) GWT.create(Type_factory__o_k_w_c_s_b_c_BPMNShapeSet__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PermissionExceptionSwitchView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionExceptionSwitchView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DisabledFeatureActivity>) GWT.create(Type_factory__o_u_e_c_d_s_DisabledFeatureActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClientDiagramServiceImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_ClientDiagramServiceImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LocationControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_LocationControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchServicesProxyBackendImpl>) GWT.create(Type_factory__o_u_c_WorkbenchServicesProxyBackendImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldSetFormGroupViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_n_f_FieldSetFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PasteToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_PasteToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeysEditorEmptyStateDisplayer>) GWT.create(Type_factory__o_u_s_c_e_c_e_SSHKeysEditorEmptyStateDisplayer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CollapsibleFormGroupViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_n_c_CollapsibleFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignedEntitiesExplorerView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_AssignedEntitiesExplorerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CommonActionsToolboxView>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_t_a_CommonActionsToolboxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_CommonActionsToolbox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoPanel>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_v_LienzoPanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StringColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_m_c_i_StringColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerWiresHandlerFactory>) GWT.create(Type_factory__o_k_w_c_s_c_l_w_StunnerWiresHandlerFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DiagramsNavigatorScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_DiagramsNavigatorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PartManager>) GWT.create(Type_factory__o_u_c_w_p_s_PartManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ActivityNotFoundPresenter>) GWT.create(Type_factory__o_u_e_w_c_c_w_w_p_a_n_ActivityNotFoundPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AccessImpl>) GWT.create(Type_factory__o_j_e_i_c_l_i_AccessImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Undefined>) GWT.create(Type_factory__o_u_w_c_Undefined__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PluginExplorerScreenActivity>) GWT.create(Type_factory__o_u_e_p_c_e_PluginExplorerScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionPresenterView>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_s_i_SessionPresenterView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ToolbarItem>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_i_ToolbarItem__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchToolBarView>) GWT.create(Type_factory__o_u_c_v_p_t_WorkbenchToolBarView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientDiagramValidator>) GWT.create(Type_factory__o_k_w_c_s_c_c_v_ClientDiagramValidator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TransitionTo>) GWT.create(ContextualProvider_factory__o_j_e_u_n_c_l_TransitionTo__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_TextBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CreateNodeAction>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_CreateNodeAction__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_FlowActionsToolbox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PagedTable>) GWT.create(Type_factory__o_u_e_w_c_c_t_PagedTable__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariableNameTextBox>) GWT.create(ExtensionProvided_factory__o_k_w_c_s_b_c_f_w_VariableNameTextBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToPdfSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToPdfSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefinitionPaletteGroupWidget>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_c_g_DefinitionPaletteGroupWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultCanvasCommandFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_DefaultCanvasCommandFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetResourceType>) GWT.create(Type_factory__o_k_w_c_s_b_r_BPMNDefinitionSetResourceType__quals__j_e_i_Any_j_e_i_Default_o_u_w_a_VisibleAsset.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldRequiredEntryProcessor>) GWT.create(Type_factory__o_k_w_c_f_s_s_m_p_i_p_FieldRequiredEntryProcessor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CrudComponentViewImpl>) GWT.create(Type_factory__o_k_w_c_f_c_c_c_CrudComponentViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CDIClientFeatureDefRegistry>) GWT.create(Type_factory__o_u_e_c_s_i_CDIClientFeatureDefRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultDiagramEditor>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_d_i_DefaultDiagramEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetMorphDefinitionProvider>) GWT.create(Type_factory__o_k_w_c_s_b_d_m_BPMNDefinitionSetMorphDefinitionProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldPlaceHolderEntryProcessor>) GWT.create(Type_factory__o_k_w_c_f_f_s_m_m_e_FieldPlaceHolderEntryProcessor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ActivityNotFoundView>) GWT.create(Type_factory__o_u_c_v_p_n_ActivityNotFoundView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BindableTypeAhead>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_t_BindableTypeAhead__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HtmlEditor>) GWT.create(Type_factory__o_u_e_e_c_c_h_HtmlEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VFSLockServiceProxyClientImpl>) GWT.create(Type_factory__o_u_c_w_VFSLockServiceProxyClientImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RegisterElementOptionsType>) GWT.create(JsType_factory__e_d_Document_RegisterElementOptionsType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignmentListItemWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_AssignmentListItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SliderFormGroupViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_s_SliderFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CollapsibleFormGroup>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_n_c_CollapsibleFormGroup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<KebabMenuItem>) GWT.create(Type_factory__o_u_c_v_p_w_KebabMenuItem__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SplashScreenJSExporter>) GWT.create(Type_factory__o_u_c_e_SplashScreenJSExporter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeListFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_ScriptTypeListFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormGenerationToolboxAction>) GWT.create(Type_factory__o_k_w_c_s_f_c_c_t_FormGenerationToolboxAction__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Map>) GWT.create(JsType_factory__j_u_Map__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CheckBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_CheckBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LoadingBox>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_LoadingBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ToggleCommentView>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_c_ToggleCommentView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldHelpViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_l_h_FieldHelpViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SplashModalFooter>) GWT.create(Type_factory__o_u_c_v_p_s_SplashModalFooter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultFileEditorPresenterActivity>) GWT.create(Type_factory__o_u_e_w_c_c_e_d_DefaultFileEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PluginExplorerScreen>) GWT.create(Type_factory__o_u_e_p_c_e_PluginExplorerScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PartContextMenusView>) GWT.create(Type_factory__o_u_c_v_p_m_PartContextMenusView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldLayoutComponent>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_FieldLayoutComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientMetaDataEntryManager>) GWT.create(Type_factory__o_k_w_c_f_a_e_c_f_ClientMetaDataEntryManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ResizeTabPanel>) GWT.create(Type_factory__o_u_c_v_p_t_ResizeTabPanel__quals__j_e_i_Any_o_u_c_v_p_t_Resize.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UndoToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_UndoToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClearSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_ClearSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoImageStripGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_g_LienzoImageStripGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ListBoxFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_s_ListBoxFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldChangeHandlerManagerImpl>) GWT.create(Type_factory__o_k_w_c_f_p_e_h_i_FieldChangeHandlerManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionScreenViewImpl>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionScreenViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GenerateDiagramFormsSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_f_c_s_c_GenerateDiagramFormsSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PartDefinition>) GWT.create(JsType_factory__o_u_w_m_PartDefinition__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Paragraph>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Paragraph__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UnanchoredStaticWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_UnanchoredStaticWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuItemCommand>) GWT.create(JsType_factory__o_u_w_m_m_MenuItemCommand__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DynamicMenuResourceType>) GWT.create(Type_factory__o_u_e_p_c_t_DynamicMenuResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Anchor>) GWT.create(ExtensionProvided_factory__o_g_c_u_Anchor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HashChangeEventEventInitDictType>) GWT.create(JsType_factory__e_d_HashChangeEvent_HashChangeEventEventInitDictType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeFieldEditorPresenter>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeFieldEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UnorderedList>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_UnorderedList__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserEditorDriver>) GWT.create(Type_factory__o_u_e_s_m_c_e_u_UserEditorDriver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupsList>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_l_GroupsList__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<EditorTreeProvider>) GWT.create(Type_factory__o_u_c_a_EditorTreeProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DiagramsNavigatorScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_DiagramsNavigatorScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldReadOnlyEntryProcessor>) GWT.create(Type_factory__o_k_w_c_f_s_s_m_p_i_p_FieldReadOnlyEntryProcessor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNTextPreferences>) GWT.create(Type_factory__o_k_w_c_s_b_c_p_BPMNTextPreferences__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerListBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_l_IntegerListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RootPanel>) GWT.create(Provider_factory__c_g_g_u_c_u_RootPanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefinitionsCacheRegistry>) GWT.create(Producer_factory__o_k_w_c_s_c_r_i_DefinitionsCacheRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementLocationControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_CaseManagementLocationControlImpl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PreferenceScopeTypesProducer>) GWT.create(Type_factory__o_u_p_s_i_PreferenceScopeTypesProducer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveLayoutPluginResourceType>) GWT.create(Type_factory__o_u_e_p_c_t_PerspectiveLayoutPluginResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DateTimePickerCell>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_DateTimePickerCell__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StaticWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_StaticWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NodeDragProxyImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_d_NodeDragProxyImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<HTML5DndSeleniumSupport>) GWT.create(Type_factory__o_u_c_v_p_HTML5DndSeleniumSupport__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StunnerPreferencesBeanGeneratedImpl>) GWT.create(Type_factory__o_k_w_c_s_c_p_StunnerPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LiveSearchSelectorDropDownItemViewImpl>) GWT.create(Type_factory__o_u_e_w_c_c_d_i_LiveSearchSelectorDropDownItemViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiScreenWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_MultiScreenWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchMenuBarPresenter>) GWT.create(Producer_factory__o_u_c_w_w_m_WorkbenchMenuBarPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ToolBar>) GWT.create(JsType_factory__o_u_w_m_t_ToolBar__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HasRowsFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_HasRowsFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<KeyValueSerializer>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_KeyValueSerializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StunnerDiagramEditorPreferencesBeanGeneratedImpl>) GWT.create(Type_factory__o_k_w_c_s_c_p_StunnerDiagramEditorPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GAVPreferencesBeanGeneratedImpl>) GWT.create(Type_factory__o_g_c_s_p_p_GAVPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DecimalListBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_l_DecimalListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupEditorDriver>) GWT.create(Type_factory__o_u_e_s_m_c_e_g_GroupEditorDriver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeyEditor>) GWT.create(Type_factory__o_u_s_c_e_c_k_k_SSHKeyEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MapIndexBuilder>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_i_m_MapIndexBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SplashEditorView>) GWT.create(Type_factory__o_u_e_p_c_e_SplashEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<NodeContainmentEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_NodeContainmentEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchViewModeSwitcherView>) GWT.create(Type_factory__o_u_c_v_p_m_WorkbenchViewModeSwitcherView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveTreeProvider>) GWT.create(Type_factory__o_u_c_a_PerspectiveTreeProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PartContextMenusPresenter>) GWT.create(Type_factory__o_u_c_w_w_m_PartContextMenusPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SplashScreenMenuView>) GWT.create(Type_factory__o_u_c_v_p_m_SplashScreenMenuView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefinitionPaletteItemWidget>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_c_i_DefinitionPaletteItemWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DestructionImpl>) GWT.create(Type_factory__o_j_e_i_c_l_i_DestructionImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefinitionPaletteGroupWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_c_g_DefinitionPaletteGroupWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UberfireBreadcrumbsView>) GWT.create(Type_factory__o_u_e_w_c_c_b_UberfireBreadcrumbsView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HtmlEditorView>) GWT.create(Type_factory__o_u_e_e_c_c_h_HtmlEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CompassDropController>) GWT.create(Type_factory__o_u_c_w_w_d_CompassDropController__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeysEditor>) GWT.create(Type_factory__o_u_s_c_e_c_SSHKeysEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultModelValidator>) GWT.create(Type_factory__o_k_w_c_f_p_e_h_i_DefaultModelValidator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultToolbarView>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_i_DefaultToolbarView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ProcessMessageRefProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_ProcessMessageRefProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ACLEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_ACLEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ScreenPluginResourceType>) GWT.create(Type_factory__o_u_e_p_c_t_ScreenPluginResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BootstrapLayoutGenerator>) GWT.create(Type_factory__o_u_e_l_e_c_g_BootstrapLayoutGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ProcessErrorRefProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_ProcessErrorRefProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PluginNameValidator>) GWT.create(Type_factory__o_u_e_p_c_v_PluginNameValidator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ImageElementRendererViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_ImageElementRendererViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserGroupsExplorerView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_UserGroupsExplorerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UsersList>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_l_UsersList__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormGroup>) GWT.create(Type_factory__o_u_c_v_p_w_FormGroup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultDefinitionsCacheRegistry>) GWT.create(Type_factory__o_k_w_c_s_c_r_i_DefaultDefinitionsCacheRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultRestSecurityErrorCallback>) GWT.create(Type_factory__o_j_e_s_c_l_c_DefaultRestSecurityErrorCallback__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiTabWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_v_p_t_MultiTabWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementContainmentStateHolder>) GWT.create(Type_factory__o_k_w_c_s_c_c_w_CaseManagementContainmentStateHolder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ChildContextMenuItemView>) GWT.create(Type_factory__o_u_c_v_p_m_m_c_ChildContextMenuItemView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<HistoryTokenFactory>) GWT.create(Type_factory__o_j_e_u_n_c_l_HistoryTokenFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SubFormFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_r_s_p_SubFormFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecurityManagementPerspectiveActivity>) GWT.create(Type_factory__o_u_e_s_m_c_p_SecurityManagementPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClientRequiredRolesExtractorImpl>) GWT.create(Type_factory__o_j_e_s_c_l_r_ClientRequiredRolesExtractorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ManagedClientSessionCommands>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_ManagedClientSessionCommands__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExperimentalFeatureEditor>) GWT.create(Type_factory__o_u_e_c_e_g_f_ExperimentalFeatureEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToBpmnSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToBpmnSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecurityExplorerView>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_SecurityExplorerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CommonActionsToolboxFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_CommonActionsToolboxFactory__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_CommonActionsToolbox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RuleManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_r_RuleManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IceServersFieldType>) GWT.create(JsType_factory__e_d_RTCConfigurationInterface_IceServersFieldType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupUsersAssignment>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_g_GroupUsersAssignment__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<InitBallotProvider>) GWT.create(Type_factory__o_j_e_i_c_a_b_InitBallotProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClipboardControlCleaner>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_c_ClipboardControlCleaner__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientFormGenerationManager>) GWT.create(Type_factory__o_k_w_c_s_f_c_g_ClientFormGenerationManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<MenuItemFactory>) GWT.create(Type_factory__o_u_e_w_c_c_m_MenuItemFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DateEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_DateEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BooleanColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_m_c_i_BooleanColumnGenerator__quals__j_e_i_Any.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RadioGroupFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_s_r_p_RadioGroupFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TimerSettingsFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_t_TimerSettingsFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LocalClipboardControl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_c_LocalClipboardControl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RestorePopUpView>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_RestorePopUpView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Command>) GWT.create(JsType_factory__o_u_m_Command__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RuntimePluginsServiceProxyBackendImpl>) GWT.create(Type_factory__o_u_c_RuntimePluginsServiceProxyBackendImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LocaleListBox>) GWT.create(Type_factory__o_j_e_u_c_w_LocaleListBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LienzoCanvasExport>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_e_LienzoCanvasExport__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SplashEditorPresenterActivity>) GWT.create(Type_factory__o_u_e_p_c_e_SplashEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScreenErrorViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_s_ScreenErrorViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MetaFileEditorPresenterActivity>) GWT.create(Type_factory__o_u_e_w_c_c_e_m_MetaFileEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Notifications>) GWT.create(Type_factory__o_k_w_c_s_s_c_w_Notifications__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<FormsClientEntryPoint>) GWT.create(Type_factory__o_k_w_c_s_f_c_FormsClientEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FlowActionsToolboxFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_FlowActionsToolboxFactory__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_FlowActionsToolbox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LiveSearchDropDownView>) GWT.create(Type_factory__o_u_e_w_c_c_d_LiveSearchDropDownView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<UtilityMenuBarPresenter>) GWT.create(Type_factory__o_u_c_w_w_m_UtilityMenuBarPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextField>) GWT.create(Type_factory__o_u_e_p_e_c_f_TextField__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ChildrenTraverseProcessorImpl>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_t_c_ChildrenTraverseProcessorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<TargetDivDragComponent>) GWT.create(Type_factory__o_u_e_p_c_p_e_l_e_TargetDivDragComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PlaceRequest>) GWT.create(JsType_factory__o_u_m_PlaceRequest__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementDiagramFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_f_CaseManagementDiagramFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WidgetWrapperViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_WidgetWrapperViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DockingAcceptorControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_DockingAcceptorControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LongestPathVertexLayerer>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_l_s_s_LongestPathVertexLayerer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ContainmentAcceptorControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_ContainmentAcceptorControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ShowcaseEntryPoint>) GWT.create(Type_factory__o_k_w_c_s_s_c_ShowcaseEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_IntegerEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WidgetElementRendererViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_WidgetElementRendererViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SSHKeysEditorScreen>) GWT.create(Type_factory__o_u_s_c_e_SSHKeysEditorScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToPngToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_ExportToPngToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WiresCanvasView>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_w_WiresCanvasView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerWiresControlFactory>) GWT.create(Type_factory__o_k_w_c_s_c_l_w_StunnerWiresControlFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LogSelectedEdgeDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_LogSelectedEdgeDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNSVGViewFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_r_BPMNSVGViewFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SelectorFieldInitilizer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_s_SelectorFieldInitilizer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientFormGenerator>) GWT.create(Type_factory__o_k_w_c_f_a_e_c_f_ClientFormGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DeletePopUpPresenter>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_DeletePopUpPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComboBox>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_w_ComboBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionPropertiesScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionPropertiesScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeLiveSearchService>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeLiveSearchService__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClientPropertuValueExtractor>) GWT.create(Type_factory__o_k_w_c_f_a_e_c_f_u_ClientPropertuValueExtractor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NewUserAttributeEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_NewUserAttributeEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextEditorSingleLineBox>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_a_TextEditorSingleLineBox__quals__j_e_i_Any_o_k_w_c_s_c_c_c_c_a_SingleLineTextEditorBox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiListWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_MultiListWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClearCommandHistoryDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_ClearCommandHistoryDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementDeleteSelectionToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_t_c_CaseManagementDeleteSelectionToolbarCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ColumnWithComponentsView>) GWT.create(Type_factory__o_u_e_l_e_c_c_c_ColumnWithComponentsView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<org.uberfire.client.workbench.widgets.notfound.ActivityNotFoundPresenter>) GWT.create(Type_factory__o_u_c_w_w_n_ActivityNotFoundPresenter__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<EdgeCardinalityEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_EdgeCardinalityEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComboBoxViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_w_ComboBoxViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextEditorMultiLineBoxView>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_a_TextEditorMultiLineBoxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_c_a_MultiLineTextEditorBox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ToolbarItemView>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_i_ToolbarItemView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClosableSimpleWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_ClosableSimpleWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserEditorWorkflow>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_w_UserEditorWorkflow__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LockDemandDetector>) GWT.create(Type_factory__o_u_c_m_LockDemandDetector__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ColorField>) GWT.create(Type_factory__o_u_e_p_e_c_f_ColorField__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLDivElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLDivElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CommonsEntryPoint>) GWT.create(Type_factory__o_u_e_w_c_c_CommonsEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<RoleStyleBindingProvider>) GWT.create(Type_factory__o_j_e_s_c_l_s_RoleStyleBindingProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DecimalBoxFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_d_p_DecimalBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasLayoutUtils>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_u_CanvasLayoutUtils__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ACLSettingsView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_ACLSettingsView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToJpgToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_ExportToJpgToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultEditorSession>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_i_DefaultEditorSession__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<UberfireBreadcrumbsContainerImpl>) GWT.create(Type_factory__o_u_e_w_c_c_b_h_UberfireBreadcrumbsContainerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecurityManagementHomeScreen>) GWT.create(Type_factory__o_u_e_s_m_c_s_h_SecurityManagementHomeScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoImageStripLoader>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_g_LienzoImageStripLoader__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SimpleWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_SimpleWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ShapeViewFactory>) GWT.create(Type_factory__o_k_w_c_s_s_c_v_ShapeViewFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RoleEditorScreen>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_RoleEditorScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JsTypeProvider>) GWT.create(JsType_factory__o_j_e_i_c_c_JsTypeProvider__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLElement>) GWT.create(ContextualProvider_factory__e_d_HTMLElement__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WiresManagerFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_w_WiresManagerFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SimpleDnDWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_SimpleDnDWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PreferenceScopeFactoryImpl>) GWT.create(Type_factory__o_u_p_s_i_PreferenceScopeFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeListItem>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeListItem__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeyEditorViewImpl>) GWT.create(Type_factory__o_u_s_c_e_c_k_k_SSHKeyEditorViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LiveSearchFooter>) GWT.create(Type_factory__o_u_e_w_c_c_d_f_LiveSearchFooter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecurityManagementHomeView>) GWT.create(Type_factory__o_u_e_s_m_c_s_h_SecurityManagementHomeView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLTextAreaElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLTextAreaElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchMenuBarView>) GWT.create(Type_factory__o_u_c_v_p_m_WorkbenchMenuBarView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionLoader>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_i_SessionLoader__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<EventProvider>) GWT.create(Type_factory__o_j_e_e_c_c_EventProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DoubleEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_d_DoubleEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SvgElementGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_g_SvgElementGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EntityWorkflowViewImpl>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_w_EntityWorkflowViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuItem>) GWT.create(JsType_factory__o_u_w_m_m_MenuItem__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DecimalRadioGroupFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_r_DecimalRadioGroupFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LogSelectedItemsDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_LogSelectedItemsDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LayoutComponentPalettePresenter>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutComponentPalettePresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutHelper>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_l_LayoutHelper__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Instance>) GWT.create(ContextualProvider_factory__j_e_i_Instance__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DynamicValidator>) GWT.create(DynamicValidatorFactory.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HeaderPanel>) GWT.create(ExtensionProvided_factory__c_g_g_u_c_u_HeaderPanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeFieldInitializer>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_AssigneeFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<EventBus>) GWT.create(Producer_factory__c_g_w_b_e_s_EventBus__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementShapeDefFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_f_CaseManagementShapeDefFactory__quals__j_e_i_Any_j_e_i_Default.class));
  }

  private void registerFactories1(final Context org_jboss_errai_ioc_client_container_ApplicationScopedContext_context, final Context org_jboss_errai_ioc_client_container_DependentScopeContext_context) {
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PlaceManagerJSExporter>) GWT.create(Type_factory__o_u_c_e_PlaceManagerJSExporter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AuthoringPerspectiveActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_p_AuthoringPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeysDisplayerViewImpl>) GWT.create(Type_factory__o_u_s_c_e_c_k_SSHKeysDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RoleEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_r_RoleEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveEditorCoreGroupProvider>) GWT.create(Type_factory__o_u_e_p_c_p_e_a_PerspectiveEditorCoreGroupProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LogNodeEdgesDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_LogNodeEdgesDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Heading>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Heading__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScreenEditorView>) GWT.create(Type_factory__o_u_e_p_c_e_ScreenEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldHelp>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_l_h_FieldHelp__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormsContainer>) GWT.create(Type_factory__o_k_w_c_s_f_c_w_c_FormsContainer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiPageEditorViewImpl>) GWT.create(Type_factory__o_u_c_v_p_m_MultiPageEditorViewImpl__quals__j_e_i_Any_o_u_c_w_w_m_Multiple.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_UserEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SwitchLogLevelDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_SwitchLogLevelDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultFormGroupViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_d_DefaultFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultFormGroup>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_d_DefaultFormGroup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiplePermissionNodeEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_MultiplePermissionNodeEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DeleteSelectionToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_DeleteSelectionToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientRegistryFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_r_i_ClientRegistryFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeListFieldEditorPresenter>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeListFieldEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupMenuItemView>) GWT.create(Type_factory__o_u_c_v_p_m_m_m_GroupMenuItemView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleSelectorFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_s_MultipleSelectorFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNGraphFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_b_f_BPMNGraphFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultVertexPositioning>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_l_s_s_DefaultVertexPositioning__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionDiagramViewerScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionDiagramViewerScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ProcessingEngineEntryPoint>) GWT.create(Type_factory__o_k_w_c_f_p_e_h_ProcessingEngineEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SplashEditorPresenter>) GWT.create(Type_factory__o_u_e_p_c_e_SplashEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchMegaMenuPresenter>) GWT.create(Producer_factory__o_u_c_w_w_m_m_WorkbenchMegaMenuPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BindableTypeAheadViewImpl>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_t_BindableTypeAheadViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementGraphFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_f_CaseManagementGraphFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClosableSimpleWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_ClosableSimpleWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNCanvasShortcutsControlImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_c_c_BPMNCanvasShortcutsControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CallerProvider>) GWT.create(Type_factory__o_j_e_i_c_a_b_CallerProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasInPlaceTextEditorControlMultiLine>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_a_CanvasInPlaceTextEditorControlMultiLine__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_a_MultiLineTextEditorBox.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PermissionTreeI18NImpl>) GWT.create(Type_factory__o_u_c_r_i_PermissionTreeI18NImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AttachShadowOptionsType>) GWT.create(JsType_factory__e_d_HTMLElement_AttachShadowOptionsType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasView>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_v_CanvasView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementEditorSession>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_CaseManagementEditorSession__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SubFormFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_n_SubFormFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ConnectorParentsMatchConnectionHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_e_i_ConnectorParentsMatchConnectionHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DropdownMenu>) GWT.create(Type_factory__o_u_c_v_p_w_DropdownMenu__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.uberfire.ext.security.management.client.widgets.popup.LoadingBox>) GWT.create(Type_factory__o_u_e_s_m_c_w_p_LoadingBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleSubFormFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_n_MultipleSubFormFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Span>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Span__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLInputElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLInputElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ObservablePathImpl>) GWT.create(Type_factory__o_u_b_v_i_ObservablePathImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PreferenceStore>) GWT.create(ContextualProvider_factory__o_u_p_c_s_PreferenceStore__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DynamicMenuEditorPresenterActivity>) GWT.create(Type_factory__o_u_e_p_c_e_DynamicMenuEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DynamicRendererEntryPoint>) GWT.create(Type_factory__o_k_w_c_f_d_c_DynamicRendererEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ConnectionEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_ConnectionEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLOListElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLOListElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<AdapterManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_a_AdapterManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<GraphConnectionEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_GraphConnectionEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultToolboxControl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_t_DefaultToolboxControl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BootstrapAdapterFactory>) GWT.create(Type_factory__o_k_w_c_s_c_d_a_b_BootstrapAdapterFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PlugInAuthoringPerspective>) GWT.create(Type_factory__o_u_e_p_c_p_PlugInAuthoringPerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UberTabPanel>) GWT.create(Type_factory__o_u_c_v_p_t_UberTabPanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CheckBoxFormGroup>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_c_CheckBoxFormGroup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ElementBuilderControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_b_i_ElementBuilderControlImpl__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_b_i_Element.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Activity>) GWT.create(JsType_factory__o_u_c_m_Activity__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLButtonElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLButtonElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ManagedToolbar>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_i_ManagedToolbar__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EditorPlugInEditorPresenterActivity>) GWT.create(Type_factory__o_u_e_p_c_e_EditorPlugInEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SecurityEntryPoint>) GWT.create(Type_factory__o_u_s_c_SecurityEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JSNativeSplashScreen>) GWT.create(Type_factory__o_u_c_s_JSNativeSplashScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NotificationsScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_NotificationsScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LocaleSelector>) GWT.create(Type_factory__o_j_e_u_c_w_LocaleSelector__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutComponentPaletteView>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutComponentPaletteView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecurityRolesConstraintPage>) GWT.create(Type_factory__o_j_e_s_c_l_c_SecurityContextImpl_SecurityRolesConstraintPage__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleSubFormFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_m_MultipleSubFormFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<GraphFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_f_i_GraphFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CreationImpl>) GWT.create(Type_factory__o_j_e_i_c_l_i_CreationImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UsersExplorer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_UsersExplorer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DecimalBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_DecimalBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ListComponentProvider>) GWT.create(Type_factory__o_j_e_d_c_ListComponentProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormPropertiesWidget>) GWT.create(Type_factory__o_k_w_c_s_f_c_w_FormPropertiesWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuItemPerspective>) GWT.create(JsType_factory__o_u_w_m_m_MenuItemPerspective__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionEditorImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_s_i_SessionEditorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultPreferenceScopeTypes>) GWT.create(Type_factory__o_u_p_s_i_DefaultPreferenceScopeTypes__quals__j_e_i_Any_o_u_a_FallbackImplementation.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ContextDefinition>) GWT.create(JsType_factory__o_u_w_m_ContextDefinition__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PerspectiveEditorView>) GWT.create(Type_factory__o_u_e_p_c_p_e_PerspectiveEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ResizeControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_ResizeControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<UberfireBreadcrumbs>) GWT.create(Type_factory__o_u_e_w_c_c_b_UberfireBreadcrumbs__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementShapeFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_f_CaseManagementShapeFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutEditorPluginImpl>) GWT.create(Type_factory__o_u_e_l_e_c_LayoutEditorPluginImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultPermissionTypeRegistry>) GWT.create(Type_factory__o_u_s_i_a_DefaultPermissionTypeRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SplashViewImpl>) GWT.create(Type_factory__o_u_c_v_p_s_SplashViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LienzoGlyphRenderers>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_g_LienzoGlyphRenderers__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PerspectiveActivity>) GWT.create(JsType_factory__o_u_c_m_PerspectiveActivity__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExperimentalFeaturesGroup>) GWT.create(Type_factory__o_u_e_c_e_g_ExperimentalFeaturesGroup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JsTypeEventObserver>) GWT.create(JsType_factory__o_j_e_e_c_c_JsTypeEventObserver__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchLayoutImpl>) GWT.create(Type_factory__o_u_c_w_WorkbenchLayoutImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeEditorWidget>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeEditorWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormLayoutGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_FormLayoutGenerator__quals__j_e_i_Any.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HtmlEditorPresenter>) GWT.create(Type_factory__o_u_e_e_c_c_h_HtmlEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasPerspectiveActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_p_CanvasPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BigDecimalEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_d_BigDecimalEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PluginJSExporter>) GWT.create(Type_factory__o_u_c_e_PluginJSExporter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CheckBoxFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_c_p_CheckBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleInputFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_MultipleInputFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DiagramFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_f_i_DiagramFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleInputComponent>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_MultipleInputComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionEditorPresenter>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_s_i_SessionEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UndoSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_UndoSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CrudComponentEntryPoint>) GWT.create(Type_factory__o_k_w_c_f_c_c_CrudComponentEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementCanvasView>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_CaseManagementCanvasView__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleInputProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_l_i_MultipleInputProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SavePopUpPresenter>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_SavePopUpPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasCommandManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_CanvasCommandManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ChildMenuItemView>) GWT.create(Type_factory__o_u_c_v_p_m_m_m_ChildMenuItemView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultVertexOrdering>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_l_s_s_DefaultVertexOrdering__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CommonDomainLookups>) GWT.create(Type_factory__o_k_w_c_s_c_l_d_CommonDomainLookups__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultMenuGroup>) GWT.create(JsType_factory__o_u_w_m_m_i_DefaultMenuGroup__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerLienzoCore>) GWT.create(Type_factory__o_k_w_c_s_c_l_StunnerLienzoCore__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoShapeGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_g_LienzoShapeGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNToolboxControl>) GWT.create(Type_factory__o_k_w_c_s_b_c_c_c_BPMNToolboxControl__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<URLPatternMatcher>) GWT.create(Producer_factory__o_j_e_u_n_c_l_URLPatternMatcher__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<UberfireDocksImpl>) GWT.create(Type_factory__o_u_c_d_UberfireDocksImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HtmlEditorActivity>) GWT.create(Type_factory__o_u_e_e_c_c_h_HtmlEditorActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SimpleDnDWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_SimpleDnDWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PreferenceStoreIOCProvider>) GWT.create(Type_factory__o_u_p_c_s_PreferenceStoreIOCProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiScreenPartWidget>) GWT.create(Type_factory__o_u_c_v_p_m_MultiScreenPartWidget__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariableDeleteHandler>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_v_VariableDeleteHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LifecycleListenerRegistrarImpl>) GWT.create(Type_factory__o_j_e_i_c_l_i_LifecycleListenerRegistrarImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleInputViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_MultipleInputViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RenamePopUpView>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_RenamePopUpView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExperimentalFeaturesEditorScreenViewImpl>) GWT.create(Type_factory__o_u_e_c_e_ExperimentalFeaturesEditorScreenViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientDefinitionManager>) GWT.create(Type_factory__o_k_w_c_s_c_c_a_ClientDefinitionManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CardinalityEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_CardinalityEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionDiagramPreviewScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionDiagramPreviewScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LiveSearchDropDown>) GWT.create(Type_factory__o_u_e_w_c_c_d_LiveSearchDropDown__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DatePickerFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_d_DatePickerFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EntitiesExplorerViewImpl>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_EntitiesExplorerViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecurityManagementHomeScreenActivity>) GWT.create(Type_factory__o_u_e_s_m_c_s_h_SecurityManagementHomeScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.jboss.errai.common.client.dom.HTMLElement>) GWT.create(ContextualProvider_factory__o_j_e_c_c_d_HTMLElement__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GraphBoundsIndexerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_i_b_GraphBoundsIndexerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultBreadcrumbsView>) GWT.create(Type_factory__o_u_e_w_c_c_b_w_DefaultBreadcrumbsView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientFieldElementProcessor>) GWT.create(Type_factory__o_k_w_c_f_a_e_c_f_ClientFieldElementProcessor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<AppsEntryPoint>) GWT.create(Type_factory__o_u_e_a_c_AppsEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextFileExport>) GWT.create(Producer_factory__o_u_e_e_c_c_f_e_TextFileExport__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariableListItemWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_v_VariableListItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BS3PaletteWidgetImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_BS3PaletteWidgetImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutDragComponentGroupPresenter>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutDragComponentGroupPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LongEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_LongEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToSvgSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToSvgSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ValidatorProvider>) GWT.create(Type_factory__o_j_e_v_c_ValidatorProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VersionHistoryPresenter>) GWT.create(Type_factory__o_u_e_e_c_c_h_VersionHistoryPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PortablePreferenceMockBeanGeneratedImpl>) GWT.create(Type_factory__o_u_p_s_b_m_PortablePreferenceMockBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiTabWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_MultiTabWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CustomPanelDefinition>) GWT.create(JsType_factory__o_u_w_m_CustomPanelDefinition__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleSelectorProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_l_s_MultipleSelectorProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoTextTooltip>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_v_LienzoTextTooltip__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Path>) GWT.create(JsType_factory__o_u_b_v_Path__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ImageElementGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_g_ImageElementGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Elemental2Producer>) GWT.create(Type_factory__o_u_c_v_p_w_Elemental2Producer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TreeExplorer>) GWT.create(Type_factory__o_k_w_c_s_c_w_e_t_TreeExplorer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<EdgeFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_f_i_EdgeFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Disposer>) GWT.create(ContextualProvider_factory__o_j_e_i_c_a_Disposer__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextAreaFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_TextAreaFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComboBoxFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_ComboBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ErrorBox>) GWT.create(Type_factory__o_u_e_s_m_c_w_p_ErrorBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ListComponent>) GWT.create(ContextualProvider_factory__o_j_e_d_c_c_ListComponent__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<InputDeviceCapabilitiesOptionsType>) GWT.create(JsType_factory__e_d_InputDeviceCapabilities_InputDeviceCapabilitiesOptionsType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SplitLayoutPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_SplitLayoutPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BasicFileMenuBuilderImpl>) GWT.create(Type_factory__o_u_e_e_c_c_m_BasicFileMenuBuilderImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.uberfire.client.views.pfly.modal.ErrorPopupView>) GWT.create(Type_factory__o_u_c_v_p_m_ErrorPopupView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientTranslationMessages>) GWT.create(Type_factory__o_k_w_c_s_c_c_i_ClientTranslationMessages__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.uberfire.ext.widgets.core.client.workbench.widgets.popups.activities.notfound.ActivityNotFoundView>) GWT.create(Type_factory__o_u_e_w_c_c_w_w_p_a_n_ActivityNotFoundView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExpandedPaletteDefinitionBuilder>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_p_ExpandedPaletteDefinitionBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RolesExplorer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_RolesExplorer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormPropertiesWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_f_c_w_FormPropertiesWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HasEnabledStateChangeListeners>) GWT.create(JsType_factory__o_u_w_m_m_HasEnabledStateChangeListeners__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetDefinitionSetAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetDefinitionSetAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FullContentTraverseProcessorImpl>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_t_c_FullContentTraverseProcessorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StorageHandlerProvider>) GWT.create(Type_factory__o_j_e_s_c_l_s_StorageHandlerProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<AnyResourceType>) GWT.create(Type_factory__o_u_c_w_t_AnyResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<FormWidgetsEntryPoint>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_FormWidgetsEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PlaceHistoryHandler>) GWT.create(Type_factory__o_u_c_m_PlaceHistoryHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Form>) GWT.create(Type_factory__o_g_c_s_p_c_Form__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<NotificationManager>) GWT.create(Type_factory__o_u_c_w_w_n_NotificationManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ShapeDefFunctionalFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_f_ShapeDefFunctionalFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.jboss.errai.common.client.dom.Select>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Select__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultBreadcrumbsPresenter>) GWT.create(Type_factory__o_u_e_w_c_c_b_w_DefaultBreadcrumbsPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNDiagramFilterProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_BPMNDiagramFilterProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RestoreVersionCommandProvider>) GWT.create(Type_factory__o_u_e_e_c_c_m_RestoreVersionCommandProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiplePermissionNodeViewer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_MultiplePermissionNodeViewer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutEditorPresenter>) GWT.create(Type_factory__o_u_e_l_e_c_LayoutEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DatePickerFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_d_p_DatePickerFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ToggleCommentPresenter>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_c_ToggleCommentPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SubFormFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_s_SubFormFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JSNativeClientResourceType>) GWT.create(Type_factory__o_u_c_e_t_JSNativeClientResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleSubFormWidget>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_m_MultipleSubFormWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComponentColumn>) GWT.create(Type_factory__o_u_e_l_e_c_c_c_ComponentColumn__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TransitionToRole>) GWT.create(ContextualProvider_factory__o_j_e_u_n_c_l_TransitionToRole__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeFieldEditorWidget>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeFieldEditorWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveEditorNavGroupProvider>) GWT.create(Type_factory__o_u_e_p_c_p_e_a_PerspectiveEditorNavGroupProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DotResourceType>) GWT.create(Type_factory__o_u_c_w_t_DotResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConnectorGlyphLienzoRenderer>) GWT.create(Type_factory__o_k_w_c_s_s_c_ConnectorGlyphLienzoRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CollapsedPaletteDefinitionBuilder>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_p_CollapsedPaletteDefinitionBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Document>) GWT.create(Producer_factory__o_j_e_c_c_d_Document__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLAnchorElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLAnchorElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasDiagramValidator>) GWT.create(Type_factory__o_k_w_c_s_c_c_v_c_CanvasDiagramValidator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ValidateSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_ValidateSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSetPropertySetAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetPropertySetAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoLayer>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_LienzoLayer__quals__j_e_i_Any_o_k_w_c_s_c_l_Lienzo.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserStorageHandler>) GWT.create(Provider_factory__o_j_e_s_c_l_s_UserStorageHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ManagedInstance>) GWT.create(ContextualProvider_factory__o_j_e_i_c_a_ManagedInstance__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignedEntitiesInlineEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_AssignedEntitiesInlineEditorView__quals__j_e_i_Any_o_u_e_s_m_c_w_m_e_AssignedEntitiesInlineEditor.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ExperimentalActivitiesAuthorizationManagerImpl>) GWT.create(Type_factory__o_u_e_c_s_a_ExperimentalActivitiesAuthorizationManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PasteSelectionSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_PasteSelectionSessionCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_ScriptTypeFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LoadingBoxView>) GWT.create(Type_factory__o_u_e_s_m_c_w_p_LoadingBoxView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeListFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeListFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Menus>) GWT.create(JsType_factory__o_u_w_m_m_Menus__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StringEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_StringEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementVisitGraphSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_CaseManagementVisitGraphSessionCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerBPMNEntryPoint>) GWT.create(Type_factory__o_k_w_c_s_b_StunnerBPMNEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToJpgSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_ExportToJpgSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NotificationsScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_NotificationsScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PictureProvidersManager>) GWT.create(Type_factory__o_k_w_c_s_s_c_f_PictureProvidersManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DragHelperComponentColumn>) GWT.create(Type_factory__o_u_e_l_e_c_i_DragHelperComponentColumn__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NoCanvasView>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_s_NoCanvasView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoSvgDataUriGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_g_LienzoSvgDataUriGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.jboss.errai.common.client.dom.Anchor>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Anchor__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JSNativePerspective>) GWT.create(Type_factory__o_u_c_p_JSNativePerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PreferenceScopeValidatorImpl>) GWT.create(Type_factory__o_u_p_s_i_PreferenceScopeValidatorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PropertyEditorWidget>) GWT.create(Type_factory__o_u_e_p_e_c_PropertyEditorWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserMenuViewImpl>) GWT.create(Type_factory__o_u_c_v_p_m_UserMenuViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PermissionSwitch>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionSwitch__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserAssignedGroupsEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_UserAssignedGroupsEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PlaceStatus>) GWT.create(JsType_factory__o_u_c_m_PlaceStatus__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormsCanvasSessionHandler>) GWT.create(Type_factory__o_k_w_c_s_f_c_w_FormsCanvasSessionHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormValidatorImpl>) GWT.create(Type_factory__o_k_w_c_f_p_e_h_i_FormValidatorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignedEntitiesModalEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_AssignedEntitiesModalEditorView__quals__j_e_i_Any_o_u_e_s_m_c_w_m_e_AssignedEntitiesModalEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PreferenceScopeResolutionStrategy>) GWT.create(ContextualProvider_factory__o_u_p_s_PreferenceScopeResolutionStrategy__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DeletePopUpView>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_DeletePopUpView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientTranslationService>) GWT.create(Type_factory__o_k_w_c_s_c_c_i_ClientTranslationService__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TimerSettingsFieldEditorWidget>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_t_TimerSettingsFieldEditorWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ExperimentalAwareClientTypeRegistryImpl>) GWT.create(Type_factory__o_u_e_c_w_t_ExperimentalAwareClientTypeRegistryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignmentsEditorFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_AssignmentsEditorFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Resource>) GWT.create(JsType_factory__o_u_s_Resource__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConfigErrorDisplayerViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_c_ConfigErrorDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_LayoutPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DockingEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_DockingEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JSNativeScreen>) GWT.create(Type_factory__o_u_c_s_JSNativeScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Modal>) GWT.create(Type_factory__o_u_c_v_p_w_Modal__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultWorkbenchController>) GWT.create(Type_factory__o_u_c_a_DefaultWorkbenchController__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DataBinder>) GWT.create(ContextualProvider_factory__o_j_e_d_c_a_DataBinder__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Logger>) GWT.create(ExtensionProvided_factory__o_s_Logger__quals__Universal_2.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Logger>) GWT.create(ExtensionProvided_factory__o_s_Logger__quals__Universal_1.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Logger>) GWT.create(ExtensionProvided_factory__o_s_Logger__quals__Universal_4.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Logger>) GWT.create(ExtensionProvided_factory__o_s_Logger__quals__Universal_3.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Logger>) GWT.create(ExtensionProvided_factory__o_s_Logger__quals__Universal_6.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Logger>) GWT.create(ExtensionProvided_factory__o_s_Logger__quals__Universal_5.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NewTabFilterPopup>) GWT.create(Type_factory__o_u_e_w_c_c_t_p_NewTabFilterPopup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ConnectorCardinalityEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_ConnectorCardinalityEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeFieldEditorView>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeFieldEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldTypeEntryProcessor>) GWT.create(Type_factory__o_k_w_c_f_s_s_m_p_i_p_FieldTypeEntryProcessor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuItemPlain>) GWT.create(JsType_factory__o_u_w_m_m_MenuItemPlain__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkItemDefinitionClientRegistry>) GWT.create(Type_factory__o_k_w_c_s_b_c_w_WorkItemDefinitionClientRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HasPlaceHolderFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_HasPlaceHolderFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldLabelEntryProcessor>) GWT.create(Type_factory__o_k_w_c_f_s_s_m_p_i_p_FieldLabelEntryProcessor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldStateValidatorImpl>) GWT.create(Type_factory__o_k_w_c_f_p_e_h_i_FieldStateValidatorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AppendParallelGatewayShortcut>) GWT.create(Type_factory__o_k_w_c_s_b_c_c_c_k_s_AppendParallelGatewayShortcut__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementWiresManagerFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_w_CaseManagementWiresManagerFactory__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchMenuBarProducer>) GWT.create(Type_factory__o_u_c_w_w_m_WorkbenchMenuBarProducer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionViewerPresenter>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_s_i_SessionViewerPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DynamicMenuEditorPresenter>) GWT.create(Type_factory__o_u_e_p_c_e_DynamicMenuEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiplePermissionNodeEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_MultiplePermissionNodeEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleSelectorInput>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_s_i_MultipleSelectorInput__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CrudComponent>) GWT.create(Type_factory__o_k_w_c_f_c_c_c_CrudComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RestorePopUpPresenter>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_RestorePopUpPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeFieldInitializer>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_ScriptTypeFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<RootPanelProvider>) GWT.create(Type_factory__o_j_e_i_c_a_b_RootPanelProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TableCell>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_TableCell__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultCloneProcess>) GWT.create(Type_factory__o_k_w_c_s_c_d_c_DefaultCloneProcess__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ShowcaseDiagramService>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_ShowcaseDiagramService__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VerticesTransposer>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_l_s_s_VerticesTransposer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConditionalComboBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_c_ConditionalComboBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BS3PaletteWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_BS3PaletteWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<NavigationGraph>) GWT.create(Producer_factory__o_j_e_u_n_c_l_s_NavigationGraph__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ImageStripRegistry>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_ImageStripRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SvgFileExport>) GWT.create(Producer_factory__o_u_e_e_c_c_f_e_s_SvgFileExport__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultViewerSession>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_i_DefaultViewerSession__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Logger>) GWT.create(ExtensionProvided_factory__o_s_Logger__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientExperimentalFeaturesRegistryServiceImpl>) GWT.create(Type_factory__o_u_e_c_s_i_ClientExperimentalFeaturesRegistryServiceImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefinitionPaletteCategoryWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_c_DefinitionPaletteCategoryWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PlaceRequestHistoryMapperImpl>) GWT.create(Type_factory__o_u_c_m_PlaceRequestHistoryMapperImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextEditorWorkbenchEditorActivity>) GWT.create(Type_factory__o_u_e_w_c_c_e_t_TextEditorWorkbenchEditorActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeFormGroup>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_f_AssigneeFormGroup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CreateConnectorAction>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_CreateConnectorAction__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PanelDefinitionImpl>) GWT.create(JsType_factory__o_u_w_m_i_PanelDefinitionImpl__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DynamicFormModelGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_s_s_a_DynamicFormModelGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScreenEditorPresenterActivity>) GWT.create(Type_factory__o_u_e_p_c_e_ScreenEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JQuery>) GWT.create(Producer_factory__o_u_c_v_p_w_JQueryProducer_JQuery__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ListWidgetProvider>) GWT.create(Type_factory__o_j_e_u_c_w_ListWidgetProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasBoundsIndexerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_i_b_CanvasBoundsIndexerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IFrameScreenView>) GWT.create(Type_factory__o_u_e_w_c_c_s_i_IFrameScreenView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MetaFileEditorPresenter>) GWT.create(Type_factory__o_u_e_w_c_c_e_m_MetaFileEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionDiagramViewerScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionDiagramViewerScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ConnectorParentsMatchHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_e_i_ConnectorParentsMatchHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveEditorPaletteI18nImpl>) GWT.create(Type_factory__o_u_e_p_c_r_i_PerspectiveEditorPaletteI18nImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutDragComponentWidget>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutDragComponentWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IFrameScreenPresenterActivity>) GWT.create(Type_factory__o_u_e_w_c_c_s_i_IFrameScreenPresenterActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BootstrapNavigatorView>) GWT.create(Type_factory__o_k_w_c_s_c_w_e_n_v_BootstrapNavigatorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<NodeFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_f_i_NodeFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<URLPatternMatcherProvider>) GWT.create(Type_factory__o_j_e_u_n_c_l_URLPatternMatcherProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Bs3Modal>) GWT.create(Type_factory__o_u_c_v_p_m_Bs3Modal__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RestoreUtil>) GWT.create(Type_factory__o_u_e_e_c_c_f_RestoreUtil__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<RuntimePluginStartup>) GWT.create(Type_factory__o_u_e_p_c_p_RuntimePluginStartup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSetResourceType>) GWT.create(Type_factory__o_k_w_c_s_c_r_CaseManagementDefinitionSetResourceType__quals__j_e_i_Any_j_e_i_Default_o_u_w_a_VisibleAsset.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConnectionAcceptorControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_ConnectionAcceptorControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LogSessionDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_LogSessionDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PluginController>) GWT.create(Type_factory__o_u_e_p_c_s_PluginController__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RoleEditorWorkflow>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_r_w_RoleEditorWorkflow__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GraphUtils>) GWT.create(Type_factory__o_k_w_c_s_c_g_u_GraphUtils__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ContainerView>) GWT.create(Type_factory__o_u_e_l_e_c_c_c_ContainerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MorphActionsToolboxFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_MorphActionsToolboxFactory__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_MorphActionsToolbox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ContextDisplayMode>) GWT.create(JsType_factory__o_u_w_m_ContextDisplayMode__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchLayoutInfoImpl>) GWT.create(Type_factory__o_u_c_w_WorkbenchLayoutInfoImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MorphActionsToolboxView>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_t_a_MorphActionsToolboxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_MorphActionsToolbox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SaveToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_SaveToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FlowActionsToolboxView>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_t_a_FlowActionsToolboxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_t_a_FlowActionsToolbox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TableSection>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_TableSection__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetDefinitionAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetDefinitionAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeListFieldInitializer>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_ScriptTypeListFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Label>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Label__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Validator>) GWT.create(Provider_factory__j_v_Validator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PerspectiveDefinition>) GWT.create(JsType_factory__o_u_w_m_PerspectiveDefinition__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementShapeDefFunctionalFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_f_CaseManagementShapeDefFunctionalFactory__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LayoutSelection>) GWT.create(Type_factory__o_u_c_w_LayoutSelection__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VisitGraphSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_VisitGraphSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ACLViewerView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_ACLViewerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefinitionLookupManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_d_DefinitionLookupManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ServerTemplateProvider>) GWT.create(Type_factory__o_j_e_u_s_ServerTemplateProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ElementShapeGlyphRenderer>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_g_ElementShapeGlyphRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLUListElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLUListElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.gwtbootstrap3.client.ui.Button>) GWT.create(ExtensionProvided_factory__o_g_c_u_Button__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NodeBuilderControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_b_i_NodeBuilderControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FlowPanel>) GWT.create(ExtensionProvided_factory__o_g_c_u_g_FlowPanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ValidateToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_ValidateToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GraphCommandManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_g_c_GraphCommandManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CutToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_CutToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserEditorScreenActivity>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_UserEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ShapeManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_a_ShapeManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CoreEntryPoint>) GWT.create(Type_factory__o_u_e_w_c_c_CoreEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientSecurityExceptionMessageResolver>) GWT.create(Type_factory__o_u_e_s_m_c_ClientSecurityExceptionMessageResolver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PlaceManagerImpl>) GWT.create(Type_factory__o_u_c_m_PlaceManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CustomSplashHelp>) GWT.create(Type_factory__o_u_c_m_CustomSplashHelp__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LeafPermissionNodeEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_LeafPermissionNodeEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecretTextField>) GWT.create(Type_factory__o_u_e_p_e_c_f_SecretTextField__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VersionMenuDropDownButtonViewImpl>) GWT.create(Type_factory__o_u_e_e_c_c_h_VersionMenuDropDownButtonViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TimerSettingsFieldEditorPresenter>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_t_TimerSettingsFieldEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VersionHistoryPresenterViewImpl>) GWT.create(Type_factory__o_u_e_e_c_c_h_VersionHistoryPresenterViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DiagramTypeServiceImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_DiagramTypeServiceImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultAuthorizationManager>) GWT.create(Type_factory__o_u_s_i_a_DefaultAuthorizationManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<IOCBeanManagerProvider>) GWT.create(Type_factory__o_j_e_i_c_a_b_IOCBeanManagerProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PictureWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_p_w_PictureWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeysDisplayer>) GWT.create(Type_factory__o_u_s_c_e_c_k_SSHKeysDisplayer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ResizeFlowPanel>) GWT.create(ExtensionProvided_factory__o_u_c_w_w_l_ResizeFlowPanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PlugInAuthoringPerspectiveActivity>) GWT.create(Type_factory__o_u_e_p_c_p_PlugInAuthoringPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<KebabMenu>) GWT.create(Type_factory__o_u_c_v_p_w_KebabMenu__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ProcessSignalRefProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_ProcessSignalRefProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ErrorBoxView>) GWT.create(Type_factory__o_u_e_s_m_c_w_p_ErrorBoxView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerCoreEntryPoint>) GWT.create(Type_factory__o_k_w_c_s_c_StunnerCoreEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<AppNavBar>) GWT.create(Type_factory__o_k_w_c_s_s_c_n_AppNavBar__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DecimalBox>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_d_DecimalBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JSNativeEditor>) GWT.create(Type_factory__o_u_c_e_JSNativeEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DiagramPresenterScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_DiagramPresenterScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SelectView>) GWT.create(Type_factory__o_u_e_w_c_c_s_SelectView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementPreferencesRegistry>) GWT.create(Type_factory__o_k_w_c_s_c_c_p_CaseManagementPreferencesRegistry__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScreenEditorPresenter>) GWT.create(Type_factory__o_u_e_p_c_e_ScreenEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SignalScopeProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_SignalScopeProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LogBoundsDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_LogBoundsDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchActivity>) GWT.create(JsType_factory__o_u_c_m_WorkbenchActivity__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<TransitionAnchorFactoryProvider>) GWT.create(Type_factory__o_j_e_u_n_c_l_TransitionAnchorFactoryProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CreateEntity>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_CreateEntity__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ChangePassword>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_ChangePassword__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerBox>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_i_IntegerBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<OpenDiagramLayoutExecutor>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_l_OpenDiagramLayoutExecutor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkItemDefinitionRegistry>) GWT.create(Producer_factory__o_k_w_c_s_b_w_WorkItemDefinitionRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchViewModeSwitcherMenuBuilder>) GWT.create(Type_factory__o_u_c_m_WorkbenchViewModeSwitcherMenuBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ActivityLifecycleErrorHandler>) GWT.create(Type_factory__o_u_c_m_ActivityLifecycleErrorHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CanvasPerspective>) GWT.create(Type_factory__o_k_w_c_s_s_c_p_CanvasPerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UnanchoredStaticWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_UnanchoredStaticWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ColumnGeneratorManager>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_m_ColumnGeneratorManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextInput>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_TextInput__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NotificationsView>) GWT.create(Type_factory__o_k_w_c_s_s_c_w_NotificationsView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<GraphValidatorImpl>) GWT.create(Type_factory__o_k_w_c_s_c_v_i_GraphValidatorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DnDManager>) GWT.create(Type_factory__o_u_e_l_e_c_i_DnDManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PictureFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_p_PictureFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<HtmlResourceType>) GWT.create(Type_factory__o_u_e_e_c_c_h_HtmlResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasDefinitionTooltip>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_v_CanvasDefinitionTooltip__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<RequestDispatcherProvider>) GWT.create(Type_factory__o_j_e_i_s_b_c_RequestDispatcherProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserAttributesEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_UserAttributesEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLLIElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLLIElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetModelFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_b_d_f_BPMNDefinitionSetModelFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ConnectorParentsMatchContainmentHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_e_i_ConnectorParentsMatchContainmentHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NavigatorThumbnailItemView>) GWT.create(Type_factory__o_k_w_c_s_c_w_e_n_i_NavigatorThumbnailItemView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<InlineNotification>) GWT.create(Type_factory__o_u_c_v_p_w_InlineNotification__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Process>) GWT.create(Type_factory__o_g_c_s_p_c_Process__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScrollIntoViewTopType>) GWT.create(JsType_factory__e_d_Element_ScrollIntoViewTopType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CopyToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_CopyToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NewSSHKeyModal>) GWT.create(Type_factory__o_u_s_c_e_c_c_NewSSHKeyModal__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComboField>) GWT.create(Type_factory__o_u_e_p_e_c_f_ComboField__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultPermissionTreeFactory>) GWT.create(Type_factory__o_u_s_c_a_t_i_DefaultPermissionTreeFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RuntimeResource>) GWT.create(JsType_factory__o_u_s_a_RuntimeResource__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ActivityBeansCache>) GWT.create(Type_factory__o_u_c_m_ActivityBeansCache__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerTextBox>) GWT.create(ExtensionProvided_factory__o_k_w_c_s_c_w_c_a_IntegerTextBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MorphNodeAction>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_t_a_MorphNodeAction__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ChildMenuItemPresenter>) GWT.create(Type_factory__o_u_c_w_w_m_m_m_ChildMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComboBoxWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_c_ComboBoxWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldSetFormGroup>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_n_f_FieldSetFormGroup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DocksBars>) GWT.create(Type_factory__o_u_c_d_v_DocksBars__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerCommonIconsStrip>) GWT.create(Type_factory__o_k_w_c_s_c_c_r_StunnerCommonIconsStrip__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NewPluginPopUpViewImpl>) GWT.create(Type_factory__o_u_e_p_c_w_p_NewPluginPopUpViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetRuleAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetRuleAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DisposerProvider>) GWT.create(Type_factory__o_j_e_i_c_a_b_DisposerProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DynamicMenuResourceTypeDefinition>) GWT.create(Type_factory__o_u_e_p_t_DynamicMenuResourceTypeDefinition__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<MessageBusProvider>) GWT.create(Type_factory__o_j_e_i_s_b_c_MessageBusProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldLabel>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_l_l_FieldLabel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PdfFileExport>) GWT.create(Producer_factory__o_u_e_e_c_c_f_e_PdfFileExport__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiPageEditorImpl>) GWT.create(Type_factory__o_u_c_v_p_m_MultiPageEditorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SenderProvider>) GWT.create(Type_factory__o_j_e_i_s_b_c_SenderProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignmentsEditorWidget>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_AssignmentsEditorWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RenamePopUpPresenter>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_RenamePopUpPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AppsHomeView>) GWT.create(Type_factory__o_u_e_a_c_h_AppsHomeView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EmptyDropRow>) GWT.create(Type_factory__o_u_e_l_e_c_c_r_EmptyDropRow__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ManagedInstanceProvider>) GWT.create(Type_factory__o_j_e_i_c_a_b_ManagedInstanceProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutDragComponentHelper>) GWT.create(Type_factory__o_u_e_l_e_c_i_LayoutDragComponentHelper__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ComboBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_c_ComboBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<KeyEventHandler>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_k_KeyEventHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<GraphCommandFactory>) GWT.create(Type_factory__o_k_w_c_s_c_g_c_i_GraphCommandFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EditHTMLView>) GWT.create(Type_factory__o_u_e_p_c_p_e_l_e_p_EditHTMLView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<List>) GWT.create(JsType_factory__j_u_List__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CanvasInPlaceTextEditorControlSingleLine>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_a_CanvasInPlaceTextEditorControlSingleLine__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_a_SingleLineTextEditorBox.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Image>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Image__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupMenuItemPresenter>) GWT.create(Type_factory__o_u_c_w_w_m_m_m_GroupMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ExperimentalFeaturesEditorScreen>) GWT.create(Type_factory__o_u_e_c_e_ExperimentalFeaturesEditorScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.kie.workbench.common.stunner.core.client.session.command.impl.PasteSelectionSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_PasteSelectionSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PictureFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_i_p_PictureFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ByteEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_i_ByteEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AuthenticationServiceInterceptor>) GWT.create(Type_factory__o_j_e_s_c_l_i_AuthenticationServiceInterceptor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementCanvasHandler>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_CaseManagementCanvasHandler__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ReverseEdgesCycleBreaker>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_l_s_s_ReverseEdgesCycleBreaker__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutEditorView>) GWT.create(Type_factory__o_u_e_l_e_c_LayoutEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Promises>) GWT.create(Type_factory__o_u_c_p_Promises__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PreferenceBeanStoreImpl>) GWT.create(Type_factory__o_u_p_c_s_PreferenceBeanStoreImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ListBarWidgetImpl>) GWT.create(Type_factory__o_u_c_v_p_l_ListBarWidgetImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LogGraphDevCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_i_LogGraphDevCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchViewModeSwitcherPresenter>) GWT.create(Type_factory__o_u_c_m_WorkbenchViewModeSwitcherPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ScreenLayoutDragComponent>) GWT.create(Type_factory__o_u_e_p_c_p_e_l_e_ScreenLayoutDragComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ActivityDataIOEditorWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_ActivityDataIOEditorWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AppendEmbeddedSubprocessShortcut>) GWT.create(Type_factory__o_k_w_c_s_b_c_c_c_k_s_AppendEmbeddedSubprocessShortcut__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLElementProvider>) GWT.create(Type_factory__o_j_e_u_c_l_p_HTMLElementProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeEditorFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_AssigneeEditorFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefinitionPaletteItemWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_c_i_DefinitionPaletteItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SwitchGridToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_SwitchGridToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RolesList>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_l_RolesList__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultMenus>) GWT.create(JsType_factory__o_u_w_m_m_i_DefaultMenus__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<HtmlEditorLibraryLoader>) GWT.create(Type_factory__o_u_e_e_c_c_h_HtmlEditorLibraryLoader__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultFileNameValidator>) GWT.create(Type_factory__o_u_e_e_c_c_v_DefaultFileNameValidator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Elemental2HTMLElementProvider>) GWT.create(Type_factory__o_j_e_u_c_l_p_Elemental2HTMLElementProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.gwtbootstrap3.client.ui.html.Span>) GWT.create(ExtensionProvided_factory__o_g_c_u_h_Span__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<MenuBuilder>) GWT.create(Type_factory__o_u_c_d_v_m_MenuBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LienzoSelectionControl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_LienzoSelectionControl__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_s_SingleSelection.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<View>) GWT.create(Producer_factory__o_u_e_e_c_c_f_p_CopyPopUpPresenter_View__quals__j_e_i_Any_o_u_e_e_c_c_f_Customizable.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultRouteFormProvider>) GWT.create(Type_factory__o_k_w_c_s_b_c_d_DefaultRouteFormProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<AppsHomePresenter>) GWT.create(Type_factory__o_u_e_a_c_h_AppsHomePresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NewFilterPopup>) GWT.create(Type_factory__o_u_e_w_c_c_t_p_NewFilterPopup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SessionManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_a_SessionManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariablesEditorFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_VariablesEditorFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FilterPagedTable>) GWT.create(Type_factory__o_u_e_w_c_c_t_FilterPagedTable__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<UtilityMenuBarView>) GWT.create(Type_factory__o_u_c_v_p_m_UtilityMenuBarView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSet>) GWT.create(Type_factory__o_k_w_c_s_c_CaseManagementDefinitionSet__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CopySelectionSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_CopySelectionSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConfirmBoxView>) GWT.create(Type_factory__o_u_e_s_m_c_w_p_ConfirmBoxView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BootstrapLayoutGeneratorDriver>) GWT.create(Type_factory__o_u_e_l_e_c_g_BootstrapLayoutGeneratorDriver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ListItem>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_ListItem__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeysEditorPerspectiveActivity>) GWT.create(Type_factory__o_u_s_c_e_SSHKeysEditorPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BaseScreenView>) GWT.create(Type_factory__o_u_e_s_m_c_s_BaseScreenView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormsContainerViewImpl>) GWT.create(Type_factory__o_k_w_c_s_f_c_w_c_FormsContainerViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SessionInfo>) GWT.create(Producer_factory__o_u_r_SessionInfo__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientDefinitionsCacheRegistry>) GWT.create(Type_factory__o_k_w_c_s_c_c_r_i_ClientDefinitionsCacheRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SubFormWidget>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_s_w_SubFormWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RoleEditorScreenActivity>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_RoleEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ListWidget>) GWT.create(ContextualProvider_factory__o_j_e_u_c_w_ListWidget__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NoItemsComponentViewImpl>) GWT.create(Type_factory__o_u_e_w_c_c_d_n_NoItemsComponentViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<TextPropertyProviderFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_a_TextPropertyProviderFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DisabledExperimentalLayoutComponent>) GWT.create(Type_factory__o_u_e_l_e_c_i_e_DisabledExperimentalLayoutComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientModelValidator>) GWT.create(Type_factory__o_k_w_c_s_c_c_v_ClientModelValidator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LiveSearchFooterViewImpl>) GWT.create(Type_factory__o_u_e_w_c_c_d_f_LiveSearchFooterViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserEditorView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_UserEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SaveAndRenameCommandBuilder>) GWT.create(Type_factory__o_u_e_e_c_c_m_c_SaveAndRenameCommandBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DynamicFormHandlerGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_i_i_DynamicFormHandlerGenerator__quals__j_e_i_Any_o_k_w_c_f_d_s_s_DynamicContext.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormHandlerImpl>) GWT.create(Type_factory__o_k_w_c_f_p_e_h_i_FormHandlerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ShapeViewDragProxyImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_d_ShapeViewDragProxyImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RolesEditorWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_c_r_RolesEditorWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExperimentalFeaturesGroupViewImpl>) GWT.create(Type_factory__o_u_e_c_e_g_ExperimentalFeaturesGroupViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CharacterEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_CharacterEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LayoutElementPropertiesPresenter>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutElementPropertiesPresenter__quals__j_e_i_Any_j_e_i_Default.class));
  }

  private void registerFactories2(final Context org_jboss_errai_ioc_client_container_ApplicationScopedContext_context, final Context org_jboss_errai_ioc_client_container_DependentScopeContext_context) {
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LiveSearchSelectorDropDownItem>) GWT.create(Type_factory__o_u_e_w_c_c_d_i_LiveSearchSelectorDropDownItem__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SecurityExplorerScreenActivity>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_SecurityExplorerScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ListBoxFieldProvider>) GWT.create(Type_factory__o_k_w_c_f_f_s_f_b_s_l_p_ListBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ContainmentEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_ContainmentEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionTreeExplorerScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_SessionTreeExplorerScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HasMaxLengthFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_HasMaxLengthFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExternalPathPlaceRequest>) GWT.create(JsType_factory__o_u_m_i_ExternalPathPlaceRequest__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConditionalComboBoxFieldProvider>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_f_ConditionalComboBoxFieldProvider__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<InstanceProvider>) GWT.create(Type_factory__o_j_e_e_c_c_InstanceProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementNodeBuilderControl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_b_CaseManagementNodeBuilderControl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormDisplayerViewImpl>) GWT.create(Type_factory__o_k_w_c_s_f_c_w_c_d_FormDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultiScreenWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_MultiScreenWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LeafPermissionNodeViewerView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_LeafPermissionNodeViewerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ObserverBuilderControl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_b_i_ObserverBuilderControl__quals__j_e_i_Any_j_e_i_Default_o_k_w_c_s_c_c_c_c_b_i_Observer.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NotificationPopupsManagerView>) GWT.create(Type_factory__o_u_c_v_p_n_NotificationPopupsManagerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SplitLayoutPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_SplitLayoutPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CopyPopUpPresenter>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_CopyPopUpPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DateRangePicker>) GWT.create(Type_factory__o_u_c_v_p_w_DateRangePicker__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupContextMenuItemPresenter>) GWT.create(Type_factory__o_u_c_w_w_m_m_c_GroupContextMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefaultFileEditorView>) GWT.create(Type_factory__o_u_e_w_c_c_e_d_DefaultFileEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementViewerSession>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_CaseManagementViewerSession__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ControlPointControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_ControlPointControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuCustom>) GWT.create(JsType_factory__o_u_w_m_m_MenuCustom__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DeepCloneProcess>) GWT.create(Type_factory__o_k_w_c_s_c_d_c_DeepCloneProcess__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DateTimePickerPresenterViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_DateTimePickerPresenterViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WindowEventObservers>) GWT.create(JsType_factory__o_j_e_e_c_c_WindowEventObservers__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerSliderFieldInitializer>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_f_p_f_f_s_IntegerSliderFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LeafPermissionNodeViewer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_LeafPermissionNodeViewer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RelatedEventEventInitDictType>) GWT.create(JsType_factory__e_d_RelatedEvent_RelatedEventEventInitDictType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ForceUnlockEventObserver>) GWT.create(Type_factory__o_u_c_m_ForceUnlockEventObserver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<FormGenerationNotifier>) GWT.create(Type_factory__o_k_w_c_s_f_c_n_FormGenerationNotifier__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PageTransitionEventEventInitDictType>) GWT.create(JsType_factory__e_d_PageTransitionEvent_PageTransitionEventEventInitDictType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PrimitiveDragProxyImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_d_PrimitiveDragProxyImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchPickupDragController>) GWT.create(Type_factory__o_u_c_w_w_d_WorkbenchPickupDragController__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PanelManagerImpl>) GWT.create(Type_factory__o_u_c_w_PanelManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SplashPluginResourceType>) GWT.create(Type_factory__o_u_e_p_c_t_SplashPluginResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementCanvasCommandFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_CaseManagementCanvasCommandFactory__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SelectOptionComponent>) GWT.create(Type_factory__o_u_e_w_c_c_s_SelectOptionComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetDefinitionSetProxyImpl>) GWT.create(Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetDefinitionSetProxyImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DynamicBindingHelper>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_m_b_DynamicBindingHelper__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CheckBox>) GWT.create(ExtensionProvided_factory__o_g_c_u_CheckBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Sender>) GWT.create(ContextualProvider_factory__o_j_e_i_s_b_c_Sender__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PluginNavList>) GWT.create(Type_factory__o_u_e_p_c_w_n_PluginNavList__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AppsPerspectiveActivity>) GWT.create(Type_factory__o_u_e_a_c_AppsPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LeafPermissionNodeEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_LeafPermissionNodeEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<TextResourceType>) GWT.create(Type_factory__o_u_e_w_c_c_e_t_TextResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionPreviewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_s_i_SessionPreviewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Tree>) GWT.create(ExtensionProvided_factory__o_u_e_w_c_c_t_Tree__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SplashScreenMenuPresenter>) GWT.create(Type_factory__o_u_c_m_SplashScreenMenuPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefinitionPaletteCategoryWidget>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_c_DefinitionPaletteCategoryWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<com.google.gwt.user.client.ui.FlowPanel>) GWT.create(ExtensionProvided_factory__c_g_g_u_c_u_FlowPanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DropdownMenuItem>) GWT.create(Type_factory__o_u_c_v_p_w_DropdownMenuItem__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerPreferencesRegistries>) GWT.create(Type_factory__o_k_w_c_s_c_c_p_StunnerPreferencesRegistries__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToBpmnToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_ExportToBpmnToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkItemDefinitionCacheRegistry>) GWT.create(Type_factory__o_k_w_c_s_b_w_WorkItemDefinitionCacheRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ElementCardinalityEvaluationHandler>) GWT.create(Type_factory__o_k_w_c_s_c_r_h_i_ElementCardinalityEvaluationHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ACLSettings>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_ACLSettings__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GeneralPreferencesBeanGeneratedImpl>) GWT.create(Type_factory__o_g_c_s_p_p_GeneralPreferencesBeanGeneratedImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<FileExportScriptInjector>) GWT.create(Type_factory__o_u_e_e_c_c_f_e_j_FileExportScriptInjector__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextEditorMultiLineBox>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_a_TextEditorMultiLineBox__quals__j_e_i_Any_o_k_w_c_s_c_c_c_c_a_MultiLineTextEditorBox.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerPreferencesRegistryLoader>) GWT.create(Type_factory__o_k_w_c_s_c_c_p_StunnerPreferencesRegistryLoader__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BasicShapesFactory>) GWT.create(Type_factory__o_k_w_c_s_s_c_f_BasicShapesFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_g_GroupEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToPdfToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_ExportToPdfToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSetDefinitionAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetDefinitionAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchMegaMenuProducer>) GWT.create(Type_factory__o_u_c_w_w_m_m_WorkbenchMegaMenuProducer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<NewPluginPopUp>) GWT.create(Type_factory__o_u_e_p_c_w_p_NewPluginPopUp__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RedoToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_RedoToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ActivityDataIOEditor>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_ActivityDataIOEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PerspectiveEditorPresenterActivity>) GWT.create(Type_factory__o_u_e_p_c_p_e_PerspectiveEditorPresenterActivity__quals__j_e_i_Any_j_i_Named_o_u_c_w_a_AssociatedResources_o_u_c_w_a_Priority.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CategoriesManagerCache>) GWT.create(Type_factory__o_u_c_m_CategoriesManagerCache__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Div>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Div__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LienzoPanelUtils>) GWT.create(Type_factory__o_k_w_c_s_c_l_u_LienzoPanelUtils__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSet>) GWT.create(Type_factory__o_k_w_c_s_b_BPMNDefinitionSet__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StaticBindingHelper>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_r_m_b_StaticBindingHelper__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ResourceTypeManagerCache>) GWT.create(Type_factory__o_u_c_m_ResourceTypeManagerCache__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PictureInput>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_p_PictureInput__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextEditorView>) GWT.create(Type_factory__o_u_e_w_c_c_e_t_TextEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SaveDiagramSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_c_i_SaveDiagramSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetPropertySetAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetPropertySetAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Collection>) GWT.create(JsType_factory__j_u_Collection__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ResourceTypeDefinition>) GWT.create(JsType_factory__o_u_w_t_ResourceTypeDefinition__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextArea>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_TextArea__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StringListBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_l_StringListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MedianVertexLayerPositioning>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_l_s_s_MedianVertexLayerPositioning__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DelegateShapeFactory>) GWT.create(Type_factory__o_k_w_c_s_c_c_s_f_DelegateShapeFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Model>) GWT.create(Type_factory__o_g_c_s_p_c_Model__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StringRadioGroupFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_r_StringRadioGroupFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssigneeLiveSearchEntryCreationEditorViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_w_AssigneeLiveSearchEntryCreationEditorViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FieldLabelViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_l_l_FieldLabelViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StaticFormHandlerGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_i_i_StaticFormHandlerGenerator__quals__j_e_i_Any_o_k_w_c_f_d_s_s_StaticContext.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<JSEntryPoint>) GWT.create(Type_factory__o_u_c_JSEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TemplatedWorkbenchPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_TemplatedWorkbenchPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariablesEditorFieldInitializer>) GWT.create(Type_factory__o_k_w_c_s_b_f_s_a_p_p_f_VariablesEditorFieldInitializer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<AuthorizationManagerHelper>) GWT.create(Type_factory__o_u_s_c_a_AuthorizationManagerHelper__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<User>) GWT.create(Producer_factory__o_j_e_s_s_a_i_User__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DataBinderProvider>) GWT.create(Type_factory__o_j_e_d_c_DataBinderProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AnimatedFloatingWidgetView>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_AnimatedFloatingWidgetView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PageTransitionProvider>) GWT.create(Type_factory__o_j_e_u_n_c_l_PageTransitionProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSetDefinitionSetAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetDefinitionSetAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchPartPresenterDefault>) GWT.create(Type_factory__o_u_c_w_p_n_p_WorkbenchPartPresenterDefault__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GenerateProcessFormsSessionCommand>) GWT.create(Type_factory__o_k_w_c_s_f_c_s_c_GenerateProcessFormsSessionCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TimerSettingsFieldEditorView>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_t_TimerSettingsFieldEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SavePopUpView>) GWT.create(Type_factory__o_u_e_e_c_c_f_p_SavePopUpView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ManagedViewerToolbar>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_i_ManagedViewerToolbar__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DecimalBoxViewImpl>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_d_DecimalBoxViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<AppsPerspective>) GWT.create(Type_factory__o_u_e_a_c_AppsPerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PlaceManager>) GWT.create(JsType_factory__o_u_c_m_PlaceManager__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSetModelFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_f_CaseManagementDefinitionSetModelFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClassUtils>) GWT.create(Type_factory__o_k_w_c_s_c_u_ClassUtils__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CachedRuleManager>) GWT.create(Type_factory__o_k_w_c_s_c_r_CachedRuleManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<FormBuildingServiceImpl>) GWT.create(Type_factory__o_k_w_c_f_a_e_s_i_FormBuildingServiceImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<TransitionAnchorProvider>) GWT.create(Type_factory__o_j_e_u_n_c_l_TransitionAnchorProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserMenu>) GWT.create(Type_factory__o_u_c_v_p_m_UserMenu__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CompassWidgetImpl>) GWT.create(Type_factory__o_u_c_v_p_d_CompassWidgetImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ToolBarItem>) GWT.create(JsType_factory__o_u_w_m_t_ToolBarItem__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchMenuCompactNavBarView>) GWT.create(Type_factory__o_u_c_v_p_m_WorkbenchMenuCompactNavBarView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ClientFactoryManager>) GWT.create(Type_factory__o_k_w_c_s_c_c_a_ClientFactoryManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserAssignedRolesEditor>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_UserAssignedRolesEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSetPropertyAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetPropertyAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeysEditorScreenActivity>) GWT.create(Type_factory__o_u_s_c_e_SSHKeysEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementEditorToolbar>) GWT.create(Type_factory__o_k_w_c_s_c_c_t_i_CaseManagementEditorToolbar__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLLabelElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLLabelElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNSessionInitializer>) GWT.create(Type_factory__o_k_w_c_s_b_c_s_BPMNSessionInitializer__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PerspectiveEditorPresenter>) GWT.create(Type_factory__o_u_e_p_c_p_e_PerspectiveEditorPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PopupViewImpl>) GWT.create(Type_factory__o_u_c_v_p_p_PopupViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GeneralPluginEditor>) GWT.create(Type_factory__o_u_e_p_c_w_p_GeneralPluginEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CreateEntityView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_CreateEntityView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConfirmPopup>) GWT.create(Type_factory__o_u_c_v_p_w_ConfirmPopup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<HomePerspective>) GWT.create(Type_factory__o_k_w_c_s_s_c_p_HomePerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JsTypeEventObserver>) GWT.create(Type_factory__o_j_e_e_c_c_JsTypeEventObserver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNShapeFactory>) GWT.create(Type_factory__o_k_w_c_s_b_c_s_f_BPMNShapeFactory__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ColumnWithComponents>) GWT.create(Type_factory__o_u_e_l_e_c_c_c_ColumnWithComponents__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<WorkbenchToolBarPresenter>) GWT.create(Type_factory__o_u_c_w_w_t_WorkbenchToolBarPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DisabledFeatureComponent>) GWT.create(Type_factory__o_u_e_c_d_c_DisabledFeatureComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LockTarget>) GWT.create(JsType_factory__o_u_c_m_LockTarget__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerBoxViewImpl>) GWT.create(Type_factory__o_k_w_c_f_c_r_c_w_i_IntegerBoxViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClearToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_ClearToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BaseEditorEntryPoint>) GWT.create(Type_factory__o_u_e_e_c_c_BaseEditorEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FormDisplayer>) GWT.create(Type_factory__o_k_w_c_s_f_c_w_c_d_FormDisplayer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MultipleInput>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_MultipleInput__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DefineOptionsType>) GWT.create(JsType_factory__e_d_CustomElementRegistry_DefineOptionsType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IntegerBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_IntegerBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<EditorPluginResourceType>) GWT.create(Type_factory__o_u_e_p_c_t_EditorPluginResourceType__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LayoutEditorCssHelper>) GWT.create(Type_factory__o_u_e_l_e_c_i_LayoutEditorCssHelper__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NoItemsComponent>) GWT.create(Type_factory__o_u_e_w_c_c_d_n_NoItemsComponent__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AssignmentsEditorFieldRenderer>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_AssignmentsEditorFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<FileExportProducer>) GWT.create(Type_factory__o_u_e_e_c_c_f_e_FileExportProducer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Position>) GWT.create(JsType_factory__o_u_w_m_Position__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HTMLTableRowElement>) GWT.create(ExtensionProvided_factory__e_d_HTMLTableRowElement__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EditorPlugInEditorView>) GWT.create(Type_factory__o_u_e_p_c_e_EditorPlugInEditorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<NewSSHKeyModalViewImpl>) GWT.create(Type_factory__o_u_s_c_e_c_c_NewSSHKeyModalViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SSHKeysEditorEmptyStateDisplayerViewImpl>) GWT.create(Type_factory__o_u_s_c_e_c_e_SSHKeysEditorEmptyStateDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PanelDefinition>) GWT.create(JsType_factory__o_u_w_m_PanelDefinition__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AppendNoneTaskShortcut>) GWT.create(Type_factory__o_k_w_c_s_b_c_c_c_k_s_AppendNoneTaskShortcut__quals__j_e_i_Any_o_k_w_c_s_b_q_BPMN.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BasicUserCacheImpl>) GWT.create(Type_factory__o_j_e_s_c_l_c_BasicUserCacheImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<KeyboardControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_k_KeyboardControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TreeWalkTraverseProcessorImpl>) GWT.create(Type_factory__o_k_w_c_s_c_g_p_t_t_TreeWalkTraverseProcessorImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CopyPopUpViewProducer>) GWT.create(Type_factory__o_u_e_e_c_c_f_CopyPopUpViewProducer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MediaLibraryWidget>) GWT.create(Type_factory__o_u_e_p_c_w_m_MediaLibraryWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<EditorJSExporter>) GWT.create(Type_factory__o_u_c_e_EditorJSExporter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClientBindableMorphAdapter>) GWT.create(Type_factory__o_k_w_c_s_c_c_d_a_b_ClientBindableMorphAdapter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionContainer>) GWT.create(ExtensionProvided_factory__o_k_w_c_s_c_w_p_s_i_SessionContainer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<RuntimePluginsEntryPoint>) GWT.create(Type_factory__o_u_e_p_c_RuntimePluginsEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Heading>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Heading__quals__j_e_i_Any_j_e_i_Default_j_i_Named_1.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TreeExplorerView>) GWT.create(Type_factory__o_k_w_c_s_c_w_e_t_TreeExplorerView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<AuthoringPerspective>) GWT.create(Type_factory__o_k_w_c_s_s_c_p_AuthoringPerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExportToSvgToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_w_t_c_ExportToSvgToolbarCommand__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TitleProvider>) GWT.create(JsType_factory__o_u_c_m_LockTarget_TitleProvider__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SimpleTable>) GWT.create(Type_factory__o_u_e_w_c_c_t_SimpleTable__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EnumListBoxFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_s_l_EnumListBoxFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SliderFormGroup>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_s_SliderFormGroup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<com.google.gwt.user.client.ui.Label>) GWT.create(ExtensionProvided_factory__c_g_g_u_c_u_Label__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WindowEventObservers>) GWT.create(Type_factory__o_j_e_e_c_c_WindowEventObservers__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ActivityDataIOEditorViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_a_ActivityDataIOEditorViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<FloatEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_n_d_FloatEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SecurityContextHoldingSingleton>) GWT.create(Type_factory__o_j_e_s_c_l_n_SecurityContextHoldingSingleton__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultTextPropertyProviderImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_a_DefaultTextPropertyProviderImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BusyIndicatorView>) GWT.create(Type_factory__o_u_e_w_c_c_c_BusyIndicatorView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutEditorPropertiesView>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutEditorPropertiesView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuItemDividerView>) GWT.create(Type_factory__o_u_e_w_c_c_m_MenuItemDividerView__quals__j_e_i_Any_o_u_e_w_c_c_m_MenuItemDivider.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UserAssignedRolesExplorer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_u_UserAssignedRolesExplorer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ModalFormDisplayer>) GWT.create(Type_factory__o_k_w_c_f_c_c_c_f_m_ModalFormDisplayer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EmbeddedFormDisplayer>) GWT.create(Type_factory__o_k_w_c_f_c_c_c_f_e_EmbeddedFormDisplayer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<NavigationPanelProvider>) GWT.create(Type_factory__o_j_e_u_n_c_l_NavigationPanelProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerPreferencesEntryPoint>) GWT.create(Type_factory__o_k_w_c_s_c_c_p_StunnerPreferencesEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ConfirmBox>) GWT.create(Type_factory__o_u_e_s_m_c_w_p_ConfirmBox__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DiagramPresenterScreenActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_DiagramPresenterScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<JsArray>) GWT.create(JsType_factory__o_j_e_i_c_JsArray__quals__Universal.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ExperimentalFeaturesPerspective>) GWT.create(Type_factory__o_u_e_c_p_ExperimentalFeaturesPerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementSVGViewFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_r_CaseManagementSVGViewFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CompassPosition>) GWT.create(JsType_factory__o_u_w_m_CompassPosition__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ExperimentalFeaturesPerspectiveActivity>) GWT.create(Type_factory__o_u_e_c_p_ExperimentalFeaturesPerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ActivityManagerImpl>) GWT.create(Type_factory__o_u_c_m_ActivityManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<EditableColumnGeneratorManagerImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_EditableColumnGeneratorManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VersionMenuDropDownButton>) GWT.create(Type_factory__o_u_e_e_c_c_h_VersionMenuDropDownButton__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RoleEditorDriver>) GWT.create(Type_factory__o_u_e_s_m_c_e_r_RoleEditorDriver__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PermissionSwitchView>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionSwitchView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuVisitor>) GWT.create(JsType_factory__o_u_w_m_m_MenuVisitor__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RolesListItemWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_c_r_RolesListItemWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TemplatedWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_TemplatedWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<DynamicFormRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_DynamicFormRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScreenPanelViewImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_s_ScreenPanelViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SvgDataUriGenerator>) GWT.create(Type_factory__o_k_w_c_s_c_c_u_SvgDataUriGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SliderFieldRenderer>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_SliderFieldRenderer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PreferenceScopeResolutionStrategyIOCProvider>) GWT.create(Type_factory__o_u_p_c_s_PreferenceScopeResolutionStrategyIOCProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BooleanEditableColumnGenerator>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_r_l_c_i_w_i_BooleanEditableColumnGenerator__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveJSExporter>) GWT.create(Type_factory__o_u_c_e_PerspectiveJSExporter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StateChangeImpl>) GWT.create(Type_factory__o_j_e_i_c_l_i_StateChangeImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RequestDispatcher>) GWT.create(Provider_factory__o_j_e_b_c_a_m_RequestDispatcher__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementContainmentAcceptorControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_c_CaseManagementContainmentAcceptorControlImpl__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SSHClientEntryPoint>) GWT.create(Type_factory__o_u_s_c_SSHClientEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<StunnerWidgetsEntryPoint>) GWT.create(Type_factory__o_k_w_c_s_c_w_StunnerWidgetsEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<BPMNDiagramFactoryImpl>) GWT.create(Type_factory__o_k_w_c_s_b_f_BPMNDiagramFactoryImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<KebabWidget>) GWT.create(Type_factory__o_u_e_l_e_c_w_KebabWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ZoomWheelControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_l_c_c_ZoomWheelControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<BPMNDefinitionSetPropertyAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_b_d_a_b_BPMNDefinitionSetPropertyAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ErrorPopup>) GWT.create(Type_factory__o_u_c_v_p_w_ErrorPopup__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ScriptTypeListFieldEditorWidget>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_s_ScriptTypeListFieldEditorWidget__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EditHTMLPresenter>) GWT.create(Type_factory__o_u_e_p_c_p_e_l_e_p_EditHTMLPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TextEditorSingleLineBoxView>) GWT.create(Type_factory__o_k_w_c_s_c_w_c_a_TextEditorSingleLineBoxView__quals__j_e_i_Any_o_k_w_c_s_c_c_c_c_a_SingleLineTextEditorBox.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<TranslationServiceProvider>) GWT.create(Type_factory__o_j_e_u_c_l_s_TranslationServiceProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AceEditor>) GWT.create(ExtensionProvided_factory__o_u_e_w_c_c_a_AceEditor__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClientUsernameProvider>) GWT.create(Type_factory__o_u_p_c_s_ClientUsernameProvider__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<VFSJSExporter>) GWT.create(Type_factory__o_u_c_e_VFSJSExporter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PerspectiveDropDown>) GWT.create(Type_factory__o_u_e_w_c_c_d_PerspectiveDropDown__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<EdgeBuilderControlImpl>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_c_b_i_EdgeBuilderControlImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WelcomeScreen>) GWT.create(Type_factory__o_k_w_c_s_s_c_s_WelcomeScreen__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RedoCommandHandler>) GWT.create(Type_factory__o_k_w_c_s_c_c_u_RedoCommandHandler__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<AdaptiveWorkbenchPanelPresenter>) GWT.create(Type_factory__o_u_c_w_p_i_AdaptiveWorkbenchPanelPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementCanvasPresenter>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_CaseManagementCanvasPresenter__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<MenuDevCommandsBuilder>) GWT.create(Type_factory__o_k_w_c_s_c_w_m_d_MenuDevCommandsBuilder__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<VariablesEditorWidgetViewImpl>) GWT.create(Type_factory__o_k_w_c_s_b_c_f_f_v_VariablesEditorWidgetViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupEditorScreenActivity>) GWT.create(Type_factory__o_u_e_s_m_c_s_e_GroupEditorScreenActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultBusSecurityErrorCallback>) GWT.create(Type_factory__o_j_e_s_c_l_c_DefaultBusSecurityErrorCallback__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<PermissionExceptionSwitch>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_n_PermissionExceptionSwitch__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PluginConfigService>) GWT.create(Type_factory__o_u_e_p_c_c_PluginConfigService__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CheckBoxFormGroupViewImpl>) GWT.create(Type_factory__o_k_w_c_f_d_c_r_f_i_c_CheckBoxFormGroupViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SessionViewerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_w_p_s_i_SessionViewerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementDefinitionSetRuleAdapterImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_a_b_CaseManagementDefinitionSetRuleAdapterImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CloneManagerImpl>) GWT.create(Type_factory__o_k_w_c_s_c_d_c_CloneManagerImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<RequestCommandManager>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_RequestCommandManager__quals__j_e_i_Any_o_k_w_c_s_c_c_c_Request.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ClientResourceType>) GWT.create(JsType_factory__o_u_c_w_t_ClientResourceType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.kie.workbench.common.stunner.client.widgets.views.LoadingBoxView>) GWT.create(Type_factory__o_k_w_c_s_c_w_v_LoadingBoxView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefinitionUtils>) GWT.create(Type_factory__o_k_w_c_s_c_u_DefinitionUtils__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<HomePerspectiveActivity>) GWT.create(Type_factory__o_k_w_c_s_s_c_p_HomePerspectiveActivity__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ActivityResourceType>) GWT.create(JsType_factory__o_u_w_m_ActivityResourceType__quals__Universal.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<UnanchoredWorkbenchPartPresenter>) GWT.create(Type_factory__o_u_c_w_p_u_p_UnanchoredWorkbenchPartPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<Decision>) GWT.create(Type_factory__o_g_c_s_p_c_Decision__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<GroupContextMenuItemView>) GWT.create(Type_factory__o_u_c_v_p_m_m_c_GroupContextMenuItemView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<SyncBeanManager>) GWT.create(Producer_factory__o_j_e_i_c_c_SyncBeanManager__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IFrameScreenPresenter>) GWT.create(Type_factory__o_u_e_w_c_c_s_i_IFrameScreenPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<Row>) GWT.create(Type_factory__o_u_e_l_e_c_c_r_Row__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<DefaultPreferencesRegistry>) GWT.create(Type_factory__o_k_w_c_s_c_c_p_DefaultPreferencesRegistry__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ModalFormDisplayerViewImpl>) GWT.create(Type_factory__o_k_w_c_f_c_c_c_f_m_ModalFormDisplayerViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<LayoutEditorPropertiesPresenter>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutEditorPropertiesPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<CaseManagementVisitGraphToolbarCommand>) GWT.create(Type_factory__o_k_w_c_s_c_c_t_c_CaseManagementVisitGraphToolbarCommand__quals__j_e_i_Any_o_k_w_c_s_c_q_CaseManagementEditor.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<org.jboss.errai.common.client.dom.Button>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_Button__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<SSHKeysEditorPerspective>) GWT.create(Type_factory__o_u_s_c_e_SSHKeysEditorPerspective__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutElementPropertiesView>) GWT.create(Type_factory__o_u_e_l_e_c_w_LayoutElementPropertiesView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WizardViewImpl>) GWT.create(Type_factory__o_u_e_w_c_c_w_WizardViewImpl__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ResourceTypeDefinitionJSExporter>) GWT.create(Type_factory__o_u_c_e_ResourceTypeDefinitionJSExporter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<StaticFocusedResizePanel>) GWT.create(ExtensionProvided_factory__o_u_c_w_w_p_StaticFocusedResizePanel__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<ResourceBundleEntryPoint>) GWT.create(Type_factory__o_u_e_s_m_c_ResourceBundleEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<WorkbenchPartView>) GWT.create(Type_factory__o_u_c_w_p_n_p_WorkbenchPartView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CaseManagementEntryPoint>) GWT.create(Type_factory__o_k_w_c_s_c_CaseManagementEntryPoint__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ChildContextMenuItemPresenter>) GWT.create(Type_factory__o_u_c_w_w_m_m_c_ChildContextMenuItemPresenter__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<CanvasFileExport>) GWT.create(Type_factory__o_k_w_c_s_c_c_c_u_CanvasFileExport__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<PerspectiveEditorSettings>) GWT.create(Type_factory__o_u_e_p_c_p_e_l_e_PerspectiveEditorSettings__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<TranslationService>) GWT.create(Provider_factory__o_j_e_u_c_l_s_TranslationService__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<IconMenuItemView>) GWT.create(Type_factory__o_u_c_v_p_m_m_m_IconMenuItemView__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<RadioInput>) GWT.create(ExtensionProvided_factory__o_j_e_c_c_d_RadioInput__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
    org_jboss_errai_ioc_client_container_ApplicationScopedContext_context.registerFactory((Factory<UberfireDocksContainer>) GWT.create(Type_factory__o_u_c_w_d_UberfireDocksContainer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<ACLViewer>) GWT.create(Type_factory__o_u_e_s_m_c_w_m_e_a_ACLViewer__quals__j_e_i_Any_j_e_i_Default.class));
    org_jboss_errai_ioc_client_container_DependentScopeContext_context.registerFactory((Factory<LayoutPanelView>) GWT.create(Type_factory__o_u_c_w_p_i_LayoutPanelView__quals__j_e_i_Any_j_e_i_Default_j_i_Named.class));
  }

  private void registerFactories(final Context org_jboss_errai_ioc_client_container_ApplicationScopedContext_context, final Context org_jboss_errai_ioc_client_container_DependentScopeContext_context) {
    registerFactories0(org_jboss_errai_ioc_client_container_ApplicationScopedContext_context, org_jboss_errai_ioc_client_container_DependentScopeContext_context);
    registerFactories1(org_jboss_errai_ioc_client_container_ApplicationScopedContext_context, org_jboss_errai_ioc_client_container_DependentScopeContext_context);
    registerFactories2(org_jboss_errai_ioc_client_container_ApplicationScopedContext_context, org_jboss_errai_ioc_client_container_DependentScopeContext_context);
  }

  // The main IOC bootstrap method.
  public ContextManager bootstrapContainer() {
    final Context org_jboss_errai_ioc_client_container_ApplicationScopedContext_context = new ApplicationScopedContext();
    final Context org_jboss_errai_ioc_client_container_DependentScopeContext_context = new DependentScopeContext();
    logger.debug("Registering factories with contexts.");
    long start = System.currentTimeMillis();
    registerFactories(org_jboss_errai_ioc_client_container_ApplicationScopedContext_context, org_jboss_errai_ioc_client_container_DependentScopeContext_context);
    logger.debug("Registered 1276 factories in {}ms", System.currentTimeMillis() - start);
    logger.debug("Adding contexts to context manager...");
    start = System.currentTimeMillis();
    contextManager.addContext(org_jboss_errai_ioc_client_container_ApplicationScopedContext_context);
    contextManager.addContext(org_jboss_errai_ioc_client_container_DependentScopeContext_context);
    logger.debug("Added 2 contexts in {}ms", System.currentTimeMillis() - start);
    logger.debug("Calling finishInit on ContextManager");
    start = System.currentTimeMillis();
    contextManager.finishInit();
    logger.debug("ContextManager#finishInit ran in {}ms", System.currentTimeMillis() - start);
    return contextManager;
  }
}